Imports System.ComponentModel
Imports System.Drawing.Design
Imports System.ComponentModel.Design.Serialization
Imports System.Reflection
Imports System.Globalization
Imports System
Imports System.Windows.Forms
Imports System.Collections

Imports Stax.UI

Namespace ComponentModel
    <AttributeUsage(AttributeTargets.All)> _
    Public Class DisplayInfoAttribute
        Inherits DisplayNameAttribute

        Public Sub New(ByVal name As String)
            DisplayNameValue = name
        End Sub

        Public Shared Function GetValueForEnum(Of T)(ByVal value As T) As String
            For Each i In value.GetType.GetFields
                If i.GetValue(value).Equals(value) Then
                    For Each i2 In i.GetCustomAttributes(False)
                        If i2.GetType Is GetType(DisplayInfoAttribute) Then
                            Return DirectCast(i2, DisplayInfoAttribute).DisplayName
                        End If
                    Next

                    Return i.Name
                End If
            Next

            Throw New Exception
        End Function

        Public Shared Function GetNamesForEnum(Of T)() As String()
            Dim l As New List(Of String)

            For Each i As T In System.Enum.GetValues(GetType(T))
                l.Add(GetValueForEnum(Of T)(i))
            Next

            Return l.ToArray
        End Function

        Public Shared Function GetValue(ByVal attributes As Object()) As String
            For Each i In attributes
                If TypeOf i Is DisplayInfoAttribute Then
                    Return DirectCast(i, DisplayInfoAttribute).DisplayName
                End If
            Next

            Return Nothing
        End Function
    End Class

    <AttributeUsage(AttributeTargets.All)> _
    Public Class OrderAttribute
        Inherits Attribute

        Public Sub New(ByVal order As Integer)
            Me.Order = order
        End Sub

        Private OrderValue As Integer

        Public Property Order() As Integer
            Get
                Return OrderValue
            End Get
            Set(ByVal Value As Integer)
                OrderValue = Value
            End Set
        End Property
    End Class

    Public Class PropertyOrderConverter
        Inherits TypeConverter

        Public Overloads Overrides Function GetPropertiesSupported(ByVal context As ITypeDescriptorContext) As Boolean
            Return True
        End Function

        Public Overloads Overrides Function GetProperties(ByVal context As ITypeDescriptorContext, ByVal value As Object, ByVal attributes As Attribute()) As PropertyDescriptorCollection
            Dim c As PropertyDescriptorCollection = TypeDescriptor.GetProperties(value, attributes)
            Dim s As New Sorter

            For Each i As PropertyDescriptor In c
                Dim oa As OrderAttribute = DirectCast(i.Attributes(GetType(OrderAttribute)), OrderAttribute)

                If oa Is Nothing Then
                    s.Add(999, i)
                Else
                    s.Add(oa.Order, i)
                End If
            Next

            Dim l As New List(Of String)

            For Each i As PropertyDescriptor In s.GetSortedList
                l.Add(i.Name)
            Next

            Return c.Sort(l.ToArray)
        End Function
    End Class

    Public Class DescriptionAttributeHelp
        Public Shared Function GetDescription(ByVal attributes As Object()) As String
            For Each i In attributes
                If TypeOf i Is DescriptionAttribute Then
                    Return DirectCast(i, DescriptionAttribute).Description
                End If
            Next

            Return Nothing
        End Function
    End Class

    Public Class EditorAttributeHelp
        Public Shared Function GetEditor(ByVal attributes As Object()) As Type
            For Each i In attributes
                If TypeOf i Is EditorAttribute Then
                    Return Type.GetType(DirectCast(i, EditorAttribute).EditorTypeName)
                End If
            Next

            Return Nothing
        End Function
    End Class
End Namespace