﻿Imports System.Runtime.CompilerServices
Imports System.Reflection
Imports System.IO

Public Module MiscExtensions
    <Extension()> _
    Public Function Sort(Of T)(ByVal value As IEnumerable(Of T)) As IEnumerable(Of T)
        Dim a As T() = value.ToArray
        Array.Sort(Of T)(a)
        Return a
    End Function

    <Extension()> _
    Public Function Join(ByVal value As IEnumerable(Of String), ByVal delimiter As String) As String
        Return String.Join(delimiter, value.ToArray)
    End Function

    <Extension()> _
    Public Function GetAttribute(Of T)(ByVal mi As MemberInfo) As T
        Dim attributes = mi.GetCustomAttributes(True)

        If OK(attributes) Then
            If attributes.Length = 1 Then
                If TypeOf attributes(0) Is T Then
                    Return DirectCast(attributes(0), T)
                End If
            Else
                For Each i In attributes
                    If TypeOf i Is T Then
                        Return DirectCast(i, T)
                    End If
                Next
            End If
        End If
    End Function

    <Extension()> _
    Public Function IsDigit(ByVal c As Char) As Boolean
        Return Char.IsDigit(c)
    End Function
End Module

Public Module ControlExtensions
    <Extension()> _
    Public Function ClientMousePos(ByVal c As Control) As Point
        Return c.PointToClient(Control.MousePosition)
    End Function

    <Extension()> _
    Public Sub SetDefaultPath(ByVal d As FolderBrowserDialog, ByVal path As String)
        If Directory.Exists(path) Then
            d.SelectedPath = path
        End If
    End Sub

    <Extension()> _
    Public Sub SetWidth(ByVal parent As Control, ByVal child As Control, ByVal items As IEnumerable, ByVal offset As Integer)
        Dim max As Integer

        For Each i In items
            Dim w = TextRenderer.MeasureText(i.ToString, child.Font).Width + offset

            If w > max Then
                max = w
            End If
        Next

        If parent.Width < max + (parent.Width - child.Width) Then
            parent.Width = max + (parent.Width - child.Width)
        End If
    End Sub

    <Extension()> _
    Public Function RemoveAndDispose(ByVal c As Control, ByVal ParamArray controls As Control()) As Point
        For Each i In controls
            c.Controls.Remove(i)
            i.Dispose()
        Next
    End Function
End Module