﻿Imports System.Text
Imports System.IO
Imports System.Security.Cryptography
Imports System.Runtime.CompilerServices

Imports VB6 = Microsoft.VisualBasic

Public Module StringExtensions
    <Extension()> _
    Public Function ContainsUnicode(ByVal value As String) As Boolean
        For Each i In value
            If Convert.ToInt32(i) > 255 Then
                Return True
            End If
        Next

        Return False
    End Function

    '.NET method CultureInfo.CurrentCulture.TextInfo.ToTitleCase
    'don't convert all upper string like 'ALL UPPER'
    <Extension()> _
    Public Function ToTitleCase(ByVal value As String) As String
        Dim chars = value.ToLower.ToCharArray

        For i = 0 To chars.Length - 1
            If i = 0 OrElse (i > 0 AndAlso Not Char.IsLetter(chars(i - 1))) Then
                chars(i) = CChar(chars(i).ToString.ToUpper)
            End If
        Next

        Return New String(chars)
    End Function

    <Extension()> _
    Public Function ToInt(ByVal value As String) As Integer
        Return ToInt(value, 0)
    End Function

    <Extension()> _
    Public Function IsInt(ByVal value As String) As Boolean
        Return Integer.TryParse(value, Nothing)
    End Function

    <Extension()> _
    Public Function ToInt(ByVal value As String, ByVal defaultValue As Integer) As Integer
        If Not Integer.TryParse(value, Nothing) Then
            Return defaultValue
        End If

        Return CInt(value)
    End Function

    <Extension()> _
    Public Sub WriteFile(ByVal value As String, ByVal path As String)
        WriteFile(value, path, Encoding.Default)
    End Sub

    <Extension()> _
    Public Sub WriteFile(ByVal value As String, ByVal path As String, ByVal encoding As Encoding)
        Using sw As New StreamWriter(File.Create(path), encoding)
            sw.Write(value)
        End Using
    End Sub

    <Extension()> _
    Public Function RightLast(ByVal value As String, ByVal start As String) As String
        If Not value.Contains(start) Then
            Return ""
        End If

        Return value.Substring(value.LastIndexOf(start) + start.Length)
    End Function

    <Extension()> _
    Public Function Left(ByVal value As String, ByVal index As Integer) As String
        If Not OK(value) OrElse index < 0 Then
            Return ""
        End If

        If index > value.Length Then
            Return value
        End If

        Return value.Substring(0, index)
    End Function

    <Extension()> _
    Public Function Left(ByVal value As String, ByVal start As String) As String
        If Not value.Contains(start) Then
            Return ""
        End If

        Return value.Substring(0, value.IndexOf(start))
    End Function

    <Extension()> _
    Public Function FormatColumn(ByVal value As String, ByVal delimiter As String) As String
        Dim positions As New List(Of Integer)
        Dim lines = value.SplitLines

        For Each i In lines
            positions.Add(i.IndexOf(delimiter))
        Next

        If positions.Count > 0 Then
            Dim highest = Aggregate i In positions Into Max(i)

            For i = 0 To lines.Length - 1
                If positions(i) <> highest AndAlso positions(i) > -1 Then
                    lines(i) = lines(i).Insert(positions(i), " ".PadRight(highest - positions(i)))
                End If
            Next
        End If

        Return lines.Join(CrLf)
    End Function

    <Extension()> _
    Public Function EqualIgnoreCase(ByVal a As String, ByVal b As String) As Boolean
        If Not OK(a, b) Then
            Return False
        End If

        Return String.Compare(a, b, StringComparison.OrdinalIgnoreCase) = 0
    End Function

    <Extension()> _
    Public Function LeftLast(ByVal value As String, ByVal start As String) As String
        If Not value.Contains(start) Then
            Return ""
        End If

        Return value.Substring(0, value.LastIndexOf(start))
    End Function

    <Extension()> _
    Public Function Right(ByVal value As String, ByVal index As Integer) As String
        If Not OK(value) OrElse index < 0 Then
            Return ""
        End If

        If index > value.Length Then
            Return value
        End If

        Return value.Substring(value.Length - index)
    End Function

    <Extension()> _
    Public Function Right(ByVal value As String, ByVal start As String) As String
        If Not value.Contains(start) Then
            Return ""
        End If

        Return value.Substring(value.IndexOf(start) + start.Length)
    End Function

    <Extension()> _
    Public Function Shorten(ByVal value As String, ByVal maxLength As Integer) As String
        If Not OK(value) OrElse value.Length <= maxLength Then
            Return value
        End If

        Return value.Substring(0, maxLength)
    End Function

    <Extension()> _
    Public Function FixBreak(ByVal value As String) As String
        value = value.Replace(CrLf, VB6.vbLf)
        value = value.Replace(VB6.vbCr, VB6.vbLf)
        Return value.Replace(VB6.vbLf, CrLf)
    End Function

    <Extension()> _
    Public Function Split(ByVal value As String, ByVal ParamArray delimiters As String()) As String()
        Dim a = value.Split(delimiters, StringSplitOptions.RemoveEmptyEntries)

        Dim emptyFound As Boolean

        For Each i In a
            If Not OK(i) Then
                emptyFound = True
            End If
        Next

        If emptyFound Then
            Dim l As New List(Of String)

            For Each i In a
                If OK(i) Then
                    l.Add(i)
                End If
            Next

            a = l.ToArray
        End If

        Return a
    End Function

    <Extension()> _
    Public Function Split(ByVal value As String, ByVal trimSpaces As Boolean, ByVal ParamArray delimiters As String()) As String()
        Dim a = Split(value, delimiters)

        If trimSpaces Then
            Dim emptyFound As Boolean

            For i = 0 To a.Length - 1
                If a(i).First = " " Then
                    a(i) = a(i).TrimStart
                End If

                If a(i) = "" Then
                    emptyFound = True
                Else
                    If a(i).Last = " " Then
                        a(i) = a(i).TrimEnd

                        If a(i) = "" Then
                            emptyFound = True
                        End If
                    End If
                End If
            Next

            If emptyFound Then
                Dim l As New List(Of String)

                For Each i In a
                    If OK(i) Then
                        l.Add(i)
                    End If
                Next

                a = l.ToArray
            End If
        End If

        Return a
    End Function

    <Extension()> _
    Public Function SplitNonLetterOrDigit(ByVal value As String) As String()
        For Each i In value
            If Not Char.IsLetterOrDigit(i) AndAlso i <> " " Then
                value = value.Replace(i, " "c)
            End If
        Next

        Return value.Split(" ")
    End Function

    <Extension()> _
    Public Function SplitLines(ByVal value As String) As String()
        Return Split(value, Environment.NewLine)
    End Function

    <Extension()> _
    Public Function AllowedChars(ByVal value As String, ByVal chars As String) As String
        Dim ret As String = value

        For Each i As Char In value
            If chars.IndexOf(i) = -1 Then
                ret = ret.Replace(i, "")
            End If
        Next

        Return ret
    End Function

    <Extension()> _
    Public Function TrimStart(ByVal value As String, ByVal trimValue As String) As String
        If Not OK(value) Then
            Return ""
        End If

        If Not OK(trimValue) Then
            Return value
        End If

        If value.StartsWith(trimValue) Then
            Return Right(value, trimValue)
        End If

        Return value
    End Function

    <Extension()> _
    Public Function DeleteLeft(ByVal value As String, ByVal count As Integer) As String
        Return Right(value, value.Length - count)
    End Function

    <Extension()> _
    Public Function DeleteRight(ByVal value As String, ByVal count As Integer) As String
        Return Left(value, value.Length - count)
    End Function

    <Extension()> _
    Public Function ReplaceUnicode(ByVal value As String) As String
        If value.Contains(Convert.ToChar(&H2212)) Then
            value = value.Replace(Convert.ToChar(&H2212), "-"c)
        End If

        Return value
    End Function

    <Extension()> _
    Public Function MD5Hash(ByVal value As String) As String
        Dim crypt As MD5 = MD5CryptoServiceProvider.Create()
        Dim hash As Byte() = crypt.ComputeHash(ASCIIEncoding.ASCII.GetBytes(value))
        Dim sb As New StringBuilder()

        For Each i As Byte In hash
            sb.Append(i.ToString("x2"))
        Next

        Return sb.ToString()
    End Function

    <Extension()> _
    Public Function Path(ByVal value As String) As Filepath
        Return New Filepath(value)
    End Function

    <Extension()> _
    Public Sub ToClipboard(ByVal value As String)
        If OK(value) Then
            Clipboard.SetText(value)
        Else
            Clipboard.Clear()
        End If
    End Sub
End Module