Imports Microsoft.Win32
Imports System.Globalization
Imports System.IO
Imports System.Reflection
Imports System.Runtime.InteropServices
Imports System.Runtime.Serialization
Imports System.Runtime.Serialization.Formatters.Binary
Imports System.Text
Imports System.Text.RegularExpressions
Imports System.Threading
Imports System.Windows.Forms
Imports System.Security.Cryptography
Imports System.ComponentModel
Imports System.Xml.Serialization
Imports System.Xml

Imports Stax.UI

Public Class StringHelp
    Public Shared Function ReadFile(ByVal path As String) As String
        Using sr As New StreamReader(File.OpenRead(path))
            Return sr.ReadToEnd()
        End Using
    End Function
End Class

Public Class ObjectHelp
    'parse recursive serializable fields and lists 
    Public Shared Function GetCompareString(ByVal obj As Object) As String
        Dim sb As New StringBuilder
        ParseCompareString(obj, Nothing, sb)
        Return sb.ToString
    End Function

    Public Shared Sub ParseCompareString(ByVal obj As Object, ByVal declaringObj As Object, ByVal sb As StringBuilder)
        If TypeOf obj Is ICollection Then
            For Each i As Object In DirectCast(obj, ICollection)
                If IsGoodType(i) Then
                    If IsToString(i) Then
                        sb.Append(i.ToString)
                    Else
                        ParseCompareString(i, obj, sb)
                    End If
                End If
            Next
        Else
            If IsGoodType(obj) Then
                Dim t = obj.GetType

                While Not t Is Nothing
                    For Each i As FieldInfo In t.GetFields(BindingFlags.Public Or BindingFlags.NonPublic Or BindingFlags.Instance Or BindingFlags.DeclaredOnly)
                        If Not i.IsNotSerialized Then
                            Dim o = i.GetValue(obj)

                            If IsGoodType(o) Then
                                If IsToString(o) Then
                                    sb.Append(i.Name + "=" + o.ToString + CrLf)
                                    'sb.Append(i.Name + "=" + o.ToString + " [" + t.Name + "]" + CrLf)
                                Else
                                    If Not o Is declaringObj Then
                                        ParseCompareString(o, obj, sb)
                                    End If
                                End If
                            End If
                        End If
                    Next

                    t = t.BaseType
                End While
            End If
        End If
    End Sub

    Private Shared Function IsGoodType(ByVal o As Object) As Boolean
        If o Is Nothing Then
            Return False
        End If

        If TypeOf o Is Pointer Then
            Return False
        End If

        If Not o.GetType.IsSerializable Then
            Return False
        End If

        Return True
    End Function

    Private Shared Function IsToString(ByVal o As Object) As Boolean
        If Not o Is Nothing Then
            If o.GetType.IsPrimitive Then
                Return True
            End If

            If TypeOf o Is String Then
                Return True
            End If

            If TypeOf o Is CultureInfo Then 'some fields change here
                Return True
            End If

            If TypeOf o Is StringBuilder Then 'some fields change here
                Return True
            End If
        End If
    End Function

    <DebuggerHidden()> _
    Public Shared Function GetCopy(ByVal o As Object) As Object
        Using ms As New MemoryStream
            Dim bf As New BinaryFormatter
            bf.Serialize(ms, o)
            ms.Position = 0
            Return bf.Deserialize(ms)
        End Using
    End Function

    <DebuggerHidden()> _
    Public Shared Function GetCopy(Of T)(ByVal o As Object) As T
        Using ms As New MemoryStream
            Dim bf As New BinaryFormatter
            bf.Serialize(ms, o)
            ms.Position = 0
            Return DirectCast(bf.Deserialize(ms), T)
        End Using
    End Function

    Public Shared Sub Serialize(ByVal o As Object, ByVal path As String)
        Dim fs As FileStream = New FileStream(path, FileMode.OpenOrCreate)
        Dim bf As BinaryFormatter = New BinaryFormatter
        bf.Serialize(fs, o)
        fs.Close()
    End Sub

    Public Shared Function Deserialize(ByVal path As String) As Object
        Dim fs As FileStream = New FileStream(path, FileMode.Open)
        Dim bf As BinaryFormatter = New BinaryFormatter
        Dim o As Object = bf.Deserialize(fs)
        fs.Close()
        Return o
    End Function
End Class

Public Class RegistryHelp
    Public Shared CurrentUser As New RootKey(Registry.CurrentUser)
    Public Shared LocalMachine As New RootKey(Registry.LocalMachine)
    Public Shared ClassesRoot As New RootKey(Registry.ClassesRoot)

    Private Shared ApplicationKey As String = "Software\" + Application.ProductName

    Public Class RootKey
        Private RootRegistryKey As RegistryKey

        Public Sub New(ByVal rootKey As RegistryKey)
            RootRegistryKey = rootKey
        End Sub

        Public Function GetValue(ByVal key As String, ByVal name As String) As Object
            Using k = RootRegistryKey.OpenSubKey(key)
                If Not k Is Nothing Then
                    Return k.GetValue(name)
                End If

                Return Nothing
            End Using
        End Function

        Public Function GetValue(Of T)(ByVal key As String, ByVal name As String) As T
            Using k = RootRegistryKey.OpenSubKey(key)
                If Not k Is Nothing Then
                    Dim r = k.GetValue(name)

                    If Not r Is Nothing Then
                        Return DirectCast(r, T)
                    End If
                End If
            End Using
        End Function

        Public Function GetString(ByVal key As String) As String
            Return GetString(key, Nothing)
        End Function

        Public Function GetString(ByVal key As String, ByVal name As String) As String
            Return DirectCast(GetValue(key, name), String)
        End Function

        Public Sub SetValue(ByVal key As String, ByVal name As String, ByVal value As Object)
            Dim k = RootRegistryKey.OpenSubKey(key, True)

            If k Is Nothing Then
                k = RootRegistryKey.CreateSubKey(key)
            End If

            k.SetValue(name, value)
            k.Close()
        End Sub

        Public Sub DeleteKey(ByVal key As String)
            Using k = RootRegistryKey.OpenSubKey(key)
                If Not k Is Nothing Then
                    If k.SubKeyCount = 0 Then
                        RootRegistryKey.DeleteSubKey(key)
                    Else
                        RootRegistryKey.DeleteSubKeyTree(key)
                    End If
                End If
            End Using
        End Sub

        Public Sub DeleteValue(ByVal key As String, ByVal name As String)
            Using k = RootRegistryKey.OpenSubKey(key, True)
                If Not k Is Nothing Then
                    If name Is Nothing Then
                        name = ""
                    End If

                    k.DeleteValue(name, False)
                End If
            End Using
        End Sub

        Public Sub SetAppValue(ByVal name As String, ByVal value As Object)
            SetValue(ApplicationKey, name, value)
        End Sub

        Public Sub SetAppValue(ByVal subKey As String, ByVal name As String, ByVal value As Object)
            SetValue(ApplicationKey + "\" + subKey, name, value)
        End Sub

        Public Function GetAppInt(ByVal name As String, ByVal defaultValue As Integer) As Integer
            Dim ret As Object = GetAppValue(name)

            If TypeOf ret Is Integer Then
                Return DirectCast(ret, Integer)
            Else
                Return defaultValue
            End If
        End Function

        Public Function GetAppInt(ByVal subKey As String, ByVal name As String) As Integer
            Return GetAppInt(subKey, name, 0)
        End Function

        Public Function GetAppInt(ByVal subKey As String, ByVal name As String, ByVal defaultValue As Integer) As Integer
            Dim ret As Object = GetAppValue(subKey, name)

            If TypeOf ret Is Integer Then
                Return DirectCast(ret, Integer)
            Else
                Return defaultValue
            End If
        End Function

        Public Function GetAppValue(ByVal name As String) As Object
            Return GetValue(ApplicationKey, name)
        End Function

        Public Function GetAppValue(ByVal subKey As String, ByVal name As String) As Object
            Return GetValue(ApplicationKey + "\" + subKey, name)
        End Function

        Public Function GetAppString(ByVal name As String) As String
            Return GetString(ApplicationKey, name)
        End Function
    End Class
End Class

Public Class DirectoryHelp
    Public Shared Sub Delete(ByVal filepath As String)
        Try
            If Directory.Exists(filepath) Then
                Directory.Delete(filepath, True)
            End If
        Catch
            If Msg.Show("Failed to delete directory:" + CrLf2 + filepath + CrLf2 + Strings.IOErrorMessage, _
                MsgIcon.Error, MessageBoxButtons.RetryCancel) = DialogResult.Retry Then

                Try
                    If Directory.Exists(filepath) Then
                        Directory.Delete(filepath, True)
                    End If
                Catch
                    Msg.Show("Failed to delete directory." + CrLf2 + Strings.IOErrorMessage, MsgIcon.Error)
                End Try
            End If
        End Try
    End Sub

    Public Shared Sub Copy(ByVal source As String, ByVal target As String)
        Try
            Microsoft.VisualBasic.FileIO.FileSystem.CopyDirectory(source, target, True)
        Catch ex As Exception
            If Msg.Show("Failed to copy directory from:" + CrLf2 + source + CrLf2 + "to:" + CrLf2 + _
                target + CrLf2 + Strings.IOErrorMessage, MsgIcon.Error, MessageBoxButtons.RetryCancel) = DialogResult.Retry Then

                Try
                    Microsoft.VisualBasic.FileIO.FileSystem.CopyDirectory(source, target, True)
                Catch ex2 As Exception
                    Msg.Show("Failed to copy directory." + CrLf2 + Strings.IOErrorMessage, MsgIcon.Error)
                End Try
            End If
        End Try
    End Sub

    Public Shared Sub Move(ByVal source As String, ByVal target As String)
        Try
            Microsoft.VisualBasic.FileIO.FileSystem.MoveDirectory(source, target, True)
        Catch ex As Exception
            If Msg.Show("Failed to move directory from:" + CrLf2 + source + CrLf2 + "to:" + CrLf2 + _
                target + CrLf2 + Strings.IOErrorMessage, MsgIcon.Error, MessageBoxButtons.RetryCancel) = DialogResult.Retry Then

                Try
                    Microsoft.VisualBasic.FileIO.FileSystem.MoveDirectory(source, target, True)
                Catch ex2 As Exception
                    Msg.Show("Failed to move directory." + CrLf2 + Strings.IOErrorMessage, MsgIcon.Error)
                End Try
            End If
        End Try
    End Sub

    Public Shared Function FindFiles(ByVal startDir As String, ByVal filename As String, ByVal subdirs As Boolean) As List(Of String)
        Dim ret As New List(Of String)
        AddFiles(startDir, filename, ret, subdirs)
        Return ret
    End Function

    Public Shared Function Compare(ByVal dir1 As String, ByVal dir2 As String) As Boolean
        Return DirPath.AppendSeparator(dir1).ToUpper = DirPath.AppendSeparator(dir2).ToUpper
    End Function

    Private Shared Sub AddFiles(ByVal startDir As String, ByVal filename As String, ByVal filePaths As List(Of String), ByVal subdirs As Boolean)
        If Directory.Exists(startDir) Then
            Dim di As New DirectoryInfo(startDir)

            For Each i As FileSystemInfo In di.GetFileSystemInfos()
                If TypeOf i Is FileInfo Then
                    If Filepath.GetName(i.FullName).EqualIgnoreCase(filename) Then
                        filePaths.Add(i.FullName)
                    End If
                Else
                    If subdirs Then
                        AddFiles(i.FullName, filename, filePaths, subdirs)
                    End If
                End If
            Next
        End If
    End Sub

    <DebuggerNonUserCode()> _
    Public Shared Sub FileCallback(ByVal startDir As String, ByVal callback As Action(Of String), ByVal cancelArgs As CancelEventArgs)
        Try
            Dim di As New DirectoryInfo(startDir)

            For Each i As FileSystemInfo In di.GetFileSystemInfos()
                If cancelArgs.Cancel Then
                    Exit Sub
                End If

                If TypeOf i Is FileInfo Then
                    callback(i.FullName)
                Else
                    FileCallback(i.FullName, callback, cancelArgs)
                End If
            Next
        Catch
        End Try
    End Sub
End Class

Public Class ClipboardHelp
    Public Shared Function GetText() As String
        Dim data As Object = Clipboard.GetDataObject.GetData(DataFormats.Text)

        If Not data Is Nothing Then
            Return data.ToString
        End If

        Return ""
    End Function
End Class

Public Class XMLSettingsUtil(Of T)
    Private Shared Value As T
    Private Shared Filepath As String = CommonDirs.Startup + "Settings.xml"

    Public Shared Sub SaveXML(ByVal obj As T)
        Dim s As XmlSerializer = New XmlSerializer(GetType(T))

        Using w As New XmlTextWriter(Filepath, Encoding.UTF8)
            w.Formatting = Formatting.Indented
            s.Serialize(w, obj)
        End Using
    End Sub

    Public Shared Function LoadXML() As T
        Return LoadXML(Nothing)
    End Function

    Public Shared Function LoadXML(ByVal form As Form) As T
        If Not form Is Nothing Then
            AddHandler form.FormClosing, AddressOf FormClosing
        End If

        If File.Exists(Filepath) Then
            Using r As New XmlTextReader(Filepath)
                Dim s As New XmlSerializer(GetType(T))
                Value = DirectCast(s.Deserialize(r), T)
            End Using
        Else
            Value = Activator.CreateInstance(Of T)()
        End If

        Return Value
    End Function

    Private Shared Sub FormClosing(ByVal sender As Object, ByVal e As FormClosingEventArgs)
        SaveXML(Value)
    End Sub
End Class

Public Class EnvironmentHelp
    Public Shared Function GetArgs() As List(Of String)
        Dim ret As New List(Of String)(Environment.GetCommandLineArgs)
        ret.RemoveAt(0)
        Return ret
    End Function
End Class

Public Class ListHelp
    Public Shared Function GetIntegerList(ByVal startValue As Integer, ByVal endValue As Integer, ByVal stepValue As Integer) As List(Of Object)
        Dim ret As New List(Of Object)
        Dim value As Integer = startValue

        While value <= endValue
            ret.Add(value)
            value += stepValue
        End While

        Return ret
    End Function

    Public Shared Function GetDoubleList(ByVal startValue As Single, ByVal endValue As Single, ByVal stepValue As Double, ByVal digits As Integer) As List(Of Object)
        Dim ret As New List(Of Object)
        Dim value As Double = startValue

        While value <= endValue
            ret.Add(value)
            value += stepValue
            value = Math.Round(value, digits)
        End While

        Return ret
    End Function
End Class

Public Class FileHelp
    Public Shared Function Move(ByVal src As String, ByVal dest As String) As Boolean
        Try
            If File.Exists(dest) Then
                File.Delete(dest)
            End If

            File.Move(src, dest)
            Return True
        Catch
            If Msg.Show("Failed to move file from:" + CrLf2 + src + CrLf2 + "to:" + CrLf2 + dest + CrLf2 + _
                        Strings.IOErrorMessage, MsgIcon.Error, MessageBoxButtons.RetryCancel) = DialogResult.Retry Then
                Try
                    File.Move(src, dest)
                    Return True
                Catch
                    Msg.Show("Failed to move file." + CrLf2 + Strings.IOErrorMessage, MsgIcon.Error)
                End Try
            End If
        End Try
    End Function

    Public Shared Function Copy(ByVal src As String, ByVal dest As String) As Boolean
        Try
            File.Copy(src, dest, True)
            Return True
        Catch
            If Msg.Show("Failed to copy file from:" + CrLf2 + src + CrLf2 + "to:" + CrLf2 + dest + CrLf2 + _
                        Strings.IOErrorMessage, MsgIcon.Error, MessageBoxButtons.RetryCancel) = DialogResult.Retry Then
                Try
                    File.Copy(src, dest, True)
                    Return True
                Catch
                    Msg.Show("Failed to copy file." + CrLf2 + Strings.IOErrorMessage, MsgIcon.Error)
                End Try
            End If
        End Try
    End Function

    Public Shared Function Delete(ByVal path As String) As Boolean
        Try
            If File.Exists(path) Then
                File.Delete(path)
            End If

            Return True
        Catch
            If Msg.Show("Failed to delete the file:" + CrLf2 + path + CrLf2 + Strings.IOErrorMessage, _
                MsgIcon.Error, MessageBoxButtons.RetryCancel) = DialogResult.Retry Then

                Try
                    If File.Exists(path) Then
                        File.Delete(path)
                    End If

                    Return True
                Catch
                    Msg.Show("Failed to delete file." + CrLf2 + Strings.IOErrorMessage, MsgIcon.Error)
                End Try
            End If
        End Try
    End Function
End Class