Imports System
Imports System.Runtime.Serialization
Imports System.Collections.Specialized
Imports System.ComponentModel
Imports System.Drawing.Design
Imports System.Globalization
Imports System.IO
Imports System.Runtime.Serialization.Formatters.Binary
Imports System.Runtime.InteropServices
Imports System.Reflection
Imports System.Text.RegularExpressions
Imports System.Net
Imports System.Resources
Imports System.Xml
Imports System.Text
Imports System.Security.Permissions
Imports Microsoft.Win32
Imports System.Security.Principal
Imports System.Management

Imports Stax.ComponentModel
Imports Stax
Imports Stax.UI
Imports Stax.AppFramework

Imports VBN = Microsoft.VisualBasic

Public Class CommonDirs
    Public Shared ReadOnly Property Startup() As String
        Get
            Return DirPath.AppendSeparator(Application.StartupPath)
        End Get
    End Property

    Public Shared ReadOnly Property CurrentDirectory() As String
        Get
            Return DirPath.AppendSeparator(Environment.CurrentDirectory)
        End Get
    End Property

    Public Shared ReadOnly Property Temp() As String
        Get
            'weird extension method behaviour
            Return DirPath.AppendSeparator(IO.Path.GetTempPath)
        End Get
    End Property

    Public Shared ReadOnly Property System() As String
        Get
            Return DirPath.AppendSeparator(Environment.SystemDirectory)
        End Get
    End Property

    Public Shared ReadOnly Property Programs() As String
        Get
            Return DirPath.AppendSeparator(GetFolderPath(Environment.SpecialFolder.ProgramFiles))
        End Get
    End Property

    Public Shared ReadOnly Property AllAppData() As String
        Get
            Return DirPath.AppendSeparator(GetFolderPath(Environment.SpecialFolder.CommonApplicationData))
        End Get
    End Property

    Public Shared ReadOnly Property UserAppDataRoaming() As String
        Get
            Return DirPath.AppendSeparator(GetFolderPath(Environment.SpecialFolder.ApplicationData))
        End Get
    End Property

    Public Shared ReadOnly Property UserAppDataLocal() As String
        Get
            Return DirPath.AppendSeparator(GetFolderPath(Environment.SpecialFolder.LocalApplicationData))
        End Get
    End Property

    Public Shared ReadOnly Property UserDesktop() As String
        Get
            Const CSIDL_DESKTOPDIRECTORY As Int32 = &H10 'users desktop dir
            Dim sb As New StringBuilder(260)
            SHGetFolderPath(IntPtr.Zero, CSIDL_DESKTOPDIRECTORY, IntPtr.Zero, 0, sb)
            Return DirPath.AppendSeparator(sb.ToString)
        End Get
    End Property

    Public Shared ReadOnly Property Windows() As String
        Get
            Const CSIDL_WINDOWS As Int32 = &H24
            Dim sb As New StringBuilder(260)
            SHGetFolderPath(IntPtr.Zero, CSIDL_WINDOWS, IntPtr.Zero, 0, sb)
            Return DirPath.AppendSeparator(sb.ToString)
        End Get
    End Property

    <DllImport("shfolder.dll", CharSet:=CharSet.Unicode)> _
    Private Shared Function SHGetFolderPath(ByVal hwndOwner As IntPtr, ByVal nFolder As Integer, ByVal hToken As IntPtr, ByVal dwFlags As Integer, ByVal lpszPath As StringBuilder) As Integer
    End Function

    Public Shared Function GetFolderPath(ByVal folder As Environment.SpecialFolder) As String
        Dim sb As New StringBuilder(260)
        SHGetFolderPath(IntPtr.Zero, CInt(folder), IntPtr.Zero, 0, sb)
        Dim ret As String = DirPath.AppendSeparator(sb.ToString) '.NET fails on 'D:'
        Call New FileIOPermission(FileIOPermissionAccess.PathDiscovery, ret).Demand()
        Return ret
    End Function
End Class

Public Class PathBase
    Public Shared ReadOnly Property Separator() As Char
        Get
            'weird extension method behaviour
            Return IO.Path.DirectorySeparatorChar
        End Get
    End Property

    Public Shared Function AppendSeparator(ByVal path As String) As String
        If String.IsNullOrEmpty(path) Then
            Return ""
        End If

        If path.EndsWith(Separator) Then
            Return path
        End If

        Return path + Separator
    End Function

    Public Shared Function IsSameBase(ByVal a As String, ByVal b As String) As Boolean
        Return Filepath.GetBase(a).EqualIgnoreCase(Filepath.GetBase(b))
    End Function

    Public Shared Function IsSameDir(ByVal a As String, ByVal b As String) As Boolean
        Return Filepath.GetDir(a).EqualIgnoreCase(Filepath.GetDir(b))
    End Function

    Public Shared Function IsValidFileSystemName(ByVal name As String) As Boolean
        If Not OK(name) Then
            Return False
        End If

        For i = 0 To name.Length - 1
            Dim value = name.Chars(i)

            For Each i2 In """*/:<>?\|"
                If value = i2 Then
                    Return False
                End If
            Next

            If Convert.ToInt32(value) < 32 Then
                Return False
            End If
        Next

        Return True
    End Function
End Class

Public Class DirPath
    Inherits PathBase

    Public Shared Function GetRoot(ByVal path As String) As String
        Return Regex.Match(path, "^.:\\").Value
    End Function

    Public Shared Function TrimTrailingSeparator(ByVal path As String) As String
        If Not OK(path) Then
            Return ""
        End If

        If path.EndsWith(Separator) AndAlso Not path.EndsWith(":\") Then
            Return path.TrimEnd(Separator)
        End If

        Return path
    End Function

    Public Shared Function FixSeperator(ByVal path As String) As String
        If path.Contains("\") AndAlso Separator <> "\" Then
            path = path.Replace("\", Separator)
        End If

        If path.Contains("/") AndAlso Separator <> "/" Then
            path = path.Replace("/", Separator)
        End If

        Return path
    End Function

    Public Shared Function GetParent(ByVal path As String) As String
        If String.IsNullOrEmpty(path) Then
            Return ""
        End If

        Dim tmp As String = TrimTrailingSeparator(path)

        If tmp.Contains(Separator) Then
            path = tmp.LeftLast(Separator) + Separator
        End If

        Return path
    End Function

    Public Shared Function GetName(ByVal path As String) As String
        If String.IsNullOrEmpty(path) Then
            Return ""
        End If

        path = TrimTrailingSeparator(path)

        Return path.RightLast(Separator)
    End Function

    Public Shared Function IsInSysDir(ByVal path As String) As Boolean
        If Not OK(path) Then
            Return False
        End If

        If Not path.EndsWith("\") Then
            path += "\"
        End If

        Return path.ToUpper.Contains(CommonDirs.Programs.ToUpper)
    End Function
End Class

Public Class Filepath
    Inherits PathBase

    Private Value As String

    Public Sub New(ByVal path As String)
        Value = path
    End Sub

    Public ReadOnly Property Dir() As String
        Get
            Return GetDir(Value)
        End Get
    End Property

    Public Shared Function GetDir(ByVal path As String) As String
        If String.IsNullOrEmpty(path) Then
            Return ""
        End If

        If path.Contains(Separator) Then
            path = path.LeftLast(Separator) + Separator
        End If

        Return path
    End Function

    Public ReadOnly Property DirAndBase() As String
        Get
            Return GetDirAndBase(Value)
        End Get
    End Property

    Public Shared Function GetDirAndBase(ByVal path As String) As String
        Return GetDir(path) + GetBase(path)
    End Function

    Public ReadOnly Property Name() As String
        Get
            Return GetName(Value)
        End Get
    End Property

    Public Shared Function GetName(ByVal path As String) As String
        If Not path Is Nothing Then
            Dim index = path.LastIndexOf(IO.Path.DirectorySeparatorChar)

            If index > -1 Then
                Return path.Substring(index + 1)
            End If
        End If

        Return path
    End Function

    Public ReadOnly Property Ext() As String
        Get
            Return GetExt(Value)
        End Get
    End Property

    Public Shared Function GetExt(ByVal path As String) As String
        Dim x = path.Length

        While x > 0
            x -= 1

            Dim c = path(x)

            If c = Separator Then
                Return ""
            End If

            If c = "."c Then
                Return path.Substring(x).ToLowerInvariant
            End If
        End While

        Return ""
    End Function

    Public ReadOnly Property ExtNoDot() As String
        Get
            Return GetExtNoDot(Value)
        End Get
    End Property

    Public Shared Function GetExtNoDot(ByVal path As String) As String
        Dim x As Integer = path.Length

        While x > 0
            x -= 1

            Dim c As Char = path(x)

            If c = Separator Then
                Return ""
            End If

            If c = "."c Then
                Return path.Substring(x + 1).ToLowerInvariant
            End If
        End While

        Return ""
    End Function

    Public ReadOnly Property ChangeExt(ByVal extWithDot As String) As String
        Get
            Return GetChangeExt(Value, extWithDot)
        End Get
    End Property

    Public Shared Function GetChangeExt(ByVal path As String, ByVal value As String) As String
        If Not OK(path) Then
            Return Nothing
        End If

        If Not value.StartsWith(".") Then
            value = "." + value
        End If

        Return GetDirAndBase(path) + value
    End Function

    Public ReadOnly Property DirNoSep() As String
        Get
            Return GetDirNoSep(Value)
        End Get
    End Property

    Public Shared Function GetDirNoSep(ByVal path As String) As String
        path = GetDir(path)

        If path.EndsWith(Separator) Then
            path = TrimSep2(path)
        End If

        Return path
    End Function

    Public ReadOnly Property Equal(ByVal val As String) As Boolean
        Get
            Return IsEqual(Me.Value, val)
        End Get
    End Property

    Public Shared Function IsEqual(ByVal a As String, ByVal b As String) As Boolean
        If Not OK(a, b) Then
            Return False
        End If

        Return a.EqualIgnoreCase(b)
    End Function

    Public ReadOnly Property Base() As String
        Get
            Return GetBase(Value)
        End Get
    End Property

    Public Shared Function GetBase(ByVal path As String) As String
        If String.IsNullOrEmpty(path) Then Return ""

        Dim ret As String = path

        If ret.Contains(Separator) Then
            ret = ret.RightLast(Separator)
        End If

        If ret.Contains(".") Then
            ret = ret.LeftLast(".")
        End If

        Return ret
    End Function

    Public ReadOnly Property TrimSep() As String
        Get
            Return TrimSep2(Value)
        End Get
    End Property

    Public Shared Function TrimSep2(ByVal path As String) As String
        If path = "" OrElse path Is Nothing Then
            Return ""
        End If

        If path.EndsWith(Separator) AndAlso Not path.EndsWith(":" + Separator) Then
            Return path.TrimEnd(Separator)
        End If

        Return path
    End Function

    Public Shared Function GetDirNameOnly(ByVal path As String) As String
        Return Filepath.GetDirNoSep(path).RightLast("\")
    End Function
End Class

Public Class SafeSerialization
    Public Shared Sub Serialize(ByVal o As Object, ByVal path As String)
        Dim list As New List(Of Object)

        For Each i In o.GetType.GetFields(BindingFlags.Public Or BindingFlags.NonPublic Or BindingFlags.Instance)
            If Not i.IsNotSerialized Then
                Dim mc As New FieldContainer
                mc.Name = i.Name

                Dim value = i.GetValue(o)

                If Not value Is Nothing Then
                    If IsSimpleType(i.FieldType) Then
                        mc.Value = value
                    Else
                        mc.Value = GetObjectData(value)
                    End If

                    list.Add(mc)
                End If
            End If
        Next

        Dim bf As New BinaryFormatter

        Using fs As New FileStream(path, FileMode.Create)
            bf.Serialize(fs, list)
        End Using
    End Sub

    <DebuggerNonUserCode()> _
    Public Shared Function Deserialize(ByVal instance As Object, ByVal path As String, ByVal binder As SerializationBinder) As Object
        Dim safeInstance = DirectCast(instance, ISafeSerialization)

        If File.Exists(path) Then
            Dim list As List(Of Object) = Nothing
            Dim bf As New BinaryFormatter

            Using fs As New FileStream(path, FileMode.Open)
                Try
                    list = DirectCast(bf.Deserialize(fs), List(Of Object))
                Catch
                End Try
            End Using

            If Not list Is Nothing Then
                For Each i As FieldContainer In list
                    For Each iFieldInfo In instance.GetType.GetFields(BindingFlags.Public Or BindingFlags.NonPublic Or BindingFlags.Instance)
                        If Not iFieldInfo.IsNotSerialized Then
                            Dim a = iFieldInfo.GetAttribute(Of LegacyNameAttribute)()

                            If Not a Is Nothing AndAlso i.Name = a.Name Then
                                i.Name = iFieldInfo.Name
                            End If

                            If i.Name = iFieldInfo.Name Then
                                Try
                                    If i.Value.GetType Is GetType(Byte()) Then
                                        iFieldInfo.SetValue(instance, GetObjectInstance(DirectCast(i.Value, Byte()), binder))
                                    Else
                                        iFieldInfo.SetValue(instance, i.Value)
                                    End If
                                Catch ex As Exception
#If DEBUG Then
                                    Msg.Show(ex.ToString)
#End If
                                    safeInstance.WasUpdated = True
                                End Try
                            End If
                        End If
                    Next
                Next
            End If
        End If

        safeInstance.Init()

        If safeInstance.WasUpdated Then
            safeInstance.WasUpdated = False
            Serialize(instance, path)
        End If

        Return instance
    End Function

    Private Shared Function IsSimpleType(ByVal t As Type) As Boolean
        Return t.IsPrimitive OrElse t Is GetType(String) OrElse _
            t Is GetType(SettingBag(Of String)) OrElse _
            t Is GetType(SettingBag(Of Boolean)) OrElse _
            t Is GetType(SettingBag(Of Integer))
    End Function

    <DebuggerNonUserCode()> _
    Private Shared Function GetObjectInstance(ByVal ba As Byte(), ByVal binder As SerializationBinder) As Object
        Using ms As New MemoryStream(ba)
            Dim bf As New BinaryFormatter

            If Not binder Is Nothing Then
                bf.Binder = binder
            End If

            Return bf.Deserialize(ms)
        End Using
    End Function

    Private Shared Function GetObjectData(ByVal o As Object) As Byte()
        Using ms As New MemoryStream
            Dim bf As New BinaryFormatter
            bf.Serialize(ms, o)
            Return ms.ToArray()
        End Using
    End Function

    <Serializable()> _
    Public Class FieldContainer
        Public Value As Object
        Public Name As String
    End Class

    Public Shared Function Check(ByVal iface As ISafeSerialization, ByVal o As Object, ByVal key As String, ByVal minimal As Integer, ByVal current As Integer) As Boolean
        If o Is Nothing Then
            iface.WasUpdated = True

            If iface.Versions.ContainsKey(key) Then
                Msg.Show("A setting had to be reset: " + key, MsgIcon.Info)
            End If

            iface.Versions(key) = current
            Return True
        End If

        If Not iface.Versions.ContainsKey(key) Then
            iface.Versions(key) = 0
        End If

        If iface.Versions(key) < minimal Then
            iface.WasUpdated = True
            iface.Versions(key) = current
            Msg.Show("A setting had to be reset: " + key, MsgIcon.Info)
            Return True
        ElseIf iface.Versions(key) < current Then
            iface.WasUpdated = True
            iface.Versions(key) = current

            If Msg.Show("A setting has improved default values: " + key + _
                CrLf2 + "Would you like to update?", _
                MsgIcon.Question, MessageBoxButtons.YesNo) = DialogResult.Yes Then

                Return True
            End If
        End If

        iface.Versions(key) = current

        Return False
    End Function
End Class

<AttributeUsage(AttributeTargets.Field)> _
Public Class LegacyNameAttribute
    Inherits Attribute

    Public Sub New(ByVal name As String)
        Me.Name = name
    End Sub

    Private NameValue As String

    Public Property Name() As String
        Get
            Return NameValue
        End Get
        Set(ByVal Value As String)
            NameValue = Value
        End Set
    End Property
End Class

Public Interface ISafeSerialization
    Property WasUpdated() As Boolean
    ReadOnly Property Versions() As Dictionary(Of String, Integer)
    Sub Init()
End Interface

Public Class Sorter
    Private List As New List(Of Object)

    Public Sub Add(ByVal sortValue As IComparable, ByVal o As Object)
        List.Add(New SortObject(sortValue, o))
    End Sub

    Public Function GetSortedList() As List(Of Object)
        List.Sort()
        Dim ret As New List(Of Object)

        For Each i As SortObject In List
            ret.Add(i.Object)
        Next

        Return ret
    End Function

    Private Class SortObject
        Implements IComparable

        Public SortValue As IComparable
        Public [Object] As Object

        Public Sub New(ByVal sortValue As IComparable, ByVal o As Object)
            Me.SortValue = sortValue
            Me.Object = o
        End Sub

        Public Function CompareTo(ByVal obj As Object) As Integer Implements IComparable.CompareTo
            Return SortValue.CompareTo(CType(obj, SortObject).SortValue)
        End Function
    End Class
End Class

Public Class HelpDocument
    Private Path As String
    Private Title As String
    Private IsClosed As Boolean

    Public Shared TopSection As String

    Public Sub New(ByVal path As String)
        Me.Path = path
    End Sub

    Private WriterValue As XmlTextWriter

    Public Property Writer() As XmlTextWriter
        Get
            Return WriterValue
        End Get
        Set(ByVal Value As XmlTextWriter)
            WriterValue = Value
        End Set
    End Property

    Public Sub WriteStart(ByVal title As String)
        WriteStart(title, True)
    End Sub

    Public Sub WriteStart(ByVal title As String, ByVal showTitle As Boolean)
        WriteStart(title, showTitle, True)
    End Sub

    Public Sub WriteStart(ByVal title As String, ByVal showTitle As Boolean, ByVal showTopSection As Boolean)
        Dim script As XElement = _
<script type="text/javascript">
</script>

        Dim style As XElement = _
<style type="text/css">
body {
    background-image:url('StaxRipBackground.gif');
    background-repeat:repeat-x;
    font-family:Arial,Helvetica,sans-serif;
}

h1 {
    font-size:150%;
    color:#333333;
    margin-bottom:-4pt;
}

h2 {
    font-size:120%;
    color:#666666;
    margin-bottom:-8pt;
}

h3 {
    font-size:100%;
    color:#333333;
    margin-bottom:-8pt;
}

a {
    color: #666666;
}

</style>

        Me.Title = title
        Writer = New XmlTextWriter(Path, Encoding.UTF8)
        Writer.Formatting = Formatting.Indented
        Writer.WriteStartElement("html")
        Writer.WriteStartElement("head")
        Writer.WriteElementString("title", title)
        Writer.WriteRaw(CrLf + style.ToString + CrLf)
        Writer.WriteRaw(CrLf + script.ToString + CrLf)
        Writer.WriteEndElement() 'head
        Writer.WriteStartElement("body")
        WriteElement("p", "<img src=""" + StaxApplication.MainImagePath + """ style=""margin-bottom:-8pt;margin-left:-4pt"">")

        If showTitle Then
            If Not TopSection Is Nothing AndAlso showTopSection Then
                Writer.WriteRaw(TopSection)
            End If

            Writer.WriteElementString("h1", title)
        End If

        Dim dummy = StaxApplication.BackgroundImagePath
    End Sub

    Public Sub WriteP(ByVal rawText As String)
        WriteElement("p", rawText)
    End Sub

    Public Sub WriteH2(ByVal rawText As String)
        WriteElement("h2", rawText)
    End Sub

    Public Sub WriteH3(ByVal rawText As String)
        WriteElement("h3", rawText)
    End Sub

    Public Sub WriteP(ByVal title As String, ByVal rawText As String)
        WriteElement("h2", title)
        WriteElement("p", rawText)
    End Sub

    Public Sub WriteElement(ByVal elementName As String, ByVal rawText As String)
        Writer.WriteStartElement(elementName)

        If MustConvert(rawText) Then
            Writer.WriteRaw(ConvertMarkup(rawText, False))
        Else
            Writer.WriteRaw(rawText)
        End If

        Writer.WriteEndElement()
    End Sub

    Public Shared Function ConvertChars(ByVal value As String) As String
        If value.Contains("<") Then value = value.Replace("<", "&lt;")
        If value.Contains(">") Then value = value.Replace("<", "&gt;")

        Return value
    End Function

    Public Shared Function ConvertMarkup(ByVal value As String, ByVal stripOnly As Boolean) As String
        If stripOnly Then
            If value.Contains("[") Then
                Dim re As New Regex("\[(.+?) (.+?)\]")

                If re.IsMatch(value) Then
                    value = re.Replace(value, "$2")
                End If
            End If

            If value.Contains("'''") Then
                Dim re As New Regex("'''(.+?)'''")

                If re.IsMatch(value) Then
                    value = re.Replace(value, "$1")
                End If
            End If
        Else
            If value.Contains("[") Then
                Dim re As New Regex("\[(.+?) (.+?)\]")
                Dim m = re.Match(value)

                If m.Success Then
                    value = re.Replace(value, "<a href=""$1"">$2</a>")
                End If
            End If

            If value.Contains("'''") Then
                Dim re As New Regex("'''(.+?)'''")
                Dim m = re.Match(value)

                If m.Success Then
                    value = re.Replace(value, "<b>$1</b>")
                End If
            End If
        End If

        Return value
    End Function

    Public Shared Function MustConvert(ByVal value As String) As Boolean
        If OK(value) AndAlso (value.Contains("[") OrElse value.Contains("'''")) Then
            Return True
        End If
    End Function

    Public Sub WriteTips(ByVal ParamArray tips As StringPairList())
        Dim l As New StringPairList

        For Each i In tips
            l.AddRange(i)
        Next

        l.Sort()

        For Each i In l
            WriteH3(i.Name)

            For Each i2 In i.Value.Split(CrLf2)
                If i2.Contains(CrLf) Then
                    i2 = i2.Replace(CrLf, "<br>")
                End If

                WriteP(i2)
            Next
        Next
    End Sub

    Public Sub WriteList(ByVal ParamArray values As String())
        Writer.WriteStartElement("ul")

        For Each i As String In values
            Writer.WriteStartElement("li")
            Writer.WriteRaw(ConvertMarkup(i, False))
            Writer.WriteEndElement()
        Next

        Writer.WriteEndElement()
    End Sub

    Public Sub WriteTable(ByVal list As StringPairList)
        WriteTable(Nothing, Nothing, list, True)
    End Sub

    Public Sub WriteTable(ByVal title As String, ByVal list As StringPairList)
        WriteTable(title, Nothing, list, True)
    End Sub

    Public Sub WriteTable(ByVal title As String, ByVal list As StringPairList, ByVal sort As Boolean)
        WriteTable(title, Nothing, list, sort)
    End Sub

    Public Sub WriteTable(ByVal title As String, ByVal text As String, ByVal list As StringPairList)
        WriteTable(title, text, list, True)
    End Sub

    Private Sub WriteTable(ByVal title As String, ByVal text As String, ByVal list As StringPairList, ByVal sort As Boolean)
        If sort Then
            list.Sort()
        End If

        If Not title Is Nothing Then
            Writer.WriteElementString("h2", title)
        End If

        If text Is Nothing Then
            Writer.WriteElementString("p", "")
        Else
            WriteP(text)
        End If

        Writer.WriteStartElement("table")
        Writer.WriteAttributeString("border", "1")
        Writer.WriteAttributeString("cellspacing", "0")
        Writer.WriteAttributeString("cellpadding", "3")
        Writer.WriteAttributeString("bordercolordark", "white")
        Writer.WriteAttributeString("bordercolorlight", "black")

        For Each i As StringPair In list
            Writer.WriteStartElement("tr")

            Writer.WriteStartElement("td")
            WriteElement("p", i.Name)
            Writer.WriteEndElement() 'td

            Writer.WriteStartElement("td")

            If i.Value Is Nothing Then
                WriteElement("p", "&nbsp;")
            Else
                WriteElement("p", i.Value)
            End If

            Writer.WriteEndElement() 'td

            Writer.WriteEndElement() 'tr
        Next

        Writer.WriteEndElement() 'table
    End Sub

    Public Sub WriteDocument()
        If Not IsClosed Then
            IsClosed = True

            Writer.WriteRaw("<p>&nbsp;</p>" + CrLf)
            Writer.WriteRaw("<h5 align=""center"">Copyright &copy; " & DateTime.Now.Year & " by stax. All rights reserved.</h5><br>")
            Writer.WriteEndElement() 'body
            Writer.WriteEndElement() 'html
            Writer.Close()
        End If
    End Sub

    Public Sub WriteDocument(ByVal browser As WebBrowser)
        WriteDocument()
        browser.Navigate(Path)
    End Sub
End Class

<Serializable()> _
Public Class SettingBag(Of T)
    Public Sub New(ByVal value As T)
        Me.Value = value
    End Sub

    Private ValueValue As T

    Public Overridable Property Value() As T
        Get
            Return ValueValue
        End Get
        Set(ByVal Value As T)
            ValueValue = Value
        End Set
    End Property
End Class

Public Class PropertySettingBag(Of T)
    Inherits SettingBag(Of T)

    Private Obj As Object
    Private Name As String

    Public Sub New(ByVal obj As Object, ByVal propName As String)
        MyBase.New(Nothing)
        Me.Obj = obj
        Me.Name = propName
    End Sub

    Public Overrides Property Value() As T
        Get
            Return DirectCast(Obj.GetType.GetProperty(Name).GetValue(Obj, Nothing), T)
        End Get
        Set(ByVal value As T)
            If Not Obj Is Nothing Then
                Obj.GetType.GetProperty(Name).SetValue(Obj, value, Nothing)
            End If
        End Set
    End Property
End Class

Public Class FieldSettingBag(Of T)
    Inherits SettingBag(Of T)

    Private Obj As Object
    Private Name As String

    Public Sub New(ByVal obj As Object, ByVal fieldName As String)
        MyBase.New(Nothing)
        Me.Obj = obj
        Me.Name = fieldName
    End Sub

    Public Overrides Property Value() As T
        Get
            Return DirectCast(Obj.GetType.GetField(Name).GetValue(Obj), T)
        End Get
        Set(ByVal value As T)
            If Not Obj Is Nothing Then
                Obj.GetType.GetField(Name).SetValue(Obj, value)
            End If
        End Set
    End Property
End Class

Public Class StringPair
    Implements IComparable(Of StringPair)

    Public Sub New()
    End Sub

    Public Sub New(ByVal name As String, ByVal text As String)
        Me.Name = name
        Me.Value = text
    End Sub

    Private NameValue As String

    Public Property Name() As String
        Get
            Return NameValue
        End Get
        Set(ByVal Value As String)
            NameValue = Value
        End Set
    End Property

    Private ValueValue As String

    Public Property Value() As String
        Get
            Return ValueValue
        End Get
        Set(ByVal Value As String)
            ValueValue = Value
        End Set
    End Property

    Public Function CompareTo(ByVal other As StringPair) As Integer Implements System.IComparable(Of StringPair).CompareTo
        Return Name.CompareTo(other.Name)
    End Function
End Class

Public Class Misc
    Public Shared Sub UpdateMostRecentList(ByVal list As List(Of String), ByVal value As String, ByVal max As Integer)
        If Not OK(value) Then
            Exit Sub
        End If

        If list.Count > 0 AndAlso value = list(0) Then
            Exit Sub
        End If

        list.Remove(value)
        list.Insert(0, value)

        While list.Count > max
            list.RemoveAt(list.Count - 1)
        End While
    End Sub

    Public Shared Function BrowseFile() As String
        Using d As New OpenFileDialog
            If d.ShowDialog = DialogResult.OK Then
                Return d.FileName
            End If
        End Using

        Return ""
    End Function

    Public Shared Function GetHtmlToVbb(ByVal value As String) As String
        value = value.Replace("<p>", "")
        value = value.Replace("</p>", "")
        value = value.Replace("<br>", "")
        value = value.Replace("</br>", "")
        value = value.Replace("<ul>", "[list]")
        value = value.Replace("</ul>", "[/list]")
        value = value.Replace("<li>", "[*]")
        value = value.Replace("</li>", "")
        value = value.Replace("<b>", "[b]")
        value = value.Replace("</b>", "[/b]")
        value = value.Replace("<strong>", "[b]")
        value = value.Replace("</strong>", "[/b]")
        value = value.Replace("&quot;", """")
        value = value.Replace("&nbsp;", " ")
        value = value.Replace("&amp;", "&")
        value = value.Replace("<h1>", "[size=+2]")
        value = value.Replace("</h1>", "[/size]")
        value = value.Replace("<h2>", "[size=+1]")
        value = value.Replace("</h2>", "[/size]")

        Dim eval As New MatchEvaluator(AddressOf EvalMatch)
        value = Regex.Replace(value, "<a href=""(.+?)"">(.+?)</a>", eval)

        value = Regex.Replace(value, "<font color=""(.+?)"">(.+?)</font>", "[color=$1]$2[/color]")

        Return value
    End Function

    Private Shared IsAdminValue As Boolean?

    Public Shared Function IsAdmin() As Boolean
        If Not IsAdminValue.HasValue Then
            Dim id = WindowsIdentity.GetCurrent()
            Dim wp = New WindowsPrincipal(id)
            IsAdminValue = wp.IsInRole(WindowsBuiltInRole.Administrator)
        End If

        Return IsAdminValue.Value
    End Function

    Public Shared Sub PlayAudioFile(ByVal path As String, ByVal volume As Integer)
        Try
            Static r As New Reflector("WMPlayer.OCX.7")
            Dim s = r.Invoke("settings", BindingFlags.GetProperty)
            s.Invoke("volume", BindingFlags.SetProperty, volume)
            s.Invoke("setMode", "loop", False)
            r.Invoke("URL", BindingFlags.SetProperty, path)
            r.Invoke("controls", BindingFlags.GetProperty).Invoke("play")
        Catch ex As Exception
            Msg.Show(ex.Message, MsgIcon.Error)
        End Try
    End Sub

    Public Shared Function Eval(ByVal value As String) As String
        Static dt As New DataTable()
        Static dr As DataRow = dt.Rows.Add()
        Static dc As DataColumn = dt.Columns.Add()

        dc.Expression = value
        Dim r = dr(0).ToString

        If r.Contains(",") Then
            r = r.Replace(",", ".")
        End If

        Return r
    End Function

    Private Shared Function EvalMatch(ByVal match As Match) As String
        If match.Groups(1).Value.Contains("http://") Then
            Return "[url=" + match.Groups(1).Value + "]" + match.Groups(2).Value + "[/url]"
        Else
            Return match.Groups(2).Value
        End If
    End Function

    Public Shared Sub SendPaste(ByVal value As String)
        Dim tmp = Clipboard.GetText()
        Clipboard.SetText(value)
        SendKeys.SendWait("^v")
        Clipboard.SetText(tmp)
    End Sub

    Public Shared Function Validate(ByVal value As String, ByVal pattern As String) As Boolean
        If Not Regex.IsMatch(value, pattern) Then
            Msg.Show("""" + value + """ is no valid input.", "Invalid Input", MsgIcon.Warning)
            Return False
        End If

        Return True
    End Function
End Class

Public Class ErrorAbortException
    Inherits ApplicationException

    Public Sub New(ByVal message As String)
        MyBase.New(message)
    End Sub
End Class

Public Class AbortException
    Inherits ApplicationException
End Class

Public Class CLIArg
    Public Sub New(ByVal value As String)
        Me.Value = value
    End Sub

    Private ValueValue As String

    Public Property Value() As String
        Get
            Return ValueValue
        End Get
        Set(ByVal Value As String)
            ValueValue = Value
        End Set
    End Property

    Public Shared Function GetArgs(ByVal a As String()) As List(Of CLIArg)
        Dim ret As New List(Of CLIArg)
        Dim args As New List(Of String)(a)
        args.RemoveAt(0)

        For Each i As String In args
            ret.Add(New CLIArg(i))
        Next

        Return ret
    End Function

    Public Function IsMatch(ByVal ParamArray values As String()) As Boolean
        For Each i As String In values
            i = i.ToUpper
            Dim val As String = Value.ToUpper

            If "-" + i = val OrElse "/" + i = val OrElse _
                val.ToUpper.StartsWith("-" + i + ":") OrElse _
                val.ToUpper.StartsWith("/" + i + ":") Then

                Return True
            End If
        Next
    End Function

    Public Function GetInt() As Integer
        Return CInt(Value.Right(":"))
    End Function

    Public Function GetString() As String
        Return Value.Right(":").Trim(""""c)
    End Function

    Public Function IsFile() As Boolean
        Return File.Exists(Value)
    End Function
End Class

Public Class StringPairList
    Inherits List(Of StringPair)

    Public Overloads Sub Add(ByVal name As String, ByVal text As String)
        Add(New StringPair(name, text))
    End Sub
End Class

<Serializable()> _
Public Class CommandParameters
    Public Sub New(ByVal methodName As String, ByVal ParamArray params As Object())
        Me.MethodName = methodName
        Parameters = New List(Of Object)(params)
    End Sub

    Private MethodNameValue As String

    Public Property MethodName() As String
        Get
            Return MethodNameValue
        End Get
        Set(ByVal Value As String)
            MethodNameValue = Value
        End Set
    End Property

    Private ParametersValue As List(Of Object)

    Public Property Parameters() As List(Of Object)
        Get
            Return ParametersValue
        End Get
        Set(ByVal Value As List(Of Object))
            ParametersValue = Value
        End Set
    End Property
End Class

Public Class Command
    Implements IComparable(Of Command)

    Private FunctionAttributeValue As CommandAttribute

    Public Property Attribute() As CommandAttribute
        Get
            Return FunctionAttributeValue
        End Get
        Set(ByVal Value As CommandAttribute)
            FunctionAttributeValue = Value
        End Set
    End Property

    Private MethodInfoValue As MethodInfo

    Public Property MethodInfo() As MethodInfo
        Get
            Return MethodInfoValue
        End Get
        Set(ByVal Value As MethodInfo)
            MethodInfoValue = Value
        End Set
    End Property

    Public Function FixParameters(ByVal params As List(Of Object)) As List(Of Object)
        Dim copiedParams As New List(Of Object)(params)
        Dim checkedParams As New List(Of Object)(GetDefaultParameters)

        For i As Integer = 0 To checkedParams.Count - 1
            If copiedParams.Count > 0 Then
                If checkedParams(i).GetType Is copiedParams(0).GetType Then
                    checkedParams(i) = copiedParams(0)
                    copiedParams.RemoveAt(0)
                End If
            End If
        Next

        Return checkedParams
    End Function

    Public Function GetDefaultParameters() As List(Of Object)
        Dim l As New List(Of Object)

        For Each iParameterInfo As ParameterInfo In MethodInfo.GetParameters
            If Not iParameterInfo.ParameterType.IsValueType AndAlso _
                Not iParameterInfo.ParameterType Is GetType(String) Then

                Throw New Exception("Methods must have string or value type params.")
            End If

            Dim a As DefaultValueAttribute() = DirectCast(iParameterInfo.GetCustomAttributes(GetType(DefaultValueAttribute), False), DefaultValueAttribute())

            If a.Length > 0 Then
                l.Add(a(0).Value)
            Else
                If iParameterInfo.ParameterType Is GetType(String) Then
                    l.Add("")
                ElseIf iParameterInfo.ParameterType.IsValueType Then
                    l.Add(Activator.CreateInstance(iParameterInfo.ParameterType))
                End If
            End If
        Next

        Return l
    End Function

    Public Overrides Function ToString() As String
        If Not Attribute Is Nothing Then
            Return Attribute.Name
        End If

        Return ""
    End Function

    Private ObjectValue As Object

    Public Property [Object]() As Object
        Get
            Return ObjectValue
        End Get
        Set(ByVal Value As Object)
            ObjectValue = Value
        End Set
    End Property

    Public Function CompareTo(ByVal other As Command) As Integer Implements System.IComparable(Of Command).CompareTo
        Return Attribute.Name.CompareTo(other.Attribute.Name)
    End Function

    Public Shared Sub PopulateCommandMenu(ByVal items As ToolStripItemCollection, ByVal commands As List(Of Command), ByVal clickSub As Action(Of Command))
        commands.Sort()

        For Each i In commands
            ParamToolStripMenuItem(Of Command).Populate(items, i.Attribute.Name, clickSub, i, Keys.None, i.Attribute.Description)
        Next
    End Sub

    Public Function GetParameterHelp(ByVal parameters As List(Of Object)) As String
        If parameters.Count > 0 Then
            Dim paramList As New List(Of String)
            Dim params As ParameterInfo() = MethodInfo.GetParameters

            For iParams As Integer = 0 To params.Length - 1
                Dim paramInfo As ParameterInfo = params(iParams)
                Dim attributs As Object() = paramInfo.GetCustomAttributes(GetType(DisplayInfoAttribute), False)

                If attributs.Length > 0 Then
                    paramList.Add(DirectCast(attributs(0), DisplayInfoAttribute).DisplayName + " = " + FixParameters(parameters)(iParams).ToString)
                End If
            Next

            Return String.Join(", ", paramList.ToArray)
        End If

        Return Nothing
    End Function
End Class

<AttributeUsage(AttributeTargets.Method)> _
Public Class CommandAttribute
    Inherits Attribute

    Public Sub New(ByVal name As String, ByVal description As String)
        Me.Name = name
        Me.Description = description
    End Sub

    Private NameValue As String

    Public Property Name() As String
        Get
            Return NameValue
        End Get
        Set(ByVal Value As String)
            NameValue = Value
        End Set
    End Property

    Private DescriptionValue As String

    Public Property Description() As String
        Get
            Return DescriptionValue
        End Get
        Set(ByVal Value As String)
            DescriptionValue = Value
        End Set
    End Property

    Private TopicValue As String

    Public Property Topic() As String
        Get
            Return TopicValue
        End Get
        Set(ByVal Value As String)
            TopicValue = Value
        End Set
    End Property

    Private SwitchValue As String

    Public Property Switch() As String
        Get
            Return SwitchValue
        End Get
        Set(ByVal Value As String)
            SwitchValue = Value
        End Set
    End Property
End Class

Public Class CommandManager
    Private CommandsValue As New Dictionary(Of String, Command)

    Public Property Commands() As Dictionary(Of String, Command)
        Get
            Return CommandsValue
        End Get
        Set(ByVal Value As Dictionary(Of String, Command))
            CommandsValue = Value
        End Set
    End Property

    Public Function HasCommand(ByVal name As String) As Boolean
        Return Not name Is Nothing AndAlso Commands.ContainsKey(name)
    End Function

    Public Function GetCommand(ByVal name As String) As Command
        If HasCommand(name) Then
            Return Commands(name)
        End If

        Return Nothing
    End Function

    Public Sub AddCommandsFromObject(ByVal obj As Object)
        For Each i As MethodInfo In obj.GetType.GetMethods(BindingFlags.Instance Or BindingFlags.NonPublic Or BindingFlags.Public)
            Dim attributes As CommandAttribute() = DirectCast(i.GetCustomAttributes(GetType(CommandAttribute), False), CommandAttribute())

            If attributes.Length > 0 Then
                Dim c As New Command

                c.MethodInfo = i
                c.Attribute = attributes(0)
                c.Object = obj

                AddCommand(c)
            End If
        Next
    End Sub

    Public Sub AddCommand(ByVal command As Command)
        If Commands.ContainsKey(command.MethodInfo.Name) Then
            Msg.Show("Command '" + command.MethodInfo.Name + "' already exists.", MsgIcon.Warning)
        Else
            Commands(command.MethodInfo.Name) = command
        End If
    End Sub

    Public Sub Process(ByVal cp As CommandParameters)
        If Not cp Is Nothing Then
            Process(cp.MethodName, cp.Parameters)
        End If
    End Sub

    Public Sub Process(ByVal name As String, ByVal params As List(Of Object))
        If HasCommand(name) Then
            Process(GetCommand(name), params)
        End If
    End Sub

    Public Sub Process(ByVal command As Command, ByVal params As List(Of Object))
        Try
            command.MethodInfo.Invoke(command.Object, command.FixParameters(params).ToArray)
        Catch ex As TargetParameterCountException
            Msg.Show("Parameter mismatch, for the command :" + command.Attribute.Name, MsgIcon.Error)
        End Try
    End Sub

    Public Function ProcessCmdlArgument(ByVal value As String) As Boolean
        If value.Contains("/") Then
            value = value.Replace("/", "|")
        End If

        For Each i As Command In Commands.Values
            Dim s As String = i.Attribute.Name.ToUpper.Replace(" ", "")

            Dim switches As New List(Of String)
            switches.Add(s)

            If OK(i.Attribute.Switch) Then
                switches.AddRange(i.Attribute.Switch.ToUpper.Split("|"c))
            End If

            For Each iSwitch As String In switches
                If value.ToUpper = "-" + iSwitch OrElse value.ToUpper = "/" + iSwitch Then
                    Process(i.MethodInfo.Name, New List(Of Object))
                    Return True
                ElseIf value.ToUpper.StartsWith("-" + iSwitch + ":") OrElse _
                    value.ToUpper.StartsWith("/" + iSwitch + ":") Then

                    Dim mc = Regex.Matches(value.Right(":"), """(?<a>.+?)""|(?<a>[^,]+)")
                    Dim args As New List(Of Object)

                    For Each iMatch As Match In mc
                        args.Add(iMatch.Groups("a").Value)
                    Next

                    Dim params = i.MethodInfo.GetParameters

                    For x = 0 To params.Length - 1
                        args(x) = TypeDescriptor.GetConverter(params(x).ParameterType).ConvertFrom(args(x))
                    Next

                    Process(i.MethodInfo.Name, args)
                    Return True
                End If
            Next
        Next

        Return False
    End Function

    Public Function GetTips() As StringPairList
        Dim l As New StringPairList

        For Each i As Command In Commands.Values
            If Not i.Attribute.Description Is Nothing Then
                l.Add(New StringPair(i.Attribute.Name, i.Attribute.Description))
            End If
        Next

        Return l
    End Function
End Class

Public Module MainModule
    Public Const CrLf As String = VBN.vbCrLf
    Public Const CrLf2 As String = VBN.vbCrLf + VBN.vbCrLf

    Public Function OK(ByVal value As String) As Boolean
        Return Not String.IsNullOrEmpty(value)
    End Function

    Public Function OK(ByVal ParamArray values As String()) As Boolean
        If values Is Nothing OrElse values.Length = 0 Then
            Return False
        End If

        For Each i As String In values
            If String.IsNullOrEmpty(i) Then
                Return False
            End If
        Next

        Return True
    End Function

    Public Function OK(ByVal ParamArray values As Object()) As Boolean
        If values Is Nothing OrElse values.Length = 0 Then
            Return False
        End If

        For Each i As Object In values
            If i Is Nothing Then
                Return False
            End If
        Next

        Return True
    End Function

    Public Function IsOneOf(ByVal value As Integer, ByVal ParamArray values As Integer()) As Boolean
        For Each i In values
            If value = i Then
                Return True
            End If
        Next

        Return False
    End Function

    Public Function IsOneOf(ByVal value As String, ByVal ParamArray values As String()) As Boolean
        For Each i In values
            If value = i Then
                Return True
            End If
        Next

        Return False
    End Function
End Module

Public Class Reflector
    Public Type As Type
    Private BasicFlags As BindingFlags = BindingFlags.Static Or BindingFlags.Instance Or BindingFlags.Public Or BindingFlags.NonPublic

    Public Sub New(ByVal obj As Object)
        Me.Value = obj
    End Sub

    Public Sub New(ByVal obj As Object, ByVal type As Type)
        Me.Value = obj
        Me.Type = type
    End Sub

    Public Sub New(ByVal progID As String)
        Me.New(Activator.CreateInstance(Type.GetTypeFromProgID(progID)))
    End Sub

    Private ValueValue As Object

    Public Property Value() As Object
        Get
            Return ValueValue
        End Get
        Set(ByVal Value As Object)
            ValueValue = Value

            If Not Value Is Nothing Then
                Type = Value.GetType
            End If
        End Set
    End Property

    Public Function ToBool() As Boolean
        Return DirectCast(Value, Boolean)
    End Function

    Public Function ToInt() As Integer
        Return DirectCast(Value, Integer)
    End Function

    Public Overrides Function ToString() As String
        Return DirectCast(Value, String)
    End Function

    Public Function Cast(Of T)() As T
        Return DirectCast(Value, T)
    End Function

    Public Function Invoke(ByVal name As String) As Reflector
        'returns overloads as single MemberInfo
        For Each i In Type.GetMember(name, BasicFlags)
            Select Case i.MemberType
                Case MemberTypes.Property
                    Return Invoke(name, BasicFlags Or BindingFlags.GetProperty, New Object() {})
                Case MemberTypes.Field
                    Return Invoke(name, BasicFlags Or BindingFlags.GetField, New Object() {})
                Case MemberTypes.Method
                    Return Invoke(name, BasicFlags Or BindingFlags.InvokeMethod, New Object() {})
            End Select
        Next

        Return Invoke(name, BindingFlags.InvokeMethod) 'COM
    End Function

    Public Function Invoke(ByVal name As String, ByVal ParamArray args As Object()) As Reflector
        'returns overloads as single MemberInfo
        For Each i In Type.GetMember(name, BasicFlags)
            Select Case i.MemberType
                Case MemberTypes.Property
                    Return Invoke(name, BasicFlags Or BindingFlags.SetProperty, args)
                Case MemberTypes.Field
                    Return Invoke(name, BasicFlags Or BindingFlags.SetField, args)
                Case MemberTypes.Method
                    Return Invoke(name, BasicFlags Or BindingFlags.InvokeMethod, args)
                Case Else
                    Throw New NotImplementedException
            End Select
        Next

        Return Invoke(name, BindingFlags.InvokeMethod, args) 'COM 
    End Function

    Public Function Invoke(ByVal name As String, ByVal flags As BindingFlags, ByVal ParamArray args As Object()) As Reflector
        Return New Reflector(Type.InvokeMember(name, flags, Nothing, Value, args))
    End Function
End Class

Public Class WindowsScript
    Private R As New Reflector("MSScriptControl.ScriptControl")

    Public Sub AddCode(ByVal code As String)
        R.Invoke("AddCode", code)
    End Sub

    Public Sub AddObject(ByVal name As String, ByVal obj As Object)
        R.Invoke("AddObject", name, obj, False)
    End Sub

    Private Sub ThrowException()
        Throw New Exception("Line" + R.Invoke("Error", _
            BindingFlags.GetProperty).Invoke("Line", _
            BindingFlags.GetProperty).ToInt.ToString + ": " + _
            R.Invoke("Error", BindingFlags.GetProperty).Invoke( _
            "Description", BindingFlags.GetProperty).ToString)
    End Sub

    Public WriteOnly Property Language() As WshLanguage
        Set(ByVal value As WshLanguage)
            R.Invoke("Language", BindingFlags.PutDispProperty, value.ToString)
        End Set
    End Property

    Public Sub ExecuteStatement(ByVal statement As String)
        Try
            R.Invoke("ExecuteStatement", statement)
        Catch ex As Exception
            ThrowException()
        End Try
    End Sub

    Public Shared Sub ExecuteVB(ByVal code As String)
        Execute(code, WshLanguage.VBScript)
    End Sub

    Public Shared Sub ExecuteJS(ByVal code As String)
        Execute(code, WshLanguage.JScript)
    End Sub

    Private Shared Sub Execute(ByVal code As String, ByVal language As WshLanguage)
        Static ws As WindowsScript

        If ws Is Nothing Then
            Try
                ws = New WindowsScript
                ws.Language = language
            Catch
                Throw New Exception("Failed to create scripting control.")
            End Try
        End If

        ws.ExecuteStatement(code)
    End Sub

    Public Function Eval(ByVal expression As String) As Object
        Try
            Return R.Invoke("Eval", expression).Value
        Catch ex As Exception
            ThrowException()
        End Try

        Return Nothing
    End Function

    Public Shared Function EvalVB(ByVal code As String) As Object
        Return Eval(code, WshLanguage.VBScript)
    End Function

    Public Shared Function EvalJS(ByVal code As String) As Object
        Return Eval(code, WshLanguage.JScript)
    End Function

    Private Shared Function Eval(ByVal code As String, ByVal language As WshLanguage) As Object
        Static ws As WindowsScript

        If ws Is Nothing Then
            Try
                ws = New WindowsScript
                ws.Language = language
            Catch
                Throw New Exception("Failed to create scripting control.")
            End Try
        End If

        Return ws.Eval(code)
    End Function
End Class

Public Enum WshLanguage
    VBScript
    JScript
End Enum

Public Class Shutdown
    Public Shared Sub Commit(ByVal mode As ShutdownMode)
        If mode = ShutdownMode.Standby Then
            SetSuspendState(False, False, False)
        ElseIf mode = ShutdownMode.Hibernate Then
            SetSuspendState(True, False, False)
        ElseIf mode <> ShutdownMode.Nothing Then
            Dim c As New ManagementClass("Win32_OperatingSystem")
            c.Scope.Options.EnablePrivileges = True
            Dim params = c.GetMethodParameters("Win32Shutdown")
            params("Flags") = CInt(mode).ToString
            DirectCast(c.GetInstances()(0), ManagementObject).InvokeMethod("Win32Shutdown", params, Nothing)
        End If
    End Sub

    Declare Function SetSuspendState Lib "powrprof.dll" (ByVal hibernate As Boolean, ByVal forceCritical As Boolean, ByVal disableWakeEvent As Boolean) As Boolean
End Class

Public Enum ShutdownMode
    [Nothing] = 50
    Standby = 51
    Shutdown = 1
    Hibernate = 52
End Enum

Public Class FileAssociation
    Public Shared Sub Create(ByVal extension As String)
        Create(Application.ExecutablePath, extension, """%1"" %*")
    End Sub

    Public Shared Sub Create(ByVal executable As String, ByVal extension As String, ByVal parameter As String)
        If Not extension.StartsWith(".") Then
            extension = "." + extension
        End If

        Dim backup = RegistryHelp.ClassesRoot.GetString(extension, Nothing)

        If OK(backup) Then
            RegistryHelp.ClassesRoot.SetValue(extension, "backup", backup)
        End If

        Dim name = Application.ProductName + extension
        RegistryHelp.ClassesRoot.SetValue(extension, Nothing, name)
        RegistryHelp.ClassesRoot.DeleteKey(name)
        RegistryHelp.ClassesRoot.SetValue(name + "\shell\open\command", Nothing, """" + executable + """ " + parameter)
        RegistryHelp.CurrentUser.DeleteKey("Software\Microsoft\Windows\CurrentVersion\Explorer\FileExts\" + extension)
    End Sub

    Public Shared Sub Remove(ByVal extension As String)
        If Not extension.StartsWith(".") Then
            extension = "." + extension
        End If

        Dim backup = RegistryHelp.ClassesRoot.GetString(extension, "backup")
        Dim value = RegistryHelp.ClassesRoot.GetString(extension, Nothing)

        If OK(backup) Then
            RegistryHelp.ClassesRoot.SetValue(extension, Nothing, backup)
            RegistryHelp.ClassesRoot.DeleteValue(extension, "backup")
        Else
            RegistryHelp.ClassesRoot.DeleteValue(extension, Nothing)
        End If

        RegistryHelp.ClassesRoot.DeleteKey(Application.ProductName + extension)
    End Sub

    Public Shared Sub RefreshIconCache()
        Dim SMTO_ABORTIFHUNG = &H2
        Dim HWND_BROADCAST As New IntPtr(&HFFFF)
        Dim old = RegistryHelp.CurrentUser.GetString("Control Panel\Desktop\WindowMetrics", "Shell Icon Size")
        RegistryHelp.CurrentUser.SetValue("Control Panel\Desktop\WindowMetrics", "Shell Icon Size", "33")
        Win32.NativeFunctions.SendMessageTimeout(HWND_BROADCAST, Win32.WM.SETTINGCHANGE, IntPtr.Zero, IntPtr.Zero, SMTO_ABORTIFHUNG, 3, IntPtr.Zero)
        RegistryHelp.CurrentUser.SetValue("Control Panel\Desktop\WindowMetrics", "Shell Icon Size", old)
        Win32.NativeFunctions.SendMessageTimeout(HWND_BROADCAST, Win32.WM.SETTINGCHANGE, IntPtr.Zero, IntPtr.Zero, SMTO_ABORTIFHUNG, 3, IntPtr.Zero)
    End Sub
End Class

Public Class Strings
    Public Const IOErrorMessage As String = "Possibly another application locks or the privilegs are insufficient. Please close all applications including StaxRip or reboot and then right-click the StaxRip shortcut or executable and select 'Run as...' from the menu in order to run StaxRip as admin."
End Class