Imports System.Threading
Imports System.Text.RegularExpressions

Imports Stax.UI

Public Class Proc
    Implements IDisposable

    Private TrowExceptionValue As Boolean

    Public Property TrowException() As Boolean
        Get
            Return TrowExceptionValue
        End Get
        Set(ByVal Value As Boolean)
            TrowExceptionValue = Value
        End Set
    End Property

    Private ReTrowExceptionValue As Boolean

    Public Property ReTrowException() As Boolean
        Get
            Return ReTrowExceptionValue
        End Get
        Set(ByVal Value As Boolean)
            ReTrowExceptionValue = Value
        End Set
    End Property

    Private ProcessValue As New Process

    Public Property Process() As Process
        Get
            Return ProcessValue
        End Get
        Set(ByVal Value As Process)
            ProcessValue = Value
        End Set
    End Property

    Private WaitValue As Boolean

    Public Property Wait() As Boolean
        Get
            Return WaitValue
        End Get
        Set(ByVal value As Boolean)
            WaitValue = value
        End Set
    End Property

    Private BeginErrorReadLineValue As Boolean

    Public Property BeginErrorReadLine() As Boolean
        Get
            Return BeginErrorReadLineValue
        End Get
        Set(ByVal Value As Boolean)
            Process.StartInfo.CreateNoWindow = True
            Process.StartInfo.UseShellExecute = False
            Process.StartInfo.RedirectStandardError = True
            BeginErrorReadLineValue = Value
        End Set
    End Property

    Private RedirectValue As Boolean

    Public Property Redirect() As Boolean
        Get
            Return RedirectValue
        End Get
        Set(ByVal Value As Boolean)
            Process.StartInfo.CreateNoWindow = True
            Process.StartInfo.UseShellExecute = False
            Process.StartInfo.RedirectStandardOutput = True
            RedirectValue = Value
        End Set
    End Property

    Private RedirectOutputValue As String

    Public ReadOnly Property RedirectOutput() As String
        Get
            Return RedirectOutputValue
        End Get
    End Property

    Private BeginOutputReadLineValue As Boolean

    Public Property BeginOutputReadLine() As Boolean
        Get
            Return BeginOutputReadLineValue
        End Get
        Set(ByVal Value As Boolean)
            Process.StartInfo.CreateNoWindow = True
            Process.StartInfo.UseShellExecute = False
            Process.StartInfo.RedirectStandardOutput = True
            BeginOutputReadLineValue = Value
        End Set
    End Property

    Public WriteOnly Property Minimized() As Boolean
        Set(ByVal Value As Boolean)
            If Value Then
                Process.StartInfo.WindowStyle = ProcessWindowStyle.Minimized
            Else
                Process.StartInfo.WindowStyle = ProcessWindowStyle.Normal
            End If
        End Set
    End Property

    Private PriorityValue As ProcessPriorityClass = ProcessPriorityClass.Normal

    Public Property Priority() As ProcessPriorityClass
        Get
            Return PriorityValue
        End Get
        Set(ByVal value As ProcessPriorityClass)
            PriorityValue = value
        End Set
    End Property

    Public Property Directory() As String
        Get
            Return Process.StartInfo.WorkingDirectory
        End Get
        Set(ByVal Value As String)
            If IO.Directory.Exists(Value) Then
                Process.StartInfo.WorkingDirectory = Value
            End If
        End Set
    End Property

    Private ExitCodeValue As Integer

    Public ReadOnly Property ExitCode() As Integer
        Get
            Return ExitCodeValue
        End Get
    End Property

    Public Property File() As String
        Get
            Return Process.StartInfo.FileName
        End Get
        Set(ByVal Value As String)
            If Value.Contains("%") Then
                Process.StartInfo.FileName = Environment.ExpandEnvironmentVariables(Value)
            Else
                Process.StartInfo.FileName = Value
            End If
        End Set
    End Property

    Public Property CommandLine() As String
        Get
            Return """" + File + """ " + Arguments
        End Get
        Set(ByVal Value As String)
            Try
                Dim m = Regex.Match(Value, "((?<file>[^\s""]+)|""(?<file>.+?)"") *(?<args>[^\f\r]*)")
                File = m.Groups("file").Value
                Arguments = m.Groups("args").Value
            Catch
                Throw New Exception("Failed to parse command line: " + Value)
            End Try
        End Set
    End Property

    Public Property Arguments() As String
        Get
            Return Process.StartInfo.Arguments
        End Get
        Set(ByVal Value As String)
            Process.StartInfo.Arguments = Value

            If Process.StartInfo.Arguments.Contains("\""") Then
                Process.StartInfo.Arguments = Process.StartInfo.Arguments.Replace("\""", "\\""")
            End If

            If Process.StartInfo.Arguments.Contains("%") Then
                Process.StartInfo.Arguments = Environment.ExpandEnvironmentVariables(Process.StartInfo.Arguments)
            End If
        End Set
    End Property

    Public Sub KillAndThrow()
        TrowException = True

        Try
            Process.Kill()
        Catch
        End Try
    End Sub

    Private HideAfterStartValue As Boolean

    Public Property HideAfterStart() As Boolean
        Get
            Return HideAfterStartValue
        End Get
        Set(ByVal Value As Boolean)
            HideAfterStartValue = Value
        End Set
    End Property

    <DebuggerNonUserCode()> _
    Public Sub Start()
        Try
            Dim h = Win32.NativeFunctions.GetForegroundWindow()
            Process.Start()

            If Redirect Then
                RedirectOutputValue = Process.StandardOutput.ReadToEnd
            End If

            If BeginErrorReadLine Then
                Process.BeginErrorReadLine()
            End If

            If BeginOutputReadLine Then
                Process.BeginOutputReadLine()
            End If

            'WindowStyle.Hidden yield to window handle zero
            If HideAfterStart Then
                Dim counter As Integer

                While counter < 99 AndAlso Not Process.HasExited
                    Process.Refresh()
                    Dim handle As IntPtr = Process.MainWindowHandle

                    If handle <> IntPtr.Zero Then
                        Win32.Win32Window.Hide(handle)
                        Exit While
                    End If

                    Thread.Sleep(50)
                    counter += 1
                End While
            End If

            Win32.NativeFunctions.SetForegroundWindow(h)
        Catch ex As Exception
            Dim m As String = Nothing

            If OK(File) Then
                m += "File: " + File + CrLf2
            End If

            If OK(Arguments) Then
                m += "Arguments: " + Arguments + CrLf2
            End If

            Msg.Show(m + ex.Message.ToString, MsgIcon.Error)

            If ReTrowException Then
                Throw New AbortException
            End If
        End Try

        Try
            If Priority <> ProcessPriorityClass.Normal AndAlso Not Process.HasExited Then
                Process.PriorityClass = Priority
            End If

            If Wait Then
                Process.WaitForExit()
                ExitCodeValue = Process.ExitCode
                Process.Close()
            End If
        Catch
        End Try

        If TrowException Then
            Throw New AbortException
        End If
    End Sub

    Public Shared Sub StartFile(ByVal path As String)
        Using pw As New Proc
            pw.File = path
            pw.Start()
        End Using
    End Sub

    Public Shared Sub StartCmdl(ByVal cmdl As String)
        Using pw As New Proc
            pw.CommandLine = cmdl
            pw.Start()
        End Using
    End Sub

    Private DisposedValue As Boolean = False

    Protected Overridable Sub Dispose(ByVal disposing As Boolean)
        If Not DisposedValue Then
            If disposing Then
                If Not Process Is Nothing Then
                    Process.Dispose()
                End If
            End If
        End If

        DisposedValue = True
    End Sub

    Public Sub Dispose() Implements IDisposable.Dispose
        Dispose(True)
        GC.SuppressFinalize(Me)
    End Sub
End Class