Imports Microsoft.VisualBasic

Imports System
Imports System.Collections.Generic
Imports System.ComponentModel
Imports System.Drawing
Imports System.Drawing.Design
Imports System.Drawing.Drawing2D
Imports System.Globalization
Imports System.Linq
Imports System.Runtime.InteropServices
Imports System.Text
Imports System.Windows.Forms
Imports System.Windows.Forms.VisualStyles

Imports Stax.Win32

Public Class CustomTrackBar
    Inherits TrackBar

    Private ChannelBounds As Rectangle
    Private ThumbBounds As Rectangle
    Private ThumbState As Integer

    Public Sub New()
        SetStyle(ControlStyles.SupportsTransparentBackColor, True)
    End Sub

    Protected Overrides Sub WndProc(ByRef m As Message)
        If m.Msg = 20 Then
            m.Result = IntPtr.Zero
        Else
            MyBase.WndProc(m)

            If m.Msg = &H204E Then
                Dim struct As NMHDR = CType(Marshal.PtrToStructure(m.LParam, GetType(NMHDR)), NMHDR)

                If struct.code = -12 Then
                    Dim ptr As IntPtr
                    Marshal.StructureToPtr(struct, m.LParam, False)
                    Dim nmcustomdraw As NativeMethods.NMCUSTOMDRAW = CType(Marshal.PtrToStructure(m.LParam, GetType(NativeMethods.NMCUSTOMDRAW)), NativeMethods.NMCUSTOMDRAW)

                    If nmcustomdraw.dwDrawStage = NativeMethods.CustomDrawDrawStage.CDDS_PREPAINT Then
                        Dim graphics As Graphics = graphics.FromHdc(nmcustomdraw.hdc)
                        Dim e As New PaintEventArgs(graphics, Me.Bounds)
                        e.Graphics.TranslateTransform(CSng(0 - Me.Left), CSng(0 - Me.Top))
                        InvokePaintBackground(Me.Parent, e)
                        InvokePaint(Me.Parent, e)
                        Dim brush As New SolidBrush(Me.BackColor)
                        e.Graphics.FillRectangle(brush, Me.Bounds)
                        brush.Dispose()
                        e.Graphics.ResetTransform()
                        e.Dispose()
                        graphics.Dispose()
                        ptr = New IntPtr(&H30)
                        m.Result = ptr
                    ElseIf nmcustomdraw.dwDrawStage = NativeMethods.CustomDrawDrawStage.CDDS_POSTPAINT Then
                        OnDrawTicks(nmcustomdraw.hdc)
                        DrawChannel(nmcustomdraw.hdc)
                        OnDrawThumb(nmcustomdraw.hdc)
                    ElseIf nmcustomdraw.dwDrawStage = NativeMethods.CustomDrawDrawStage.CDDS_ITEMPREPAINT Then
                        If nmcustomdraw.dwItemSpec.ToInt32() = 2 Then
                            ThumbBounds = nmcustomdraw.rc.ToRectangle()

                            If Enabled Then
                                If nmcustomdraw.uItemState = NativeMethods.CustomDrawItemState.CDIS_SELECTED Then
                                    ThumbState = 3
                                Else
                                    ThumbState = 1
                                End If
                            Else
                                ThumbState = 5
                            End If

                            OnDrawThumb(nmcustomdraw.hdc)
                        ElseIf nmcustomdraw.dwItemSpec.ToInt32() = 3 Then
                            ChannelBounds = nmcustomdraw.rc.ToRectangle()
                            DrawChannel(nmcustomdraw.hdc)
                        ElseIf nmcustomdraw.dwItemSpec.ToInt32() = 1 Then
                            OnDrawTicks(nmcustomdraw.hdc)
                        End If

                        ptr = New IntPtr(4)
                        m.Result = ptr
                    End If
                End If
            End If
        End If
    End Sub

    Private Sub DrawHorizontalTicks(ByVal g As Graphics, ByVal color As Color)
        Dim innerTickRect As RectangleF
        Dim numofTicks = Maximum \ TickFrequency - 1
        Dim tickPen As New Pen(color)
        Dim endTickRRect As New RectangleF(CSng(ChannelBounds.Left + ThumbBounds.Width / 2), CSng(ThumbBounds.Top - 5), 0.0F, 3.0F)
        Dim endTickLRect As New RectangleF(CSng((ChannelBounds.Right - ThumbBounds.Width / 2) - 1), CSng(ThumbBounds.Top - 5), 0.0F, 3.0F)
        Dim tickPitch As Single = (endTickLRect.Right - endTickRRect.Left) / (CSng(numofTicks + 1))

        'Draw upper (top) ticks
        If Me.TickStyle <> TickStyle.BottomRight Then
            'Draw right outer tick
            g.DrawLine(tickPen, endTickRRect.Left, endTickRRect.Top, endTickRRect.Right, endTickRRect.Bottom)
            'Draw left outer tick
            g.DrawLine(tickPen, endTickLRect.Left, endTickLRect.Top, endTickLRect.Right, endTickLRect.Bottom)
            'Draw inner ticks
            innerTickRect = endTickRRect
            innerTickRect.Height -= 1
            innerTickRect.Offset(tickPitch, 1.0F)
            Dim numOfInnerTicks As Integer = numofTicks - 1

            For i = 0 To numOfInnerTicks
                g.DrawLine(tickPen, innerTickRect.Left, innerTickRect.Top, innerTickRect.Left, innerTickRect.Bottom)
                innerTickRect.Offset(tickPitch, 0.0F)
            Next
        End If

        endTickRRect.Offset(0.0F, CSng(Me.ThumbBounds.Height + 6))
        endTickLRect.Offset(0.0F, CSng(Me.ThumbBounds.Height + 6))

        'Draw lower (bottom) ticks
        If TickStyle <> TickStyle.TopLeft Then
            'Draw right outer tick
            g.DrawLine(tickPen, endTickRRect.Left, endTickRRect.Top, endTickRRect.Left, endTickRRect.Bottom)
            'Draw left outer tick
            g.DrawLine(tickPen, endTickLRect.Left, endTickLRect.Top, endTickLRect.Left, endTickLRect.Bottom)
            'Draw innerticks
            innerTickRect = endTickRRect
            innerTickRect.Height -= 1
            innerTickRect.Offset(tickPitch, 0.0F)
            Dim numOfInnerTicks = numofTicks - 1

            For j As Integer = 0 To numOfInnerTicks
                g.DrawLine(tickPen, innerTickRect.Left, innerTickRect.Top, innerTickRect.Left, innerTickRect.Bottom)
                innerTickRect.Offset(tickPitch, 0.0F)
            Next
        End If

        tickPen.Dispose()
    End Sub

    Private Sub DrawVerticalTicks(ByVal g As Graphics, ByVal color As Color)
        Dim innerTickRect As RectangleF
        Dim numOfTicks As Integer = (Me.Maximum \ Me.TickFrequency) - 1
        Dim tickPen As New Pen(color)
        Dim endTickBottomRect As New RectangleF(CSng(Me.ThumbBounds.Left - 5), CSng((Me.ChannelBounds.Bottom - (Me.ThumbBounds.Height / 2)) - 1), 3.0F, 0.0F)
        Dim endTickTopRect As New RectangleF(CSng(Me.ThumbBounds.Left - 5), CSng(Me.ChannelBounds.Top + (Me.ThumbBounds.Height / 2)), 3.0F, 0.0F)
        Dim y As Single = (endTickTopRect.Bottom - endTickBottomRect.Top) / (CSng(numOfTicks + 1))

        'Draw left-hand ticks
        If Me.TickStyle <> TickStyle.BottomRight Then
            'Draw lower (bottom) outer tick
            g.DrawLine(tickPen, endTickBottomRect.Left, endTickBottomRect.Top, endTickBottomRect.Right, endTickBottomRect.Bottom)
            'Draw upper (top) outer tick
            g.DrawLine(tickPen, endTickTopRect.Left, endTickTopRect.Top, endTickTopRect.Right, endTickTopRect.Bottom)
            'Draw inner ticks
            innerTickRect = endTickBottomRect
            innerTickRect.Width -= 1
            innerTickRect.Offset(1.0F, y)
            Dim numOfInnerTicks As Integer = numOfTicks - 1

            For i = 0 To numOfInnerTicks
                g.DrawLine(tickPen, innerTickRect.Left, innerTickRect.Top, innerTickRect.Right, innerTickRect.Bottom)
                innerTickRect.Offset(0.0F, y)
            Next
        End If

        endTickBottomRect.Offset(CSng(Me.ThumbBounds.Width + 6), 0.0F)
        endTickTopRect.Offset(CSng(Me.ThumbBounds.Width + 6), 0.0F)

        'Draw right-hand ticks
        If TickStyle <> TickStyle.TopLeft Then
            'Draw lower (bottom) tick
            g.DrawLine(tickPen, endTickBottomRect.Left, endTickBottomRect.Top, endTickBottomRect.Right, endTickBottomRect.Bottom)
            'Draw upper (top) tick
            g.DrawLine(tickPen, endTickTopRect.Left, endTickTopRect.Top, endTickTopRect.Right, endTickTopRect.Bottom)
            'Draw inner ticks
            innerTickRect = endTickBottomRect
            innerTickRect.Width -= 1
            innerTickRect.Offset(0.0F, y)
            Dim numOfInnerTicks = numOfTicks - 1

            For i = 0 To numOfInnerTicks
                g.DrawLine(tickPen, innerTickRect.Left, innerTickRect.Top, innerTickRect.Right, innerTickRect.Bottom)
                innerTickRect.Offset(0.0F, y)
            Next
        End If

        tickPen.Dispose()
    End Sub

    ''' Draw a downward pointer
    Private Sub DrawPointerDown(ByVal g As Graphics)
        Dim points() As Point = {New Point(ThumbBounds.Left + ThumbBounds.Width \ 2, ThumbBounds.Bottom - 1), New Point(ThumbBounds.Left, (ThumbBounds.Bottom - ThumbBounds.Width \ 2) - 1), ThumbBounds.Location, New Point(ThumbBounds.Right - 1, ThumbBounds.Top), New Point(ThumbBounds.Right - 1, (ThumbBounds.Bottom - ThumbBounds.Width \ 2) - 1), New Point(ThumbBounds.Left + ThumbBounds.Width \ 2, ThumbBounds.Bottom - 1)}
        Dim path As New GraphicsPath()
        path.AddLines(points)
        Dim region As New Region(path)
        g.Clip = region

        If ThumbState = 3 OrElse Not Enabled Then
            ControlPaint.DrawButton(g, Me.ThumbBounds, ButtonState.All)
        Else
            g.Clear(SystemColors.Control)
        End If

        g.ResetClip()
        region.Dispose()
        path.Dispose()
        'Draw light shadow
        Dim shadowPoints() As Point = {points(0), points(1), points(2), points(3)}
        g.DrawLines(SystemPens.ControlLightLight, shadowPoints)
        'Draw dark shadow
        shadowPoints = New Point() {points(3), points(4), points(5)}
        g.DrawLines(SystemPens.ControlDarkDark, shadowPoints)
        points(0).Offset(0, -1)
        points(1).Offset(1, 0)
        points(2).Offset(1, 1)
        points(3).Offset(-1, 1)
        points(4).Offset(-1, 0)
        points(5) = points(0)
        shadowPoints = New Point() {points(0), points(1), points(2), points(3)}
        g.DrawLines(SystemPens.ControlLight, shadowPoints)
        shadowPoints = New Point() {points(3), points(4), points(5)}
        g.DrawLines(SystemPens.ControlDark, shadowPoints)
    End Sub

    Private Sub DrawPointerLeft(ByVal g As Graphics)
        Dim points() As Point = {New Point(ThumbBounds.Left, ThumbBounds.Top + (ThumbBounds.Height \ 2)), New Point(ThumbBounds.Left + (ThumbBounds.Height \ 2), ThumbBounds.Top), New Point(ThumbBounds.Right - 1, Me.ThumbBounds.Top), New Point(ThumbBounds.Right - 1, ThumbBounds.Bottom - 1), New Point(ThumbBounds.Left + ThumbBounds.Height \ 2, ThumbBounds.Bottom - 1), New Point(ThumbBounds.Left, ThumbBounds.Top + (ThumbBounds.Height \ 2))}
        Dim path As New GraphicsPath()
        path.AddLines(points)
        Dim region As New Region(path)
        g.Clip = region
        If (Me.ThumbState = 3) OrElse (Not Me.Enabled) Then
            ControlPaint.DrawButton(g, Me.ThumbBounds, ButtonState.All)
        Else
            g.Clear(SystemColors.Control)
        End If
        g.ResetClip()
        region.Dispose()
        path.Dispose()
        ' Draw light shadow
        Dim shadowPoints() As Point = {points(0), points(1), points(2)}
        g.DrawLines(SystemPens.ControlLightLight, shadowPoints)
        ' Draw dark shadow
        shadowPoints = New Point() {points(2), points(3), points(4), points(5)}
        g.DrawLines(SystemPens.ControlDarkDark, shadowPoints)
        points(0).Offset(1, 0)
        points(1).Offset(0, 1)
        points(2).Offset(-1, 1)
        points(3).Offset(-1, -1)
        points(4).Offset(0, -1)
        points(5) = points(0)
        shadowPoints = New Point() {points(0), points(1), points(2)}
        g.DrawLines(SystemPens.ControlLight, shadowPoints)
        shadowPoints = New Point() {points(2), points(3), points(4), points(5)}
        g.DrawLines(SystemPens.ControlDark, shadowPoints)
    End Sub

    Private Sub DrawPointerRight(ByVal g As Graphics)
        Dim points() As Point = {New Point(ThumbBounds.Left, ThumbBounds.Bottom - 1), New Point(ThumbBounds.Left, ThumbBounds.Top), New Point((ThumbBounds.Right - ThumbBounds.Height \ 2) - 1, ThumbBounds.Top), New Point(ThumbBounds.Right - 1, ThumbBounds.Top + ThumbBounds.Height \ 2), New Point((ThumbBounds.Right - ThumbBounds.Height \ 2) - 1, ThumbBounds.Bottom - 1), New Point(ThumbBounds.Left, ThumbBounds.Bottom - 1)}
        Dim path As New GraphicsPath()
        path.AddLines(points)
        Dim region As New Region(path)
        g.Clip = region

        If (Me.ThumbState = 3) OrElse (Not Me.Enabled) Then
            ControlPaint.DrawButton(g, Me.ThumbBounds, ButtonState.All)
        Else
            g.Clear(SystemColors.Control)
        End If

        g.ResetClip()
        region.Dispose()
        path.Dispose()
        'Draw light shadow
        Dim shadowPoints() As Point = {points(0), points(1), points(2), points(3)}
        g.DrawLines(SystemPens.ControlLightLight, shadowPoints)
        'Draw dark shadow
        shadowPoints = New Point() {points(3), points(4), points(5)}
        g.DrawLines(SystemPens.ControlDarkDark, shadowPoints)
        points(0).Offset(1, -1)
        points(1).Offset(1, 1)
        points(2).Offset(0, 1)
        points(3).Offset(-1, 0)
        points(4).Offset(0, -1)
        points(5) = points(0)
        shadowPoints = New Point() {points(0), points(1), points(2), points(3)}
        g.DrawLines(SystemPens.ControlLight, shadowPoints)
        shadowPoints = New Point() {points(3), points(4), points(5)}
        g.DrawLines(SystemPens.ControlDark, shadowPoints)
    End Sub

    Private Sub DrawPointerUp(ByVal g As Graphics)
        Dim points() As Point = {New Point(ThumbBounds.Left, ThumbBounds.Bottom - 1), New Point(ThumbBounds.Left, ThumbBounds.Top + ThumbBounds.Width \ 2), New Point(ThumbBounds.Left + ThumbBounds.Width \ 2, ThumbBounds.Top), New Point(ThumbBounds.Right - 1, ThumbBounds.Top + ThumbBounds.Width \ 2), New Point(ThumbBounds.Right - 1, ThumbBounds.Bottom - 1), New Point(ThumbBounds.Left, ThumbBounds.Bottom - 1)}
        Dim path As New GraphicsPath()
        path.AddLines(points)
        Dim region As New Region(path)
        g.Clip = region

        If ThumbState = 3 OrElse Not Enabled Then
            ControlPaint.DrawButton(g, ThumbBounds, ButtonState.All)
        Else
            g.Clear(SystemColors.Control)
        End If

        g.ResetClip()
        region.Dispose()
        path.Dispose()
        'Draw light shadow
        Dim shadowPoints() As Point = {points(0), points(1), points(2)}
        g.DrawLines(SystemPens.ControlLightLight, shadowPoints)
        'Draw dark shadow
        shadowPoints = New Point() {points(2), points(3), points(4), points(5)}
        g.DrawLines(SystemPens.ControlDarkDark, shadowPoints)
        points(0).Offset(1, -1)
        points(1).Offset(1, 0)
        points(2).Offset(0, 1)
        points(3).Offset(-1, 0)
        points(4).Offset(-1, -1)
        points(5) = points(0)
        shadowPoints = New Point() {points(0), points(1), points(2)}
        g.DrawLines(SystemPens.ControlLight, shadowPoints)
        shadowPoints = New Point() {points(2), points(3), points(4), points(5)}
        g.DrawLines(SystemPens.ControlDark, shadowPoints)
    End Sub

    Protected Overridable Sub OnDrawTicks(ByVal hdc As IntPtr)
        Using graphics = Drawing.Graphics.FromHdc(hdc)
            If TickStyle = TickStyle.None OrElse ThumbBounds.Equals(Rectangle.Empty) Then
                Return
            End If

            Dim black = Color.Black

            If VisualStyleRenderer.IsSupported Then
                Dim vsr As New VisualStyleRenderer("TRACKBAR", CInt(Fix(NativeMethods.TrackBarParts.TKP_TICS)), ThumbState)
                black = vsr.GetColor(ColorProperty.TextColor)
            End If

            If Orientation = Orientation.Horizontal Then
                DrawHorizontalTicks(graphics, black)
            Else
                DrawVerticalTicks(graphics, black)
            End If
        End Using
    End Sub

    Protected Overridable Sub OnDrawThumb(ByVal hdc As IntPtr)
        Using g = Graphics.FromHdc(hdc)
            g.Clip = New Region(ThumbBounds)
            'Determine the style of the thumb, based on the tickstyle
            Dim part = NativeMethods.TrackBarParts.TKP_THUMB

            If ThumbBounds.Equals(Rectangle.Empty) Then
                Exit Sub
            End If

            Select Case Me.TickStyle
                Case TickStyle.None, TickStyle.BottomRight
                    part = If((Orientation <> Orientation.Horizontal), NativeMethods.TrackBarParts.TKP_THUMBRIGHT, NativeMethods.TrackBarParts.TKP_THUMBBOTTOM)
                Case TickStyle.TopLeft
                    part = If((Orientation <> Orientation.Horizontal), NativeMethods.TrackBarParts.TKP_THUMBLEFT, NativeMethods.TrackBarParts.TKP_THUMBTOP)

                Case TickStyle.Both
                    part = If((Orientation <> Orientation.Horizontal), NativeMethods.TrackBarParts.TKP_THUMBVERT, NativeMethods.TrackBarParts.TKP_THUMB)
            End Select

            'Perform drawing
            If VisualStyleRenderer.IsSupported Then
                Dim vsr As New VisualStyleRenderer("TRACKBAR", CInt(Fix(part)), ThumbState)
                vsr.DrawBackground(g, Me.ThumbBounds)
                g.ResetClip()
                Exit Sub
            Else
                Select Case part
                    Case NativeMethods.TrackBarParts.TKP_THUMBBOTTOM
                        DrawPointerDown(g)
                    Case NativeMethods.TrackBarParts.TKP_THUMBTOP
                        DrawPointerUp(g)
                    Case NativeMethods.TrackBarParts.TKP_THUMBLEFT
                        DrawPointerLeft(g)
                    Case NativeMethods.TrackBarParts.TKP_THUMBRIGHT
                        DrawPointerRight(g)
                    Case Else
                        If (ThumbState = 3) OrElse (Not Me.Enabled) Then
                            ControlPaint.DrawButton(g, Me.ThumbBounds, ButtonState.All)
                        Else
                            'Tick-style is both - draw the thumb as a solid rectangle
                            g.FillRectangle(SystemBrushes.Control, Me.ThumbBounds)
                        End If

                        ControlPaint.DrawBorder3D(g, Me.ThumbBounds, Border3DStyle.Raised)
                End Select
            End If

            g.ResetClip()
        End Using
    End Sub

    Protected Overridable Sub DrawChannel(ByVal hdc As IntPtr)
        Using g = Graphics.FromHdc(hdc)
            If ChannelBounds.Equals(Rectangle.Empty) Then
                Exit Sub
            End If

            If VisualStyleRenderer.IsSupported Then
                Dim vsr As New VisualStyleRenderer("TRACKBAR", CInt(Fix(NativeMethods.TrackBarParts.TKP_TRACK)), CInt(Fix(TrackBarItemState.Normal)))
                vsr.DrawBackground(g, ChannelBounds)
                g.ResetClip()
                Return
            End If

            ControlPaint.DrawBorder3D(g, ChannelBounds, Border3DStyle.Sunken)
        End Using
    End Sub
End Class

Public Class NativeMethods
    Public Const NM_CUSTOMDRAW As Integer = -12
    Public Const NM_FIRST As Integer = 0
    Public Const S_OK As Integer = 0
    Public Const TMT_COLOR As Integer = &HCC

    Private Sub New()
    End Sub

    <DllImport("UxTheme.dll", CallingConvention:=CallingConvention.Cdecl)> _
    Public Shared Function CloseThemeData(ByVal hTheme As IntPtr) As Integer
    End Function

    <DllImport("Comctl32.dll", EntryPoint:="DllGetVersion", CallingConvention:=CallingConvention.Cdecl)> _
    Public Shared Function CommonControlsGetVersion(ByRef pdvi As DLLVERSIONINFO) As Integer
    End Function

    <DllImport("UxTheme.dll", CallingConvention:=CallingConvention.Cdecl)> _
    Public Shared Function DrawThemeBackground(ByVal hTheme As IntPtr, ByVal hdc As IntPtr, ByVal iPartId As Integer, ByVal iStateId As Integer, ByRef pRect As RECT, ByRef pClipRect As RECT) As Integer
    End Function

    <DllImport("UxTheme.dll", CallingConvention:=CallingConvention.Cdecl)> _
    Public Shared Function GetThemeColor(ByVal hTheme As IntPtr, ByVal iPartId As Integer, ByVal iStateId As Integer, ByVal iPropId As Integer, ByRef pColor As Integer) As Integer
    End Function

    <DllImport("UxTheme.dll", CallingConvention:=CallingConvention.Cdecl)> _
    Public Shared Function IsAppThemed() As <MarshalAs(UnmanagedType.Bool)> Boolean
    End Function

    <DllImport("UxTheme.dll", CallingConvention:=CallingConvention.Cdecl, CharSet:=CharSet.Unicode)> _
    Public Shared Function OpenThemeData(ByVal hwnd As IntPtr, ByVal pszClassList As String) As IntPtr
    End Function

    Public Enum CustomDrawDrawStage
        CDDS_ITEM = &H10000
        CDDS_ITEMPOSTERASE = &H10004
        CDDS_ITEMPOSTPAINT = &H10002
        CDDS_ITEMPREERASE = &H10003
        CDDS_ITEMPREPAINT = &H10001
        CDDS_POSTERASE = 4
        CDDS_POSTPAINT = 2
        CDDS_PREERASE = 3
        CDDS_PREPAINT = 1
        CDDS_SUBITEM = &H20000
    End Enum

    Public Enum CustomDrawItemState
        CDIS_CHECKED = 8
        CDIS_DEFAULT = &H20
        CDIS_DISABLED = 4
        CDIS_FOCUS = &H10
        CDIS_GRAYED = 2
        CDIS_HOT = &H40
        CDIS_INDETERMINATE = &H100
        CDIS_MARKED = &H80
        CDIS_SELECTED = 1
        CDIS_SHOWKEYBOARDCUES = &H200
    End Enum

    Public Enum CustomDrawReturnFlags
        CDRF_DODEFAULT = 0
        CDRF_NEWFONT = 2
        CDRF_NOTIFYITEMDRAW = &H20
        CDRF_NOTIFYPOSTERASE = &H40
        CDRF_NOTIFYPOSTPAINT = &H10
        CDRF_NOTIFYSUBITEMDRAW = &H20
        CDRF_SKIPDEFAULT = 4
    End Enum

    Public Enum TrackBarCustomDrawPart
        TBCD_CHANNEL = 3
        TBCD_THUMB = 2
        TBCD_TICS = 1
    End Enum

    Public Enum TrackBarParts
        TKP_THUMB = 3
        TKP_THUMBBOTTOM = 4
        TKP_THUMBLEFT = 7
        TKP_THUMBRIGHT = 8
        TKP_THUMBTOP = 5
        TKP_THUMBVERT = 6
        TKP_TICS = 9
        TKP_TICSVERT = 10
        TKP_TRACK = 1
        TKP_TRACKVERT = 2
    End Enum

    <StructLayout(LayoutKind.Sequential)> _
    Public Structure DLLVERSIONINFO
        Public cbSize As Integer
        Public dwMajorVersion As Integer
        Public dwMinorVersion As Integer
        Public dwBuildNumber As Integer
        Public dwPlatformID As Integer
    End Structure

    <StructLayout(LayoutKind.Sequential)> _
    Public Structure NMCUSTOMDRAW
        Public hdr As NMHDR
        Public dwDrawStage As NativeMethods.CustomDrawDrawStage
        Public hdc As IntPtr
        Public rc As RECT
        Public dwItemSpec As IntPtr
        Public uItemState As NativeMethods.CustomDrawItemState
        Public lItemlParam As IntPtr
    End Structure
End Class

Public Enum TrackBarItemState
    Active = 3
    Disabled = 5
    Hot = 2
    Normal = 1
End Enum