Imports System.ComponentModel
Imports System.Runtime.InteropServices

Imports Stax
Imports Stax.Win32
Imports Stax.AppFramework

Namespace UI
    Public Class OptionListBox
        Inherits ListBox
        Implements IHostedControl

        Private Tooltip As ToolTip
        Private TooltipItem As Object
        Private Timer As Timer

        Public Sub New()
            DrawMode = DrawMode.OwnerDrawFixed
            ItemHeight = 20
            IntegralHeight = True

            If StaxApplication.ShowToolTips Then
                Tooltip = New ToolTip
                Timer = New Timer
                Timer.Interval = Tooltip.InitialDelay
                AddHandler Timer.Tick, AddressOf TimerTick
            End If
        End Sub

        Private Sub TimerTick(ByVal sender As Object, ByVal e As EventArgs)
            If Not TooltipItem Is Nothing AndAlso TooltipItem Is SelectedItem Then
                Dim p = PointToClient(Control.MousePosition)
                p.Y = p.Y + (Cursor.Size.Height - Cursor.HotSpot.Y) + 2
                Tooltip.Show(Cast(TooltipItem).ToolTip, Me, p, 20000)
                Timer.Stop()
            End If
        End Sub

        Private TextOffsetValue As Integer

        <Category("OptionListBox"), DefaultValue(0)> _
        Public Property TextOffset() As Integer
            Get
                Return TextOffsetValue
            End Get
            Set(ByVal Value As Integer)
                TextOffsetValue = Value
            End Set
        End Property

        Protected Overrides Sub RefreshItem(ByVal index As Integer)
            MyBase.RefreshItem(index)
        End Sub

        Protected Overrides Sub SetItemsCore(ByVal items As IList)
            MyBase.SetItemsCore(items)
        End Sub

        Protected Overrides Sub OnDrawItem(ByVal e As DrawItemEventArgs)
            MyBase.OnDrawItem(e)

            If Items.Count > 0 AndAlso e.Index > -1 Then
                Dim g = e.Graphics
                Dim isSelected = ((e.State And DrawItemState.Selected) = DrawItemState.Selected)
                Dim item = Cast(Items(e.Index))

                If isSelected Then
                    Using b As New SolidBrush(SystemColors.Control)
                        g.FillRectangle(b, e.Bounds)
                    End Using

                    ControlPaint.DrawFocusRectangle(g, e.Bounds, SystemColors.Control, BackColor)
                Else
                    Using b As New SolidBrush(BackColor)
                        g.FillRectangle(b, e.Bounds)
                    End Using
                End If

                If Not item.Image Is Nothing Then
                    g.DrawImage(item.Image, e.Bounds.Left, e.Bounds.Top + (ItemHeight - item.Image.Height) \ 2, 16, 16)
                End If

                Using b As New SolidBrush(ForeColor)
                    g.DrawString(item.Text, Font, b, e.Bounds.Left + TextOffset, e.Bounds.Top + (ItemHeight - Font.Height) \ 2)
                End Using
            End If
        End Sub

        Protected Overrides Sub OnSelectedIndexChanged(ByVal e As EventArgs)
            MyBase.OnSelectedIndexChanged(e)

            If Not Tooltip Is Nothing Then
                Dim item = Cast(SelectedItem)
                Tooltip.Hide(Me)

                If Not item Is Nothing AndAlso OK(item.ToolTip) Then
                    TooltipItem = SelectedItem

                    If Not Timer Is Nothing Then
                        Timer.Start()
                    End If
                End If
            End If
        End Sub

        Protected Overrides Sub OnMouseMove(ByVal e As MouseEventArgs)
            MyBase.OnMouseMove(e)
            SelectAtMouse()
        End Sub

        Public Sub SelectAtMouse()
            Dim index = IndexFromPoint(PointToClient(Control.MousePosition))

            If index <> SelectedIndex Then
                SelectedIndex = index
            End If
        End Sub

        Private Function Cast(ByVal o As Object) As OptionListBoxItem
            Return DirectCast(o, OptionListBoxItem)
        End Function

        Protected Overrides Sub OnMouseUp(ByVal e As MouseEventArgs)
            MyBase.OnMouseUp(e)
            Dim item = Cast(SelectedItem)

            If Not item Is Nothing Then
                If e.Button = MouseButtons.Left Then
                    If Not Tooltip Is Nothing Then
                        Tooltip.Dispose() 'hide don't work when comp check is invoked
                        Tooltip = New ToolTip
                    End If

                    item.ClickDelegate.Invoke()
                ElseIf e.Button = MouseButtons.Right Then
                    If Not String.IsNullOrEmpty(item.ToolTip) Then
                        Dim p As New PopUp
                        p.Title = item.Text
                        p.Text = item.ToolTip
                        p.Show()
                    End If
                End If
            End If
        End Sub

        Protected Overrides Sub OnMouseLeave(ByVal e As EventArgs)
            SelectedItem = Nothing
            MyBase.OnMouseLeave(e)
        End Sub

        Protected Overrides Sub Dispose(ByVal disposing As Boolean)
            MyBase.Dispose(disposing)

            If Not Tooltip Is Nothing Then
                Tooltip.Dispose()
            End If
        End Sub

        Public Sub ShowHelp() Implements IHostedControl.ShowHelp
            Dim l As New StringPairList

            For Each i As OptionListBoxItem In Items
                If OK(i.ToolTip) Then
                    l.Add(i.Text, i.ToolTip)
                End If
            Next

            Dim f As New HelpForm()
            f.Doc.WriteStart(FindForm.Text)
            f.Doc.WriteTips(l)
            f.Show()
        End Sub

        Private HelpIDValue As String

        Public Property HelpID() As String Implements IHostedControl.HelpID
            Get
                Return HelpIDValue
            End Get
            Set(ByVal value As String)
                HelpIDValue = value
            End Set
        End Property
    End Class

    Public Class OptionListBoxItem
        Public Sub New()
        End Sub

        Private TextValue As String

        Public Property Text() As String
            Get
                Return TextValue
            End Get
            Set(ByVal Value As String)
                TextValue = Value
            End Set
        End Property

        Private ToolTipValue As String

        Public Property ToolTip() As String
            Get
                Return ToolTipValue
            End Get
            Set(ByVal Value As String)
                ToolTipValue = Value
            End Set
        End Property

        Private ClickDelegateValue As Action

        Public Property ClickDelegate() As Action
            Get
                Return ClickDelegateValue
            End Get
            Set(ByVal Value As Action)
                ClickDelegateValue = Value
            End Set
        End Property

        Private ImageValue As Image

        Public Property Image() As Image
            Get
                Return ImageValue
            End Get
            Set(ByVal Value As Image)
                ImageValue = Value
            End Set
        End Property

        Public Overrides Function ToString() As String
            Return Text
        End Function
    End Class
End Namespace