Imports System
Imports System.Drawing
Imports System.Drawing.Design
Imports System.Collections
Imports System.ComponentModel
Imports System.ComponentModel.Design
Imports System.Windows.Forms
Imports System.Text
Imports System.Reflection
Imports System.Windows.Forms.Design
Imports System.Text.RegularExpressions
Imports System.Globalization

Namespace UI.Design
    Public Class CommonActionList
        Inherits DesignerActionList

        Private Items As New DesignerActionItemCollection()
        Private Lists As DesignerActionListCollection

        Public Sub New(ByVal component As IComponent, ByVal lists As DesignerActionListCollection)
            MyBase.New(component)
            Me.Lists = lists
        End Sub

        Protected Function FixControlName(ByVal value As String) As String
            Dim ret = CultureInfo.CurrentCulture.TextInfo.ToTitleCase(value)

            For Each i As Char In value
                If Not Char.IsLetterOrDigit(i) Then
                    ret = ret.Replace(i, "")
                End If
            Next

            Return ret
        End Function

        Protected Sub RefreshComponent()
            DirectCast(GetService(GetType(DesignerActionUIService)), DesignerActionUIService).Refresh(Component)
        End Sub

        Public Sub SetValue(ByVal propertyName As String, ByVal value As Object)
            SetValue(Component, propertyName, value)
        End Sub

        Public Sub SetValue(ByVal c As IComponent, ByVal propertyName As String, ByVal value As Object)
            Dim pd = TypeDescriptor.GetProperties(c)(propertyName)

            If Not pd Is Nothing Then
                pd.SetValue(c, value)
            End If
        End Sub

        Public Function GetValue(Of T)(ByVal propertyName As String) As T
            Return GetValue(Of T)(Component, propertyName)
        End Function

        Public Function GetValue(Of T)(ByVal c As IComponent, ByVal propertyName As String) As T
            Dim pd = TypeDescriptor.GetProperties(c)(propertyName)

            If Not pd Is Nothing Then
                Return DirectCast(pd.GetValue(c), T)
            End If
        End Function

        Private Function GetPrefix() As String
            Dim ret As String = Nothing

            Dim typeName = GetComponent.GetType.Name

            For Each i In typeName
                If Char.IsUpper(i) Then
                    ret += i
                End If
            Next

            If typeName.StartsWith("Extended") Then
                ret = ret.Substring(1)
            End If

            Return ret.ToLower
        End Function

        Private Function GetComponent() As Component
            Return DirectCast(Component, Component)
        End Function

        Public Property Text() As String
            Get
                Return GetValue(Of String)("Text")
            End Get
            Set(ByVal Value As String)
                SetValue("Text", Value)

                If OK(Value) Then
                    SetValue("Name", GetPrefix() + FixControlName(Value))
                End If

                RefreshComponent()
            End Set
        End Property

        Public Property Name() As String
            Get
                Return GetValue(Of String)("Name")
            End Get
            Set(ByVal value As String)
                SetValue("Name", value)
            End Set
        End Property

        Public Property TipName() As String
            Get
                Return GetValue(Of String)("TipName")
            End Get
            Set(ByVal value As String)
                SetValue("TipName", value)
            End Set
        End Property

        <Editor(GetType(StringEditor), GetType(UITypeEditor))> _
        Public Property TipText() As String
            Get
                Return GetValue(Of String)("TipText")
            End Get
            Set(ByVal value As String)
                SetValue("TipText", value)
            End Set
        End Property

        Public Property Multiline() As Boolean
            Get
                Return GetValue(Of Boolean)("Multiline")
            End Get
            Set(ByVal value As Boolean)
                SetValue("Multiline", value)
            End Set
        End Property

        Public Property Title() As String
            Get
                Return GetValue(Of String)("Title")
            End Get
            Set(ByVal Value As String)
                SetValue("Title", Value)
                Dim name As String = GetPrefix() + FixControlName(Value)
                SetValue("Name", name)
                RefreshComponent()
            End Set
        End Property

        Public Property Description() As String
            Get
                Dim ret As String = GetValue(Of String)("Description")

                If ret Is Nothing Then
                    ret = ""
                End If

                Return ret
            End Get
            Set(ByVal value As String)
                SetValue("Description", value)
            End Set
        End Property

        <Editor(GetType(AnchorTypeEditor), GetType(UITypeEditor))> _
        Public Property Anchor() As AnchorStyles
            Get
                Return GetValue(Of AnchorStyles)("Anchor")
            End Get
            Set(ByVal Value As AnchorStyles)
                SetValue("Anchor", Value)
            End Set
        End Property

        Public Property TextAlign() As ContentAlignment
            Get
                Return GetValue(Of ContentAlignment)("TextAlign")
            End Get
            Set(ByVal Value As ContentAlignment)
                SetValue("TextAlign", Value)
            End Set
        End Property

        Public Property AutoSize() As Boolean
            Get
                Return GetValue(Of Boolean)("AutoSize")
            End Get
            Set(ByVal Value As Boolean)
                SetValue("AutoSize", Value)
            End Set
        End Property

        Public Property DropDownStyle() As ComboBoxStyle
            Get
                Return GetValue(Of ComboBoxStyle)("DropDownStyle")
            End Get
            Set(ByVal value As ComboBoxStyle)
                SetValue("DropDownStyle", value)
            End Set
        End Property

        Private Sub SetSize80x24()
            SetValue("Size", New Size(80, 24))
        End Sub

        Public Sub Size24x24()
            SetValue("Size", New Size(24, 24))
        End Sub

        Public Sub AddButton()
            Dim host = DirectCast(GetService(GetType(IDesignerHost)), IDesignerHost)
            Dim control = DirectCast(host.CreateComponent(GetType(ExtendedButton)), Control)
            Dim container = DirectCast(Component, Control)
            container.Controls.Add(control)
            SetValue(control, "Text", Me.GetValue(Of String)(control, "Name"))
            SetValue(control, "Location", New Point(0, 100))
            SetValue(control, "Size", New Size(80, 24))
            control.BringToFront()
        End Sub

        Public Sub AddCheckBox()
            Dim host = DirectCast(GetService(GetType(IDesignerHost)), IDesignerHost)
            Dim control = DirectCast(host.CreateComponent(GetType(ExtendedCheckBox)), Control)
            Dim container = DirectCast(Component, Control)
            container.Controls.Add(control)
            SetValue(control, "Text", Me.GetValue(Of String)(control, "Name"))
            SetValue(control, "Location", New Point(0, 100))
            SetValue(control, "AutoSize", True)
            control.BringToFront()
        End Sub

        Public Sub AddLabel()
            Dim host = DirectCast(GetService(GetType(IDesignerHost)), IDesignerHost)
            Dim control = DirectCast(host.CreateComponent(GetType(ExtendedLabel)), Control)
            Dim container = DirectCast(Component, Control)
            container.Controls.Add(control)
            SetValue(control, "Text", Me.GetValue(Of String)(control, "Name"))
            SetValue(control, "Location", New Point(0, 100))
            SetValue(control, "AutoSize", True)
            control.BringToFront()
        End Sub

        Public Sub AddTextBox()
            Dim host = DirectCast(GetService(GetType(IDesignerHost)), IDesignerHost)
            Dim control = DirectCast(host.CreateComponent(GetType(ExtendedTextbox)), Control)
            Dim container = DirectCast(Component, Control)
            container.Controls.Add(control)
            SetValue(control, "Location", New Point(0, 100))
            control.BringToFront()
        End Sub

        Public Property Width() As Integer
            Get
                Return GetValue(Of Integer)("Width")
            End Get
            Set(ByVal Value As Integer)
                SetValue("Width", Value)
            End Set
        End Property

        Public Property Height() As Integer
            Get
                Return GetValue(Of Integer)("Height")
            End Get
            Set(ByVal Value As Integer)
                SetValue("Height", Value)
            End Set
        End Property

        Public Sub FlipOrientation()
            Dim h As Integer = GetValue(Of Integer)("Height")
            Dim w As Integer = GetValue(Of Integer)("Width")
            SetValue("Height", w)
            SetValue("Width", h)
        End Sub

        Private Function IsContaining(ByVal name As String) As Boolean
            For Each i As DesignerActionList In Lists
                If Not i Is Me Then
                    For Each i2 As DesignerActionItem In i.GetSortedActionItems
                        If i2.DisplayName = name Then
                            Return True
                        End If
                    Next
                End If
            Next

            Return False
        End Function

        Public Sub AddPropertyItem(ByVal name As String, ByVal ParamArray t As Type())
            If Not IsContaining(name) Then
                For Each i As Type In t
                    If GetComponent.GetType Is i OrElse _
                        GetComponent.GetType.IsSubclassOf(i) Then

                        Items.Add(New DesignerActionPropertyItem(name, name))
                    End If
                Next
            End If
        End Sub

        Public Sub AddMethodItem(ByVal name As String, ByVal displayName As String, ByVal ParamArray t As Type())
            If Not IsContaining(displayName) Then
                For Each i As Type In t
                    If GetComponent.GetType Is i OrElse _
                        GetComponent.GetType.IsSubclassOf(i) Then

                        Items.Add(New DesignerActionMethodItem(Me, name, displayName, True))
                    End If
                Next
            End If
        End Sub

        Public Overrides Function GetSortedActionItems() As DesignerActionItemCollection
            Items.Clear()

            AddPropertyItem("Text", _
                GetType(TextBox), _
                GetType(GroupBox), _
                GetType(Label), _
                GetType(Button), _
                GetType(CheckBox))

            AddPropertyItem("Title", GetType(CommandButton))
            AddPropertyItem("Name", GetType(Control))
            AddPropertyItem("Description", GetType(CommandButton))

            If Not TypeDescriptor.GetProperties(Component)("TipText") Is Nothing Then
                AddPropertyItem("TipName", _
                                GetType(TextBox), _
                                GetType(Label), _
                                GetType(TrackBar), _
                                GetType(ComboBox), _
                                GetType(NumericUpDown))

                AddPropertyItem("TipText", GetType(Control))
            End If

            AddPropertyItem("TextAlign", GetType(Label))
            AddPropertyItem("DropDownStyle", GetType(ComboBox))
            AddMethodItem("Width", "Width", GetType(LineControl), GetType(CommandButton))
            AddMethodItem("Height", "Height", GetType(LineControl), GetType(CommandButton))
            AddMethodItem("FlipOrientation", "Flip Orientation", GetType(LineControl))
            AddPropertyItem("Anchor", GetType(Control))

            AddPropertyItem("AutoSize", GetType(Label), GetType(CheckBox))
            AddPropertyItem("Multiline", GetType(TextBox))

            AddMethodItem("Size24x24", "24x24", GetType(Button))
            AddMethodItem("SetSize80x24", "80x24", GetType(Button))
            AddMethodItem("AddButton", "Add Button", GetType(Form), GetType(Panel), GetType(GroupBox))
            AddMethodItem("AddCheckBox", "Add CheckBox", GetType(Form), GetType(Panel), GetType(GroupBox))
            AddMethodItem("AddLabel", "Add Label", GetType(Form), GetType(Panel), GetType(GroupBox))
            AddMethodItem("AddTextBox", "Add TextBox", GetType(Form), GetType(Panel), GetType(GroupBox))

            Return Items
        End Function
    End Class

    Public Class AnchorTypeEditor
        Inherits UITypeEditor

        Public Overloads Overrides Function EditValue(ByVal context As ITypeDescriptorContext, ByVal provider As IServiceProvider, ByVal value As Object) As Object
            Dim editorService = DirectCast(provider.GetService(GetType(IWindowsFormsEditorService)), IWindowsFormsEditorService)
            Dim c As New AnchorStylesEditorControl(editorService, DirectCast(value, AnchorStyles))
            editorService.DropDownControl(c)
            Return c.Style
        End Function

        Public Overrides Function GetEditStyle(ByVal context As System.ComponentModel.ITypeDescriptorContext) As System.Drawing.Design.UITypeEditorEditStyle
            Return UITypeEditorEditStyle.DropDown
        End Function
    End Class
End Namespace