﻿Imports System.Drawing.Design
Imports System.ComponentModel
Imports System.Text.RegularExpressions

Namespace UI
    <ProvideProperty("Pattern", GetType(Control))> _
    Public Class ValidationProvider
        Inherits Component
        Implements IExtenderProvider

        Private Patterns As New Dictionary(Of Control, String)
        Private Defaults As New Dictionary(Of Control, String)

        Public Function CanExtend(ByVal obj As Object) As Boolean Implements IExtenderProvider.CanExtend
            Return TypeOf obj Is Control
        End Function

        <DefaultValue("")> _
           Public Function GetPattern(ByVal c As Control) As String
            If Patterns.ContainsKey(c) Then
                Return Patterns(c)
            End If

            Return ""
        End Function

        Public Sub SetPattern(ByVal c As Control, ByVal value As String)
            If Not Patterns.ContainsKey(c) Then
                AddHandler c.Validating, AddressOf Validating
                AddHandler c.Enter, AddressOf Enter
            End If

            Patterns(c) = value
        End Sub

        Public Sub Validating(ByVal sender As Object, ByVal e As CancelEventArgs)
            Dim c As Control = CType(sender, Control)

            If Not Regex.IsMatch(c.Text, Patterns(c)) Then
                Msg.Show("""" + c.Text + """ is no valid input.", "Invalid Input", _
                    MsgIcon.Warning, MessageBoxButtons.OK)

                c.Text = Defaults(c)
            End If
        End Sub

        Public Sub Enter(ByVal sender As Object, ByVal e As EventArgs)
            Dim c As Control = CType(sender, Control)
            Defaults(c) = c.Text
        End Sub
    End Class
End Namespace
