Imports System.Threading
Imports System.Runtime.InteropServices
Imports System.ComponentModel
Imports System.Drawing.Design
Imports System.Reflection
Imports System.Text

Imports Stax.UI
Imports Stax.Win32
Imports Stax.ComponentModel
Imports Stax.AppFramework

Namespace UI
    <Serializable()> _
    Public Class CustomMenuItem
        Public Sub New()
        End Sub

        Public Sub New(ByVal text As String)
            Me.Text = text
        End Sub

        <NonSerialized()> _
        Private CustomMenuValue As CustomMenu

        Public Property CustomMenu() As CustomMenu
            Get
                Return CustomMenuValue
            End Get
            Set(ByVal Value As CustomMenu)
                CustomMenuValue = Value
            End Set
        End Property

        Private TextValue As String

        Public Overridable Property Text() As String
            Get
                Return TextValue
            End Get
            Set(ByVal Value As String)
                TextValue = Value
            End Set
        End Property

        Private SubItemsValue As New List(Of CustomMenuItem)

        Public Property SubItems() As List(Of CustomMenuItem)
            Get
                Return SubItemsValue
            End Get
            Set(ByVal Value As List(Of CustomMenuItem))
                SubItemsValue = Value
            End Set
        End Property

        Private KeyDataValue As Keys

        Public Property KeyData() As Keys
            Get
                Return KeyDataValue
            End Get
            Set(ByVal Value As Keys)
                KeyDataValue = Value
            End Set
        End Property

        Private MethodNameValue As String

        Public Property MethodName() As String
            Get
                Return MethodNameValue
            End Get
            Set(ByVal Value As String)
                MethodNameValue = Value
            End Set
        End Property

        Private ParametersValue As List(Of Object)

        Public Property Parameters() As List(Of Object)
            Get
                If ParametersValue Is Nothing Then
                    ParametersValue = New List(Of Object)
                End If

                Return ParametersValue
            End Get
            Set(ByVal Value As List(Of Object))
                ParametersValue = Value
            End Set
        End Property

        Public Sub Add(ByVal path As String)
            Add(path, Nothing, Keys.None, Nothing)
        End Sub

        Public Sub Add(ByVal path As String, ByVal methodName As String)
            Add(path, methodName, Keys.None, Nothing)
        End Sub

        Public Sub Add(ByVal path As String, ByVal methodName As String, ByVal ParamArray params As Object())
            Add(path, methodName, Keys.None, params)
        End Sub

        Public Sub Add(ByVal path As String, ByVal methodName As String, ByVal keyData As Keys, ByVal ParamArray params As Object())
            Add(path.Split("/"c), methodName, keyData, params)
        End Sub

        Public Sub Add(ByVal pathArray As String(), ByVal methodName As String, ByVal keyData As Keys, ByVal ParamArray params As Object())
            Dim l As List(Of CustomMenuItem) = SubItems

            For i = 0 To pathArray.Length - 1
                Dim found As Boolean = False

                For Each iItem In l
                    If i < pathArray.Length - 1 Then
                        If iItem.Text = pathArray(i) Then
                            found = True
                            l = iItem.SubItems
                        End If
                    End If
                Next

                If Not found Then
                    Dim item As New CustomMenuItem(pathArray(i))
                    l.Add(item)
                    l = item.SubItems

                    If i = pathArray.Length - 1 Then
                        item.MethodName = methodName
                        item.KeyData = keyData

                        If Not params Is Nothing Then
                            item.Parameters.AddRange(params)
                        End If
                    End If
                End If
            Next
        End Sub

        <NonSerialized()> _
        Private ParentValue As CustomMenuItem

        Public Property Parent() As CustomMenuItem
            Get
                Return ParentValue
            End Get
            Set(ByVal Value As CustomMenuItem)
                ParentValue = Value
            End Set
        End Property

        Public Shared Sub SetParents(ByVal item As CustomMenuItem)
            For Each i In item.SubItems
                i.Parent = item
                SetParents(i)
            Next
        End Sub

        Public Sub Remove()
            Parent.SubItems.Remove(Me)
        End Sub

        Public Function GetAllItems() As List(Of CustomMenuItem)
            Dim l As New List(Of CustomMenuItem)
            AddToList(Me, l)
            Return l
        End Function

        Private Sub AddToList(ByVal item As CustomMenuItem, ByVal list As List(Of CustomMenuItem))
            For Each i In item.SubItems
                list.Add(i)
                AddToList(i, list)
            Next
        End Sub

        Public Function GetClone() As CustomMenuItem
            Return DirectCast(ObjectHelp.GetCopy(Me), CustomMenuItem)
        End Function
    End Class

    Public Class CustomMenu
        Private Items As New List(Of CustomMenuItem)

        Public Event Command(ByVal e As CustomMenuItemEventArgs)
        Public Event Topic(ByVal item As CustomMenuItem)

        Public Sub New( _
            ByVal defaultMenu As Func(Of CustomMenuItem), _
            ByVal menuItem As CustomMenuItem, _
            ByVal commandManager As CommandManager, _
            ByVal toolStrip As ToolStrip)

            Me.CommandManager = commandManager
            Me.DefaultMenu = defaultMenu
            Me.MenuItem = menuItem
            Me.ToolStrip = toolStrip
        End Sub

        Private MenuValue As Menu

        Public Property Menu() As Menu
            Get
                Return MenuValue
            End Get
            Set(ByVal Value As Menu)
                MenuValue = Value
            End Set
        End Property

        Private MenuStripValue As MenuStrip

        Public Property MenuStrip() As MenuStrip
            Get
                Return MenuStripValue
            End Get
            Set(ByVal Value As MenuStrip)
                MenuStripValue = Value
            End Set
        End Property

        Private ToolStripValue As ToolStrip

        Public Property ToolStrip() As ToolStrip
            Get
                Return ToolStripValue
            End Get
            Set(ByVal Value As ToolStrip)
                ToolStripValue = Value
            End Set
        End Property

        Private MenuItemsValue As New List(Of ExtendedToolStripMenuItem)

        Public Property MenuItems() As List(Of ExtendedToolStripMenuItem)
            Get
                Return MenuItemsValue
            End Get
            Set(ByVal Value As List(Of ExtendedToolStripMenuItem))
                MenuItemsValue = Value
            End Set
        End Property

        Private DefaultMenuValue As Func(Of CustomMenuItem)

        Public Property DefaultMenu() As Func(Of CustomMenuItem)
            Get
                Return DefaultMenuValue
            End Get
            Set(ByVal Value As Func(Of CustomMenuItem))
                DefaultMenuValue = Value
            End Set
        End Property

        Private MenuItemValue As CustomMenuItem

        Public Property MenuItem() As CustomMenuItem
            Get
                Return MenuItemValue
            End Get
            Set(ByVal Value As CustomMenuItem)
                MenuItemValue = Value
            End Set
        End Property

        Private CommandManagerValue As CommandManager

        Public Property CommandManager() As CommandManager
            Get
                Return CommandManagerValue
            End Get
            Set(ByVal Value As CommandManager)
                CommandManagerValue = Value
            End Set
        End Property

        Public Sub AddKeyDownHandler(ByVal control As Control)
            AddHandler control.KeyDown, AddressOf OnKeyDown
        End Sub

        Public Function GetKeys() As StringPairList
            Dim ret As New StringPairList

            For Each i As ExtendedToolStripMenuItem In MenuItems
                If i.ShortcutKeyDisplayString <> "" Then
                    Dim sp As New StringPair

                    If i.Text.EndsWith("...") Then
                        sp.Name = i.Text.TrimEnd("."c)
                    Else
                        sp.Name = i.Text
                    End If

                    sp.Value = i.ShortcutKeyDisplayString
                    ret.Add(sp)
                End If
            Next

            Return ret
        End Function

        Public Function GetTips() As StringPairList
            Dim ret As New StringPairList

            For Each i As ExtendedToolStripMenuItem In MenuItems
                If Not i.Tip Is Nothing Then
                    ret.Add(i.Tip)
                End If
            Next

            Return ret
        End Function

        Public Function Edit() As CustomMenuItem
            Using f As New CustomMenuEditor(Me)
                If f.ShowDialog = DialogResult.OK Then
                    MenuItem = f.GetState
                    BuildMenu()
                End If
            End Using

            Return MenuItem
        End Function

        Public Sub OnKeyDown(ByVal sender As Object, ByVal e As KeyEventArgs)
            For Each i As CustomMenuItem In Items
                If i.KeyData = e.KeyData Then
                    OnCommand(i)
                    Exit For
                End If
            Next
        End Sub

        Public Sub MenuClick(ByVal sender As Object, ByVal e As EventArgs)
            If TypeOf sender Is ExtendedToolStripMenuItem Then
                OnCommand(DirectCast(sender, ExtendedToolStripMenuItem).CustomMenuItem)
            End If
        End Sub

        Private Sub OnCommand(ByVal item As CustomMenuItem)
            If item.MethodName <> "" Then
                Dim e As New CustomMenuItemEventArgs(item)
                RaiseEvent Command(e)

                If Not e.Handled Then
                    Process(item)
                End If

                Dim f As Form = ToolStrip.FindForm

                If Not f Is Nothing Then
                    f.Refresh()
                End If
            End If
        End Sub

        Public Sub Process(ByVal item As CustomMenuItem)
            CommandManager.Process(item.MethodName, item.Parameters)
        End Sub

        Public Sub BuildMenu()
            ToolStrip.Items.Clear()
            Items.Clear()
            MenuItems.Clear()

            BuildMenu(ToolStrip, MenuItem)
        End Sub

        Private Sub BuildMenu(ByVal menu As Object, ByVal item As CustomMenuItem)
            For Each i As CustomMenuItem In item.SubItems
                i.CustomMenu = Me
                Dim mi As ToolStripItem

                If i.Text = "-" Then
                    mi = New ToolStripSeparator
                Else
                    Dim emi As New ExtendedToolStripMenuItem
                    MenuItems.Add(emi)
                    mi = emi
                    emi.CustomMenuItem = i

                    Dim keys = KeysHelp.GetKeyString(i.KeyData)

                    If keys <> "" Then
                        emi.ShortcutKeyDisplayString = keys
                    End If

                    AddHandler mi.Click, AddressOf MenuClick

                    If CommandManager.HasCommand(i.MethodName) Then
                        Dim ca As CommandAttribute = CommandManager.GetCommand(i.MethodName).Attribute

                        If TypeOf menu Is ToolStrip AndAlso _
                            Not TypeOf menu Is ContextMenuStrip Then

                            emi.Font = New Font(emi.Font, FontStyle.Bold)
                        End If
                    End If
                End If

                Items.Add(i)
                mi.Text = i.Text

                If TypeOf menu Is ToolStripMenuItem Then
                    DirectCast(menu, ToolStripMenuItem).DropDownItems.Add(mi)
                ElseIf TypeOf menu Is ToolStrip Then
                    DirectCast(menu, ToolStrip).Items.Add(mi)
                End If

                BuildMenu(mi, i)
            Next
        End Sub
    End Class

    Public Class CustomMenuItemEventArgs
        Inherits EventArgs

        Public Sub New(ByVal item As CustomMenuItem)
            Me.Item = item
        End Sub

        Private HandledValue As Boolean

        Public Property Handled() As Boolean
            Get
                Return HandledValue
            End Get
            Set(ByVal Value As Boolean)
                HandledValue = Value
            End Set
        End Property

        Private ItemValue As CustomMenuItem

        Public Property Item() As CustomMenuItem
            Get
                Return ItemValue
            End Get
            Set(ByVal Value As CustomMenuItem)
                ItemValue = Value
            End Set
        End Property
    End Class

    Public Class ExtendedToolStripMenuItem
        Inherits ToolStripMenuItem

        Private TipValue As StringPair

        <Browsable(False), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)> _
        Public ReadOnly Property Tip() As StringPair
            Get
                If TipValue Is Nothing AndAlso _
                    Not CustomMenuItem Is Nothing AndAlso _
                    Not CustomMenuItem.CustomMenu Is Nothing Then

                    If CustomMenuItem.CustomMenu.CommandManager.HasCommand(CustomMenuItem.MethodName) Then
                        Dim command = CustomMenuItem.CustomMenu.CommandManager.GetCommand(CustomMenuItem.MethodName)

                        If Not command.Attribute.Description Is Nothing Then
                            Dim sp As New StringPair

                            If Text.EndsWith("...") Then
                                sp.Name = Text.TrimEnd("."c)
                            Else
                                sp.Name = Text
                            End If

                            sp.Value = command.Attribute.Description
                            Dim paramHelp = command.GetParameterHelp(CustomMenuItem.Parameters)

                            If Not paramHelp Is Nothing Then
                                sp.Value += " (" + paramHelp + ")"
                            End If

                            TipValue = sp
                        End If
                    End If
                End If

                Return TipValue
            End Get
        End Property

        Private CustomMenuItemValue As CustomMenuItem

        <Browsable(False), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)> _
        Public Property CustomMenuItem() As CustomMenuItem
            Get
                Return CustomMenuItemValue
            End Get
            Set(ByVal Value As CustomMenuItem)
                CustomMenuItemValue = Value

                If Not Value Is Nothing AndAlso _
                    Not Value.CustomMenu Is Nothing AndAlso _
                    Value.CustomMenu.CommandManager.HasCommand(CustomMenuItem.MethodName) Then

                    Dim c = CustomMenuItem.CustomMenu.CommandManager.GetCommand(CustomMenuItem.MethodName)

                    If c.MethodInfo.Name <> "DynamicMenuItem" Then
                        If c.MethodInfo.Name = "ExecuteCmdl" Then
                            HelpText = CustomMenuItem.Parameters(0).ToString.Trim(""""c)
                        Else
                            HelpTopic = c.Attribute.Topic
                            HelpText = c.Attribute.Description
                        End If
                    End If
                End If
            End Set
        End Property

        Private Function ShouldSerializeHelpText() As Boolean
            Return Not String.IsNullOrEmpty(HelpTextValue)
        End Function

        Private HelpTextValue As String

        <Editor(GetType(StringEditor), GetType(UITypeEditor))> _
        Public Property HelpText() As String
            Get
                Return HelpTextValue
            End Get
            Set(ByVal Value As String)
                HelpTextValue = Value

                If StaxApplication.ShowToolTips AndAlso OK(HelpTextValue) Then
                    If String.IsNullOrEmpty(HelpTextValue) Then
                        If Not String.IsNullOrEmpty(HelpTopic) Then
                            ToolTipText = "Right-click to open the help topic '" + HelpTopic + "'"
                        End If
                    Else
                        If HelpTextValue.Length < 80 Then
                            If String.IsNullOrEmpty(HelpTopic) Then
                                ToolTipText = HelpTextValue.TrimEnd("."c)
                            Else
                                ToolTipText = HelpTextValue.TrimEnd("."c) + ". Right-click to open the help topic '" + HelpTopic + "'"
                            End If
                        Else
                            ToolTipText = "Right-click for help"
                        End If
                    End If
                End If
            End Set
        End Property

        Private Function ShouldSerializeHelpTopic() As Boolean
            Return Not String.IsNullOrEmpty(HelpTopicValue)
        End Function

        Private HelpTopicValue As String

        Public Property HelpTopic() As String
            Get
                Return HelpTopicValue
            End Get
            Set(ByVal Value As String)
                HelpTopicValue = Value
            End Set
        End Property

        Protected Overrides Sub OnMouseDown(ByVal e As MouseEventArgs)
            MyBase.OnMouseDown(e)

            If e.Button = MouseButtons.Right Then
                Dim args As New ExtendedHelpEventArgs(e.Location)

                args.HelpTitle = Text
                args.HelpText = HelpText
                args.HelpTopic = HelpTopic

                StaxApplication.RaiseHelpRequested(args)
            End If
        End Sub

        Protected Overrides Sub OnClick(ByVal e As EventArgs)
            Application.DoEvents()
            MyBase.OnClick(e)
        End Sub
    End Class

    Public Class SubToolStripMenuItem
        Inherits ExtendedToolStripMenuItem

        Private A As Action

        Public Sub New(ByVal text As String, ByVal a As Action)
            Me.New(text, a, Nothing)
        End Sub

        Public Sub New(ByVal text As String, ByVal a As Action, ByVal toolTip As String)
            Me.Text = text
            Me.A = a
            Me.HelpText = toolTip
        End Sub

        Protected Overrides Sub OnClick(ByVal e As EventArgs)
            MyBase.OnClick(e)
            Application.DoEvents()
            A()
        End Sub
    End Class

    Public Class ParamToolStripMenuItem(Of T)
        Inherits ExtendedToolStripMenuItem

        Private PD As Action(Of T)
        Private Argument As T

        Public Sub New()
        End Sub

        Public Sub New(ByVal text As String, ByVal pd As Action(Of T), ByVal argument As T)
            Me.New(text, pd, argument, Nothing)
        End Sub

        Public Sub New(ByVal text As String, ByVal pd As Action(Of T), ByVal argument As T, ByVal tip As String)
            Me.New(text, pd, argument, tip, Keys.None)
        End Sub

        Public Sub New(ByVal text As String, ByVal pd As Action(Of T), ByVal argument As T, ByVal tip As String, ByVal key As Keys)
            Me.New(text, pd, argument, tip, key, Nothing)
        End Sub

        Public Sub New(ByVal text As String, ByVal pd As Action(Of T), ByVal argument As T, ByVal tip As String, ByVal key As Keys, ByVal image As Image)
            Me.Text = text
            Me.PD = pd
            Me.Argument = argument
            Me.HelpText = tip
            Me.Image = image

            If key <> Keys.None Then
                Me.ShortcutKeyDisplayString = KeysHelp.GetKeyString(key)
            End If
        End Sub

        Protected Overrides Sub OnClick(ByVal e As EventArgs)
            MyBase.OnClick(e)
            Application.DoEvents()
            PD(Argument)
        End Sub

        Public Shared Sub Populate( _
            ByVal items As ToolStripItemCollection, _
            ByVal path As String, _
            ByVal action As Action(Of T), _
            ByVal argument As T, _
            ByVal shortcutKey As Keys, _
            ByVal tip As String)

            Dim a = path.Split("|"c)
            Dim l = items

            For x = 0 To a.Length - 1
                Dim found = False

                For Each i As ToolStripMenuItem In l
                    If x < a.Length - 1 Then
                        If i.Text = a(x) Then
                            found = True
                            l = i.DropDownItems
                        End If
                    End If
                Next

                If Not found Then
                    If x = a.Length - 1 Then
                        Dim item As New ParamToolStripMenuItem(Of T)(a(x), action, argument, tip, shortcutKey)

                        l.Add(item)
                        l = item.DropDownItems
                    Else
                        Dim item As New ToolStripMenuItem(a(x))
                        l.Add(item)
                        l = item.DropDownItems
                    End If
                End If
            Next
        End Sub
    End Class
End Namespace