Namespace UI
    Public Class Msg
        Private ButtonListValue As New List(Of Button)
        Private Form As MsgForm

        Public ReadOnly Property ButtonList() As List(Of Button)
            Get
                Return ButtonListValue
            End Get
        End Property

        Public ButtonResults As New Dictionary(Of Button, DialogResult)

        Private PressedButtonValue As Button

        Public ReadOnly Property PressedButton() As Button
            Get
                Return PressedButtonValue
            End Get
        End Property

        Public ReadOnly Property PressedButtonIndex() As Integer
            Get
                If PressedButton Is Nothing Then
                    Return -1
                End If

                Return ButtonList.IndexOf(PressedButton)
            End Get
        End Property

        Public Function AddButton(ByVal text As String) As Button
            Return AddButton(text, DialogResult.Ignore)
        End Function

        Public Function AddButton(ByVal text As String, ByVal result As DialogResult) As Button
            If result = DialogResult.None Then
                Throw New ArgumentException("The Form class seem to not return DialogResult.None nor does it close.")
            End If

            Dim ret As New Button
            ret.Text = text
            ButtonList.Add(ret)
            ButtonResults(ret) = result
            Return ret
        End Function

        Private FontValue As Font

        Public Property Font() As Font
            Get
                If FontValue Is Nothing Then
                    FontValue = Control.DefaultFont
                End If

                Return FontValue
            End Get
            Set(ByVal value As Font)
                FontValue = value
            End Set
        End Property

        Private TextColorValue As Color = Color.Black

        Public Property TextColor() As Color
            Get
                Return TextColorValue
            End Get
            Set(ByVal Value As Color)
                TextColorValue = Value
            End Set
        End Property

        Private TextValue As String

        Public Property Text() As String
            Get
                Return TextValue
            End Get
            Set(ByVal value As String)
                TextValue = value
            End Set
        End Property

        Private TitleValue As String

        Public Property Title() As String
            Get
                If TitleValue Is Nothing Then
                    TitleValue = Application.ProductName
                End If

                Return TitleValue
            End Get
            Set(ByVal value As String)
                TitleValue = value
            End Set
        End Property

        Private ImageValue As Image

        Public Property Image() As Image
            Get
                If ImageValue Is Nothing Then
                    If Icon = MsgIcon.Auto Then
                        Icon = GetIcon(Buttons)
                    End If

                    Select Case Icon
                        Case MsgIcon.Error
                            ImageValue = SystemIcons.Error.ToBitmap
                        Case MsgIcon.Info
                            ImageValue = My.Resources.Info32
                        Case MsgIcon.Question
                            ImageValue = My.Resources.Help32
                        Case MsgIcon.Warning
                            ImageValue = My.Resources.Warning32
                        Case MsgIcon.None
                            ImageValue = Nothing
                        Case Else
                            Throw New NotImplementedException()
                    End Select
                End If

                Return ImageValue
            End Get
            Set(ByVal value As Image)
                ImageValue = value
            End Set
        End Property

        Private CheckBoxTextValue As String

        Public Property CheckBoxText() As String
            Get
                If CheckBoxTextValue Is Nothing Then
                    CheckBoxTextValue = "Don't show this again"
                End If

                Return CheckBoxTextValue
            End Get
            Set(ByVal Value As String)
                CheckBoxTextValue = Value
            End Set
        End Property

        Private ShowCheckBoxValue As Boolean

        Public Property ShowCheckBox() As Boolean
            Get
                Return ShowCheckBoxValue
            End Get
            Set(ByVal value As Boolean)
                ShowCheckBoxValue = value
            End Set
        End Property

        Private CheckedValue As Boolean

        Public Property Checked() As Boolean
            Get
                Return CheckedValue
            End Get
            Set(ByVal value As Boolean)
                CheckedValue = value
            End Set
        End Property

        Private CheckedListValue As List(Of String)

        Public Property CheckedList() As List(Of String)
            Get
                Return CheckedListValue
            End Get
            Set(ByVal value As List(Of String))
                If Not value Is Nothing Then
                    ShowCheckBox = True
                End If

                CheckedListValue = value
            End Set
        End Property

        Private ButtonsValue As MessageBoxButtons

        Public Property Buttons() As MessageBoxButtons
            Get
                Return ButtonsValue
            End Get
            Set(ByVal Value As MessageBoxButtons)
                ButtonsValue = Value

                If Icon = MsgIcon.Auto Then
                    Icon = GetIcon(ButtonsValue)
                End If

                Select Case Value
                    Case MessageBoxButtons.OK
                        AddButton("OK", DialogResult.OK)
                    Case MessageBoxButtons.OKCancel
                        AddButton("OK", DialogResult.OK)
                        AddButton("Cancel", DialogResult.Cancel)
                    Case MessageBoxButtons.YesNo
                        AddButton("Yes", DialogResult.Yes)
                        AddButton("No", DialogResult.No)
                    Case MessageBoxButtons.YesNoCancel
                        AddButton("Yes", DialogResult.Yes)
                        AddButton("No", DialogResult.No)
                        AddButton("Cancel", DialogResult.Cancel)
                    Case MessageBoxButtons.RetryCancel
                        AddButton("Retry", DialogResult.Retry)
                        AddButton("Cancel", DialogResult.Cancel)
                    Case Else
                        Throw New NotImplementedException()
                End Select
            End Set
        End Property

        Private IconValue As MsgIcon = MsgIcon.Auto

        Public Property Icon() As MsgIcon
            Get
                Return IconValue
            End Get
            Set(ByVal Value As MsgIcon)
                IconValue = Value
            End Set
        End Property

        Private Function GetIcon(ByVal b As MessageBoxButtons) As MsgIcon
            Select Case b
                Case MessageBoxButtons.OK
                    Return MsgIcon.Info
                Case _
                    MessageBoxButtons.OKCancel, _
                    MessageBoxButtons.YesNo, _
                    MessageBoxButtons.YesNoCancel

                    Return MsgIcon.Question
                Case MessageBoxButtons.RetryCancel, MessageBoxButtons.AbortRetryIgnore
                    Return MsgIcon.Error
                Case Else
                    Return MsgIcon.None
            End Select
        End Function

        Public Shared Function Show(ByVal value As Integer) As DialogResult
            Return Show(value.ToString, MsgIcon.Auto)
        End Function

        Public Shared Function Show(ByVal value As Boolean) As DialogResult
            Return Show(value.ToString, MsgIcon.Auto)
        End Function

        Public Shared Function Show(ByVal text As String) As DialogResult
            Return Show(text, MsgIcon.Auto)
        End Function

        Public Shared Function Show( _
            ByVal text As String, _
            ByVal title As String) As DialogResult

            Return Show(text, title, MsgIcon.Auto)
        End Function

        Public Shared Function Show( _
            ByVal text As String, _
            ByVal icon As MsgIcon) As DialogResult

            Return Show(text, icon, MessageBoxButtons.OK)
        End Function

        Public Shared Function Show( _
            ByVal text As String, _
            ByVal buttons As MessageBoxButtons) As DialogResult

            Return Show(text, Application.ProductName, buttons)
        End Function

        Public Shared Function Show( _
            ByVal text As String, _
            ByVal title As String, _
            ByVal buttons As MessageBoxButtons) As DialogResult

            Return Show(text, title, MsgIcon.Auto, buttons)
        End Function

        Public Shared Function Show( _
            ByVal text As String, _
            ByVal title As String, _
            ByVal icon As MsgIcon) As DialogResult

            Return Show(text, title, icon, MessageBoxButtons.OK, Nothing)
        End Function

        Public Shared Function Show( _
            ByVal text As String, _
            ByVal icon As MsgIcon, _
            ByVal buttons As MessageBoxButtons) As DialogResult

            Return Show(text, Application.ProductName, icon, buttons, Nothing)
        End Function

        Public Shared Function Show( _
            ByVal text As String, _
            ByVal title As String, _
            ByVal checkedList As List(Of String)) As DialogResult

            Return Show(text, title, MsgIcon.Auto, MessageBoxButtons.OK, checkedList)
        End Function

        Public Shared Function Show( _
            ByVal text As String, _
            ByVal icon As MsgIcon, _
            ByVal buttons As MessageBoxButtons, _
            ByVal checkedList As List(Of String)) As DialogResult

            Return Show(text, Application.ProductName, icon, buttons, checkedList)
        End Function

        Public Shared Function Show( _
            ByVal text As String, _
            ByVal title As String, _
            ByVal icon As MsgIcon, _
            ByVal buttons As MessageBoxButtons) As DialogResult

            Return Show(text, title, icon, buttons, Nothing)
        End Function

        Public Shared Function Show( _
            ByVal text As String, _
            ByVal checkedList As List(Of String)) As DialogResult

            Return Show(text, MsgIcon.Auto, checkedList)
        End Function

        Public Shared Function Show( _
            ByVal text As String, _
            ByVal icon As MsgIcon, _
            ByVal checkedList As List(Of String)) As DialogResult

            Return Show(text, Application.ProductName, icon, _
                MessageBoxButtons.OK, checkedList)
        End Function

        Public Shared Function Show( _
            ByVal text As String, _
            ByVal title As String, _
            ByVal icon As MsgIcon, _
            ByVal buttons As MessageBoxButtons, _
            ByVal checkedList As List(Of String)) As DialogResult

            If icon = MsgIcon.Error Then
                AppFramework.StaxApplication.RaiseErrorMessage(text)
            End If

            If checkedList IsNot Nothing AndAlso checkedList.Contains(text.MD5Hash) Then
                Return DialogResult.None
            End If

            Dim m As New Msg

            m.Text = text
            m.Title = title
            m.Icon = icon
            m.Buttons = buttons
            m.CheckedList = checkedList

            Return m.Show
        End Function

        Public Function Show() As DialogResult
            If Not CheckedList Is Nothing AndAlso _
                CheckedList.Contains(Text.MD5Hash) Then

                Return DialogResult.None
            End If

            Dim ret As DialogResult

            Using f As New MsgForm(Me)
                Form = f
                ret = f.ShowDialog()
                Application.DoEvents()
                PressedButtonValue = f.PressedButton
            End Using

            If CheckedList IsNot Nothing AndAlso Checked Then
                CheckedList.Add(Text.MD5Hash)
            End If

            Return ret
        End Function

        Public Sub Close()
            If Not Form Is Nothing AndAlso _
                Not Form.IsDisposed AndAlso _
                Form.IsHandleCreated Then

                If Form.InvokeRequired Then
                    Form.Invoke(New Action(AddressOf Form.Close))
                Else
                    Form.Close()
                End If
            End If
        End Sub
    End Class

    Public Enum MsgIcon
        [Error]
        Warning
        Question
        Info
        Auto
        None
    End Enum
End Namespace