Imports System.Threading
Imports System.Runtime.InteropServices
Imports System.Drawing.Drawing2D

Imports Stax.Win32
Imports Stax.AppFramework

Namespace UI
    Public Class PopUpForm
        Inherits Form

#Region " Designer "

        <System.Diagnostics.DebuggerStepThrough()> Private Sub InitializeComponent()
            Me.SuspendLayout()
            '
            'PopUp
            '
            Me.ClientSize = New System.Drawing.Size(198, 190)
            Me.ControlBox = False
            Me.FormBorderStyle = System.Windows.Forms.FormBorderStyle.None
            Me.KeyPreview = True
            Me.Name = "PopUp"
            Me.ShowInTaskbar = False
            Me.SizeGripStyle = System.Windows.Forms.SizeGripStyle.Hide
            Me.StartPosition = System.Windows.Forms.FormStartPosition.Manual
            Me.ResumeLayout(False)

        End Sub

#End Region

        Public TextValue As String
        Public Title As String
        Public Image As Image
        Public CenterOwner As Control

        Private WordWrapValue As Integer = 300
        Private WordWrap As Boolean = True
        Private MarginValue As Integer = 4
        Private SizeName As SizeF
        Private SizeText As SizeF
        Private HeaderHeight As Integer
        Private Middle As Integer = 2
        Private CenterLocation As Point
        Private ShownTick As Integer

        Public Sub New()
            MyBase.New()
            InitializeComponent()
        End Sub

        Private HeaderFontValue As Font

        Public Property HeaderFont() As Font
            Get
                If HeaderFontValue Is Nothing Then
                    HeaderFontValue = New Font(Font.FontFamily, 9, FontStyle.Bold)
                End If

                Return HeaderFontValue
            End Get
            Set(ByVal Value As Font)
                HeaderFontValue = Value
            End Set
        End Property

        Protected Overrides Sub OnLoad(ByVal e As EventArgs)
            MyBase.OnLoad(e)

            'set size values
            Using g As Graphics = CreateGraphics()
                If Not String.IsNullOrEmpty(Title) Then
                    SizeName = g.MeasureString(Title, HeaderFont)
                End If

                If WordWrap Then
                    SizeText = g.MeasureString(TextValue, Font, WordWrapValue)
                Else
                    SizeText = g.MeasureString(TextValue, Font)
                End If
            End Using

            'set the width
            Width = CInt(SizeName.Width)

            If Image IsNot Nothing Then
                Width += Image.Width + MarginValue
            End If

            If SizeText.Width > Width Then
                Width = CInt(SizeText.Width)
            End If

            Width += MarginValue + MarginValue

            'set the height
            Dim tempHeight As Integer = CInt(SizeName.Height)

            If Image IsNot Nothing Then
                If Image.Height > tempHeight Then
                    tempHeight = Image.Height
                End If
            End If

            HeaderHeight = tempHeight

            If SizeName.Height = 0 Then
                Height = tempHeight + CInt(SizeText.Height) + MarginValue + MarginValue
            Else
                Height = tempHeight + CInt(SizeText.Height) + MarginValue + Middle + MarginValue
            End If

            If CenterOwner Is Nothing Then
                Left = Control.MousePosition.X
                Dim b = Screen.PrimaryScreen.Bounds

                If Left + Width + MarginValue + MarginValue > b.Width Then
                    Left = b.Width - Width - MarginValue - MarginValue
                End If

                If Control.MousePosition.Y < b.Height \ 2 Then
                    Top = Control.MousePosition.Y + (Cursor.Size.Height - Cursor.HotSpot.Y) + 2
                Else
                    Top = Control.MousePosition.Y - Height - 4
                End If
            Else
                Dim p = CenterOwner.Location
                p.X += CenterOwner.Width \ 2
                p.Y += CenterOwner.Height \ 2
                p.X -= Width \ 2
                p.Y -= Height \ 2
                Location = CenterOwner.PointToScreen(p)
            End If
        End Sub

        Protected Overrides Sub OnPaint(ByVal e As PaintEventArgs)
            MyBase.OnPaint(e)

            Dim g = e.Graphics

            Dim gradientBrush As New LinearGradientBrush(ClientRectangle, _
                Color.White, Appearance.GradientColorLight, LinearGradientMode.ForwardDiagonal)

            g.FillRectangle(gradientBrush, ClientRectangle)
            gradientBrush.Dispose()

            'draw name

            If Not Title Is Nothing AndAlso Title <> "" Then
                If Image Is Nothing Then
                    g.DrawString(Title, HeaderFont, Brushes.Black, MarginValue, MarginValue)
                Else
                    g.DrawString(Title, HeaderFont, Brushes.Black, MarginValue + Image.Width + Middle, ((HeaderHeight - HeaderFont.Height) \ 2) + MarginValue)
                End If
            End If

            Dim textStart As Point
            textStart.X = MarginValue

            'draw text
            If HeaderHeight = 0 Then
                textStart.Y = MarginValue
            Else
                textStart.Y = MarginValue + HeaderHeight + Middle
            End If

            g.DrawString(TextValue, Font, Brushes.Black, New RectangleF(textStart, SizeText))

            'draw image
            If Image IsNot Nothing Then
                g.DrawImage(Image, MarginValue, MarginValue)
            End If

            ControlPaint.DrawBorder(g, ClientRectangle, Color.Black, ButtonBorderStyle.Solid)
        End Sub

        Protected Overrides Sub WndProc(ByRef m As Message)
            Select Case m.Msg
                Case WM.NCACTIVATE, WM.LBUTTONUP, WM.RBUTTONUP, WM.KEYUP
                    If ShownTick > 0 AndAlso Environment.TickCount > ShownTick + 1000 Then
                        Close()
                    End If
            End Select

            MyBase.WndProc(m)
        End Sub

        Private Sub PopUpForm_LostFocus() Handles Me.LostFocus
            If Environment.TickCount < ShownTick + 1000 Then
                Activate()
            End If
        End Sub

        Private Sub PopUpForm_Shown() Handles Me.Shown
            ShownTick = Environment.TickCount
        End Sub
    End Class
End Namespace