Imports System.Runtime.InteropServices
Imports System.Text
Imports System.IO

Namespace Win32
    Public Delegate Function NativeCallbackHandler( _
        ByVal handle As IntPtr, _
        ByVal parameter As Integer) As Boolean

    Public Class NativeFunctions
        <DllImport("user32.dll", CharSet:=CharSet.Unicode)> _
        Public Shared Function RegisterWindowMessage(ByVal id As String) As Integer
        End Function

        <DllImport("user32.dll")> _
        Public Shared Function SendMessageTimeout(ByVal windowHandle As IntPtr, ByVal Msg As Integer, ByVal wParam As IntPtr, ByVal lParam As IntPtr, ByVal flags As SendMessageTimeoutFlags, ByVal timeout As Integer, ByRef result As IntPtr) As IntPtr
        End Function

        <DllImport("user32.dll")> _
        Public Shared Function RegisterHotKey( _
            ByVal hWnd As IntPtr, _
            ByVal id As Integer, _
            ByVal fsModifiers As Integer, _
            ByVal vk As Integer) As Boolean
        End Function

        <DllImport("user32.dll")> _
        Public Shared Function MapVirtualKey( _
            ByVal wCode As Integer, _
            ByVal wMapType As MAPVK) As Integer
        End Function

        <DllImport("user32.dll")> _
        Public Shared Function UnregisterHotKey( _
            ByVal hWnd As IntPtr, _
            ByVal id As Integer) As Boolean
        End Function

        <DllImport("user32.dll")> _
        Public Shared Function GetForegroundWindow() As IntPtr
        End Function

        <DllImport("user32.dll", CharSet:=CharSet.Unicode)> _
        Public Shared Function SetWindowText( _
            ByVal hWnd As IntPtr, _
            ByVal lpString As String) As Boolean
        End Function

        <DllImport("user32.dll", CharSet:=CharSet.Unicode)> _
        Public Shared Function GetWindowText( _
            ByVal hWnd As IntPtr, _
            ByVal lpString As StringBuilder, _
            ByVal nMaxCount As Integer) As Integer
        End Function

        <DllImport("user32.dll")> _
        Public Shared Function ShowWindow( _
            ByVal handle As IntPtr, _
            ByVal nCmdShow As SW) As Integer
        End Function

        <DllImport("user32.dll", CharSet:=CharSet.Unicode)> _
        Public Shared Function FindWindow( _
            ByVal lpClassName As String, _
            ByVal lpWindowName As String) As IntPtr
        End Function

        <DllImport("user32.dll")> _
        Public Shared Function SetForegroundWindow( _
            ByVal handle As IntPtr) As Boolean
        End Function

        <DllImport("user32.dll")> _
        Public Shared Function EnumWindows( _
            ByVal ewp As NativeCallbackHandler, _
            ByVal ByVallParam As IntPtr) As Boolean
        End Function

        <DllImport("user32.dll")> _
        Public Shared Function EnumChildWindows( _
            ByVal hwndParent As IntPtr, _
            ByVal ByVallpEnumProc As NativeCallbackHandler, _
            ByVal parameter As IntPtr) As Boolean
        End Function

        <DllImport("user32.dll")> _
        Public Shared Function SendMessage( _
            ByVal handle As IntPtr, _
            ByVal message As Integer, _
            ByVal wParam As IntPtr, _
            ByVal lParam As IntPtr) As Integer
        End Function

        <DllImport("user32.dll")> _
        Public Shared Function SendMessageTimeout( _
            ByVal windowHandle As IntPtr, _
            ByVal msg As Integer, _
            ByVal wParam As IntPtr, _
            ByVal lParam As IntPtr, _
            ByVal flags As Integer, _
            ByVal timeout As Integer, _
            ByRef result As IntPtr) As IntPtr
        End Function

        <DllImport("user32.dll", CharSet:=CharSet.Unicode)> _
        Public Shared Function PostMessage( _
           ByVal hwnd As IntPtr, _
           ByVal wMsg As Integer, _
           ByVal wParam As IntPtr, _
           ByVal lParam As IntPtr) As Int32
        End Function

        <DllImport("wininet.dll")> _
        Public Shared Function InternetGetConnectedState( _
            ByRef description As Integer, _
            ByVal reservedValue As Integer) As Boolean
        End Function

        <DllImport("kernel32", CharSet:=CharSet.Unicode)> _
        Public Shared Function LoadLibrary(ByVal path As String) As IntPtr
        End Function
    End Class

    Public Class Win32Window
        Private Shared WindowHandles As New ArrayList
        Private Shared ChildHandles As New ArrayList

        Public Shared Sub Show(ByVal title As String)
            Show(title, SW.RESTORE)
        End Sub

        Private Shared Sub Show(ByVal title As String, ByVal nCmdShow As SW)
            For Each handle In GetHandles()
                If GetTitle(handle).Contains(title) Then
                    NativeFunctions.ShowWindow(handle, nCmdShow)
                End If
            Next
        End Sub

        Public Shared Sub Hide(ByVal handle As IntPtr)
            NativeFunctions.ShowWindow(handle, SW.HIDE)
        End Sub

        Public Shared Sub Hide(ByVal title As String)
            For Each handle As IntPtr In GetHandles()
                If GetTitle(handle).Contains(title) Then
                    NativeFunctions.ShowWindow(handle, SW.HIDE)
                End If
            Next
        End Sub

        Public Shared Function GetHandles() As IntPtr()
            NativeFunctions.EnumWindows(AddressOf EnumWindowsProc, Nothing)
            Dim winHandles(WindowHandles.Count - 1) As IntPtr
            WindowHandles.CopyTo(winHandles)
            WindowHandles.Clear()
            Return winHandles
        End Function

        Private Shared Function EnumWindowsProc(ByVal handle As IntPtr, ByVal parameter As Integer) As Boolean
            WindowHandles.Add(handle)
            Return True
        End Function

        Public Shared Function GetTitle(ByVal handle As IntPtr) As String
            Dim sb As New StringBuilder(255)
            NativeFunctions.GetWindowText(handle, sb, sb.Capacity)
            Return sb.ToString
        End Function

        Private Shared Function GetChildHandles(ByVal handle As IntPtr) As IntPtr()
            NativeFunctions.EnumChildWindows(handle, AddressOf HandleChildCallback, Nothing)
            Dim cHandles(ChildHandles.Count - 1) As IntPtr
            ChildHandles.CopyTo(cHandles)
            ChildHandles.Clear()
            Return cHandles
        End Function

        Private Shared Function HandleChildCallback(ByVal handle As IntPtr, ByVal parameter As Integer) As Boolean
            ChildHandles.Add(handle)
            Return True
        End Function

        Public Shared Function GetChildText(ByVal handle As IntPtr) As String
            Dim ip As IntPtr = Marshal.AllocHGlobal(1000)
            NativeFunctions.SendMessage(handle, WM.GETTEXT, New IntPtr(1000), ip)
            Dim ret As String = Marshal.PtrToStringAuto(ip)
            Marshal.FreeHGlobal(ip)
            Return ret
        End Function

        Public Shared Function GetHandle(ByVal title As String) As IntPtr
            For Each handle As IntPtr In GetHandles()
                If GetTitle(handle).Contains(title) Then
                    Return handle
                End If
            Next
        End Function

        Public Shared Function Exists(ByVal title As String, ByVal text As String) As Boolean
            Dim currentTitle, currentChildTitle As String

            For Each handle In GetHandles()
                currentTitle = GetTitle(handle)

                If currentTitle <> "" AndAlso currentTitle.Contains(title) Then
                    For Each childhandle In GetChildHandles(handle)
                        currentChildTitle = GetChildText(childhandle)

                        If currentChildTitle <> "" AndAlso currentChildTitle.Contains(text) Then
                            Return True
                        End If
                    Next
                End If
            Next
        End Function

        Public Shared Sub Close(ByVal title As String)
            Dim handle As IntPtr
            Dim currentTitle As String

            For Each handle In GetHandles()
                currentTitle = GetTitle(handle)

                If currentTitle <> "" AndAlso currentTitle.Contains(title) Then
                    NativeFunctions.SendMessage(handle, WM.CLOSE, IntPtr.Zero, IntPtr.Zero)
                End If
            Next
        End Sub
    End Class

    Public Class Win32Help
        Public Shared Function IsConnectedToInternet() As Boolean
            Dim Desc As Integer
            Return NativeFunctions.InternetGetConnectedState(Desc, 0)
        End Function
    End Class

#Region " Structures "
    Public Structure RECT
        Public Left As Integer
        Public Top As Integer
        Public Right As Integer
        Public Bottom As Integer

        Public Sub New(ByVal r As Rectangle)
            Left = r.Left
            Top = r.Top
            Right = r.Right
            Bottom = r.Bottom
        End Sub

        Public Function ToRectangle() As Rectangle
            Return Rectangle.FromLTRB(Left, Top, Right, Bottom)
        End Function
    End Structure

    Public Structure SHFILEINFO
        Public hIcon As IntPtr
        Public iIcon As Integer
        Public dwAttributes As Integer
        <MarshalAs(UnmanagedType.ByValTStr, SizeConst:=260)> _
        Public szDisplayName As String
        <MarshalAs(UnmanagedType.ByValTStr, SizeConst:=80)> _
        Public szTypeName As String
    End Structure

    Public Structure NMHDR
        Public hwndFrom As Integer
        Public idFrom As Integer
        Public code As Integer
    End Structure

    Public Enum SendMessageTimeoutFlags
        SMTO_NORMAL = 0
        SMTO_BLOCK = 1
        SMTO_ABORTIFHUNG = 2
        SMTO_NOTIMEOUTIFNOTHUNG = 8
    End Enum
#End Region

#Region " Enums "
    Public Enum WM
        ACTIVATE = &H6
        ACTIVATEAPP = &H1C
        AFXFIRST = &H360
        AFXLAST = &H37F
        APP = &H8000
        APPCOMMAND = &H319
        ASKCBFORMATNAME = &H30C
        CANCELJOURNAL = &H4B
        CANCELMODE = &H1F
        CAPTURECHANGED = &H215
        CHANGECBCHAIN = &H30D
        [CHAR] = &H102
        CHARTOITEM = &H2F
        CHILDACTIVATE = &H22
        CLEAR = &H303
        CLOSE = &H10
        COMMAND = &H111
        COMPACTING = &H41
        COMPAREITEM = &H39
        CONTEXTMENU = &H7B
        COPY = &H301
        COPYDATA = &H4A
        CREATE = &H1
        CTLCOLORBTN = &H135
        CTLCOLORDLG = &H136
        CTLCOLOREDIT = &H133
        CTLCOLORLISTBOX = &H134
        CTLCOLORMSGBOX = &H132
        CTLCOLORSCROLLBAR = &H137
        CTLCOLORSTATIC = &H138
        CUT = &H300
        DEADCHAR = &H103
        DELETEITEM = &H2D
        DESTROY = &H2
        DESTROYCLIPBOARD = &H307
        DEVICECHANGE = &H219
        DEVMODECHANGE = &H1B
        DISPLAYCHANGE = &H7E
        DRAWCLIPBOARD = &H308
        DRAWITEM = &H2B
        DROPFILES = &H233
        ENABLE = &HA
        ENDSESSION = &H16
        ENTERIDLE = &H121
        ENTERMENULOOP = &H211
        ENTERSIZEMOVE = &H231
        ERASEBKGND = &H14
        EXITMENULOOP = &H212
        EXITSIZEMOVE = &H232
        FONTCHANGE = &H1D
        GETDLGCODE = &H87
        GETFONT = &H31
        GETHOTKEY = &H33
        GETICON = &H7F
        GETMINMAXINFO = &H24
        GETOBJECT = &H3D
        GETTEXT = &HD
        GETTEXTLENGTH = &HE
        HANDHELDFIRST = &H358
        HANDHELDLAST = &H35F
        HELP = &H53
        HOTKEY = &H312
        HSCROLL = &H114
        HSCROLLCLIPBOARD = &H30E
        ICONERASEBKGND = &H27
        IME_CHAR = &H286
        IME_COMPOSITION = &H10F
        IME_COMPOSITIONFULL = &H284
        IME_CONTROL = &H283
        IME_ENDCOMPOSITION = &H10E
        IME_KEYDOWN = &H290
        IME_KEYLAST = &H10F
        IME_KEYUP = &H291
        IME_NOTIFY = &H282
        IME_REQUEST = &H288
        IME_SELECT = &H285
        IME_SETCONTEXT = &H281
        IME_STARTCOMPOSITION = &H10D
        INITDIALOG = &H110
        INITMENU = &H116
        INITMENUPOPUP = &H117
        INPUT = &HFF
        INPUTLANGCHANGE = &H51
        INPUTLANGCHANGEREQUEST = &H50
        KEYDOWN = &H100
        KEYFIRST = &H100
        KEYLAST = &H108
        KEYUP = &H101
        KILLFOCUS = &H8
        LBUTTONDBLCLK = &H203
        LBUTTONDOWN = &H201
        LBUTTONUP = &H202
        MBUTTONDBLCLK = &H209
        MBUTTONDOWN = &H207
        MBUTTONUP = &H208
        MDIACTIVATE = &H222
        MDICASCADE = &H227
        MDICREATE = &H220
        MDIDESTROY = &H221
        MDIGETACTIVE = &H229
        MDIICONARRANGE = &H228
        MDIMAXIMIZE = &H225
        MDINEXT = &H224
        MDIREFRESHMENU = &H234
        MDIRESTORE = &H223
        MDISETMENU = &H230
        MDITILE = &H226
        MEASUREITEM = &H2C
        MENUCHAR = &H120
        MENUCOMMAND = &H126
        MENUDRAG = &H123
        MENUGETOBJECT = &H124
        MENURBUTTONUP = &H122
        MENUSELECT = &H11F
        MOUSEACTIVATE = &H21
        MOUSEFIRST = &H200
        MOUSEHOVER = &H2A1
        MOUSELAST = &H20A
        MOUSELEAVE = &H2A3
        MOUSEMOVE = &H200
        MOUSEWHEEL = &H20A
        MOVE = &H3
        MOVING = &H216
        NCACTIVATE = &H86
        NCCALCSIZE = &H83
        NCCREATE = &H81
        NCDESTROY = &H82
        NCHITTEST = &H84
        NCLBUTTONDBLCLK = &HA3
        NCLBUTTONDOWN = &HA1
        NCLBUTTONUP = &HA2
        NCMBUTTONDBLCLK = &HA9
        NCMBUTTONDOWN = &HA7
        NCMBUTTONUP = &HA8
        NCMOUSEMOVE = &HA0
        NCPAINT = &H85
        NCRBUTTONDBLCLK = &HA6
        NCRBUTTONDOWN = &HA4
        NCRBUTTONUP = &HA5
        NEXTDLGCTL = &H28
        NEXTMENU = &H213
        NOTIFY = &H4E
        NOTIFYFORMAT = &H55
        NULL = &H0
        PAINT = &HF
        PAINTCLIPBOARD = &H309
        PAINTICON = &H26
        PALETTECHANGED = &H311
        PALETTEISCHANGING = &H310
        PARENTNOTIFY = &H210
        PASTE = &H302
        PENWINFIRST = &H380
        PENWINLAST = &H38F
        POWER = &H48
        PRINT = &H317
        PRINTCLIENT = &H318
        QUERYDRAGICON = &H37
        QUERYENDSESSION = &H11
        QUERYNEWPALETTE = &H30F
        QUERYOPEN = &H13
        QUEUESYNC = &H23
        QUIT = &H12
        REFLECT = USER + &H1C00
        RBUTTONDBLCLK = &H206
        RBUTTONDOWN = &H204
        RBUTTONUP = &H205
        RENDERALLFORMATS = &H306
        RENDERFORMAT = &H305
        SETCURSOR = &H20
        SETFOCUS = &H7
        SETFONT = &H30
        SETHOTKEY = &H32
        SETICON = &H80
        SETREDRAW = &HB
        SETTEXT = &HC
        SETTINGCHANGE = &H1A
        SHOWWINDOW = &H18
        SIZE = &H5
        SIZECLIPBOARD = &H30B
        SIZING = &H214
        SPOOLERSTATUS = &H2A
        STYLECHANGED = &H7D
        STYLECHANGING = &H7C
        SYNCPAINT = &H88
        SYSCHAR = &H106
        SYSCOLORCHANGE = &H15
        SYSCOMMAND = &H112
        SYSDEADCHAR = &H107
        SYSKEYDOWN = &H104
        SYSKEYUP = &H105
        TCARD = &H52
        THEMECHANGED = &H31A
        TIMECHANGE = &H1E
        TIMER = &H113
        UNDO = &H304
        UNINITMENUPOPUP = &H125
        USER = &H400
        USERCHANGED = &H54
        VKEYTOITEM = &H2E
        VSCROLL = &H115
        VSCROLLCLIPBOARD = &H30A
        WINDOWPOSCHANGED = &H47
        WINDOWPOSCHANGING = &H46
        WININICHANGE = &H1A
    End Enum

    Public Enum WS As UInteger
        ACTIVECAPTION = &H1
        BORDER = &H800000
        CAPTION = &HC00000
        CHILD = &H40000000
        CHILDWINDOW = CHILD
        CLIPCHILDREN = &H2000000
        CLIPSIBLINGS = &H4000000
        DISABLED = &H8000000
        DLGFRAME = &H400000
        EX_ACCEPTFILES = &H10I
        EX_APPWINDOW = &H40000I
        EX_CLIENTEDGE = &H200I
        EX_CONTEXTHELP = &H400I
        EX_CONTROLPARENT = &H10000I
        EX_DLGMODALFRAME = &H1I
        EX_LAYERED = &H80000
        EX_LAYOUTRTL = &H400000I
        EX_LEFT = &H0I
        EX_LEFTSCROLLBAR = &H4000I
        EX_LTRREADING = &H0I
        EX_MDICHILD = &H40I
        EX_NOACTIVATE = &H8000000I
        EX_NOINHERITLAYOUT = &H100000I
        EX_NOPARENTNOTIFY = &H4I
        EX_OVERLAPPEDWINDOW = (EX_WINDOWEDGE Or EX_CLIENTEDGE)
        EX_PALETTEWINDOW = (EX_WINDOWEDGE Or EX_TOOLWINDOW Or EX_TOPMOST)
        EX_RIGHT = &H1000I
        EX_RIGHTSCROLLBAR = &H0I
        EX_RTLREADING = &H2000I
        EX_STATICEDGE = &H20000I
        EX_TOOLWINDOW = &H80I
        EX_TOPMOST = &H8I
        EX_TRANSPARENT = &H20I
        EX_WINDOWEDGE = &H100I
        GROUP = &H20000
        GT = (GROUP Or TABSTOP)
        HSCROLL = &H100000
        ICONIC = MINIMIZE
        MAXIMIZE = &H1000000
        MAXIMIZEBOX = &H10000
        MINIMIZE = &H20000000
        MINIMIZEBOX = &H20000
        OVERLAPPED = &H0I
        OVERLAPPEDWINDOW = OVERLAPPED Or CAPTION Or SYSMENU Or THICKFRAME Or MINIMIZEBOX Or MAXIMIZEBOX
        'POPUP = &H80000000 'shouldn't this work?
        'POPUPWINDOW = (POPUP Or BORDER Or SYSMENU)
        SIZEBOX = THICKFRAME
        SYSMENU = &H80000
        TABSTOP = &H10000
        THICKFRAME = &H40000
        TILED = OVERLAPPED
        TILEDWINDOW = OVERLAPPEDWINDOW
        VISIBLE = &H10000000
        VSCROLL = &H200000
    End Enum

    Public Enum APPCOMMAND
        BROWSER_BACKWARD = 1
        BROWSER_FORWARD = 2
        BROWSER_REFRESH = 3
        BROWSER_STOP = 4
        BROWSER_SEARCH = 5
        BROWSER_FAVORITES = 6
        BROWSER_HOME = 7
        VOLUME_MUTE = 8
        VOLUME_DOWN = 9
        VOLUME_UP = 10
        MEDIA_NEXTTRACK = 11
        MEDIA_PREVIOUSTRACK = 12
        MEDIA_STOP = 13
        MEDIA_PLAY_PAUSE = 14
        LAUNCH_MAIL = 15
        LAUNCH_MEDIA_SELECT = 16
        LAUNCH_APP1 = 17
        LAUNCH_APP2 = 18
        BASS_DOWN = 19
        BASS_BOOST = 20
        BASS_UP = 21
        TREBLE_DOWN = 22
        TREBLE_UP = 23
        MICROPHONE_VOLUME_MUTE = 24
        MICROPHONE_VOLUME_DOWN = 25
        MICROPHONE_VOLUME_UP = 26
        HELP = 27
        FIND = 28
        [NEW] = 29
        OPEN = 30
        CLOSE = 31
        SAVE = 32
        PRINT = 33
        UNDO = 34
        REDO = 35
        COPY = 36
        CUT = 37
        PASTE = 38
        REPLY_TO_MAIL = 39
        FORWARD_MAIL = 40
        SEND_MAIL = 41
        SPELL_CHECK = 42
        DICTATE_OR_COMMAND_CONTROL_TOGGLE = 43
        MIC_ON_OFF_TOGGLE = 44
        CORRECTION_LIST = 45
        MEDIA_PLAY = 46
        MEDIA_PAUSE = 47
        MEDIA_RECORD = 48
        MEDIA_FAST_FORWARD = 49
        MEDIA_REWIND = 50
        MEDIA_CHANNEL_UP = 51
        MEDIA_CHANNEL_DOWN = 52
    End Enum

    Public Enum SC
        ARRANGE = &HF110
        CLOSE = &HF060I
        CONTEXTHELP = &HF180
        [DEFAULT] = &HF160
        DLG_FORCE_UI = &H4
        DLG_MINIMAL_UI = &H1
        DLG_NO_UI = &H2
        FLAGS = &H400
        HOTKEY = &HF150
        HSCROLL = &HF080
        ICON = MINIMIZE
        KEYMENU = &HF100
        MANAGER_CONNECT = &H1
        MANAGER_CREATE_SERVICE = &H2
        MANAGER_ENUMERATE_SERVICE = &H4
        MANAGER_LOCK = &H8
        MANAGER_MODIFY_BOOT_CONFIG = &H20
        MANAGER_QUERY_LOCK_STATUS = &H10
        MAXIMIZE = &HF030
        MINIMIZE = &HF020
        MONITORPOWER = &HF170
        MOUSEMENU = &HF090
        MOVE = &HF010
        NEXTWINDOW = &HF040
        PREVWINDOW = &HF050
        RESTORE = &HF120
        SCREENSAVE = &HF140
        SEPARATOR = &HF00F
        SIZE = &HF000
        TASKLIST = &HF130
        VSCROLL = &HF070
        ZOOM = MAXIMIZE
    End Enum

    Public Enum SW
        HIDE = 0
        SHOW = 5
        SHOWNA = 8
        RESTORE = 9
    End Enum

    Public Enum MAPVK
        VK_TO_VSC = 0
        VSC_TO_VK = 1
        VK_TO_CHAR = 2
        VSC_TO_VK_EX = 3
        VK_TO_VSC_EX = 4
    End Enum
#End Region

End Namespace