Imports System.ComponentModel

Imports Stax
Imports Stax.UI
Imports Stax.AppFramework

Public Class AviSynthListView
    Inherits ExtendedListView
    Implements IHostedControl

    Private BlockItemCheck As Boolean
    WithEvents Menu As New ContextMenuStrip

    Public Event ScriptChanged()

    Public Sub New()
        AllowDrop = True
        Anchor = AnchorStyles.Top Or AnchorStyles.Bottom Or AnchorStyles.Left Or AnchorStyles.Right
        CheckBoxes = True
        View = Windows.Forms.View.Details
        HideSelection = False
        FullRowSelect = True
        MultiSelect = False
        HeaderStyle = ColumnHeaderStyle.None
        Columns.Add("", 20)
        Columns.Add("Category", 70)
        Columns.Add("Name", 124)
        ContextMenuStrip = Menu
        HelpID = Me.GetType.FullName
    End Sub

    Private GetProfileMethodValue As Func(Of AviSynthDocument)

    <Browsable(False), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)> _
    Public Property GetProfileMethod() As Func(Of AviSynthDocument)
        Get
            Return GetProfileMethodValue
        End Get
        Set(ByVal Value As Func(Of AviSynthDocument))
            GetProfileMethodValue = Value
        End Set
    End Property

    Private HelpIDValue As String

    Public Property HelpID() As String Implements IHostedControl.HelpID
        Get
            Return HelpIDValue
        End Get
        Set(ByVal value As String)
            HelpIDValue = value
        End Set
    End Property

    Public Sub Load()
        BlockItemCheck = True
        Items.Clear()

        For Each i In GetProfileMethod.Invoke.Filters
            Dim item As New ListViewItem
            item.Tag = i
            item.Checked = i.Active
            item.SubItems.Add(i.Category)
            item.SubItems.Add(i.Name)
            item.SubItems.Add(i.Script)
            Items.Add(item)
        Next

        BlockItemCheck = False
    End Sub

    Public Sub UpdateMenu()
        Menu.Items.Clear()

        If SelectedItems.Count > 0 Then
            Dim selectedFilter = DirectCast(SelectedItems(0).Tag, AviSynthFilter)

            For Each i In s.AviSynthCategories
                If i.Name = selectedFilter.Category Then
                    For Each iFilter In i.Filters
                        Dim tip = iFilter.Script

                        If Not tip.Contains("%application:") Then
                            tip = Macro.Solve(tip)
                        End If

                        Menu.Items.Add(New ParamToolStripMenuItem(Of AviSynthFilter)( _
                            iFilter.Name, AddressOf ChangeClick, iFilter.GetCopy, tip))
                    Next
                End If
            Next

            If Menu.Items.Count > 0 Then Menu.Items.Add(New ToolStripSeparator)

            Menu.Items.Add(New SubToolStripMenuItem("Replace...", AddressOf ReplaceClick, "Replaces the selected filter."))
            Menu.Items.Add(New SubToolStripMenuItem("Insert...", AddressOf InsertClick, "Inserts a filter at the selection."))
        End If

        Menu.Items.Add(New SubToolStripMenuItem("Add...", AddressOf AddClick, "Adds a filter at the end."))

        If SelectedItems.Count > 0 Then
            Menu.Items.Add(New ToolStripSeparator)
            Menu.Items.Add(New SubToolStripMenuItem("Remove", AddressOf RemoveClick, "Removes the selected filter."))
        End If

        If Not Editable Then
            Menu.Items.Add(New SubToolStripMenuItem("Edit...", AddressOf EditClick, "Dialog to edit filters."))
        End If

        Menu.Items.Add(New SubToolStripMenuItem("Profiles...", AddressOf g.MainForm.OpenAviSynthFilterProfilesDialog, "Dialog to edit profiles."))
    End Sub

    Public Sub RaiseScriptChanged()
        RaiseEvent ScriptChanged()
    End Sub

    Public Sub EditClick()
        EditClick(AddressOf g.MainForm.GetTargetAviSynthDocument)
    End Sub

    Public Shared Sub EditClick(ByVal docDelegate As Func(Of AviSynthDocument))
        Using f As New ControlHostForm("Filters")
            f.FormBorderStyle = FormBorderStyle.Sizable
            f.Width = 600
            f.Height = 280
            s.WindowPositions.Restore(f, WindowPositions.Mode.Size)

            Dim c As New AviSynthListView
            c.SmallImageList = New ImageList() With {.ImageSize = New Size(18, 18)}
            c.Editable = True
            c.Dock = DockStyle.Fill
            c.GetProfileMethod = docDelegate
            c.Columns(1).Width += 20
            c.Columns.Add("Script")
            c.HeaderStyle = ColumnHeaderStyle.Nonclickable
            c.DoubleClickDoesCheck = False
            Dim categories = From a In s.AviSynthCategories Select a.Name
            Dim cb = c.AddComboBox(categories.ToArray, 1)
            cb.DropDownStyle = ComboBoxStyle.DropDownList
            cb.Sorted = True
            c.AddTextBox(2)
            c.AddTextBox(3)
            c.Load()
            f.AddControl(c)
            f.ShowDialog()
            s.WindowPositions.Save(f)
            g.MainForm.AviSynthListView.Load()
            g.MainForm.AviSynthListView.RaiseScriptChanged()
        End Using
    End Sub

    Protected Overrides Sub OnLayout(ByVal e As LayoutEventArgs)
        MyBase.OnLayout(e)

        If Columns.Count > 3 Then
            Columns(3).Width = Width - Columns(0).Width - Columns(1).Width - Columns(2).Width - 8
        End If
    End Sub

    Protected Overrides Sub OnBeforeShowControl(ByVal e As BeforeShowControlEventArgs)
        MyBase.OnBeforeShowControl(e)

        If Not e.Cancel AndAlso e.Position.X = 1 Then
            Dim cb = DirectCast(e.Control, ComboBox)
            cb.Items.Clear()

            Dim categories As New List(Of String)

            For Each i In s.AviSynthCategories
                categories.Add(i.Name)
            Next

            cb.Items.AddRange(categories.ToArray)
        End If
    End Sub

    Protected Overrides Sub OnEdited(ByVal value As Object, ByVal pos As Point)
        Dim f = DirectCast(Items(pos.Y).Tag, AviSynthFilter)

        Select Case pos.X
            Case 1
                f.Category = value.ToString
            Case 2
                f.Name = value.ToString
            Case 3
                f.Script = value.ToString
        End Select

        MyBase.OnEdited(value, pos)
    End Sub

    Private Sub ChangeClick(ByVal f As AviSynthFilter)
        Dim index = SelectedItems(0).Index
        GetProfileMethod.Invoke.Filters(index) = f
        Load()
        Items(index).Selected = True
        RaiseEvent ScriptChanged()
    End Sub

    Private Sub RemoveClick()
        Dim index = SelectedItems(0).Index
        GetProfileMethod.Invoke.Filters.RemoveAt(SelectedItems(0).Index)
        Load()
        RaiseEvent ScriptChanged()
    End Sub

    Private Sub ReplaceClick()
        Using f As New FilterSelectionForm
            f.Text = "Replace Filter"
            f.ShowDialog()

            If Not f.Filter Is Nothing Then
                Dim newFilter = f.Filter.GetCopy
                Dim index = SelectedItems(0).Index
                GetProfileMethod.Invoke.Filters(index) = newFilter
                Load()
                Items(index).Selected = True
                RaiseEvent ScriptChanged()
            End If
        End Using
    End Sub

    Private Sub InsertClick()
        Using f As New FilterSelectionForm
            f.Text = "Insert Filter"
            f.ShowDialog()

            If Not f.Filter Is Nothing Then
                Dim newFilter = f.Filter.GetCopy
                Dim index = SelectedItems(0).Index
                GetProfileMethod.Invoke.Filters.Insert(index, newFilter)
                Load()
                Items(index).Selected = True
                RaiseEvent ScriptChanged()
            End If
        End Using
    End Sub

    Private Sub AddClick()
        Using f As New FilterSelectionForm
            f.Text = "Add Filter"
            f.ShowDialog()

            If Not f.Filter Is Nothing Then
                Dim newFilter = f.Filter.GetCopy
                GetProfileMethod.Invoke.Filters.Add(newFilter)
                Load()
                Items(Items.Count - 1).Selected = True
                RaiseEvent ScriptChanged()
            End If
        End Using
    End Sub

    Public Sub UpdateDocument()
        GetProfileMethod.Invoke.Filters.Clear()

        For Each i As ListViewItem In Items
            GetProfileMethod.Invoke.Filters.Add(DirectCast(i.Tag, AviSynthFilter))
        Next

        RaiseEvent ScriptChanged()
    End Sub

    Protected Overrides Sub OnDragDrop(ByVal e As DragEventArgs)
        BlockItemCheck = True
        MyBase.OnDragDrop(e)
        BlockItemCheck = False
        UpdateDocument()
    End Sub

    Protected Overrides Sub OnItemCheck(ByVal e As ItemCheckEventArgs)
        MyBase.OnItemCheck(e)

        If Not BlockItemCheck Then
            Dim filter = DirectCast(Items(e.Index).Tag, AviSynthFilter)
            filter.Active = e.NewValue = CheckState.Checked
            RaiseEvent ScriptChanged()
        End If
    End Sub

    Protected Overrides Sub OnMouseUp(ByVal e As MouseEventArgs)
        If e.Button = MouseButtons.Right Then
            UpdateMenu()
        End If

        MyBase.OnMouseUp(e)

        If Control.ModifierKeys = Keys.Control AndAlso SelectedItems.Count > 0 Then
            EditScript()
        End If
    End Sub

    Public Sub ShowHelp() Implements IHostedControl.ShowHelp
        Dim f As New HelpForm()
        f.Doc.WriteStart("Filters")
        f.Doc.WriteP("The filters dialog allows to edit filters. Help about filters in general can be found at [http://www.avisynth.org www.avisynth.org].")
        f.Doc.WriteH2("Editing Features")

        f.Doc.WriteList( _
            "Left-click a selected filter to edit the category, name or script", _
            "Left-click while pressing Ctrl opens a script editor enabling multi line editing", _
            "Right-click on a filter shows a context menu with profile related features", _
            "Double-click in the empty area adds a filter", _
            "The filter order can be changed using Drag & Drop", _
            "The check boxes are used to apply or disable filters")

        f.Doc.WriteP("The mini filter editor in the main dialog has a similar but limited feature set. It supports the context menu, Drag & Drop to reorder, the script editor shows with a simple double-click.")
        f.Doc.WriteTable("Macros", Strings.MacrosHelp, Macro.GetTips())
        f.Show()
    End Sub

    Public Sub EditScript()
        If SelectedItems.Count > 0 Then
            Dim filter = DirectCast(SelectedItems(0).Tag, AviSynthFilter)

            Using f As New AviSynthScriptEditor(filter.Script)
                f.Text = filter.Name
                f.AutoResize()

                If f.ShowDialog = DialogResult.OK AndAlso filter.Script <> f.MacroEditorControl.Value Then
                    filter.Script = f.MacroEditorControl.Value

                    If Columns.Count > 3 Then
                        SetText(New Point(3, SelectedItems(0).Index), filter.Script)
                    End If

                    RaiseEvent ScriptChanged()
                End If
            End Using
        End If
    End Sub

    Protected Overrides Sub OnDoubleClick()
        MyBase.OnDoubleClick()

        If SelectedItems.Count = 0 Then
            AddClick()
        End If
    End Sub
End Class