Imports System.Threading
Imports System.ComponentModel
Imports System.Text

Imports Stax
Imports Stax.UI
Imports Stax.AppFramework

Public Class QuickSearchForm
    Inherits DialogBase

#Region " Designer "

    Protected Overloads Overrides Sub Dispose(ByVal disposing As Boolean)
        If disposing Then
            If Not (components Is Nothing) Then
                components.Dispose()
            End If
        End If
        MyBase.Dispose(disposing)
    End Sub

    Private components As System.ComponentModel.IContainer

    Friend WithEvents TipProvider As Stax.UI.TipProvider
    Friend WithEvents ResultColumnHeader As System.Windows.Forms.ColumnHeader
    Friend WithEvents lv As System.Windows.Forms.ListView
    Friend WithEvents tbSearch As System.Windows.Forms.TextBox
    Friend WithEvents VersionColumnHeader As System.Windows.Forms.ColumnHeader
    <System.Diagnostics.DebuggerStepThrough()> Private Sub InitializeComponent()
        Me.components = New System.ComponentModel.Container
        Me.lv = New System.Windows.Forms.ListView
        Me.ResultColumnHeader = New System.Windows.Forms.ColumnHeader
        Me.VersionColumnHeader = New System.Windows.Forms.ColumnHeader
        Me.TipProvider = New Stax.UI.TipProvider(Me.components)
        Me.tbSearch = New System.Windows.Forms.TextBox
        CType(Me.TipProvider, System.ComponentModel.ISupportInitialize).BeginInit()
        Me.SuspendLayout()
        '
        'lv
        '
        Me.lv.Anchor = CType((((System.Windows.Forms.AnchorStyles.Top Or System.Windows.Forms.AnchorStyles.Bottom) _
                    Or System.Windows.Forms.AnchorStyles.Left) _
                    Or System.Windows.Forms.AnchorStyles.Right), System.Windows.Forms.AnchorStyles)
        Me.lv.Columns.AddRange(New System.Windows.Forms.ColumnHeader() {Me.ResultColumnHeader, Me.VersionColumnHeader})
        Me.lv.FullRowSelect = True
        Me.lv.HeaderStyle = System.Windows.Forms.ColumnHeaderStyle.Nonclickable
        Me.lv.HideSelection = False
        Me.lv.Location = New System.Drawing.Point(13, 13)
        Me.lv.MultiSelect = False
        Me.lv.Name = "lv"
        Me.lv.Size = New System.Drawing.Size(604, 157)
        Me.lv.TabIndex = 0
        Me.lv.UseCompatibleStateImageBehavior = False
        Me.lv.View = System.Windows.Forms.View.Details
        '
        'ResultColumnHeader
        '
        Me.ResultColumnHeader.Text = "Result"
        Me.ResultColumnHeader.Width = 481
        '
        'VersionColumnHeader
        '
        Me.VersionColumnHeader.Text = "Version"
        Me.VersionColumnHeader.Width = 112
        '
        'tbSearch
        '
        Me.tbSearch.Anchor = CType(((System.Windows.Forms.AnchorStyles.Bottom Or System.Windows.Forms.AnchorStyles.Left) _
                    Or System.Windows.Forms.AnchorStyles.Right), System.Windows.Forms.AnchorStyles)
        Me.tbSearch.BorderStyle = System.Windows.Forms.BorderStyle.None
        Me.tbSearch.Location = New System.Drawing.Point(12, 157)
        Me.tbSearch.Name = "tbSearch"
        Me.tbSearch.ReadOnly = True
        Me.tbSearch.Size = New System.Drawing.Size(605, 13)
        Me.tbSearch.TabIndex = 5
        '
        'QuickSearchForm
        '
        Me.ClientSize = New System.Drawing.Size(629, 212)
        Me.Controls.Add(Me.lv)
        Me.Controls.Add(Me.tbSearch)
        Me.Name = "QuickSearchForm"
        Me.Text = "Quick Search"
        Me.Controls.SetChildIndex(Me.tbSearch, 0)
        Me.Controls.SetChildIndex(Me.lv, 0)
        CType(Me.TipProvider, System.ComponentModel.ISupportInitialize).EndInit()
        Me.ResumeLayout(False)
        Me.PerformLayout()

    End Sub

#End Region

    Private Package As Package
    Private StartDir As String
    Private CancelArgs As New CancelEventArgs
    Private Files As Integer
    WithEvents DialogCancelButton As Button = DialogButtons.bCancel

    Public Sub New(ByVal package As Package)
        MyBase.New()
        InitializeComponent()
        Me.Package = package
        lv.Items.Clear()

        For Each i In package.GetSearchResult
            AddItem(i)
        Next

        UpdateControls()

        If lv.Items.Count > 0 Then
            lv.Items(0).Selected = True
        End If
    End Sub

    Private Sub AddItem(ByVal path As String)
        Dim version = ""

        If OK(path) Then
            If Package.IsCorrectVersion(path) Then
                version = Package.Version
            Else
                version = "Unknown"
            End If
        Else
            path = "File don't exist"
        End If

        Dim lvi As New ListViewItem(New String() {path, version})

        If OK(Package.Version) AndAlso Package.Version = version Then
            lvi.Selected = True
            ActiveControl = DialogButtons.bOK
        End If

        lv.Items.Add(lvi)
    End Sub

    Private Sub lv_SelectedIndexChanged() Handles lv.SelectedIndexChanged
        UpdateControls()
    End Sub

    Private Sub UpdateControls()
        DialogButtons.bOK.Enabled = lv.SelectedItems.Count > 0
    End Sub

    Private Sub lv_DoubleClick() Handles lv.DoubleClick
        If lv.SelectedItems.Count > 0 Then
            DialogResult = Windows.Forms.DialogResult.OK
        End If
    End Sub

    Private Sub QuickSearchForm_FormClosed() Handles Me.FormClosed
        If DialogResult = DialogResult.OK Then
            Package.SetPath(lv.SelectedItems(0).SubItems(0).Text)
        End If
    End Sub

    Private Sub SearchPackageForm_HelpRequested() Handles Me.HelpRequested
        HelpForm.ShowDialog(Text, TipProvider.GetTips, "In order to use a application StaxRip must know where the application is located. The Quick Search dialog performes a quick search checking expected locations and registry hints.")
    End Sub
End Class