Imports System
Imports System.Runtime.Serialization
Imports System.Runtime.Serialization.Formatters.Binary
Imports System.Reflection
Imports System.Resources
Imports System.Globalization
Imports System.Xml.Linq

Imports Stax
Imports Stax.UI

<Serializable()> _
Public Class ApplicationSettings
    Implements ISafeSerialization

    <LegacyName("MKVCmdlPresets")> Public CmdlPresetsMKV As String
    Public AudioFileTypes As String()
    Public AudioProfiles As List(Of AudioProfile)
    Public AviSynthCategories As List(Of AviSynthCategory)
    Public AviSynthProfiles As List(Of TargetAviSynthDocument)
    Public CheckForUpdates As New SettingBag(Of Boolean)(True)
    Public CmdlPresetsMP4 As String
    Public CmdlPresetsX264 As String
    Public CustomMacros As List(Of Macro)
    Public CustomMenuCrop As CustomMenuItem
    Public CustomMenuMainForm As CustomMenuItem
    Public CustomMenuPreview As CustomMenuItem
    Public CustomMenuSize As CustomMenuItem
    Public DownloadDir As New SettingBag(Of String)("")
    Public EventCommands As List(Of EventCommand)
    Public ExtractDir As New SettingBag(Of String)("")
    Public HidePreviewButtons As Boolean
    Public Jobs As Dictionary(Of String, Boolean)
    Public LastPosition As Integer
    Public LastSourceDirValue As String
    Public MuxerProfiles As List(Of Muxer)
    Public PackagePaths As Dictionary(Of String, String)
    Public PreviewShowsDAR As Boolean = True
    Public ProcessPriority As New SettingBag(Of ProcessPriorityClass)(ProcessPriorityClass.Idle)
    Public RecentFramePositions As List(Of String)
    Public RecentOptionsPage As String
    Public RecentPackageSearchDirs As List(Of String)
    Public RecentProjects As List(Of String)
    Public RunAppsMinimized As New SettingBag(Of Boolean)(True)
    Public ShowInitialHelp As New SettingBag(Of Boolean)(True)
    Public ShowToolTips As New SettingBag(Of Boolean)(True)
    Public StartupTemplate As SettingBag(Of String)
    Public Storage As ObjectStorage
    Public StringDictionary As Dictionary(Of String, String)
    Public StringList As List(Of String)
    Public TrayIconDoubleClick As New SettingBag(Of Boolean)(False)
    Public UseShortcutToOpenSourceFiles As New SettingBag(Of Boolean)(False)
    Public VersionHelpPMPTemplate As String
    Public Versions As Dictionary(Of String, Integer)
    Public VideoEncoderProfiles As List(Of VideoEncoder)
    Public VideoFileTypes As String()
    Public WindowPositions As New WindowPositions
    Public WindowPositionsCenterScreen As String()
    Public WindowPositionsRemembered As String()
    Public WindowStatePreview As FormWindowState
    Public WrongVersionWarnings As List(Of String)

    Private WasUpdatedValue As Boolean

    Public Property WasUpdated() As Boolean Implements Stax.ISafeSerialization.WasUpdated
        Get
            Return WasUpdatedValue
        End Get
        Set(ByVal value As Boolean)
            WasUpdatedValue = value
        End Set
    End Property

    Public ReadOnly Property VersionsProperty() As Dictionary(Of String, Integer) Implements ISafeSerialization.Versions
        Get
            Return Versions
        End Get
    End Property

    Private Function Check(ByVal obj As Object, ByVal key As String, ByVal minimalVersion As Integer, ByVal currentVersion As Integer) As Boolean
        Return SafeSerialization.Check(Me, obj, key, minimalVersion, currentVersion)
    End Function

    Public Sub Init() Implements ISafeSerialization.Init
        If Versions Is Nothing Then
            Versions = New Dictionary(Of String, Integer)
        End If

        If Check(Storage, "Misc", 2, 2) Then
            Storage = New ObjectStorage
        End If

        If Check(VideoEncoderProfiles, "Video Encoder Profiles", 32, 77) Then
            VideoEncoderProfiles = VideoEncoder.GetDefaults()
            Storage.SetInt("Video Encoder Profiles Update", 1)
        ElseIf Not Storage.GetInt("Video Encoder Profiles Update") = 1 Then 'legacy 1.1.0.1
            Storage.SetInt("Video Encoder Profiles Update", 1)

            Dim a = From i In VideoEncoderProfiles Where TypeOf i Is NullEncoder

            For Each i In a.ToArray
                VideoEncoderProfiles.Remove(i)
            Next

            VideoEncoderProfiles.Add(New NullEncoder())
        End If

        If Check(AudioProfiles, "Audio Profiles", 38, 44) Then
            AudioProfiles = AudioProfile.GetDefaults(False)
        End If

        If Check(EventCommands, "Event Commands", 0, 0) Then
            EventCommands = New List(Of EventCommand)
        End If

        If Check(WindowPositionsRemembered, "Remembered Window Positions", 0, 0) Then
            WindowPositionsRemembered = New String() {"StaxRip", "Crop", "Preview", "Help"}
        End If

        If WindowPositionsCenterScreen Is Nothing Then
            WindowPositionsCenterScreen = New String() {}
        End If

        If StartupTemplate Is Nothing Then
            StartupTemplate = New SettingBag(Of String)("DVD")
        End If

        If Check(CustomMacros, "Custom Macros", 0, 0) Then
            CustomMacros = New List(Of Macro)

            CustomMacros.Add(New Macro("target_image_size", "Target Image Size", GetType(Integer), "%eval:%target_width% * %target_height%%", "Size of the target image."))
            CustomMacros.Add(New Macro("newline", "Newline", GetType(String), CrLf, "Simple line break."))
        End If

        If PackagePaths Is Nothing Then
            PackagePaths = New Dictionary(Of String, String)
        End If

        If RecentProjects Is Nothing Then
            RecentProjects = New List(Of String)
        End If

        If Check(VideoFileTypes, "Video File Types", 1, 1) Then
            VideoFileTypes = "avi avs d2v dat dga divx m2v mkv mp4 mpeg mpg mpv ogm pva rmvb ts vob wmv".Split(" "c)
        End If

        If AudioFileTypes Is Nothing Then
            AudioFileTypes = "wav mp2 mp3 mpa ac3 ogg aac mp4 dts".Split(" "c)
        End If

        If Check(MuxerProfiles, "Container Profiles", 8, 8) Then
            MuxerProfiles = New List(Of Muxer)
            MuxerProfiles.AddRange(Muxer.GetDefaults())
        End If

        If StringDictionary Is Nothing Then
            StringDictionary = New Dictionary(Of String, String)
        End If

        If RecentOptionsPage Is Nothing Then
            RecentOptionsPage = ""
        End If

        If Check(CmdlPresetsMKV, "MKV Cmdl Presets", 3, 3) Then
            CmdlPresetsMKV = _
                "File Attachment = --attach-file ""%browse_file%""" + CrLf + _
                "Attachment Description = --attachment-description ""%enter_text%""" + CrLf + _
                "Process Priority = --priority %select:lowest;lower;normal;higher;highest%"
        End If

        If Check(CmdlPresetsMP4, "MP4 Cmdl Presets", 3, 3) Then
            CmdlPresetsMP4 = "Example Switch = -example"
        End If

        If Check(CmdlPresetsX264, "X264 Cmdl Presets", 3, 3) Then
            CmdlPresetsX264 = _
                "SAR|PAL|4:3 = --sar 12:11" + CrLf + _
                "SAR|PAL|16:9 = --sar 16:11" + CrLf + _
                "SAR|NTSC|4:3 = --sar 10:11" + CrLf + _
                "SAR|NTSC|16:9 = --sar 40:33" + CrLf + _
                "Enter SAR = --sar %enter_text:Please enter the SAR.%"
        End If

        If Jobs Is Nothing Then
            Jobs = New Dictionary(Of String, Boolean)
        End If

        If WrongVersionWarnings Is Nothing Then
            WrongVersionWarnings = New List(Of String)
        End If

        If RecentPackageSearchDirs Is Nothing Then
            RecentPackageSearchDirs = New List(Of String)
        End If

        If StringList Is Nothing Then
            StringList = New List(Of String)
        End If

        If RecentFramePositions Is Nothing Then
            RecentFramePositions = New List(Of String)
        End If

        If Check(CustomMenuCrop, "Menu in crop dialog", 3, 5) Then
            CustomMenuCrop = CropForm.GetDefaultMenu
        End If

        If Check(CustomMenuMainForm, "Main menu in main window", 39, 40) Then
            CustomMenuMainForm = MainForm.GetDefaultMainMenu
        End If

        ''legacy
        'If Not Storage.GetInt("CustomMenuMainFormUpdate") = 6 Then
        '    Storage.SetInt("CustomMenuMainFormUpdate", 6)

        '    CustomMenuItem.SetParents(CustomMenuMainForm)
        '    Dim foundImportx264Profiles As Boolean

        '    For Each i In CustomMenuMainForm.GetAllItems
        '        If Not i Is Nothing Then
        '            Select Case i.MethodName
        '                Case "AddForecedSubtitle" '1.0.0.4
        '                    i.MethodName = "AddHardcodedSubtitle"
        '                    i.Text = "Add Hardcoded Subtitle..."
        '                    i.Text = i.Text
        '                Case "OpenHelpMainDialog" '1.0.0.4
        '                    i.MethodName = "OpenHelpTopic"
        '                    i.Parameters.Clear()
        '                    i.Parameters.Add("main")
        '                Case "OpenHelpTopic" '1.0.0.4
        '                    If OK(i.Parameters) Then
        '                        If i.Parameters(0).Equals("guides") Then
        '                            i.Remove()
        '                        ElseIf i.Parameters(0).Equals("changelog") Then
        '                            i.MethodName = "ExecuteCmdl"
        '                            i.Parameters.Clear()
        '                            i.Parameters.Add("http://www.planetdvb.net/staxrip/changelog")
        '                        End If
        '                    End If
        '                Case "Importx264Profiles" '1.0.0.4
        '                    foundImportx264Profiles = True
        '            End Select
        '        End If
        '    Next

        '    If Not foundImportx264Profiles Then
        '        CustomMenuMainForm.Add("Tools/Import x264 Profiles...", "Importx264Profiles")
        '    End If
        'End If

        If Check(CustomMenuPreview, "Menu in preview dialog", 20, 26) Then
            CustomMenuPreview = PreviewForm.GetDefaultMenu
        End If

        If Check(CustomMenuSize, "Target size menu in main dialog", 22, 22) Then
            CustomMenuSize = MainForm.GetDefaultMenuSize
        End If

        If Check(AviSynthCategories, "Filter Profiles", 0, 17) Then
            AviSynthCategories = AviSynthCategory.GetDefaults
        End If

        'legacy 1.0.0.3
        For Each i In AviSynthCategories
            For Each j As AviSynthFilter In i.Filters
                If j.Script.Contains(CrLf) Then
                    j.Name += " (updated)"
                    j.Script = j.Script.Replace(CrLf, " ")
                    WasUpdated = True
                End If
            Next
        Next

        If Check(AviSynthProfiles, "Filter Setup Profiles", 0, 9) Then
            AviSynthProfiles = AviSynthDocument.GetDefaults
        End If

        'legacy 1.0.0.3
        For Each i In AviSynthProfiles
            For Each i2 In i.Filters
                If i2.Script.Contains(CrLf) Then
                    i2.Name += " (updated)"
                    i2.Script = i2.Script.Replace(CrLf, " ")
                End If
            Next
        Next

        'legacy 1.1.0.1
        For Each i In AudioProfiles
            If i.StreamType = StreamType.MP3CBR OrElse i.StreamType = StreamType.MP3VBR Then
                i.StreamType = StreamType.MP3
            End If

            If i.StreamType = StreamType.AAC_HE_MP4 OrElse i.StreamType = StreamType.AAC_LC_MP4 Then
                i.StreamType = StreamType.MP4
            End If

            If i.StreamType = StreamType.AAC_HE_RAW OrElse i.StreamType = StreamType.AAC_LC_RAW Then
                i.StreamType = StreamType.AAC
            End If
        Next

        'legacy 1.1.0.1
        For Each i In MuxerProfiles.OfType(Of MkvMuxer)()
            If i.Title Is Nothing Then
                i.Title = ""
                i.VideoTrackLanguage = New Language(CultureInfo.InvariantCulture)
                i.VideoTrackName = ""
            End If
        Next

        'legacy 1.1.1.0
        For Each i In VideoEncoderProfiles.OfType(Of x264Encoder)()
            If i.Params.AQMode.Value > 1 Then
                i.Params.AQMode.Value = 1
            End If
        Next

        If Check(VersionHelpPMPTemplate, "PMP Template", 3, 3) Then
            VersionHelpPMPTemplate = ""
            g.CreatePMPTemplate()
        End If

        If LastSourceDirValue Is Nothing Then
            LastSourceDirValue = ""
        End If
    End Sub

    Public Property LastSourceDir() As String
        Get
            If LastSourceDirValue = "" Then
                Return ""
            End If

            If Not Directory.Exists(LastSourceDirValue) Then
                LastSourceDirValue = DirPath.GetParent(LastSourceDirValue)
            End If

            Return LastSourceDirValue
        End Get
        Set(ByVal Value As String)
            If Not String.IsNullOrEmpty(Value) AndAlso _
                Directory.Exists(Value) Then

                LastSourceDirValue = Value
            End If
        End Set
    End Property
End Class