Imports Stax
Imports Stax.UI
Imports Stax.AppFramework
Imports Stax.ComponentModel

<Serializable()> _
Public MustInherit Class AudioProfile
    Inherits Profile

    Public Sub New(ByVal name As String, ByVal streamName As String, ByVal typeName As String, ByVal bitrate As Integer, ByVal input As List(Of String), ByVal type As StreamType)
        MyBase.New(name, typeName)
        Me.StreamName = streamName
        Me.Bitrate = bitrate
        Me.SupportedInputFileTypes = input
        Me.StreamType = type
    End Sub

    Private FileValue As String = ""

    Public Property File() As String
        Get
            If FileValue Is Nothing Then
                FileValue = ""
            End If

            Return FileValue
        End Get
        Set(ByVal Value As String)
            FileValue = Value
        End Set
    End Property

    Private LanguageValue As New Language

    Public Property Language() As Language
        Get
            If LanguageValue Is Nothing Then
                LanguageValue = New Language
            End If

            Return LanguageValue
        End Get
        Set(ByVal Value As Language)
            LanguageValue = Value
        End Set
    End Property

    Private DelayValue As Integer

    Public Property Delay() As Integer
        Get
            Return DelayValue
        End Get
        Set(ByVal Value As Integer)
            DelayValue = Value
        End Set
    End Property

    Private SupportedInputFileTypesValue As List(Of String)

    Public Overridable Property SupportedInputFileTypes() As List(Of String)
        Get
            Return SupportedInputFileTypesValue
        End Get
        Set(ByVal value As List(Of String))
            SupportedInputFileTypesValue = value
        End Set
    End Property

    Private BitrateValue As Double

    Public Overridable Property Bitrate() As Double
        Get
            Return BitrateValue
        End Get
        Set(ByVal Value As Double)
            BitrateValue = Value
        End Set
    End Property

    Private StreamNameValue As String = ""

    Public Property StreamName() As String
        Get
            If StreamNameValue Is Nothing Then
                StreamNameValue = ""
            End If

            Return StreamNameValue
        End Get
        Set(ByVal Value As String)
            StreamNameValue = Value
        End Set
    End Property

    Private StreamTypeValue As StreamType

    Public Overridable Property StreamType() As StreamType
        Get
            Return StreamTypeValue
        End Get
        Set(ByVal Value As StreamType)
            StreamTypeValue = Value
        End Set
    End Property

    Public Overridable Sub Encode()
    End Sub

    Public Overridable Sub EditProject()
        ShowInfoDialog()
    End Sub

    Public Overridable Function HandlesDelay() As Boolean
    End Function

    Public Function SolveMacros(ByVal value As String) As String
        Return SolveMacros(value, True)
    End Function

    Public Function SolveMacros(ByVal value As String, ByVal silent As Boolean) As String
        value = value.Replace("%input%", File)
        value = value.Replace("%bitrate%", Bitrate.ToString)
        value = value.Replace("%output%", Filepath.GetDirAndBase(File) + "_AudioOutput" + GetExtension())
        value = value.Replace("%language_native%", Language.CultureInfo.NativeName)
        value = value.Replace("%language_english%", Language.Name)
        value = value.Replace("%delay%", Delay.ToString)
        Return Macro.Solve(value, silent)
    End Function

    Public Function GetExtension() As String
        Return StreamTypeUtils.GetExtension(StreamType)
    End Function

    Public Shared Function GetDefaults(ByVal advanced As Boolean) As List(Of AudioProfile)
        Dim ret As New List(Of AudioProfile)

        ret.Add(New CmdlAudioProfile("AAC/50-90 kbps", 70, _
            New String() {"ac3", "mpa", "mp2", "mp3", "wav", "ogg", "pcm"}, StreamType.MP4, _
            """%application:BeSweet%"" -core( -input ""%input%"" -output ""%output%"" ) -azid( -c normal -L -3db ) -ota( -d %delay% -g max ) -bsn( -vbr 0.3 -2ch )"))

        ret.Add(New CmdlAudioProfile("AAC/110-150 kbps", 130, _
            New String() {"ac3", "mpa", "mp2", "mp3", "wav", "ogg", "pcm"}, StreamType.MP4, _
            """%application:BeSweet%"" -core( -input ""%input%"" -output ""%output%"" ) -azid( -c normal -L -3db ) -ota( -d %delay% -g max ) -bsn( -vbr 0.5 -2ch )"))

        ret.Add(New CmdlAudioProfile("AAC/5.1 200-250 kbps", 225, _
            New String() {"ac3"}, StreamType.MP4, _
            """%application:BeSweet%"" -core( -input ""%input%"" -output ""%output%"" ) -azid( -c normal ) -ota( -d %delay% -g max ) -bsn( -vbr 0.3 -6chnew )"))

        ret.Add(New CmdlAudioProfile("MP3/110-150 kbps", 128, _
            New String() {"ac3", "mp2", "mpa", "wav", "ogg", "pcm"}, StreamType.MP3, _
            """%application:BeSweet%"" -core( -input ""%input%"" -output ""%output%"" ) -azid( -c normal -L -3db ) -ota( -d %delay% -hybridgain ) -lame( -v --vbr-new -V 5 -b 32 )"))

        ret.Add(New CmdlAudioProfile("MP3/220-260 kbps", 240, _
            New String() {"ac3", "mp2", "mpa", "wav", "ogg", "pcm"}, StreamType.MP3, _
            """%application:BeSweet%"" -core( -input ""%input%"" -output ""%output%"" ) -azid( -c normal -L -3db ) -ota( -d %delay% -hybridgain ) -lame( -v --vbr-new -V 0 -b 32 )"))

        ret.Add(New CmdlAudioProfile("MP3/128 kbps", 128, _
            New String() {"ac3", "mp2", "mpa", "wav", "ogg", "pcm"}, StreamType.MP3, _
            """%application:BeSweet%"" -core( -input ""%input%"" -output ""%output%"" ) -azid( -c normal -L -3db ) -ota( -d %delay% -hybridgain ) -lame( --alt-preset cbr %bitrate% )"))

        ret.Add(New CmdlAudioProfile("MP3/160 kbps", 160, _
            New String() {"ac3", "mp2", "mpa", "wav", "ogg", "pcm"}, StreamType.MP3, _
            """%application:BeSweet%"" -core( -input ""%input%"" -output ""%output%"" ) -azid( -c normal -L -3db ) -ota( -d %delay% -hybridgain ) -lame( --alt-preset cbr %bitrate% )"))

        ret.Add(New CmdlAudioProfile("MP3/128 kbps 44100 Khz", 128, _
            New String() {"ac3", "mp2", "mp3", "mpa", "wav", "ogg", "pcm"}, StreamType.MP3, _
            """%application:BeSweet%"" -core( -input ""%input%"" -output ""%output%"" ) -azid( -c normal -L -3db ) -ota( -d %delay% -hybridgain ) -shibatch( --rate 44100 ) -lame( --alt-preset cbr %bitrate% --resample 44.1)"))

        ret.Add(New CmdlAudioProfile("Vorbis/80-100 kbps", 90, _
            New String() {"ac3", "mpa", "mp2", "mp3", "wav", "pcm"}, StreamType.Vorbis, _
            """%application:BeSweet%"" -core( -input ""%input%"" -output ""%output%"" ) -azid( -c normal -L -3db ) -ota( -d %delay% -hybridgain ) -ogg( -q 0.200 )"))

        ret.Add(New CmdlAudioProfile("Vorbis/5.1 200-300 kbps", 300, _
            New String() {"ac3", "mpa", "mp2", "mp3", "wav", "pcm"}, StreamType.Vorbis, _
            """%application:BeSweet%"" -core( -input ""%input%"" -output ""%output%"" ) -azid( -c normal ) -ota( -d %delay% -hybridgain ) -ogg( -q 0.200 -6ch 1 )"))

        ret.Add(New MuxAudioProfile())
        ret.Add(New NullAudioProfile())

        Return ret
    End Function

    Public Overrides Function GetInfoValues() As Dictionary(Of String, String)
        Dim ret As New Dictionary(Of String, String)

        ret("Audio Profile Type") = TypeName
        ret("Supported Input File Extensions") = String.Join(", ", SupportedInputFileTypes.ToArray)
        ret("Bitrate") = Bitrate.ToString
        ret("Output Format") = DisplayInfoAttribute.GetValueForEnum(Of StreamType)(StreamType)
        ret("Output File Extension") = GetExtension()

        Return ret
    End Function
End Class

<Serializable()> _
Public Class CmdlAudioProfile
    Inherits AudioProfile

    Public Sub New(ByVal name As String, ByVal bitrate As Integer, ByVal input As String(), ByVal type As StreamType, ByVal commandLines As String)
        MyBase.New(name, "%language_english%", "Command Line Audio Profile", bitrate, New List(Of String)(input), type)
        Me.CommandLines = commandLines
        CanEditValue = True
    End Sub

    Private CommandValue As String

    Public Property CommandLines() As String
        Get
            Return CommandValue
        End Get
        Set(ByVal Value As String)
            CommandValue = Value
        End Set
    End Property

    Public Overrides Function Edit() As DialogResult
        Using f As New CmdlAudioSettingsForm(Me)
            f.cbLanguage.Visible = False
            f.lLanguage.Visible = False
            f.tbDelay.Visible = False
            f.lDelay.Visible = False
            Return f.ShowDialog()
        End Using
    End Function

    Public Overrides Sub EditProject()
        Using f As New CmdlAudioSettingsForm(Me)
            f.ShowDialog()
        End Using
    End Sub

    Public Overrides Function GetInfoValues() As Dictionary(Of String, String)
        Dim ret As Dictionary(Of String, String) = MyBase.GetInfoValues
        ret("Command Lines") = CommandLines
        Return ret
    End Function

    Public Overrides Sub Encode()
        If OK(File) Then
            Dim bitrateBefore = p.VideoBitrate
            Dim targetPath = Filepath.GetDirAndBase(File) + "_AudioOutput" + GetExtension()

            For Each i In SolveMacros(CommandLines, False).SplitLines
                Log.WriteHeader("Audio Encoding")

                Dim start = DateTime.Now

                Using pr As New Proc
                    AddHandler pr.Process.ErrorDataReceived, AddressOf ProcessForm.CmdlDataHandler
                    pr.BeginErrorReadLine = True

                    If i.Contains("|") Then
                        pr.Process.StartInfo.FileName = "cmd"
                        pr.Process.StartInfo.Arguments = "/c """ + i + """"
                    Else
                        pr.CommandLine = i
                    End If

                    ProcessForm.Proc = pr
                    Log.WriteLine(i + CrLf2)

                    pr.Minimized = s.RunAppsMinimized.Value
                    pr.Directory = Filepath.GetDir(pr.File)
                    pr.Wait = True
                    pr.Priority = s.ProcessPriority.Value
                    pr.Start()

                    RemoveHandler pr.Process.ErrorDataReceived, AddressOf ProcessForm.CmdlDataHandler
                    ProcessForm.LogCmdlOutput("[", "Processed")
                End Using

                File = targetPath

                If IO.File.Exists(File) Then
                    Bitrate = Calc.GetBitrateFromFile(File, p.TargetSeconds)
                    p.VideoBitrate = CInt(Calc.GetTotalBitrate - Calc.GetAudioBitrate())

                    Log.WriteHeader("Audio Encoding Statistics")
                    Log.WriteStats(start, p.TargetAVSDoc.GetFrames)
                    Log.WriteFilesize(File)
                    Log.WriteLine("Bitrate: ".PadRight(10) + Bitrate.ToString("f2") + " kbps")

                    If Not p.VideoEncoder.QualityMode Then
                        Log.WriteLine("Video Bitrate: " + bitrateBefore.ToString() + " -> " + p.VideoBitrate.ToString)
                    End If
                End If
            Next
        End If
    End Sub

    Public Overrides Function HandlesDelay() As Boolean
        Return CommandLines.Contains("%delay%")
    End Function
End Class

<Serializable()> _
Public Class NullAudioProfile
    Inherits AudioProfile

    Public Sub New()
        MyBase.New("No Audio", "", "Null Audio Profile", 0, New List(Of String)(New String() {}), StaxRip.StreamType.Ignore)
    End Sub

    Public Overrides Function HandlesDelay() As Boolean
    End Function

    Public Overrides Sub EditProject()
    End Sub
End Class

<Serializable()> _
Public Class MuxAudioProfile
    Inherits AudioProfile

    Public Sub New()
        MyBase.New("Add Existing File", Nothing, "Mux Audio Profile", 0, Nothing, StaxRip.StreamType.Ignore)
        CanEditValue = True
    End Sub

    <NonSerialized()> Private SupportedInputFileTypesValue As List(Of String)

    Public Overrides Property SupportedInputFileTypes() As List(Of String)
        Get
            If SupportedInputFileTypesValue Is Nothing Then
                SupportedInputFileTypesValue = New List(Of String)(New String() {"mp3", "mp4", "ac3", "mp2", "mpa", "ogg", "aac", "dts"})
            End If

            Return SupportedInputFileTypesValue
        End Get
        Set(ByVal value As List(Of String))
        End Set
    End Property

    Public Overrides Function Edit() As DialogResult
        Return Edit(False)
    End Function

    Public Overrides Sub EditProject()
        Edit(True)
    End Sub

    Private Overloads Function Edit(ByVal showProjectSettings As Boolean) As DialogResult
        Using f As New SettingTreeViewForm("Audio Mux Options", "The Audio Mux options allow to add a audio file without reencoding.", Strings.ProjectAudioSettingsHelp)
            Dim b As New PropertySettingBag(Of String)(Me, "StreamName")
            f.SettingTreeView.AddText("Audio Mux Options", "Stream Name", b, "Stream name used by the muxer. The stream name may contain macros.")

            If showProjectSettings Then
                f.SettingTreeView.AddSetting(New ComboboxTreeNodeSetting(Of Language)("Audio Mux Options", "Language", New PropertySettingBag(Of Language)(Me, "Language"), "Language of the audio track.", Language.Languages))
                f.SettingTreeView.AddInt("Audio Mux Options", "Delay", New PropertySettingBag(Of Integer)(Me, "Delay"), "Delay used by the muxer.")
            End If

            f.SettingTreeView.Nodes(0).ExpandAll()
            Return f.ShowDialog()
        End Using
    End Function

    Public Sub SetStreamType()
        If OK(File) Then
            StreamType = StreamTypeUtils.GetStreamType(Filepath.GetExt(File))
        Else
            StreamType = StaxRip.StreamType.Ignore
        End If
    End Sub
End Class