Imports System.Runtime.InteropServices

Imports Stax
Imports Stax.Win32

Public Class MediaInfo
    Implements IDisposable

    Private Handle As IntPtr

    <DllImport("MediaInfo.dll")> _
    Private Shared Function MediaInfo_New() As IntPtr
    End Function

    <DllImport("MediaInfo.dll")> _
    Private Shared Sub MediaInfo_Delete(ByVal Handle As IntPtr)
    End Sub

    <DllImport("MediaInfo.dll", CharSet:=CharSet.Unicode)> _
    Private Shared Function MediaInfo_Open(ByVal Handle As IntPtr, ByVal FileName As String) As Integer
    End Function

    <DllImport("MediaInfo.dll")> _
    Private Shared Function MediaInfo_Close(ByVal Handle As IntPtr) As Integer
    End Function

    <DllImport("MediaInfo.dll", CharSet:=CharSet.Unicode)> _
    Private Shared Function MediaInfo_Inform(ByVal Handle As IntPtr, <MarshalAs(UnmanagedType.U4)> ByVal Reserved As UInteger) As IntPtr
    End Function

    <DllImport("MediaInfo.dll", CharSet:=CharSet.Unicode)> _
    Private Shared Function MediaInfo_Get(ByVal Handle As IntPtr, <MarshalAs(UnmanagedType.U4)> ByVal StreamKind As MediaInfoStreamKind, ByVal StreamNumber As UInteger, ByVal Parameter As String, <MarshalAs(UnmanagedType.U4)> ByVal KindOfInfo As MediaInfoInfoKind, <MarshalAs(UnmanagedType.U4)> ByVal KindOfSearch As MediaInfoInfoKind) As IntPtr
    End Function

    <DllImport("MediaInfo.dll", CharSet:=CharSet.Unicode)> _
    Private Shared Function MediaInfo_Option(ByVal Handle As IntPtr, ByVal OptionString As String, ByVal Value As String) As String
    End Function

    <DllImport("MediaInfo.dll")> _
    Private Shared Function MediaInfo_State_Get(ByVal Handle As IntPtr) As Integer
    End Function

    <DllImport("MediaInfo.dll")> _
    Private Shared Function MediaInfo_Count_Get(ByVal Handle As IntPtr, <MarshalAs(UnmanagedType.U4)> ByVal StreamKind As MediaInfoStreamKind, ByVal StreamNumber As Integer) As Integer
    End Function

    Private Shared Loaded As Boolean

    Public Sub New(ByVal path As String)
        If Not Loaded Then
            Loaded = True
            NativeFunctions.LoadLibrary(Packs.MediaInfo.Path)
        End If

        Handle = MediaInfo_New()
        MediaInfo_Open(Handle, path)
    End Sub

    Private disposed As Boolean

    Protected Overridable Sub Dispose(ByVal disposing As Boolean)
        If Not disposed Then
            MediaInfo_Close(Handle)
            MediaInfo_Delete(Handle)
            disposed = True
        End If
    End Sub

    Public Sub Dispose() Implements IDisposable.Dispose
        Dispose(True)
        GC.SuppressFinalize(Me)
    End Sub

    Private AudioTracksValue As List(Of MediaInfoAudioTrack)

    Public ReadOnly Property AudioTracks() As List(Of MediaInfoAudioTrack)
        Get
            If AudioTracksValue Is Nothing Then
                AudioTracksValue = New List(Of MediaInfoAudioTrack)
                Dim count = MediaInfo_Count_Get(Handle, MediaInfoStreamKind.Audio, -1)

                If count > 0 Then
                    For i = 0 To count - 1
                        Dim at As New MediaInfoAudioTrack
                        Dim codec = Marshal.PtrToStringUni(MediaInfo_Get(Handle, MediaInfoStreamKind.Audio, Convert.ToUInt32(i), "Codec/String", MediaInfoInfoKind.Text, MediaInfoInfoKind.Name))

                        Select Case codec
                            Case "MPEG-1 Audio layer 3"
                                Dim mode = Marshal.PtrToStringUni(MediaInfo_Get(Handle, MediaInfoStreamKind.Audio, Convert.ToUInt32(i), "BitRate_Mode", MediaInfoInfoKind.Text, MediaInfoInfoKind.Name))

                                If mode = "VBR" Then
                                    at.AudioType = MediaInfoAudioType.MP3VBR
                                ElseIf mode = "CBR" Then
                                    at.AudioType = MediaInfoAudioType.MP3CBR
                                End If
                            Case "AC3"
                                at.AudioType = MediaInfoAudioType.AC3
                            Case "PCM"
                                at.AudioType = MediaInfoAudioType.WAV
                            Case "DTS"
                                at.AudioType = MediaInfoAudioType.DTS
                            Case "AAC LC-SBR", "AAC LC-SBR-PS"
                                at.AudioType = MediaInfoAudioType.AACHE
                            Case "AAC LC"
                                at.AudioType = MediaInfoAudioType.AACLC
                            Case Else
                                at.AudioType = MediaInfoAudioType.Unknown
                        End Select

                        at.LanguageTwoLetter = Marshal.PtrToStringUni(MediaInfo_Get(Handle, MediaInfoStreamKind.Audio, Convert.ToUInt32(i), "Language", MediaInfoInfoKind.Text, MediaInfoInfoKind.Name))
                        at.LanguageInfo = Marshal.PtrToStringUni(MediaInfo_Get(Handle, MediaInfoStreamKind.Audio, Convert.ToUInt32(i), "Language_More", MediaInfoInfoKind.Text, MediaInfoInfoKind.Name))
                        at.Codec = Marshal.PtrToStringUni(MediaInfo_Get(Handle, MediaInfoStreamKind.Audio, Convert.ToUInt32(i), "Codec", MediaInfoInfoKind.Text, MediaInfoInfoKind.Name))

                        AudioTracksValue.Add(at)
                    Next
                End If
            End If

            Return AudioTracksValue
        End Get
    End Property

    Public Shared Function GetAudioTracks(ByVal path As String) As List(Of MediaInfoAudioTrack)
        Using mi As New MediaInfo(path)
            Return mi.AudioTracks
        End Using
    End Function

    Private SubtitlesValue As List(Of MediaInfoSubtitle)

    Public ReadOnly Property Subtitles() As List(Of MediaInfoSubtitle)
        Get
            If SubtitlesValue Is Nothing Then
                SubtitlesValue = New List(Of MediaInfoSubtitle)
                Dim count As Integer = MediaInfo_Count_Get(Handle, MediaInfoStreamKind.Text, -1)

                If count > 0 Then
                    For i = 0 To count - 1
                        Dim s As New MediaInfoSubtitle
                        s.LanguageTwoLetter = Marshal.PtrToStringUni(MediaInfo_Get(Handle, MediaInfoStreamKind.Text, Convert.ToUInt32(i), "Language", MediaInfoInfoKind.Text, MediaInfoInfoKind.Name))
                        SubtitlesValue.Add(s)
                    Next
                End If
            End If

            Return SubtitlesValue
        End Get
    End Property

    Public Shared Function GetSummary(ByVal path As String) As String
        Using mi As New MediaInfo(path)
            Return Marshal.PtrToStringUni(MediaInfo_Inform(mi.Handle, 0))
        End Using
    End Function

    Public Function GetInfo(ByVal streamKind As MediaInfoStreamKind, ByVal parameter As String) As String
        Return Marshal.PtrToStringUni(MediaInfo_Get(Handle, streamKind, 0, parameter, MediaInfoInfoKind.Text, MediaInfoInfoKind.Name))
    End Function

    Public Shared Function GetInfo(ByVal path As String, ByVal streamKind As MediaInfoStreamKind, ByVal parameter As String) As String
        Using mi As New MediaInfo(path)
            Return mi.GetInfo(streamKind, parameter)
        End Using
    End Function
End Class

Public Class MediaInfoAudioTrack
    Public AudioType As MediaInfoAudioType
    Public LanguageTwoLetter As String
    Public LanguageInfo As String
    Public Codec As String

    Public ReadOnly Property Extension() As String
        Get
            Select Case AudioType
                Case MediaInfoAudioType.MP3CBR, MediaInfoAudioType.MP3VBR
                    Return ".mp3"
                Case Else
                    Return "." + AudioType.ToString.ToLowerInvariant
            End Select
        End Get
    End Property
End Class

Public Class MediaInfoSubtitle
    Private LanguageTwoLetterValue As String

    Public Property LanguageTwoLetter() As String
        Get
            Return LanguageTwoLetterValue
        End Get
        Set(ByVal Value As String)
            LanguageTwoLetterValue = Value
        End Set
    End Property
End Class

Public Enum MediaInfoStreamKind
    General
    Video
    Audio
    Text
    Chapters
    Image
End Enum

Public Enum MediaInfoInfoKind
    Name
    Text
    Measure
    Options
    NameText
    MeasureText
    Info
    HowTo
End Enum

Public Enum MediaInfoAudioType
    Unknown
    AACHE
    AACLC
    MP3VBR
    MP3CBR
    AC3
    WAV
    DTS
End Enum