Imports Microsoft.Win32

Imports Stax
Imports Stax.AppFramework
Imports Stax.ComponentModel
Imports Stax.DynamicObject
Imports Stax.UI
Imports Stax.Win32

Imports System.ComponentModel
Imports System.Drawing.Design
Imports System.Globalization
Imports System.Management
Imports System.Reflection
Imports System.Resources
Imports System.Runtime.InteropServices
Imports System.Runtime.Serialization
Imports System.Runtime.Serialization.Formatters.Binary
Imports System.Text
Imports System.Text.RegularExpressions
Imports System.Threading
Imports System.Xml
Imports System.Xml.Linq
Imports System.Xml.Serialization

Imports VBN = Microsoft.VisualBasic

Public Module ShortcutModule
    Public g As New GlobalClass
    Public p As New Project
    Public s As New ApplicationSettings
End Module

Public Class Paths
    Public Shared Function VerifyRequirements() As Boolean
        If Directory.Exists(Paths.AviSynthPluginsDir) Then
            If File.Exists(Paths.AviSynthPluginsDir + "MPEG2Dec.dll") OrElse _
                File.Exists(Paths.AviSynthPluginsDir + "MPEG2Dec2.dll") OrElse _
                File.Exists(Paths.AviSynthPluginsDir + "MPEG2Dec3.dll") Then

                Msg.Show("The obsolete plugin MPEG2Dec conflicts with the plugin DGDecode.", MsgIcon.Warning)
                Return False
            End If

            For Each i In Directory.GetFiles(Paths.AviSynthPluginsDir)
                If Regex.IsMatch(i, "(?i:decomb.+\.dll$)") Then
                    If Msg.Show("A legacy Decomb version was found." + CrLf2 + "Please confirm to solve this conflict.", MsgIcon.Warning, MessageBoxButtons.OKCancel) = DialogResult.OK Then
                        FileHelp.Delete(i)
                    End If

                    If File.Exists(i) Then
                        Return False
                    End If
                End If
            Next
        End If

        For Each i As Package In Packs.Packages.Values
            If i.VerifyOK = False Then
                Return False
            End If
        Next

        Dim dec = Paths.GetYV12DecoderPath

        If Not dec Is Nothing Then
            Dim decRegistered = RegistryHelp.LocalMachine.GetString("SOFTWARE\Microsoft\Windows NT\CurrentVersion\Drivers32", "VIDC.YV12")

            If dec <> decRegistered Then
                If Misc.IsAdmin Then
                    RegistryHelp.LocalMachine.SetValue("SOFTWARE\Microsoft\Windows NT\CurrentVersion\Drivers32", _
                        "VIDC.YV12", dec)
                Else
                    Msg.Show("Can't register the YV12 decoder. " + Strings.RunAsAdmin)
                    Return False
                End If
            End If

            If Filepath.GetName(dec) = "ff_vfw.dll" Then
                If RegistryHelp.CurrentUser.GetValue(Of Integer)("Software\GNU\ffdshow_vfw", "rawv") <> 1 Then
                    If Misc.IsAdmin Then
                        RegistryHelp.CurrentUser.SetValue("Software\GNU\ffdshow_vfw", "rawv", 1)
                    Else
                        Msg.Show("Can't register the YV12 decoder. " + Strings.RunAsAdmin)
                        Return False
                    End If
                End If
            End If
        End If

        If Not p.TargetAVSDoc.IsFilterActive("Source") Then
            Msg.Show("No active filter of category 'Source' found.", MsgIcon.Warning)
            Return False
        End If

        Return True
    End Function

    <DebuggerNonUserCode()> _
    Public Shared Property AviSynthPluginsDir() As String
        Get
            Dim dir = Filepath.AppendSeparator(RegistryHelp.LocalMachine.GetString("SOFTWARE\AviSynth", "plugindir2_5"))

            If Misc.IsAdmin AndAlso (Not OK(dir) OrElse Not Directory.Exists(dir)) Then
                dir = Paths.ExtractDir + "AviSynth Plugins\"

                If Not Directory.Exists(dir) Then
                    Directory.CreateDirectory(dir)
                End If

                RegistryHelp.LocalMachine.SetValue("SOFTWARE\AviSynth", "plugindir2_5", dir)
            End If

            Return dir
        End Get
        Set(ByVal Value As String)
            If Misc.IsAdmin Then
                RegistryHelp.LocalMachine.SetValue("SOFTWARE\AviSynth", "plugindir2_5", Value)
            End If
        End Set
    End Property

    Private Shared ExtractDirValue As String

    Public Shared Property ExtractDir() As String
        Get
            If ExtractDirValue Is Nothing Then
                ExtractDirValue = SolveDirMacros(s.ExtractDir.Value)

                If Not IsDirValid(ExtractDirValue) Then
                    Paths.ExtractDir = SettingsDir + "Applications\" 'Paths class ref needed, weird

                    If Not Directory.Exists(ExtractDirValue) Then
                        Directory.CreateDirectory(ExtractDirValue)
                    End If
                End If
            End If

            Return ExtractDirValue
        End Get
        Set(ByVal value As String)
            If OK(value) Then
                If Not value.EndsWith("\") Then
                    value += "\"
                End If

                ExtractDirValue = value
                s.ExtractDir.Value = InjectDirMacros(value)
            End If
        End Set
    End Property

    Private Shared DownloadDirValue As String

    Public Shared Property DownloadDir() As String
        Get
            If DownloadDirValue Is Nothing Then
                DownloadDirValue = SolveDirMacros(s.DownloadDir.Value)

                If Not IsDirValid(DownloadDirValue) Then
                    Paths.DownloadDir = SettingsDir + "Downloads\" 'Paths class ref needed, weird

                    If Not Directory.Exists(DownloadDirValue) Then
                        Directory.CreateDirectory(DownloadDirValue)
                    End If
                End If
            End If

            Return DownloadDirValue
        End Get
        Set(ByVal value As String)
            If OK(value) Then
                If Not value.EndsWith("\") Then
                    value += "\"
                End If

                DownloadDirValue = value
                s.DownloadDir.Value = InjectDirMacros(value)
            End If
        End Set
    End Property

    Private Shared Function IsDirValid(ByVal path As String) As Boolean
        Return OK(path) AndAlso path.EndsWith("\") AndAlso Directory.Exists(path)
    End Function

    Private Shared SettingsDirValue As String

    Public Shared Property SettingsDir() As String
        Get
            If SettingsDirValue Is Nothing Then
                SettingsDirValue = Paths.SolveDirMacros(RegistryHelp.CurrentUser.GetAppString("SettingsDir"))

                If Not IsDirValid(SettingsDirValue) Then
                    Dim settingsDirPage As New WizardPage(Of String)
                    settingsDirPage.Title = "Settings Directory"
                    settingsDirPage.Description = "Please choose a directory for the settings to be created or old settings to be loaded."

                    If Not DirPath.IsInSysDir(CommonDirs.Startup) Then
                        settingsDirPage.AddCommandButton("Startup", CommonDirs.Startup + "Settings", CommonDirs.Startup + "Settings")
                    End If

                    settingsDirPage.AddCommandButton("User", CommonDirs.UserAppDataLocal + "StaxRip", CommonDirs.UserAppDataLocal + "StaxRip")
                    settingsDirPage.AddDirectoryBrowserCommandButton("Custom", "Browse for a directory...")

                    Paths.SettingsDir = settingsDirPage.ShowPage

                    If Not Directory.Exists(SettingsDirValue) Then
                        Directory.CreateDirectory(SettingsDirValue)
                    End If
                End If
            End If

            Return SettingsDirValue
        End Get
        Set(ByVal value As String)
            If OK(value) Then
                If Not value.EndsWith("\") Then
                    value += "\"
                End If

                SettingsDirValue = value
                RegistryHelp.CurrentUser.SetAppValue("SettingsDir", InjectDirMacros(value))
            End If
        End Set
    End Property

    Public Shared Function SolveDirMacros(ByVal value As String) As String
        If Not OK(value) Then
            Return Nothing
        End If

        If value.Contains("%user_app_data_local_dir%") Then
            value = value.Replace("%user_app_data_local_dir%", CommonDirs.UserAppDataLocal)
        End If

        If value.Contains("%startup_dir%") Then
            value = value.Replace("%startup_dir%", CommonDirs.Startup)
        End If

        Return value
    End Function

    Private Shared Function InjectDirMacros(ByVal value As String) As String
        If Not OK(value) Then
            Return Nothing
        End If

        If value.Contains(CommonDirs.UserAppDataLocal) Then
            value = value.Replace(CommonDirs.UserAppDataLocal, "%user_app_data_local_dir%")
        End If

        If value.Contains(CommonDirs.Startup) Then
            value = value.Replace(CommonDirs.Startup, "%startup_dir%")
        End If

        Return value
    End Function

    Public Shared ReadOnly Property TemplateDir() As String
        Get
            Dim ret = SettingsDir + "Templates\"

            If Not Directory.Exists(ret) Then
                Directory.CreateDirectory(ret)
                g.CreatePMPTemplate()
            End If

            Return ret
        End Get
    End Property

    Public Shared ReadOnly Property SettingsFile() As String
        Get
            Return SettingsDir + "Settings.bin"
        End Get
    End Property

    Public Shared ReadOnly Property StartupTemplatePath() As String
        Get
            Dim ret = Paths.TemplateDir + s.StartupTemplate.Value + ".rip"

            If Not File.Exists(ret) Then
                ret = Paths.TemplateDir + "Default Template.rip"
                s.StartupTemplate.Value = "Default Template"
            End If

            Return ret
        End Get
    End Property

    Public Shared Function GetYV12DecoderPath() As String
        Dim path As String

        path = CommonDirs.System + "xvidvfw.dll"

        If File.Exists(path) Then
            Return path
        End If

        path = RegistryHelp.LocalMachine.GetString("SOFTWARE\Microsoft\Windows NT\CurrentVersion\Drivers32", "VIDC.FFDS")

        If OK(path) Then
            If Not path.Contains("\") Then
                path = CommonDirs.System + path
            End If

            If File.Exists(path) Then
                Return path
            End If
        End If

        Return Nothing
    End Function
End Class

Public Class GlobalClass
    Public ProjectPath As String
    Public MainForm As MainForm
    Public ShutdownMode As ShutdownMode = ShutdownMode.Nothing
    Public MinimizedWindows As Boolean
    Public SavedProject As New Project
    Public DefaultCommands As New GlobalCommands
    Public TempDir As String

    Public Function GetAutoSize(ByVal percentage As Integer) As Integer
        Dim ret As Integer
        Dim size = p.Size
        Dim bitrate = p.VideoBitrate

        For i = 1 To 10000
            p.Size = i
            p.VideoBitrate = CInt(Calc.GetTotalBitrate() - Calc.GetAudioBitrate())

            If CInt(Calc.GetPercent) >= percentage Then
                ret = i
                Exit For
            End If
        Next

        p.Size = size
        p.VideoBitrate = bitrate

        If ret = 0 Then
            ret = size
        End If

        Return ret
    End Function

    Public Function GetFileDialogFilter(ByVal value As String()) As String
        Return GetFileDialogFilter(value.Join(", "), value)
    End Function

    Public Function GetFileDialogFilter(ByVal text As String, ByVal value As String()) As String
        Return text + "|*." + value.Join(";*.") + "|All Files|*.*"
    End Function

    Public Sub RaiseApplicationEvent(ByVal ae As ApplicationEvent)
        For Each i In s.EventCommands
            If i.Event = ae Then
                Dim matches = 0

                For Each i2 In i.CriteriaList
                    i2.PropertyString = Macro.Solve(i2.Macro)

                    If i2.Eval Then
                        matches += 1
                    End If
                Next

                If i.CriteriaList.Count = 0 OrElse (i.OrOnly AndAlso matches > 0) OrElse _
                    (Not i.OrOnly AndAlso matches = i.CriteriaList.Count) Then

                    Log.WriteHeader("Process Event Command '" + i.Name + "'")
                    Log.WriteLine("Event: " + DisplayInfoAttribute.GetValueForEnum(Of ApplicationEvent)(i.Event))
                    Dim c = g.MainForm.CustomMainMenu.CommandManager.GetCommand(i.CommandParameters.MethodName)
                    Log.WriteLine("Arguments: " + c.GetParameterHelp(i.CommandParameters.Parameters))

                    g.MainForm.CustomMainMenu.CommandManager.Process(i.CommandParameters)
                End If
            End If
        Next
    End Sub

    Public Sub SetTempDir()
        If OK(p.SourceFile) Then
            If Not IsOneOf(Filepath.GetExt(p.SourceFile), ".d2v", ".dga") AndAlso _
                Not Filepath.GetDir(p.SourceFile).EndsWith(" temp files\") Then

                Dim dir = Macro.Solve(p.TempDir)

                If Not OK(dir) Then
                    dir = Filepath.GetDirAndBase(p.SourceFile) + " temp files\"
                End If

                dir = DirPath.AppendSeparator(dir)

                If Not Directory.Exists(dir) Then
                    Try
                        Directory.CreateDirectory(dir)
                        TempDir = dir
                    Catch ex As Exception
                        TempDir = Filepath.GetDir(p.SourceFile)
                    End Try
                Else
                    TempDir = dir
                End If
            Else
                TempDir = Filepath.GetDir(p.SourceFile)
            End If
        End If
    End Sub

    Public Function IsProjectDirty() As Boolean
        Return ObjectHelp.GetCompareString(g.SavedProject) <> ObjectHelp.GetCompareString(p)
    End Function

    Public Sub ShowCommandLinePreview(ByVal value As String)
        Using f As New StringEditorForm
            f.Text = "Command Line"
            f.cbWrap.Checked = Not value.Contains(CrLf)
            f.tb.Text = value
            f.tb.ReadOnly = True
            f.Width = 800
            f.Height = 200
            f.bOK.Visible = False
            f.bCancel.Text = "Close"
            f.ShowDialog()
        End Using
    End Sub

    Public Function IsValidSource() As Boolean
        If p.SourceAviSynthDocument.GetFrames = 0 Then
            Msg.Show("Failed to load source.", MsgIcon.Warning)
            Return False
        End If

        If Not p.SourceAviSynthDocument.GetErrorMessage Is Nothing Then
            Msg.Show(p.SourceAviSynthDocument.GetErrorMessage, MsgIcon.Error)
            Return False
        End If

        Return True
    End Function

    Public Function IsSourceSameOrSimilar(ByVal path As String) As Boolean
        Return IsSourceSame(path) OrElse IsSourceSimilar(path)
    End Function

    Public Function IsSourceSame(ByVal path As String) As Boolean
        Return Filepath.GetBase(path).StartsWith(Filepath.GetBase(p.SourceFile))
    End Function

    Public Function IsSourceSimilar(ByVal path As String) As Boolean
        If p.SourceFile.Contains("_") Then
            Dim src = Filepath.GetBase(p.SourceFile)

            While src.Length > 2 AndAlso src.ToCharArray.Last.IsDigit
                src = src.DeleteRight(1)
            End While

            If src.EndsWith("_") AndAlso Filepath.GetBase(path).StartsWith(src.TrimEnd("_"c)) Then
                Return True
            End If
        End If

        Return False
    End Function

    Public Function GetRelativePath(ByVal path As String) As String
        If OK(p.SourceFile) AndAlso Filepath.IsSameDir(p.SourceFile, path) Then
            Return Filepath.GetName(path)
        End If

        Return path
    End Function

    Public Function GetFullPath(ByVal path As String) As String
        If OK(p.SourceFile) AndAlso Not path.Contains("\") Then
            Return Filepath.GetDir(p.SourceFile) + path
        End If

        Return path
    End Function

    Public Function IsCulture(ByVal twoLetterCode As String) As Boolean
        Return CultureInfo.CurrentCulture.TwoLetterISOLanguageName = twoLetterCode
    End Function

    Public Function IsAacHE(ByVal path As String) As Boolean
        If path.Contains(" - sbr") Then
            Return True
        End If

        If File.Exists(path) AndAlso Filepath.GetExt(path) = ".mp4" AndAlso Packs.MediaInfo.VerifyOK Then
            Using mi As New MediaInfo(path)
                Return mi.AudioTracks(0).AudioType = MediaInfoAudioType.AACHE
            End Using
        End If
    End Function

    Private ExceptionHandled As Boolean

    Public Sub OnException(ByVal ex As Exception)
        If ExceptionHandled Then
            Exit Sub
        Else
            ExceptionHandled = True
        End If

        If TypeOf ex Is AbortException Then
            ProcessForm.CloseProcessForm()
        Else
            Try
                If File.Exists(p.SourceFile) Then
                    Dim name = Filepath.GetBase(p.TargetFile)

                    If name = "" Then
                        name = Filepath.GetBase(p.SourceFile)
                    End If

                    Dim path = Filepath.GetDir(p.SourceFile) + "crash.rip"
                    g.MainForm.SaveProjectByPath(path)
                End If

                g.MainForm.SaveSettings()
            Catch
            End Try

            Try
                Dim m As New Msg

                m.Icon = MsgIcon.Error
                m.Title = "StaxRip Crash"
                m.Text = "StaxRip has crashed:" + CrLf2 + ex.Message

                Try
                    g.MainForm.Invoke(New Action(AddressOf m.Show))
                Catch
                    m.Show()
                End Try

                MakeBugReport(ex)
            Catch
                Msg.Show(ex.ToString())
            End Try

            Process.GetCurrentProcess.Kill()
        End If
    End Sub

    Public Sub OnUnhandledException(ByVal sender As Object, ByVal e As UnhandledExceptionEventArgs)
        OnException(DirectCast(e.ExceptionObject, Exception))
    End Sub

    Public Sub MakeBugReport(ByVal e As Exception)
        Dim exceptions As New List(Of Exception)
        Dim dir = CommonDirs.Temp + "StaxRip Bug Report\"

        If Not e Is Nothing Then
            exceptions.Add(e)
        End If

        Try
            DirectoryHelp.Delete(dir)
            Directory.CreateDirectory(dir)
        Catch ex As Exception
            exceptions.Add(ex)
        End Try

        Try
            SafeSerialization.Serialize(p, dir + "Project.rip")
        Catch ex As Exception
            exceptions.Add(ex)
        End Try

        Dim sb As New StringBuilder

        Try
            sb.Append(Log.FormatHeader("Environment"))

            sb.Append("StaxRip version: ".PadRight(20) + Application.ProductVersion + " (" + _
                      File.GetLastWriteTime(Me.GetType.Assembly.Location).ToString("yy-MM-dd") + ")" + CrLf)

            sb.Append("OS version: ".PadRight(20) + Environment.OSVersion.ToString() + CrLf)
            sb.Append(".NET version: ".PadRight(20) + Environment.Version.ToString() + CrLf)
            sb.Append("DirectX version: ".PadRight(20) + RegistryHelp.LocalMachine.GetString("Software\Microsoft\DirectX", "Version") + CrLf)
            sb.Append("YV12 decoder: ".PadRight(20) + RegistryHelp.LocalMachine.GetString("SOFTWARE\Microsoft\Windows NT\CurrentVersion\Drivers32", "VIDC.YV12") + CrLf)
            sb.Append("Screen colordepth: ".PadRight(20) + Screen.PrimaryScreen.BitsPerPixel.ToString + CrLf)

            sb.Append(Log.FormatHeader("Not Installed"))

            For Each i In Packs.Packages.Values
                If i.GetCheckedPath Is Nothing Then
                    sb.AppendLine(i.Name)
                End If
            Next

            sb.Append(Log.FormatHeader("Wrong Version"))

            For Each i In Packs.Packages.Values
                If Not i.GetCheckedPath Is Nothing AndAlso Not i.IsCorrectVersion Then
                    sb.AppendLine(i.Name)
                End If
            Next
        Catch ex As Exception
            exceptions.Add(ex)
        End Try

        Try
            sb.Append(Log.FormatHeader("Macros"))

            For Each i In Macro.GetMacros()
                sb.AppendLine((i.FriendlyName + ": ").PadRight(20) + i.Solve)
            Next
        Catch ex As Exception
            exceptions.Add(ex)
        End Try

        Try
            sb.Append(Log.FormatHeader("Application Paths"))

            For Each i In Packs.Packages.Values
                sb.AppendLine((i.Name + ": ").PadRight(20) & i.GetCheckedPath)
            Next
        Catch ex As Exception
            exceptions.Add(ex)
        End Try

        Try
            sb.Append(Log.FormatHeader("AviSynth Plugins"))

            For Each i In Directory.GetFiles(Paths.AviSynthPluginsDir, "*.dll")
                sb.AppendLine((Filepath.GetName(i) + ": ").PadRight(20) + File.GetLastWriteTimeUtc(i).ToString("g"))
            Next
        Catch ex As Exception
            exceptions.Add(ex)
        End Try

        Try
            If File.Exists(p.SourceFile) Then
                sb.Append(Log.FormatHeader("Media Info"))
                sb.AppendLine(MediaInfo.GetSummary(p.SourceFile))
            End If
        Catch ex As Exception
            exceptions.Add(ex)
        End Try

        Try
            sb.ToString.WriteFile(dir + "Info.txt")
        Catch ex As Exception
            exceptions.Add(ex)
        End Try

        Try
            Dim l = p.Log.ToString

            If Not OK(l) Then
                l = Log.AbortLog
            End If

            If OK(l) Then
                l.WriteFile(dir + "Log.txt")
            End If
        Catch ex As Exception
            exceptions.Add(ex)
        End Try

        Try
            For Each i In exceptions
                i.ToString.WriteFile(dir + "Exception" + exceptions.IndexOf(i).ToString + ".txt")
            Next
        Catch ex As Exception
        End Try

        Try
            If File.Exists(Packs.SevenZip.Path) Then
                Dim path = CommonDirs.UserDesktop + "StaxRip Diagnostic Files.7z"
                FileHelp.Delete(path)
                Packs.SevenZip.Pack(dir, path)

                Dim m As New Msg
                m.Text = "A diagnostic files was saved on the desktop, please post this diagnostic file in the support forum!"
                m.Title = "Bug Report"

                g.MainForm.Invoke(New Action(AddressOf m.Show))
            End If
        Catch ex As Exception
        End Try

        Try
            Directory.Delete(dir, True)
        Catch ex As Exception
        End Try

        Try
            Proc.StartFile(Package.RemoteDomain + "staxrip/support")
        Catch ex As Exception
            exceptions.Add(ex)
        End Try
    End Sub

    Public Function FileWasJustWritten(ByVal path As String) As Boolean
        For x = 0 To 9
            Dim r = File.Exists(path) AndAlso CInt(DateTime.Now.Subtract(File.GetLastWriteTime(path)).TotalSeconds) < 9

            If r Then
                Return True
            End If

            Thread.Sleep(444)
        Next

        Return False
    End Function

    Public Sub ShutdownPC()
        ShutdownPC(g.ShutdownMode)
    End Sub

    Public Sub ShutdownPC(ByVal mode As ShutdownMode)
        If mode <> ShutdownMode.Nothing Then
            SavedProject = p
            g.MainForm.Close()
            Shutdown.Commit(mode)
        End If
    End Sub

    Public Sub Highlight(ByVal highlight As Boolean, ByVal c As Control)
        If highlight Then
            c.BackColor = Color.Orange
        Else
            If TypeOf c Is Label OrElse TypeOf c Is GroupBox Then
                c.BackColor = SystemColors.Control
            ElseIf TypeOf c Is TextBox AndAlso DirectCast(c, TextBox).ReadOnly Then
                c.BackColor = SystemColors.Control
            Else
                c.BackColor = SystemColors.Window
            End If
        End If
    End Sub

    Public Sub AutoResize()
        Dim cropw = p.SourceWidth - p.CropLeft - p.CropRight

        If Not p.TargetAVSDoc.IsFilterActive("Resize") Then
            g.MainForm.tbTargetWidth.Text = cropw.ToString
            g.MainForm.tbTargetHeight.Text = (p.SourceHeight - p.CropTop - p.CropBottom).ToString
        Else
            If p.AutoResizeImage <> 0 Then
                If p.TargetAVSDoc.IsFilterActive("Resize") Then
                    Dim ar = Calc.GetTargetDAR()
                    Dim w = Calc.FixMod16(CInt(Math.Sqrt(p.AutoResizeImage * ar)))

                    If w > cropw Then
                        w = (cropw \ 16) * 16
                    End If

                    Dim h = Calc.FixMod16(CInt(w / ar))

                    g.MainForm.tbTargetWidth.Text = w.ToString()
                    g.MainForm.tbTargetHeight.Text = h.ToString()
                End If
            Else
                Dim h = Calc.FixMod16(CInt(p.TargetWidth / Calc.GetTargetDAR()))
                g.MainForm.tbTargetHeight.Text = h.ToString()
            End If
        End If
    End Sub

    Public Sub AutoCrop()
        Dim d As New AviSynthDocument
        d.Path = g.TempDir + p.Name + "_AutoCrop.avs"

        For Each i In p.TargetAVSDoc.Filters
            If i.Category = "Source" OrElse i.Category = "Plugin" Then
                d.Filters.Add(i.GetCopy)
            End If
        Next

        d.Filters.Add(New AviSynthFilter("AutoCrop", "AutoCrop", "ConvertToYV12().AutoCrop(mode=2,samples=20)", True))
        d.Synchronize()

        Dim logPath = Filepath.GetDir(d.Path) + "AutoCrop.log"
        Dim line = StringHelp.ReadFile(logPath).SplitLines(0)
        File.Delete(logPath)

        Dim sa = line.Left(")").Right("(").Split(",")

        p.CropLeft = CInt(sa(0))
        p.CropTop = CInt(sa(1))
        p.CropRight = p.SourceWidth - p.CropLeft - CInt(sa(2))
        p.CropBottom = p.SourceHeight - p.CropTop - CInt(sa(3))

        CorrectCropMod()
    End Sub

    Public Sub SmartCrop()
        If Not p.TargetAVSDoc.IsFilterActive("Resize") Then
            Exit Sub
        End If

        Dim tempLeft = p.CropLeft
        Dim tempRight = p.CropRight

        Dim ae = Math.Abs(Calc.GetAspectRatioError)

        While (p.SourceWidth - p.CropLeft - p.CropRight) > 64
            p.CropLeft += 2
            p.CropRight += 2

            If Math.Abs(Calc.GetAspectRatioError()) < ae Then
                tempLeft = p.CropLeft
                tempRight = p.CropRight
                ae = Math.Abs(Calc.GetAspectRatioError())
            End If
        End While

        p.CropLeft = tempLeft
        p.CropRight = tempRight

        Dim tempTop = p.CropTop
        Dim tempBottom = p.CropBottom

        While (p.SourceHeight - p.CropTop - p.CropBottom) > 64
            p.CropTop += 2
            p.CropBottom += 2

            If Math.Abs(Calc.GetAspectRatioError()) < ae Then
                tempTop = p.CropTop
                tempBottom = p.CropBottom
                ae = Math.Abs(Calc.GetAspectRatioError())
            End If
        End While

        p.CropTop = tempTop
        p.CropBottom = tempBottom
    End Sub

    Public Sub OvercropWidth()
        If p.AutoSmartOvercrop.Value > 0 Then
            Dim newar = p.AutoSmartOvercrop.Value
            Dim croph = p.SourceHeight - p.CropTop - p.CropBottom
            g.MainForm.tbTargetHeight.Text = Calc.FixMod16(CInt(p.TargetWidth / newar)).ToString
            newar = CSng(p.TargetWidth / p.TargetHeight)
            Dim cropw = (newar / Calc.GetSourceDAR * (croph / p.SourceHeight)) * p.SourceWidth
            p.CropLeft = CInt((p.SourceWidth - cropw) / 2)
            p.CropRight = p.CropLeft
            CorrectCropMod()
        End If
    End Sub

    Public Function ConvertPath(ByVal value As String) As String
        If value.Contains("Misc/") Then
            value = value.Replace("Misc/", "")
        End If

        If value.Contains("More/") Then
            value = value.Replace("More/", "")
        End If

        If value.Contains("/") Then
            value = value.Replace("/", " ")
        End If

        If value.Contains("/") Then
            value = value.Replace("/", " ")
        End If

        If value.Contains("  ") Then
            value = value.Replace("  ", " ")
        End If

        Return value
    End Function

    Public Sub CorrectCropMod()
        CorrectCropMod(False)
    End Sub

    Public Sub ForceCropMod()
        CorrectCropMod(True)
    End Sub

    Private Sub CorrectCropMod(ByVal force As Boolean)
        If p.AutoCorrectCropValues.Value OrElse force Then
            p.CropLeft += p.CropLeft Mod 2
            p.CropRight += p.CropRight Mod 2
            p.CropTop += p.CropTop Mod 2
            p.CropBottom += p.CropBottom Mod 2

            Dim modValue = 4

            If Not p.TargetAVSDoc.IsFilterActive("Resize") Then
                modValue = p.ForcedOutputMod.Value
            End If

            Dim whalf = ((p.SourceWidth - p.CropLeft - p.CropRight) Mod modValue) \ 2

            If p.CropLeft > p.CropRight Then
                p.CropLeft += whalf - whalf Mod 2
                p.CropRight += whalf + whalf Mod 2
            Else
                p.CropRight += whalf - whalf Mod 2
                p.CropLeft += whalf + whalf Mod 2
            End If

            Dim hhalf = ((p.SourceHeight - p.CropTop - p.CropBottom) Mod modValue) \ 2

            If p.CropTop > p.CropBottom Then
                p.CropTop += hhalf - hhalf Mod 2
                p.CropBottom += hhalf + hhalf Mod 2
            Else
                p.CropBottom += hhalf - hhalf Mod 2
                p.CropTop += hhalf + hhalf Mod 2
            End If
        End If
    End Sub

    Public Sub CreatePMPTemplate()
        Dim pmp As New Project
        pmp.Init()
        pmp.TargetWidth = 480
        pmp.TargetHeight = 272
        pmp.AutoResizeImage = 0
        pmp.CompCheckAction = CompCheckAction.AdjustFileSize
        pmp.AutoSmartCrop.Value = True
        pmp.Audio0 = New CmdlAudioProfile("Misc/MP3 VBR 128 kbps 44100 Khz", 128, New String() {"ac3", "mp2", "mp3", "mpa", "wav"}, StreamType.MP3, """%application:BeSweet%"" -core( -input ""%input%"" -output ""%output%"" ) -azid( -c normal -L -3db ) -ota( -d %delay% -hybridgain ) -shibatch( --rate 44100 ) -lame( --alt-preset %bitrate% --resample 44.1)")
        pmp.VideoEncoder = VideoEncoder.GetEncoder("XviD/Exact File Size")
        pmp.VideoEncoder.Muxer = New PMPMuxer
        SafeSerialization.Serialize(pmp, Paths.TemplateDir + "PlayStation Portable (PMP).rip")
    End Sub
End Class

Public Enum AudioMode
    DirectStreamCopy
    FullProcessingMode
End Enum

Public Enum RegistryRoot
    CurrentUser
    LocalMachine
    ClassesRoot
End Enum

Public Class VirtualDubScript
    Private LinesValue As New List(Of String)

    Public Property Lines() As List(Of String)
        Get
            Return LinesValue
        End Get
        Set(ByVal Value As List(Of String))
            LinesValue = Value
        End Set
    End Property

    Public Function GetStreamSetSourceParams(ByVal path As String) As String
        Select Case Filepath.GetExt(path)
            Case ".mp3"
                Using mi As New MediaInfo(path)
                    If mi.AudioTracks(0).AudioType = MediaInfoAudioType.MP3CBR Then
                        Return "0x00000202,0"
                    Else
                        Return "0x00000202,1"
                    End If
                End Using
            Case ".mp2", ".mpa"
                Return "0x00000202,0"
            Case ".wav"
                Return "0x00000201"
            Case ".ac3"
                Return "0x00000203,0"
            Case Else
                Return Nothing
        End Select
    End Function

    Public Sub SaveScript(ByVal path As String)
        Dim script = ""

        For Each i In Lines
            If i <> "" Then
                script += i + CrLf
            End If
        Next

        Lines.Clear()

        Dim newScript = ""

        For Each i In script
            If Convert.ToByte(i) > 127 Then 'ANSII is not supported by VirtualDub
                For Each i2 In System.Text.Encoding.UTF8.GetBytes(i)
                    newScript += "\x" + System.Convert.ToString(i2, 16)
                Next
            Else
                newScript += i.ToString()
            End If
        Next

        newScript.WriteFile(path)
    End Sub

    Private Function EscapeSeparator(ByVal value As String) As String
        Return value.Replace("\\", "\").Replace("\", "\\")
    End Function

    Public Shared Function FixQuotes(ByVal st As String) As String
        Return st.Replace("""", "\""")
    End Function

    Public ReadOnly Property Text() As String
        Get
            Dim t As String = ""

            For Each i In Lines
                t += i + Environment.NewLine
            Next

            Return t
        End Get
    End Property

    Public Sub AddLine(ByVal search As String, ByVal line As String)
        Dim x As Integer
        Dim Item As String

        line = EscapeSeparator(line)

        For x = 0 To Lines.Count - 1
            Item = CStr(Lines.Item(x))

            If Item.IndexOf(search) > -1 AndAlso Not line.Contains(".Add") Then
                Lines.RemoveAt(x)
                Lines.Insert(x, line)

                Exit Sub
            End If
        Next

        Lines.Add(line)
    End Sub

    Public Sub SetLines(ByVal scriptText As String)
        For Each i As String In scriptText.SplitLines
            If i.StartsWith("VirtualDub.") Then
                AddLine(i.Left("(") + "(", i)
            End If
        Next
    End Sub

    Public Sub Delete(ByVal search As String)
        Dim x As Integer

        For x = 0 To Lines.Count - 1
            If CType(Lines.Item(x), String).IndexOf(search) > -1 Then
                Lines.RemoveAt(x)
                Exit For
            End If
        Next
    End Sub

    'script functions

    Public Sub Open(ByVal path As String)
        Lines.Add("VirtualDub.Open(""" + EscapeSeparator(path) + ""","""",0);")
    End Sub

    Public Sub Close()
        Lines.Add("VirtualDub.Close();")
    End Sub

    Public Sub Append(ByVal path As String)
        Lines.Add("VirtualDub.Append(""" + EscapeSeparator(path) + """);")
    End Sub

    Public Sub Save(ByVal path As String, ByVal splitSize As Integer)
        If splitSize > 50 Then
            Lines.Add("VirtualDub.SaveSegmentedAVI(""" + EscapeSeparator(path) + """," & splitSize & ",0);")
        Else
            Lines.Add("VirtualDub.SaveAVI(""" + EscapeSeparator(path) + """);")
        End If
    End Sub

    Public Sub SaveWAV(ByVal path As String)
        Lines.Add("VirtualDub.SaveWAV(""" + EscapeSeparator(path) + """);")
    End Sub

    Public Sub SubsetClear()
        Lines.Add("VirtualDub.subset.Clear();")
    End Sub

    Public Sub SubsetAddRange(ByVal start As Integer, ByVal [end] As Integer)
        Lines.Add("VirtualDub.subset.AddRange(" + start.ToString + ", " + [end].ToString + ");")
    End Sub

    Public Sub RemoveInputStreams()
        Lines.Add("VirtualDub.RemoveInputStreams();")
    End Sub

    Public Sub VideoSetModeFastRecompress()
        AddLine("video.SetMode", "VirtualDub.video.SetMode(1);")
    End Sub

    Public Sub VideoSetRange(ByVal start As Integer, ByVal length As Integer)
        AddLine("video.SetRange", "VirtualDub.video.SetRange(" + start.ToString() + ", " + length.ToString() + ");")
    End Sub

    Public Sub VideoSetCompression(ByVal arguments As String, ByVal kbytesPerSec As Double)
        AddLine("video.SetCompression", "VirtualDub.video.SetCompression(" + arguments.Replace("%kbytes_per_sec%", CInt(kbytesPerSec).ToString()) + ");")
    End Sub

    Public Sub StreamSetSource(ByVal index As Integer, ByVal path As String)
        Lines.Add("VirtualDub.stream[" + index.ToString() + "].SetSource(""" + EscapeSeparator(path) + """," + GetStreamSetSourceParams(path) + ");")
    End Sub

    Public Sub StreamDemux(ByVal index As Integer, ByVal path As String)
        Lines.Add("VirtualDub.stream[" + index.ToString() + "].Demux(""" + EscapeSeparator(path) + """);")
    End Sub

    Public Sub StreamSetMode(ByVal index As Integer, ByVal mode As AudioMode)
        Lines.Add("VirtualDub.stream[" + index.ToString() + "].SetMode(" + CInt(mode).ToString() + ");")
    End Sub

    Public Sub StreamSetInterleave(ByVal index As Integer, ByVal delay As Integer)
        Lines.Add("VirtualDub.stream[" + index.ToString() + "].SetInterleave(1,500,1,0," + delay.ToString + ");")
    End Sub
End Class

<Serializable()> _
Public Class Range
    Implements IComparable(Of Range)

    Public Start As Integer
    Public [End] As Integer

    Public Sub New(ByVal startPosition As Integer, ByVal endPosition As Integer)
        Me.Start = startPosition
        Me.End = endPosition
    End Sub

    Public Function GetLenght() As Integer
        Return Me.End - Start
    End Function

    Public Function CompareTo(ByVal other As Range) As Integer Implements System.IComparable(Of Range).CompareTo
        Return Start.CompareTo(other.Start)
    End Function
End Class

Public Class Log
    Public Shared Event Update(ByVal sb As StringBuilder)
    Public Shared AbortLog As String

    Public Shared Sub WriteLine(ByVal value As String)
        If value.EndsWith(CrLf) Then
            p.Log.Append(value)
        Else
            p.Log.AppendLine(value)
        End If

        RaiseUpdate()
    End Sub

    Public Shared Sub WriteDebug(ByVal header As String, ByVal message As String)
        WriteHeader(header)
        WriteLine(message)
    End Sub

    Public Shared Function FormatHeader(ByVal value As String) As String
        Return _
            CrLf + "".PadLeft(60, "-"c) + CrLf + _
            value.PadLeft(30 + value.Length \ 2) + _
            CrLf + "".PadLeft(60, "-"c) + CrLf2
    End Function

    Public Shared Sub WriteHeader(ByVal value As String)
        p.Log.Append(FormatHeader(value))
        RaiseUpdate()
    End Sub

    Public Shared Sub Save()
        If OK(p.SourceFile) Then
            p.Log.ToString.WriteFile(g.TempDir + p.Name + "_StaxRip.log")
        End If
    End Sub

    Public Shared Sub WriteStats(ByVal start As DateTime)
        WriteStats(start, 0)
    End Sub

    Public Shared Sub WriteStats(ByVal start As DateTime, ByVal frames As Integer)
        Dim n = DateTime.Now.Subtract(start)

        p.Log.Append("Start: ".PadRight(10) + start.ToLongTimeString + CrLf)
        p.Log.Append("End: ".PadRight(10) + DateTime.Now.ToLongTimeString + CrLf)
        p.Log.Append("Duration: " + CInt(Math.Floor(n.TotalHours)).ToString("d2") + ":" + n.Minutes.ToString("d2") + ":" + n.Seconds.ToString("d2") + CrLf)

        If frames > 0 Then
            p.Log.Append("Speed: ".PadRight(10) + CInt(frames / n.TotalSeconds).ToString() + " fps" + CrLf)
        End If

        RaiseUpdate()
    End Sub

    Public Shared Sub WriteFilesize(ByVal path As String)
        WriteLine("Size:".PadRight(10) + (New FileInfo(path).Length / 1024 ^ 2).ToString("f1") + " MB")
        RaiseUpdate()
    End Sub

    Private Shared Sub RaiseUpdate()
        RaiseEvent Update(p.Log)
    End Sub
End Class

Public Class Calc
    Public Shared Function GetPercent() As Double
        If p.Compressibility = 0 Then
            Return 0
        End If

        Return (GetBPF() / p.Compressibility) * 100
    End Function

    Public Shared Function GetBPF() As Double
        Dim framerate = p.TargetAVSDoc.GetFramerate

        If framerate = 0 Then
            Return 0
        End If

        If p.TargetWidth = 0 Then
            Return 0
        End If

        If p.TargetHeight = 0 Then
            Return 0
        End If

        Return (p.VideoBitrate * 1024) / (p.TargetWidth * p.TargetHeight * framerate)
    End Function

    Public Shared Function GetVideoBitrate() As Double
        Return Calc.GetTotalBitrate - Calc.GetAudioBitrate()
    End Function

    Public Shared Function GetTotalBitrate() As Double
        If p.TargetSeconds = 0 Then
            Return 0
        End If

        Dim kb As Integer = (p.Size * 1024) - GetOverheadAndSubtitlesKBytes()

        Return (kb * 8 * 1.024) / p.TargetSeconds
    End Function

    Public Shared Function GetVideoKBytes() As Double
        Return ((p.VideoBitrate * p.TargetSeconds) / 8) / 1.024
    End Function

    Public Shared Function GetOverheadAndSubtitlesKBytes() As Integer
        Dim ret As Double
        Dim frames As Integer = p.TargetAVSDoc.GetFrames

        If p.VideoEncoder.Muxer.OutputType = StreamType.AVI OrElse _
            p.VideoEncoder.Muxer.OutputType = StreamType.DMF Then

            ret += frames * 0.024

            If OK(p.Audio0.File) Then
                ret += frames * 0.04
            End If

            If OK(p.Audio1.File) Then
                ret += frames * 0.04
            End If
        ElseIf p.VideoEncoder.Muxer.OutputType = StreamType.MP4 Then
            ret += (10.4 / 1024) * frames
        ElseIf p.VideoEncoder.Muxer.OutputType = StreamType.MKV Then
            ret += frames * 0.013
        End If

        For Each i In p.VideoEncoder.Muxer.Subtitles2
            If Filepath.GetExt(g.GetFullPath(i.Path)) = ".idx" Then
                ret += CInt(p.TargetSeconds * 0.256)
            End If
        Next

        Return CInt(ret)
    End Function

    Public Shared Function GetAudioKBytes() As Double
        Return ((Calc.GetAudioBitrate() * p.TargetSeconds) / 8) / 1.024
    End Function

    Public Shared Function GetAudioBitrate() As Double
        Dim b0, b1 As Double

        If p.Audio0.File <> "" Then
            b0 = p.Audio0.Bitrate
        End If

        If p.Audio1.File <> "" Then
            b1 = p.Audio1.Bitrate
        End If

        Return b0 + b1
    End Function

    <DebuggerNonUserCode()> _
    Public Shared Function GetBitrateFromFile(ByVal path As String, ByVal seconds As Integer) As Double
        Try
            If Not OK(path) OrElse seconds = 0 Then
                Return 0
            End If

            Dim kBits = ((New FileInfo(path).Length) * 8) / 1000

            Return kBits / seconds
        Catch
            Msg.Show("File doesn't exist: " + path, MsgIcon.Error)
        End Try
    End Function

    Public Shared Function IsARSignalingRequired() As Boolean
        If Not p.TargetAVSDoc Is Nothing AndAlso p.AutoARSignaling.Value Then
            Dim par = GetTargetPAR()

            If par.X <> par.Y Then
                If p.TargetAVSDoc.IsFilterActive("Resize") Then
                    Return Math.Abs(GetAspectRatioError()) > p.MaxAspectRatioError.Value
                Else
                    Return True
                End If
            End If
        End If

        Return False
    End Function

    Private Shared Function GetPrettyPar(ByVal par As Point) As Point
        If par.Y > 0 Then
            For Each i In New Point() {New Point(12, 11), New Point(10, 11), New Point(16, 11), New Point(40, 33)}
                If Math.Abs((par.X / par.Y) / (i.X / i.Y) * 100 - 100) < 1 Then
                    Return i
                End If
            Next
        End If

        If par.X > 255 OrElse par.Y > 255 Then
            Dim x = par.X / 255
            Dim y = par.Y / 255

            If x > y Then
                par.X = CInt(par.X / x)
                par.Y = CInt(par.Y / x)
            Else
                par.X = CInt(par.X / y)
                par.Y = CInt(par.Y / y)
            End If
        End If

        Return par
    End Function

    Public Shared Function GetSourcePAR() As Point
        If OK(p.CustomPAR) Then
            If p.CustomPAR.Contains(":") OrElse p.CustomPAR.Contains("/") Then
                Dim a = p.CustomPAR.Split(":/".ToCharArray)

                If a.Length = 2 AndAlso a(0).IsInt AndAlso a(1).IsInt Then
                    Return New Point(a(0).ToInt, a(1).ToInt)
                End If
            ElseIf Single.TryParse(p.CustomPAR, Nothing) Then
                Return Reduce(New Point(CInt(CSng(p.CustomPAR) * 1000), 1000))
            End If
        End If

        Dim par As New Point(1, 1)
        Dim w = p.SourceWidth, h = p.SourceHeight

        If Filepath.GetExt(p.SourceFile) = ".d2v" AndAlso h <= 576 Then
            Dim f As VideoFormat

            For Each i In Formats
                If i.Width = p.SourceWidth AndAlso i.Height = p.SourceHeight Then
                    f = i
                    Exit For
                End If
            Next

            If f.Width > 0 Then
                Dim samplingWidth = 52.0

                If Not p.ITU.Value Then
                    samplingWidth = f.Width / f.SamplingRate
                End If

                par.X = CInt(If(p.SourceAnamorphic, 16 / 9, 4 / 3) * f.Height)
                par.Y = CInt(f.SamplingRate * samplingWidth)
            Else
                Dim dar = If(p.SourceAnamorphic, 16 / 9, 4 / 3)

                If p.ITU.Value Then
                    dar *= 1.0255
                End If

                par.X = CInt(dar * h)
                par.Y = CInt(w)
            End If
        ElseIf IsOneOf(h, 720, 1080, 1088) AndAlso p.SourceAnamorphic Then
            par.X = 16 * h
            par.Y = 9 * w
        Else
            Static last As Point
            Static lastValue As PointF
            Dim current = New Point(w, h)

            If last <> current AndAlso OK(p.SourceFile) Then
                last = current

                Dim path = p.SourceFile

                If Filepath.GetExt(path) = ".dga" Then
                    If File.Exists(Filepath.GetChangeExt(path, "mp4")) Then
                        path = Filepath.GetChangeExt(path, "mp4")
                    ElseIf File.Exists(Filepath.GetChangeExt(path, "mkv")) Then
                        path = Filepath.GetChangeExt(path, "mkv")
                    End If
                End If

                Dim r = MediaInfo.GetInfo(path, MediaInfoStreamKind.Video, "DisplayAspectRatio")

                If OK(r) Then
                    lastValue.X = Convert.ToSingle(r, CultureInfo.InvariantCulture)
                    lastValue.Y = 1
                Else
                    lastValue.X = p.SourceWidth
                    lastValue.Y = p.SourceHeight
                End If
            End If

            If lastValue.X > 0 Then
                par.X = CInt(lastValue.X * p.SourceHeight)
                par.Y = CInt(lastValue.Y * p.SourceWidth)
            End If
        End If

        Return Reduce(par)
    End Function

    Public Shared Function GetSourceDAR() As Double
        Dim par = GetSourcePAR()
        Return (par.X * p.SourceWidth) / (par.Y * p.SourceHeight)
    End Function

    Public Shared Function GetTargetPAR() As Point
        Dim par = GetSourcePAR()

        Dim cw = p.SourceWidth
        Dim ch = p.SourceHeight

        If p.TargetAVSDoc.IsFilterActive("Crop") Then
            cw -= p.CropLeft + p.CropRight
            ch -= p.CropTop + p.CropBottom
        End If

        If p.TargetWidth <> cw OrElse p.TargetHeight <> ch Then
            Dim par2 = Reduce(New Point(cw * p.TargetHeight, ch * p.TargetWidth))
            par.X = par.X * par2.X
            par.Y = par.Y * par2.Y
            par = Reduce(par)
        End If

        Return GetPrettyPar(par)
    End Function

    Public Shared Function GetTargetDAR() As Double
        Dim w = p.SourceWidth, h = p.SourceHeight
        Dim cropw = w, croph = h

        If p.TargetAVSDoc.IsFilterActive("Crop") Then
            cropw = w - p.CropLeft - p.CropRight
            croph = h - p.CropTop - p.CropBottom
        End If

        Return ((cropw / w) / (croph / h)) * GetSourceDAR()
    End Function

    Public Shared Function GetAspectRatioError() As Double
        Return (Calc.GetTargetDAR / (p.TargetWidth / p.TargetHeight)) * 100 - 100
    End Function

    Public Shared Function Reduce(ByVal p As Point) As Point
        If p.X <> 0 AndAlso p.Y <> 0 Then
            Dim gcd = GetGCD(p.X, p.Y)
            p.X \= gcd
            p.Y \= gcd
        End If

        Return p
    End Function

    Public Shared Function GetGCD(ByVal a As Integer, ByVal b As Integer) As Integer
        If b = 0 Then
            Return a
        Else
            Return GetGCD(b, a Mod b)
        End If
    End Function

    Public Shared Function FixMod16(ByVal value As Integer) As Integer
        Return CInt(value / 16) * 16
    End Function

    Public Shared Function FixMod(ByVal value As Integer, ByVal modValue As Integer) As Integer
        Return CInt(value / modValue) * modValue
    End Function

    Public Shared Function GetMod(ByVal w As Integer, ByVal h As Integer) As String
        Dim wmod, hmod As Integer

        For Each x In New Integer() {1, 2, 4, 8, 16}
            If w Mod x = 0 Then wmod = x
            If h Mod x = 0 Then hmod = x
        Next

        If wmod = 16 AndAlso hmod = 16 Then
            Return wmod & "/" & hmod
        Else
            Dim x = w - FixMod16(w)
            Dim xval As String

            If x = 8 OrElse x = -8 Then
                xval = "8"
            ElseIf x > 0 Then
                xval = "+" & x
            Else
                xval = x.ToString
            End If

            Dim y = h - FixMod16(h)
            Dim yval As String

            If y = 8 OrElse y = -8 Then
                yval = "8"
            ElseIf y > 0 Then
                yval = "+" & y
            Else
                yval = y.ToString
            End If

            Return wmod & "/" & hmod & " (" & xval & "/" & yval & ")"
        End If
    End Function


    Public Shared Function GetMod(ByVal value As Integer) As Integer
        Dim ret As Integer

        For Each i In New Integer() {1, 2, 4, 8, 16}
            If value Mod i = 0 Then
                ret = i
            End If
        Next

        Return ret
    End Function

    Public Shared Function GetNextMod(ByVal val As Integer, ByVal step1 As Integer) As Integer
        Do
            val += 1
        Loop Until val Mod step1 = 0

        Return val
    End Function

    Public Shared Function GetPreviousMod(ByVal val As Integer, ByVal step1 As Integer) As Integer
        Do
            val -= 1
        Loop Until val Mod step1 = 0

        Return val
    End Function

    Private Shared FormatsValue As VideoFormat()

    Public Shared ReadOnly Property Formats() As VideoFormat()
        Get
            If FormatsValue Is Nothing Then
                FormatsValue = New VideoFormat() { _
                    New VideoFormat(768, 576, 14.75), _
                    New VideoFormat(768, 560, 14.75), _
                    New VideoFormat(720, 576, 13.5), _
                    New VideoFormat(704, 576, 13.5), _
                    New VideoFormat(702, 576, 13.5), _
                    New VideoFormat(544, 576, 10.125), _
                    New VideoFormat(480, 576, 9.0), _
                    New VideoFormat(384, 288, 7.375), _
                    New VideoFormat(384, 280, 7.375), _
                    New VideoFormat(352, 576, 6.75), _
                    New VideoFormat(352, 288, 6.75), _
                    New VideoFormat(176, 144, 3.375), _
                    New VideoFormat(720, 486, 13.5), _
                    New VideoFormat(720, 480, 13.5), _
                    New VideoFormat(711, 486, 13.5), _
                    New VideoFormat(704, 486, 13.5), _
                    New VideoFormat(704, 480, 13.5), _
                    New VideoFormat(640, 480, 12.27272), _
                    New VideoFormat(480, 480, 9.0), _
                    New VideoFormat(352, 480, 6.75), _
                    New VideoFormat(352, 240, 6.75), _
                    New VideoFormat(320, 240, 6.13636)}
            End If

            Return FormatsValue
        End Get
    End Property
End Class

Public Structure VideoFormat
    Public Sub New(ByVal width As Integer, ByVal height As Integer, ByVal samplingRate As Double)
        Me.Width = width
        Me.Height = height
        Me.SamplingRate = samplingRate
    End Sub

    Public Width As Integer
    Public Height As Integer
    Public SamplingRate As Double
    Public Matrix As Double
End Structure

<Serializable()> _
Public Class Language
    Implements IComparable(Of Language)

    Public Sub New()
        Me.New(CultureInfo.CurrentCulture.TwoLetterISOLanguageName)
    End Sub

    Public Sub New(ByVal ci As CultureInfo)
        CultureInfoValue = ci
    End Sub

    Public Sub New(ByVal twoLetterCode As String)
        Try
            CultureInfoValue = New CultureInfo(twoLetterCode)
        Catch ex As Exception
            CultureInfoValue = CultureInfo.CurrentCulture
        End Try
    End Sub

    Private CultureInfoValue As CultureInfo

    Public ReadOnly Property CultureInfo() As CultureInfo
        Get
            Return CultureInfoValue
        End Get
    End Property

    Public ReadOnly Property TwoLetterCode() As String
        Get
            Return CultureInfo.TwoLetterISOLanguageName
        End Get
    End Property

    <NonSerialized()> Private ThreeLetterCodeValue As String

    Public ReadOnly Property ThreeLetterCode() As String
        Get
            If ThreeLetterCodeValue Is Nothing Then
                If CultureInfo.TwoLetterISOLanguageName = "iv" Then
                    ThreeLetterCodeValue = "und"
                Else
                    Select Case CultureInfo.ThreeLetterISOLanguageName
                        Case "ces"
                            ThreeLetterCodeValue = "cze"
                        Case "zho"
                            ThreeLetterCodeValue = "chi"
                        Case "deu"
                            ThreeLetterCodeValue = "ger"
                        Case "nld"
                            ThreeLetterCodeValue = "dut"
                        Case "ell"
                            ThreeLetterCodeValue = "gre"
                        Case "fra"
                            ThreeLetterCodeValue = "fre"
                        Case Else
                            ThreeLetterCodeValue = CultureInfo.ThreeLetterISOLanguageName
                    End Select
                End If
            End If

            Return ThreeLetterCodeValue
        End Get
    End Property

    Public ReadOnly Property Name() As String
        Get
            If CultureInfo.TwoLetterISOLanguageName = "iv" Then
                Return "Undetermined"
            Else
                Return CultureInfo.EnglishName
            End If
        End Get
    End Property

    Private Shared LanguagesValue As Language()

    Public Shared ReadOnly Property Languages() As Language()
        Get
            If LanguagesValue Is Nothing Then
                Dim l As New List(Of Language)

                For Each i In CultureInfo.GetCultures(CultureTypes.NeutralCultures)
                    l.Add(New Language(i))
                Next

                l.Sort()
                l.Insert(0, New Language("en"))

                If Not g.IsCulture("en") Then
                    l.Insert(0, New Language())
                End If

                l.Insert(0, New Language(""))
                LanguagesValue = l.ToArray
            End If

            Return LanguagesValue
        End Get
    End Property

    Public Shared Function Extract(ByVal value As String) As Language
        For Each i In Languages
            If value.ToUpper.Contains(i.Name.ToUpper) OrElse _
                value.ToUpper.Contains(i.CultureInfo.DisplayName.ToUpper) Then

                Return i
            End If
        Next

        Return Nothing
    End Function

    Public Overrides Function ToString() As String
        Return Name
    End Function

    Public Function CompareTo(ByVal other As Language) As Integer Implements System.IComparable(Of Language).CompareTo
        Return Name.CompareTo(other.Name)
    End Function

    Public Overrides Function Equals(ByVal o As Object) As Boolean
        If TypeOf o Is Language Then
            Return CultureInfo.Equals(DirectCast(o, Language).CultureInfo)
        End If
    End Function
End Class

Public Class CmdlTypeEditor
    Inherits UITypeEditor

    Public Overloads Overrides Function EditValue(ByVal context As ITypeDescriptorContext, ByVal provider As IServiceProvider, ByVal value As Object) As Object
        Using f As New MacroEditor
            f.SetCmdlDefaults()
            f.MacroEditorControl.Value = CStr(value)
            f.MacroEditorControl.RemoveTools()

            If f.ShowDialog = DialogResult.OK Then
                Return f.MacroEditorControl.Value
            Else
                Return value
            End If
        End Using
    End Function

    Public Overloads Overrides Function GetEditStyle(ByVal context As ITypeDescriptorContext) As System.Drawing.Design.UITypeEditorEditStyle
        Return UITypeEditorEditStyle.Modal
    End Function
End Class

Public Class ScriptTypeEditor
    Inherits UITypeEditor

    Public Overloads Overrides Function EditValue(ByVal context As ITypeDescriptorContext, ByVal provider As IServiceProvider, ByVal value As Object) As Object
        Using f As New MacroEditor
            f.SetScriptDefaults()
            f.MacroEditorControl.Value = CStr(value)
            f.MacroEditorControl.RemoveTools()

            If f.ShowDialog = DialogResult.OK Then
                Return f.MacroEditorControl.Value
            Else
                Return value
            End If
        End Using
    End Function

    Public Overloads Overrides Function GetEditStyle(ByVal context As ITypeDescriptorContext) As System.Drawing.Design.UITypeEditorEditStyle
        Return UITypeEditorEditStyle.Modal
    End Function
End Class

Public Class MacroStringTypeEditor
    Inherits UITypeEditor

    Public Overloads Overrides Function EditValue(ByVal context As ITypeDescriptorContext, ByVal provider As IServiceProvider, ByVal value As Object) As Object
        Using f As New MacroEditor
            f.SetMacroDefaults()
            f.MacroEditorControl.Value = CStr(value)
            f.MacroEditorControl.RemoveTools()
            f.AutoResize()

            If f.ShowDialog = DialogResult.OK Then
                Return f.MacroEditorControl.Value
            Else
                Return value
            End If
        End Using
    End Function

    Public Overloads Overrides Function GetEditStyle(ByVal context As ITypeDescriptorContext) As System.Drawing.Design.UITypeEditorEditStyle
        Return UITypeEditorEditStyle.Modal
    End Function
End Class

<Serializable()> _
Public MustInherit Class Profile
    Implements IComparable(Of Profile)

    Public MustOverride Function GetInfoValues() As Dictionary(Of String, String)

    Public Sub New(ByVal profileName As String, ByVal profileTypeName As String)
        Me.Name = profileName
        Me.TypeName = profileTypeName
    End Sub

    Private NameValue As String

    Public Property Name() As String
        Get
            Return NameValue
        End Get
        Set(ByVal Value As String)
            NameValue = Value
        End Set
    End Property

    Private TypeNameValue As String

    Public Property TypeName() As String
        Get
            Return TypeNameValue
        End Get
        Set(ByVal Value As String)
            TypeNameValue = Value
        End Set
    End Property

    Protected CanEditValue As Boolean

    Public Overridable ReadOnly Property CanEdit() As Boolean
        Get
            Return CanEditValue
        End Get
    End Property

    Public Overridable Function Edit() As DialogResult
    End Function

    Public Overridable Function CreateEditControl() As Control
        Return Nothing
    End Function

    Public Sub ShowInfoDialog()
        Dim l As Dictionary(Of String, String) = GetInfoValues()

        Using f As New StatusControlForm
            f.Text = Name
            f.StatusControl.AddIconTitle(Name, Nothing)
            Dim keys As New List(Of String)

            For Each i As String In l.Keys
                keys.Add(i)
            Next

            keys.Sort()

            For Each i As String In keys
                f.StatusControl.AddSection(i, l(i))
            Next

            f.SetHeight()
            f.ShowDialog()
        End Using
    End Sub

    Public Overridable Sub Clean()
    End Sub

    Public Overridable Function GetCopy() As Profile
        Return DirectCast(ObjectHelp.GetCopy(Me), Profile)
    End Function

    Public Overrides Function ToString() As String
        Return Name
    End Function

    Public Function CompareTo(ByVal other As Profile) As Integer Implements System.IComparable(Of Profile).CompareTo
        Return Name.CompareTo(other.Name)
    End Function
End Class

<Serializable(), TypeConverter(GetType(PropertyOrderConverter))> _
Public Class Macro
    Implements IComparable(Of Macro)

    Public Sub New()
        MyClass.New("", "", GetType(String), "")
    End Sub

    Public Sub New(ByVal name As String, ByVal friendlyName As String, ByVal type As Type, ByVal description As String)
        MyClass.New(name, friendlyName, type, Nothing, description)
    End Sub

    Public Sub New(ByVal name As String, ByVal friendlyName As String, ByVal type As Type, ByVal customValue As String, ByVal description As String)
        Me.Name = "%" + name + "%"
        Me.FriendlyName = friendlyName
        Me.Type = type
        Me.Description = description
        Me.CustomValue = customValue
    End Sub

    Private NameValue As String

    <DisplayName("Macro"), Order(1), RefreshProperties(RefreshProperties.All), _
    Description("Name of macro, for example %foo_bar%.")> _
    Public Property Name() As String
        Get
            If NameValue Is Nothing Then
                NameValue = ""
            End If

            Return NameValue
        End Get
        Set(ByVal Value As String)
            If Not Value.StartsWith("%") OrElse Not Value.EndsWith("%") Then
                Throw New Exception("Macro must start and end with '%'")
            End If

            NameValue = Value
        End Set
    End Property

    Private FriendlyNameValue As String

    <DisplayName("Name"), Order(2), _
    Description("Short and friendly name, for example 'Foo Bar'.")> _
    Public Property FriendlyName() As String
        Get
            If Not OK(FriendlyNameValue) AndAlso OK(NameValue) Then
                FriendlyNameValue = NameValue.Replace("_", " ").Replace("%", " ").Trim(" "c).ToTitleCase
            End If

            If FriendlyNameValue Is Nothing Then
                FriendlyNameValue = ""
            End If

            Return FriendlyNameValue
        End Get
        Set(ByVal Value As String)
            FriendlyNameValue = Value
        End Set
    End Property

    Private CustomValueValue As String

    <DisplayName("Value"), Order(3), _
    Description("Returned value, may contain other macros."), _
    Editor(GetType(MacroStringTypeEditor), GetType(UITypeEditor))> _
    Public Property CustomValue() As String
        Get
            If CustomValueValue Is Nothing Then
                CustomValueValue = ""
            End If

            Return CustomValueValue
        End Get
        Set(ByVal Value As String)
            CustomValueValue = Value
        End Set
    End Property

    Private TypeValue As Type

    <Order(4), Description("Return type."), _
    TypeConverter(GetType(BoolStringIntConverter))> _
    Public Property Type() As Type
        Get
            If TypeValue Is Nothing Then
                TypeValue = GetType(String)
            End If

            Return TypeValue
        End Get
        Set(ByVal Value As Type)
            TypeValue = Value
        End Set
    End Property

    Private DescriptionValue As String

    <Order(5), Description("A short description.")> _
    Public Property Description() As String
        Get
            If DescriptionValue Is Nothing Then
                DescriptionValue = ""
            End If

            Return DescriptionValue
        End Get
        Set(ByVal Value As String)
            DescriptionValue = Value
        End Set
    End Property

    <Browsable(False)> _
    Public ReadOnly Property HasParameters() As Boolean
        Get
            Return Name Like "%*:<*>%"
        End Get
    End Property

    Public Function Solve() As String
        Return Solve(Name, True)
    End Function

    Public Shared Function GetTips() As StringPairList
        Dim l As New StringPairList

        For Each i In GetMacrosWithParams()
            l.Add(HelpDocument.ConvertChars(i.Name), i.Description)
        Next

        Return l
    End Function

    Public Shared Function GetTipsFriendly(ByVal convertHTMLChars As Boolean) As StringPairList
        Dim l As New StringPairList

        For Each i As Macro In GetMacros()
            If convertHTMLChars Then
                l.Add(HelpDocument.ConvertChars(i.FriendlyName), i.Description)
            Else
                l.Add(i.FriendlyName, i.Description)
            End If
        Next

        Return l
    End Function

    Public Overrides Function ToString() As String
        Return Name
    End Function

    Public Function CompareTo(ByVal other As Macro) As Integer Implements System.IComparable(Of Macro).CompareTo
        Return Name.CompareTo(other.Name)
    End Function

    Public Shared Function GetMacrosWithParams() As List(Of Macro)
        Dim ret = GetMacros()

        ret.Add(New Macro("enter_text:<prompt>", "Enter Text (Params)", GetType(String), "Text entered in a input box."))
        ret.Add(New Macro("eval:<expression>", "Eval Math Expression", GetType(String), "Evaluates a math expression which may contain default macros."))
        ret.Add(New Macro("eval_js:<expression>", "Eval JScript Expression", GetType(String), "Evaluates a JScript expression which may contain default macros."))
        ret.Add(New Macro("eval_vb:<expression>", "Eval VBScript Expression", GetType(String), "Evaluates a VBScript expression which may contain default macros."))
        ret.Add(New Macro("filter:<name>", "Filter", GetType(String), "Returns the script code of a filter of the active project that matches the specified name."))
        ret.Add(New Macro("select:<param1;param2;...>", "Select", GetType(String), "String selected from dropdown."))

        Dim names As New List(Of String)

        For Each i As Package In Packs.Packages.Values
            names.Add(i.Name)
        Next

        ret.Add(New Macro("application:<name>", "Application File Path", GetType(String), "Returns the path of a aplication. Possible names are: " + String.Join(", ", names.ToArray)))
        ret.Add(New Macro("application_dir:<name>", "Application Directory", GetType(String), "Returns the directory of a aplication. Possible names are: " + String.Join(", ", names.ToArray)))

        Return ret
    End Function

    Public Shared Function GetMacros() As List(Of Macro)
        Dim ret As New List(Of Macro)

        ret.Add(New Macro("audio_bitrate", "Audio Bitrate", GetType(Integer), "Overall audio bitrate."))
        ret.Add(New Macro("audio_file1", "Audio File 1", GetType(String), "File path of the first audio file."))
        ret.Add(New Macro("audio_file2", "Audio File 2", GetType(String), "File path of the second audio file."))
        ret.Add(New Macro("browse_file", "Browse For File", GetType(String), "Filepath returned from a file browser."))
        ret.Add(New Macro("compressibility", "Compressibility", GetType(Integer), "Compressibility value."))
        ret.Add(New Macro("crop_bottom", "Crop Bottom", GetType(Integer), "Bottom crop value."))
        ret.Add(New Macro("crop_height", "Crop Height", GetType(Integer), "Crop height."))
        ret.Add(New Macro("crop_left", "Crop Left", GetType(Integer), "Left crop value."))
        ret.Add(New Macro("crop_right", "Crop Right", GetType(Integer), "Right crop value."))
        ret.Add(New Macro("crop_top", "Crop Top", GetType(Integer), "Top crop value."))
        ret.Add(New Macro("crop_width", "Crop Width", GetType(Integer), "Crop width."))
        ret.Add(New Macro("delay", "Audio Delay 1", GetType(Integer), "Audio delay of the first audio track."))
        ret.Add(New Macro("delay1", "Audio Delay 1", GetType(Integer), "Audio delay of the first audio track."))
        ret.Add(New Macro("delay2", "Audio Delay 2", GetType(Integer), "Audio delay of the second audio track."))
        ret.Add(New Macro("download_dir", "Download Directory", GetType(String), "Directory where StaxRip saves downloaded applications."))
        ret.Add(New Macro("encoder_ext", "Encoder Extension", GetType(String), "File extension of the format the encoder of the active project outputs."))
        ret.Add(New Macro("encoder_out_file", "Encoder Output File", GetType(String), "Output file of the video encoder."))
        ret.Add(New Macro("encoder_out_file_track1", "Encoder Output File Track 1", GetType(String), "Audio output file of the first track."))
        ret.Add(New Macro("encoder_out_file_track2", "Encoder Output File Track 2", GetType(String), "Audio output file of the second track."))
        ret.Add(New Macro("enter_text", "Enter Text", GetType(String), "Text entered in a input box."))
        ret.Add(New Macro("install_dir", "Install Directory", GetType(String), "Directory where StaxRip installs downloaded applications."))
        ret.Add(New Macro("plugin_dir", "AviSynth Plugin Directory", GetType(String), "AviSynth plugin directory."))
        ret.Add(New Macro("pos_frame", "Position In Frames", GetType(Integer), "Current preview position in frames."))
        ret.Add(New Macro("pos_ms", "Position In Millisecons", GetType(Integer), "Current preview position in milliseconds."))
        ret.Add(New Macro("programs_dir", "Programs Directory", GetType(String), "Programs system directory."))
        ret.Add(New Macro("sel_end", "Selection End", GetType(Integer), "End position of the first selecion in the preview."))
        ret.Add(New Macro("sel_start", "Selection Start", GetType(Integer), "Start position of the first selecion in the preview."))
        ret.Add(New Macro("settings_dir", "Settings Directory", GetType(String), "Path of the settings direcory."))
        ret.Add(New Macro("source_dir", "Source Directory", GetType(String), "Directory of the source file."))
        ret.Add(New Macro("source_dir_name", "Source Directory Name", GetType(String), "Name of the source file directory."))
        ret.Add(New Macro("source_dir_parent", "Source Directory Parent", GetType(String), "Parent directory of the source file directory."))
        ret.Add(New Macro("source_file", "Source File Path", GetType(String), "File path of the source video."))
        ret.Add(New Macro("source_files_blank", "Source Files Blank", GetType(String), "Source files in quotes separated by a blank."))
        ret.Add(New Macro("source_files_comma", "Source Files Comma", GetType(String), "Source files in quotes separated by comma."))
        ret.Add(New Macro("source_files_comma_no_quotes", "Source Files Comma No Quotes", GetType(String), "Source files separated by comma."))
        ret.Add(New Macro("source_framerate", "Source Framerate", GetType(Integer), "Frame rate of the source video."))
        ret.Add(New Macro("source_frames", "Source Frames", GetType(Integer), "Length in frames of the source video."))
        ret.Add(New Macro("source_height", "Source Image Height", GetType(Integer), "Image height of the source video."))
        ret.Add(New Macro("source_name", "Source File Name", GetType(String), "The name of the source file."))
        ret.Add(New Macro("source_seconds", "Source Seconds", GetType(Integer), "Length in seconds of the source video."))
        ret.Add(New Macro("source_width", "Source Image Width", GetType(Integer), "Image width of the source video."))
        ret.Add(New Macro("startup_dir", "Startup Directory", GetType(String), "Directory of the application."))
        ret.Add(New Macro("system_dir", "System Directory", GetType(String), "System directory."))
        ret.Add(New Macro("target_dir", "Target Directory", GetType(String), "Directory of the target file."))
        ret.Add(New Macro("target_file", "Target File Path", GetType(String), "File path of the target file."))
        ret.Add(New Macro("target_framerate", "Target Framerate", GetType(Integer), "Frame rate of the target video."))
        ret.Add(New Macro("target_frames", "Target Frames", GetType(Integer), "Length in frames of the target video."))
        ret.Add(New Macro("target_height", "Target Image Height", GetType(Integer), "Image height of the target video."))
        ret.Add(New Macro("target_name", "Target File Name", GetType(String), "Name of the target file."))
        ret.Add(New Macro("target_seconds", "Target Seconds", GetType(Integer), "Length in seconds of the target video."))
        ret.Add(New Macro("target_size", "Target Size", GetType(Integer), "Size of the target video in kilo bytes."))
        ret.Add(New Macro("target_width", "Target Image Width", GetType(Integer), "Image width of the target video."))
        ret.Add(New Macro("template_name", "Template Name", GetType(String), "Name of the template the active project is based on."))
        ret.Add(New Macro("version", "Version", GetType(String), "StaxRip version."))
        ret.Add(New Macro("video_bitrate", "Video Bitrate", GetType(Integer), "Video bitrate"))
        ret.Add(New Macro("working_dir", "Working Directory", GetType(String), "Directory of the source file or the temp directory if enabled."))

        ret.AddRange(s.CustomMacros)
        ret.Sort()

        Return ret
    End Function

    Public Shared Function Solve(ByVal value As String) As String
        Return Solve(value, False)
    End Function

    Public Shared Function Solve(ByVal value As String, ByVal silent As Boolean) As String
        Return Solve(value, True, silent)
    End Function

    Public Shared Function Solve(ByVal value As String, ByVal rekursive As Boolean, ByVal silent As Boolean) As String
        If value Is Nothing Then Return ""
        If Not value.Contains("%") Then Return value

        If value.Contains("%version%") Then value = value.Replace("%version%", Application.ProductVersion)
        If Not value.Contains("%") Then Return value

        If value.Contains("%source_width%") Then value = value.Replace("%source_width%", p.SourceWidth.ToString)
        If Not value.Contains("%") Then Return value

        If value.Contains("%source_height%") Then value = value.Replace("%source_height%", p.SourceHeight.ToString)
        If Not value.Contains("%") Then Return value

        If value.Contains("%source_seconds%") Then value = value.Replace("%source_seconds%", p.SourceSeconds.ToString)
        If Not value.Contains("%") Then Return value

        If value.Contains("%source_frames%") Then value = value.Replace("%source_frames%", p.SourceFrames.ToString)
        If Not value.Contains("%") Then Return value

        If value.Contains("%source_framerate%") Then value = value.Replace("%source_framerate%", p.SourceFramerate.ToString("f2", CultureInfo.InvariantCulture))
        If Not value.Contains("%") Then Return value

        If value.Contains("%source_file%") Then value = value.Replace("%source_file%", p.SourceFile)
        If Not value.Contains("%") Then Return value

        'legacy 1.0.0.4
        If value.Contains("%source_temp_dir%") Then
            value = value.Replace("%source_temp_dir%", "%source_dir%")
            Msg.Show("The macro '%source_temp_dir%' was removed, please use '%source_dir%' instead.", MsgIcon.Warning)
        End If

        If value.Contains("%working_dir%") Then value = value.Replace("%working_dir%", g.TempDir)
        If Not value.Contains("%") Then Return value

        If value.Contains("%source_dir%") Then value = value.Replace("%source_dir%", Filepath.GetDir(p.SourceFile))
        If Not value.Contains("%") Then Return value

        If value.Contains("%source_dir_parent%") Then value = value.Replace("%source_dir_parent%", DirPath.GetParent(Filepath.GetDir(p.SourceFile)))
        If Not value.Contains("%") Then Return value

        If value.Contains("%source_dir_name%") Then value = value.Replace("%source_dir_name%", DirPath.GetName(Filepath.GetDir(p.SourceFile)))
        If Not value.Contains("%") Then Return value

        If value.Contains("%source_name%") Then value = value.Replace("%source_name%", Filepath.GetBase(p.SourceFile))
        If Not value.Contains("%") Then Return value

        If value.Contains("%target_width%") Then value = value.Replace("%target_width%", p.TargetWidth.ToString)
        If Not value.Contains("%") Then Return value

        If value.Contains("%target_height%") Then value = value.Replace("%target_height%", p.TargetHeight.ToString)
        If Not value.Contains("%") Then Return value

        If value.Contains("%target_seconds%") Then value = value.Replace("%target_seconds%", p.TargetSeconds.ToString)
        If Not value.Contains("%") Then Return value

        If value.Contains("%target_frames%") Then value = value.Replace("%target_frames%", p.TargetAVSDoc.GetFrames.ToString)
        If Not value.Contains("%") Then Return value

        If value.Contains("%target_framerate%") Then value = value.Replace("%target_framerate%", p.TargetAVSDoc.GetFramerate.ToString("f2"))
        If Not value.Contains("%") Then Return value

        If value.Contains("%target_size%") Then value = value.Replace("%target_size%", (p.Size * 1024).ToString)
        If Not value.Contains("%") Then Return value

        If value.Contains("%target_file%") Then value = value.Replace("%target_file%", p.TargetFile)
        If Not value.Contains("%") Then Return value

        If value.Contains("%target_dir%") Then value = value.Replace("%target_dir%", Filepath.GetDir(p.TargetFile))
        If Not value.Contains("%") Then Return value

        If value.Contains("%target_name%") Then value = value.Replace("%target_name%", p.Name)
        If Not value.Contains("%") Then Return value

        If value.Contains("%crop_width%") Then value = value.Replace("%crop_width%", (p.SourceWidth - p.CropLeft - p.CropRight).ToString)
        If Not value.Contains("%") Then Return value

        If value.Contains("%crop_height%") Then value = value.Replace("%crop_height%", (p.SourceHeight - p.CropTop - p.CropBottom).ToString)
        If Not value.Contains("%") Then Return value

        If value.Contains("%crop_left%") Then value = value.Replace("%crop_left%", p.CropLeft.ToString)
        If Not value.Contains("%") Then Return value

        If value.Contains("%crop_top%") Then value = value.Replace("%crop_top%", p.CropTop.ToString)
        If Not value.Contains("%") Then Return value

        If value.Contains("%crop_right%") Then value = value.Replace("%crop_right%", p.CropRight.ToString)
        If Not value.Contains("%") Then Return value

        If value.Contains("%crop_bottom%") Then value = value.Replace("%crop_bottom%", p.CropBottom.ToString)
        If Not value.Contains("%") Then Return value

        If value.Contains("%video_bitrate%") Then value = value.Replace("%video_bitrate%", p.VideoBitrate.ToString)
        If Not value.Contains("%") Then Return value

        If value.Contains("%audio_bitrate%") Then value = value.Replace("%audio_bitrate%", (p.Audio0.Bitrate + p.Audio1.Bitrate).ToString)
        If Not value.Contains("%") Then Return value

        If value.Contains("%audio_file1%") Then value = value.Replace("%audio_file1%", p.Audio0.File)
        If Not value.Contains("%") Then Return value

        If value.Contains("%audio_file2%") Then value = value.Replace("%audio_file2%", p.Audio1.File)
        If Not value.Contains("%") Then Return value

        If value.Contains("%delay%") Then value = value.Replace("%delay%", p.Audio0.Delay.ToString)
        If Not value.Contains("%") Then Return value

        If value.Contains("%delay1%") Then value = value.Replace("%delay1%", p.Audio0.Delay.ToString)
        If Not value.Contains("%") Then Return value

        If value.Contains("%delay2%") Then value = value.Replace("%delay2%", p.Audio1.Delay.ToString)
        If Not value.Contains("%") Then Return value

        If value.Contains("%startup_dir%") Then value = value.Replace("%startup_dir%", CommonDirs.Startup)
        If Not value.Contains("%") Then Return value

        If value.Contains("%system_dir%") Then value = value.Replace("%system_dir%", CommonDirs.System)
        If Not value.Contains("%") Then Return value

        If value.Contains("%programs_dir%") Then value = value.Replace("%programs_dir%", CommonDirs.Programs)
        If Not value.Contains("%") Then Return value

        'legacy 1.0.0.4
        If value.Contains("%avisynth_plugin_dir%") Then
            value = value.Replace("%avisynth_plugin_dir%", "%plugin_dir%")
            Msg.Show("The macro '%avisynth_plugin_dir%' was removed, please use '%plugin_dir%' instead.", MsgIcon.Warning)
        End If

        If value.Contains("%plugin_dir%") Then value = value.Replace("%plugin_dir%", Paths.AviSynthPluginsDir)
        If Not value.Contains("%") Then Return value

        If value.Contains("%source_files_comma%") Then value = value.Replace("%source_files_comma%", """" + String.Join(""",""", p.SourceFiles.ToArray) + """")
        If Not value.Contains("%") Then Return value

        If value.Contains("%source_files_comma_no_quotes%") Then value = value.Replace("%source_files_comma_no_quotes%", String.Join(",", p.SourceFiles.ToArray))
        If Not value.Contains("%") Then Return value

        If value.Contains("%source_files_blank%") Then value = value.Replace("%source_files_blank%", """" + String.Join(""" """, p.SourceFiles.ToArray) + """")
        If Not value.Contains("%") Then Return value

        If value.Contains("%install_dir%") Then value = value.Replace("%install_dir%", Paths.ExtractDir)
        If Not value.Contains("%") Then Return value

        If value.Contains("%download_dir%") Then value = value.Replace("%download_dir%", Paths.DownloadDir)
        If Not value.Contains("%") Then Return value

        If value.Contains("%compressibility%") Then value = value.Replace("%compressibility%", Math.Round(p.Compressibility, 3).ToString.Replace(",", "."))
        If Not value.Contains("%") Then Return value

        If value.Contains("\\") Then value = value.Replace("\\", "\")
        If Not value.Contains("%") Then Return value

        If value.Contains("%encoder_out_file%") Then value = value.Replace("%encoder_out_file%", p.VideoEncoder.OutputPath)
        If Not value.Contains("%") Then Return value

        If value.Contains("%encoder_out_file_track1%") Then value = value.Replace("%encoder_out_file_track1%", p.Audio0.File)
        If Not value.Contains("%") Then Return value

        If value.Contains("%encoder_out_file_track2%") Then value = value.Replace("%encoder_out_file_track2%", p.Audio1.File)
        If Not value.Contains("%") Then Return value

        If value.Contains("%encoder_ext%") Then value = value.Replace("%encoder_ext%", p.VideoEncoder.GetExtension())
        If Not value.Contains("%") Then Return value

        If value.Contains("%pos_frame%") Then value = value.Replace("%pos_frame%", s.LastPosition.ToString)
        If Not value.Contains("%") Then Return value

        If value.Contains("%template_name%") Then value = value.Replace("%template_name%", p.TemplateName)
        If Not value.Contains("%") Then Return value

        If value.Contains("%settings_dir%") Then value = value.Replace("%settings_dir%", Paths.SettingsDir)
        If Not value.Contains("%") Then Return value

        If Not silent Then
            If value.Contains("%browse_file%") Then value = value.Replace("%browse_file%", Misc.BrowseFile)
            If Not value.Contains("%") Then Return value

            If value.Contains("%enter_text%") Then value = value.Replace("%enter_text%", InputBox.Show("Please enter some text."))
            If Not value.Contains("%") Then Return value

            If value.Contains("%enter_text:") Then
                Dim mc = Regex.Matches(value, "%enter_text:(.+?)%")

                For Each i As Match In mc
                    value = value.Replace(i.Value, InputBox.Show(i.Groups(1).Value))
                    If Not value.Contains("%") Then Return value
                Next
            End If

            If value.Contains("%select:") Then
                Dim mc = Regex.Matches(value, "%select:(.+?)%")

                For Each i As Match In mc
                    Dim items = i.Groups(1).Value.Split(";")

                    If items.Length > 0 Then
                        Dim f As New SelectionBox(Of String)
                        f.Text = "Please select a item."
                        f.Title = "Select"

                        For Each i2 As String In items
                            f.AddItem(i2)
                        Next

                        If f.Show = DialogResult.OK Then
                            value = value.Replace(i.Value, f.SelectedItem)
                        End If

                        If Not value.Contains("%") Then Return value
                    End If
                Next
            End If
        End If

        If p.Ranges.Count > 0 Then
            If value.Contains("%sel_start%") Then value = value.Replace("%sel_start%", p.Ranges(0).Start.ToString)
            If Not value.Contains("%") Then Return value

            If value.Contains("%sel_end%") Then value = value.Replace("%sel_end%", p.Ranges(0).End.ToString)
            If Not value.Contains("%") Then Return value
        Else
            If value.Contains("%sel_start%") Then value = value.Replace("%sel_start%", 0.ToString)
            If Not value.Contains("%") Then Return value

            If value.Contains("%sel_end%") Then value = value.Replace("%sel_end%", 0.ToString)
            If Not value.Contains("%") Then Return value
        End If

        If value.Contains("%pos_ms%") Then
            Dim r As Double = p.TargetAVSDoc.GetFramerate
            If r = 0 Then r = 25
            value = value.Replace("%pos_ms%", CInt((s.LastPosition / r) * 1000).ToString)
        End If

        If Not value.Contains("%") Then Return value

        If value.Contains("%application:") Then
            Dim mc = Regex.Matches(value, "%application:(.+?)%")

            For Each i As Match In mc
                If Packs.Packages.ContainsKey(i.Groups(1).Value) Then
                    Dim pack = Packs.Packages(i.Groups(1).Value)

                    If pack.VerifyOK(True) Then
                        Dim path = pack.Path

                        If OK(path) Then
                            value = value.Replace(i.Value, path)
                            If Not value.Contains("%") Then Return value
                        End If
                    End If
                End If
            Next
        End If

        If value.Contains("%filter:") Then
            Dim mc = Regex.Matches(value, "%filter:(.+?)%")

            For Each i As Match In mc
                For Each i2 In p.TargetAVSDoc.Filters
                    If i2.Active AndAlso i2.Name.ToUpper = i.Groups(1).Value.ToUpper Then
                        value = value.Replace(i.Value, i2.Script)
                        If Not value.Contains("%") Then Return value
                        Exit For
                    End If
                Next

                value = value.Replace(i.Value, "")
            Next
        End If

        If value.Contains("%application_dir:") Then
            Dim mc As MatchCollection = Regex.Matches(value, "%application_dir:(.+?)%")

            For Each i As Match In mc
                If Packs.Packages.ContainsKey(i.Groups(1).Value) Then
                    Dim pack = Packs.Packages(i.Groups(1).Value)

                    If pack.VerifyOK(True) Then
                        Dim path = pack.Path

                        If OK(path) Then
                            value = value.Replace(i.Value, Filepath.GetDir(path))
                            If Not value.Contains("%") Then Return value
                        End If
                    End If
                End If
            Next
        End If

        If value.Contains("%eval:") Then
            Dim mc = Regex.Matches(value, "%eval:(.+?)%")

            For Each i As Match In mc
                Try
                    value = value.Replace(i.Value, Misc.Eval(i.Groups(1).Value).ToString)

                    If Not value.Contains("%") Then
                        Return value
                    End If
                Catch ex As Exception
                    Msg.Show("Failed to solve macro '" + i.Value + "': " + ex.Message, MsgIcon.Warning)
                End Try
            Next
        End If

        If value.Contains("%eval_vb:") Then
            Dim mc = Regex.Matches(value, "%eval_vb:(.+?)%")

            For Each i As Match In mc
                Try
                    value = value.Replace(i.Value, WindowsScript.EvalVB(i.Groups(1).Value).ToString)

                    If Not value.Contains("%") Then
                        Return value
                    End If
                Catch ex As Exception
                    Msg.Show("Failed to solve macro '" + i.Value + "': " + ex.Message, MsgIcon.Warning)
                End Try
            Next
        End If

        If value.Contains("%eval_js:") Then
            Dim mc = Regex.Matches(value, "%eval_js:(.+?)%")

            For Each i As Match In mc
                Try
                    value = value.Replace(i.Value, WindowsScript.EvalJS(i.Groups(1).Value).ToString)

                    If Not value.Contains("%") Then
                        Return value
                    End If
                Catch ex As Exception
                    Msg.Show("Failed to solve macro '" + i.Value + "': " + ex.Message, MsgIcon.Warning)
                End Try
            Next
        End If

        For Each i In s.CustomMacros
            If OK(i.Name) AndAlso value.Contains(i.Name) Then
                If rekursive Then
                    value = value.Replace(i.Name, Solve(i.CustomValue, False))
                Else
                    value = value.Replace(i.Name, i.CustomValue)
                End If

                If Not value.Contains("%") Then Return value
            End If
        Next

        Return value
    End Function
End Class

'legacy 1.1.0.1
Public Enum StreamType
    <Extension("")> Ignore
    <DisplayInfo("Legacy")> MP3CBR
    <DisplayInfo("Legacy")> MP3VBR
    <DisplayInfo("Legacy")> AAC_LC_MP4
    <DisplayInfo("Legacy")> AAC_LC_RAW
    <DisplayInfo("Legacy")> AAC_HE_MP4
    <DisplayInfo("Legacy")> AAC_HE_RAW
    AC3
    WAV
    AVI
    MP4
    MPA
    MP2
    <DisplayInfo("RAW 264")> <Extension("264")> RAW264
    MKV
    <DisplayInfo("DIVX"), Extension("divx")> DMF
    <Extension("ogg")> Vorbis
    M4V
    DTS
    PMP
    MPG
    AAC
    MP3
End Enum

<AttributeUsage(AttributeTargets.Field)> _
Public Class ExtensionAttribute
    Inherits Attribute

    Public Sub New(ByVal value As String)
        If Not value.StartsWith(".") Then
            value = "." + value
        End If

        ValueValue = value.ToLowerInvariant
    End Sub

    Private ValueValue As String

    Public ReadOnly Property Value() As String
        Get
            Return ValueValue
        End Get
    End Property
End Class

Public Class StreamTypeUtils
    Public Shared Function GetStreamType(ByVal ext As String) As StreamType
        For Each i As StreamType In [Enum].GetValues(GetType(StreamType))
            If ext = GetExtension(i) Then
                Return i
            End If
        Next

        If ext = ".h264" Then
            Return StreamType.RAW264
        End If

        Return StreamType.Ignore
    End Function

    Public Shared Function GetExtension(ByVal type As StreamType) As String
        Dim fi = GetType(StreamType).GetField(type.ToString)
        Dim attribs = fi.GetCustomAttributes(GetType(ExtensionAttribute), True)

        If attribs.Length = 0 Then
            Return "." + type.ToString.ToLowerInvariant
        Else
            Return DirectCast(attribs(0), ExtensionAttribute).Value
        End If
    End Function

    Public Shared Function GetNames(ByVal types As StreamType()) As String
        Dim l As New List(Of String)

        For Each i In types
            Dim fi = GetType(StreamType).GetField(i.ToString)
            Dim a = fi.GetCustomAttributes(GetType(DisplayInfoAttribute), True)

            If a.Length > 0 Then
                l.Add(DirectCast(a(0), DisplayInfoAttribute).DisplayName)
            Else
                l.Add(i.ToString)
            End If
        Next

        Return String.Join(", ", l.ToArray)
    End Function
End Class

<Serializable()> _
Public Class ObjectStorage
    Private StringDictionary As New Dictionary(Of String, String)
    Private IntDictionary As New Dictionary(Of String, Integer)

    Private BoolDictionaryValue As Dictionary(Of String, Boolean)

    Public ReadOnly Property BoolDictionary() As Dictionary(Of String, Boolean)
        Get
            If BoolDictionaryValue Is Nothing Then
                BoolDictionaryValue = New Dictionary(Of String, Boolean)
            End If

            Return BoolDictionaryValue
        End Get
    End Property

    Public Function GetBool(ByVal key As String) As Boolean
        Return GetBool(key, False)
    End Function

    Public Function GetBool(ByVal key As String, ByVal defaultValue As Boolean) As Boolean
        If BoolDictionary.ContainsKey(key) Then
            Return BoolDictionary(key)
        End If

        Return defaultValue
    End Function

    Public Sub SetBool(ByVal key As String, ByVal Value As Boolean)
        BoolDictionary(key) = Value
    End Sub

    Public Function GetInt(ByVal key As String) As Integer
        Return GetInt(key, 0)
    End Function

    Public Function GetInt(ByVal key As String, ByVal defaultValue As Integer) As Integer
        If IntDictionary.ContainsKey(key) Then
            Return IntDictionary(key)
        End If

        Return defaultValue
    End Function

    Public Sub SetInt(ByVal key As String, ByVal value As Integer)
        IntDictionary(key) = value
    End Sub

    Public Function GetString(ByVal key As String) As String
        Return GetString(key, Nothing)
    End Function

    Public Function GetString(ByVal key As String, ByVal defaultValue As String) As String
        If StringDictionary.ContainsKey(key) Then
            Return StringDictionary(key)
        End If

        Return defaultValue
    End Function

    Public Sub SetString(ByVal key As String, ByVal value As String)
        StringDictionary(key) = value
    End Sub
End Class

<Serializable()> _
Public Class Subtitle
    Public Sub New()
    End Sub

    Public Sub New(ByVal l As Language)
        Language = l
    End Sub

    Public Path As String = ""
    Public Language As New Language("")
    Public Name As String = ""
    Public IndexData As Integer
    Public IndexIDX As Integer

    Public Shared Function Create(ByVal path As String) As List(Of Subtitle)
        Dim r As New List(Of Subtitle)

        If Filepath.GetExt(path) = ".idx" Then
            Dim indexData As Integer
            Dim s As Subtitle = Nothing

            For Each i In StringHelp.ReadFile(path).SplitLines
                If i.StartsWith("id: ") AndAlso i Like "id: ??, index: *" Then
                    s = New Subtitle

                    Try
                        s.Language = New Language(New CultureInfo(i.Substring(4, 2)))
                    Catch ex As Exception
                        Return New List(Of Subtitle)
                    End Try

                    s.IndexIDX = CInt(Regex.Match(i, ", index: (\d+)").Groups(1).Value)
                End If

                If Not s Is Nothing AndAlso i.StartsWith("timestamp: ") Then
                    s.IndexData = indexData
                    s.Path = g.GetRelativePath(path)
                    indexData += 1
                    r.Add(s)
                    s = Nothing
                End If
            Next
        Else
            Dim l = Language.Extract(path)

            If l Is Nothing Then
                l = New Language()
            End If

            Dim s As New Subtitle(l)
            s.Path = g.GetRelativePath(path)

            r.Add(s)
        End If

        Return r
    End Function
End Class

<Serializable()> _
Public Class x264Params
    Implements ISerializable

    Public AdaptiveDCT As New SettingBag(Of Boolean)(False)
    Public AddAll As New SettingBag(Of String)("")
    Public AQMode As New SettingBag(Of Integer)(1)
    Public AQStrength As New SettingBag(Of Integer)(10)
    Public B8x8 As New SettingBag(Of Boolean)(True)
    Public BAdapt As New SettingBag(Of Integer)(1)
    Public BFrames As New SettingBag(Of Integer)(4)
    Public BFramesBias As New SettingBag(Of Integer)(0)
    Public BFramesPyramid As New SettingBag(Of Boolean)(False)
    Public BFramesWeightedPrediction As New SettingBag(Of Boolean)(False)
    Public CABAC As New SettingBag(Of Boolean)(True)
    Public DeblockAlpha As New SettingBag(Of Integer)(0)
    Public DeblockBeta As New SettingBag(Of Integer)(0)
    Public DirectMode As New SettingBag(Of Integer)(x264DirectMode.spatial)
    Public GOPSizeMax As New SettingBag(Of Integer)(250)
    Public GOPSizeMin As New SettingBag(Of Integer)(25)
    Public I4x4 As New SettingBag(Of Boolean)(True)
    Public I8x8 As New SettingBag(Of Boolean)(False)
    Public IPRatio As New SettingBag(Of Single)(1.4)
    Public Level As New SettingBag(Of Integer)(x264LevelMode.Unrestricted)
    Public LoopFilter As New SettingBag(Of Boolean)(True)
    Public MEMethod As New SettingBag(Of Integer)(MEMethodMode.hex)
    Public MeRange As New SettingBag(Of Integer)(16)
    Public MixedRefs As New SettingBag(Of Boolean)(False)
    Public Mode As New SettingBag(Of Integer)(x264Mode.SingleQuant)
    Public NoChromaMe As New SettingBag(Of Boolean)(False)
    Public NoDCTDecimate As New SettingBag(Of Boolean)(False)
    Public NoFastPSkip As New SettingBag(Of Boolean)(False)
    Public NoPSNR As New SettingBag(Of Boolean)(True)
    Public NoSSIM As New SettingBag(Of Boolean)(True)
    Public P4x4 As New SettingBag(Of Boolean)(True)
    Public P8x8 As New SettingBag(Of Boolean)(True)
    Public PBRatio As New SettingBag(Of Single)(1.3)
    Public Progress As New SettingBag(Of Boolean)(True)
    Public PsyRD As New SettingBag(Of Single)(1.0)
    Public PsyTrellis As New SettingBag(Of Single)(0.0)
    Public QComp As New SettingBag(Of Single)(0.6)
    Public QPMin As New SettingBag(Of Integer)(10)
    Public Quant As New SettingBag(Of Single)(22)
    Public QuantCompCheck As New SettingBag(Of Integer)(18)
    Public ReferenceFrames As New SettingBag(Of Integer)(4)
    Public SceneCut As New SettingBag(Of Integer)(40)
    Public SubME As New SettingBag(Of Integer)(x264SubMEMode.V6)
    Public ThreadInput As New SettingBag(Of Boolean)(True)
    Public Threads As New SettingBag(Of Integer)(0)
    Public Trellis As New SettingBag(Of Integer)(x264TrellisMode.Disabled)
    Public TurboAdd As New SettingBag(Of String)("--subme 2 --partitions none --me dia")
    Public TurboRemove As New SettingBag(Of String)("--ref --mixed-refs --subme --trellis --partitions --me --8x8dct --no-fast-pskip")
    Public VBVBufSize As New SettingBag(Of Integer)(0)
    Public VBVInit As New SettingBag(Of Single)(0.9)
    Public VBVMaxRate As New SettingBag(Of Integer)(0)

    Public Sub New()
    End Sub

    <DebuggerNonUserCode()> _
    Public Sub New(ByVal info As SerializationInfo, ByVal context As StreamingContext)
        For Each i In Me.GetType.GetFields()
            Try
                i.SetValue(Me, info.GetValue(i.Name, i.FieldType))
            Catch
            End Try
        Next
    End Sub

    Public Sub GetObjectData(ByVal info As SerializationInfo, ByVal context As StreamingContext) Implements ISerializable.GetObjectData
        For Each i In Me.GetType.GetFields()
            info.AddValue(i.Name, i.GetValue(Me))
        Next
    End Sub
End Class

Public Enum x264Mode
    <DisplayInfo("Bitrate")> _
    SingleBitrate
    <DisplayInfo("Quantizer")> _
    SingleQuant
    <DisplayInfo("Quality")> _
    SingleCRF
    <DisplayInfo("Two Pass")> _
    TwoPass
    <DisplayInfo("Three Pass")> _
    ThreePass
End Enum

Public Enum x264TrellisMode
    Disabled
    <DisplayInfo("Final MB")> _
    FinalMB
    Always
End Enum

Public Enum x264SubMEMode
    <DisplayInfo("1 - QPel 1 iteration")> _
    V1 = 1
    <DisplayInfo("2 - QPel 2 iterations")> _
    V2
    <DisplayInfo("3 - HPel on MB then QPel")> _
    V3
    <DisplayInfo("4 - Always QPel")> _
    V4
    <DisplayInfo("5 - Qpel + Bidir ME")> _
    V5
    <DisplayInfo("6 - RD on I/P frames")> _
    V6
    <DisplayInfo("7 - RD on all frames")> _
    V7
    <DisplayInfo("8 - RD refinement on I/P frames")> _
    V8
    <DisplayInfo("9 - RD refinement on all frames")> _
    V9
End Enum

Public Enum x264AQMode
    Disabled
    <DisplayInfo("Variance AQ")> Variance
End Enum

Public Enum x264DirectMode
    <DisplayInfo("None")> _
    none
    <DisplayInfo("Spatial")> _
    spatial
    <DisplayInfo("Temporal")> _
    temporal
    <DisplayInfo("Auto")> _
    auto
End Enum

Public Enum MEMethodMode
    <DisplayInfo("Diamond")> _
    dia
    <DisplayInfo("Hexagon")> _
    hex
    <DisplayInfo("Multi Hex")> _
    umh
    <DisplayInfo("Exhaustive")> _
    esa
    <DisplayInfo("SATD Exhaustive")> _
    tesa
End Enum

Public Enum x264LevelMode
    <DisplayInfo("1")> L1
    <DisplayInfo("1.1")> L11
    <DisplayInfo("1.2")> L12
    <DisplayInfo("1.3")> L13
    <DisplayInfo("2")> L2
    <DisplayInfo("2.1")> L21
    <DisplayInfo("2.2")> L22
    <DisplayInfo("3")> L3
    <DisplayInfo("3.1")> L31
    <DisplayInfo("3.2")> L32
    <DisplayInfo("4")> L4
    <DisplayInfo("4.1")> L41
    <DisplayInfo("4.2")> L42
    <DisplayInfo("5")> L5
    <DisplayInfo("5.1")> L51
    <DisplayInfo("Unrestricted")> Unrestricted
End Enum

Public Enum x264BAdaptMode
    Disabled
    Fast
    Optimal
End Enum

Public Class ScriptEditorTreeNodeSetting
    Inherits TextBoxButtonTreeNodeSetting

    Public Sub New(ByVal path As String, ByVal text As String, ByVal setting As SettingBag(Of String), ByVal description As String)
        MyBase.New(path, text, setting, description)
        ImageKey = "TextBox16"
    End Sub

    Protected Overrides Sub ButtonClick()
        Using f As New MacroEditor
            f.SetScriptDefaults()
            f.MacroEditorControl.RemoveTools()
            f.MacroEditorControl.Value = Value

            If f.ShowDialog() = DialogResult.OK Then
                TextBox.Text = f.MacroEditorControl.Value
            End If
        End Using
    End Sub
End Class

Public Class CmdlEditorTreeNodeSetting
    Inherits TextBoxButtonTreeNodeSetting

    Public Sub New(ByVal path As String, ByVal text As String, ByVal setting As SettingBag(Of String), ByVal description As String)
        MyBase.New(path, text, setting, description)
        ImageKey = "TextBox16"
    End Sub

    Protected Overrides Sub ButtonClick()
        Using f As New MacroEditor
            f.SetCmdlDefaults()
            f.MacroEditorControl.RemoveTools()
            f.MacroEditorControl.Value = Value

            If f.ShowDialog() = DialogResult.OK Then
                TextBox.Text = f.MacroEditorControl.Value
            End If
        End Using
    End Sub
End Class

Public Enum CompCheckAction
    [Nothing]
    <DisplayInfo("image size")> AdjustImageSize
    <DisplayInfo("file size")> AdjustFileSize
End Enum

Public Class GlobalCommands
    <Command("Perform|Execute Command Line", "Executes command lines separated by a line break line by line.")> _
    Public Sub ExecuteCmdl( _
        <DisplayInfo("Command Line"), _
        Description("One or more command lines to be executed. " + Strings.CommandLine), _
        Editor(GetType(CmdlTypeEditor), GetType(UITypeEditor))> _
        ByVal commandLines As String, _
        <DisplayInfo("Wait For Exit"), _
        Description("This will halt the main thread until the command line returns."), _
        DefaultValue(False)> _
        ByVal waitForExit As Boolean, _
        <DisplayInfo("Show Process Window"), _
        Description("Shows the process window."), _
        DefaultValue(False)> _
        ByVal showProcessWindow As Boolean)

        Dim closeNeeded As Boolean

        If showProcessWindow AndAlso Not ProcessForm.IsActive Then
            closeNeeded = True
            ProcessForm.ShowForm(False, False)
        End If

        For Each i As String In Macro.Solve(commandLines).SplitLines
            Using pw As New Proc
                pw.CommandLine = i
                pw.Wait = waitForExit
                pw.Start()
            End Using
        Next

        If closeNeeded Then
            ProcessForm.CloseProcessForm()
        End If
    End Sub

    <Command("Perform|Play Sound", "Plays a mp3, wav or wmv sound file.")> _
    Public Sub PlaySound(<Editor(GetType(OpenFileDialogEditor), GetType(UITypeEditor)), _
        Description("Filepath to a mp3, wav or wmv sound file.")> ByVal Filepath As String, _
        <DisplayInfo("Volume (%)"), DefaultValue(20)> ByVal Volume As Integer)

        Misc.PlayAudioFile(Filepath, Volume)
    End Sub

    <Command("Dialog|Help Topic", "Opens a given help topic in the help browser.")> _
    Public Sub OpenHelpTopic( _
        <DisplayInfo("Help Topic"), _
        Description("Name of the help topic to be opened.")> ByVal topic As String)

        Dim f As New HelpForm()

        Select Case topic
            Case "info"
                f.Doc.WriteStart("StaxRip " + Application.ProductVersion)
                f.Doc.WriteP("This program is free software and may be distributed according to the terms of the [http://www.gnu.org/licenses/gpl.html GNU General Public License].")
                f.Doc.WriteH2("Credits")

                f.Doc.WriteList( _
                    "[http://www.icon-king.com Icons by David Vignoni]", _
                    "[http://www.icsharpcode.net/OpenSource/SharpZipLib/Default.aspx Zip Library by Mike Krger]", _
                    "[http://forum.doom9.org/showthread.php?t=101813 x264 profiles by Sharktooth]")

                f.Doc.WriteH2("Patches")
                f.Doc.WriteList("paulotwo fixing MediaInfo crash on Vista")
            Case "projects"
                f.Doc.WriteStart("Projects")
                f.Doc.WriteP("StaxRip has normal settings used globally like audio profiles or filter profiles and settings&nbsp;that are saved in project files. Project files&nbsp;contain&nbsp;all settings related to the opened source file like the path of the source file, the path of the audio files, the codec settings and bitrate and all other properties&nbsp;about how a movie is encoded. Projects related features can be found in the file section of the main windows main menu.")
                f.Doc.WriteP("If a source file was opened and configured a project can be saved (File/Save Project) in order to load (File/Open Project) it later. A projects can be saved (File/Save Project As Template) to serve as template for future encoding sessions so a template is also a project. Projects can be added to the job list to automatically encode a batch of projects. Technically a job is a mere project file. Recently used projects can easily accessed (File/Recent Projects). A project has the file extension rip. At startup StaxRip loads a template which can be defined in the settings.")
            Case "main"
                f.Doc.WriteStart("Main Dialog")
                f.Doc.WriteH2("Guides")
                f.Doc.WriteP(Strings.GuideIntro)

                If g.IsCulture("de") Then
                    f.Doc.WriteH3("Deutsche Anleitungen")
                    f.Doc.WriteList( _
                        "[http://encodingwissen.de Das Encodingwissen]", _
                        "[http://board.serienjunkies.org/showthread.php?t=12465 Umwandlung von TV Serien]")
                ElseIf g.IsCulture("it") Then
                    f.Doc.WriteH3("Italian Guides")
                    f.Doc.WriteList( _
                        "[http://www.planetdvb.net/non_drupal/staxrip/guides/boombastic/index.htm translation of ajp's guide by boombastic]")
                ElseIf g.IsCulture("ru") Then
                    f.Doc.WriteH3("Russian Guides")
                    f.Doc.WriteList("[http://newage.gcnet.ru/index.php?showtopic=14526 russian guide by LRN]")
                End If

                f.Doc.WriteH3("English Guides")

                f.Doc.WriteList( _
                    "[http://www.thecrave.info/walkthroughs/Staxrip.Walkthrough/index.html screencast by ORION]", _
                    "[http://www.youtube.com/watch?v=AFJidV-VBTY video guide by ohotis]", _
                    "[http://forum.indimovi.com/viewtopic.php?t=8766 guide by ajayxlnc]", _
                    "[http://forum.videohelp.com/topic333817.html guide by Trac]", _
                    "[http://forum.videohelp.com/topic332724.html guide by Trac (2)]", _
                    "[http://www.bestdvdexpert.com/dedecms/html/article/2006/0618/24.html guide on bestdvdexpert.com]", _
                    "[http://alex.nigma.info/2006/how-to-convert-your-movies-into-streamable-mpeg-4s guide by Alex Mejia]", _
                    "[http://www.digital-digest.com/articles/x264_options_page1.html guide by DVDGuy]", _
                    "[http://www.digital-digest.com/articles/staxrip_h264_page1.html guide by DVDGuy (2)]", _
                    "[http://forum.digital-digest.com/showthread.php?t=63261 guide by ajp]", _
                    "[http://avisynth.org/StaxRipDVDGuide guide on avisynth.org]")

                f.Doc.WriteTips(g.MainForm.CustomMainMenu.GetTips, g.MainForm.TipProvider.GetTips)
                f.Doc.WriteTable("Shortcut Keys", g.MainForm.CustomMainMenu.GetKeys, False)
            Case Else
                f.Doc.WriteStart("unknown topic")
                f.Doc.WriteP("The requested help topic '''" + topic + "''' is unknown.")
        End Select

        f.Show()
    End Sub

    <Command("Perform|Execute Windows Script", "Executes a windows script such as VBScript or JScript.")> _
    Public Sub ExecuteWindowsScript( _
        <DisplayInfo("Script Language"), _
        DefaultValue(GetType(WshLanguage), "VBScript")> _
        ByVal language As WshLanguage, _
        <DisplayInfo("Show Process Window"), _
        Description("Shows the process window.")> _
        ByVal showProcessWindow As Boolean, _
        <DisplayInfo("Script Code"), _
        Description("The script code may contain macros."), _
        Editor(GetType(ScriptTypeEditor), GetType(UITypeEditor))> _
        ByVal code As String)

        Dim ws As New WindowsScript
        ws.Language = language

        Dim closeNeeded As Boolean

        If showProcessWindow AndAlso Not ProcessForm.IsActive Then
            closeNeeded = True
            ProcessForm.ShowForm(False, False)
            Log.WriteLine(Macro.Solve(code))
        End If

        Try
            ws.ExecuteStatement(Macro.Solve(code))
        Catch ex As Exception
            Msg.Show(ex.Message, MsgIcon.Error)
        End Try

        If closeNeeded Then
            ProcessForm.CloseProcessForm()
        End If
    End Sub

    <Command("Perform|Show Message Box", "Shows a message box with given arguments.")> _
    Public Sub ShowMsgBox( _
        <Description("The message may contain macros."), Editor(GetType(MacroStringTypeEditor), _
        GetType(UITypeEditor))> ByVal Message As String, <Description("The title may contain macros."), _
        DefaultValue("StaxRip"), Editor(GetType(MacroStringTypeEditor), GetType(UITypeEditor))> _
        ByVal Title As String, <DefaultValue(GetType(MsgIcon), "Info")> ByVal Icon As MsgIcon)

        Msg.Show(Macro.Solve(Message), Macro.Solve(Title), Icon)
    End Sub

    <Command("Perform|Copy To Clipboard", "Copies a string to the clipboard.")> _
    Public Sub CopyToClipboard( _
        <DisplayInfo("Value"), Description("Copies the text to the clipboard. The text may contain macros."), _
        Editor(GetType(MacroStringTypeEditor), GetType(UITypeEditor))> ByVal value As String)

        Clipboard.SetText(Macro.Solve(value))
    End Sub

    <Command("Perform|Write Log Message", "Writes a log message to the process window.")> _
    Public Sub WriteLog( _
        <DisplayInfo("Header"), Description("Header is optional.")> _
        ByVal header As String, _
        <DisplayInfo("Message"), Description("Message is optional and may contain macros."), _
        Editor(GetType(MacroStringTypeEditor), GetType(UITypeEditor))> _
        ByVal message As String)

        Log.WriteHeader(header)
        Log.WriteLine(Macro.Solve(message))
    End Sub

    <Command("Perform|Delete Files", "Deletes files in a given directory.")> _
    Public Sub DeleteFiles( _
        <DisplayInfo("Directory"), _
        Description("Directory in which to delete files."), _
        Editor(GetType(MacroStringTypeEditor), GetType(UITypeEditor))> _
        ByVal dir As String, _
        <DisplayInfo("Filter"), _
        Description("Example: '.txt .log'")> _
        ByVal filter As String)

        For Each i In ",;*"
            If filter.Contains(i) Then
                filter = filter.Replace(i, " ")
            End If
        Next

        Try
            For Each i In Directory.GetFiles(Macro.Solve(dir))
                For Each i2 In filter.Split(" ")
                    If i.ToUpper.EndsWith(i2.ToUpper) Then
                        FileHelp.Delete(i)
                    End If
                Next
            Next
        Catch ex As Exception
            Msg.Show(ex.Message, MsgIcon.Error)
        End Try
    End Sub

    <Command("Perform|Add Encoder Parameters", "Adds parameters to command line based video encoders.")> _
    Public Sub AddEncoderParameters( _
        <DisplayInfo("Parameters"), _
        Description("Parameters to be added."), _
        Editor(GetType(MacroStringTypeEditor), GetType(UITypeEditor))> _
        ByVal params As String)

        If TypeOf p.VideoEncoder Is x264Encoder Then
            params = Macro.Solve(params)

            Dim b = DirectCast(p.VideoEncoder, x264Encoder).Params.AddAll

            If Not b.Value.Contains(params) Then
                If Not OK(b.Value) Then
                    b.Value = params
                Else
                    b.Value += " " + params
                End If
            End If

            Msg.Show(params)
        Else
            Msg.Show("The active video encoder is not command line based.", MsgIcon.Warning)
        End If
    End Sub

    <Command("Perform|Add Filter", "Adds a filter.")> _
    Public Sub AddFilter( _
        <DisplayInfo("Name")> ByVal name As String, _
        <DisplayInfo("Category")> ByVal category As String, _
        <DisplayInfo("Script"), Editor(GetType(MacroStringTypeEditor), _
        GetType(UITypeEditor))> ByVal script As String)

        p.TargetAVSDoc.Filters.Add(New AviSynthFilter(category, name, script, True))
        g.MainForm.AviSynthListView.Load()
        g.MainForm.Assistant()
    End Sub
End Class

<Serializable()> _
Public Class EventCommand
    Private NameValue As String = "???"

    Public Property Name() As String
        Get
            Return NameValue
        End Get
        Set(ByVal Value As String)
            NameValue = Value
        End Set
    End Property

    Private CriteriaListValue As List(Of Criteria)

    Public Property CriteriaList() As List(Of Criteria)
        Get
            If CriteriaListValue Is Nothing Then
                CriteriaListValue = New List(Of Criteria)
            End If

            Return CriteriaListValue
        End Get
        Set(ByVal Value As List(Of Criteria))
            CriteriaListValue = Value
        End Set
    End Property

    Private OrOnlyValue As Boolean

    Public Property OrOnly() As Boolean
        Get
            Return OrOnlyValue
        End Get
        Set(ByVal Value As Boolean)
            OrOnlyValue = Value
        End Set
    End Property

    Private CommandParametersValue As CommandParameters

    Public Property CommandParameters() As CommandParameters
        Get
            Return CommandParametersValue
        End Get
        Set(ByVal Value As CommandParameters)
            CommandParametersValue = Value
        End Set
    End Property

    Private EventValue As ApplicationEvent

    Public Property [Event]() As ApplicationEvent
        Get
            Return EventValue
        End Get
        Set(ByVal Value As ApplicationEvent)
            EventValue = Value
        End Set
    End Property

    Public Overrides Function ToString() As String
        Return Name
    End Function
End Class

Public Enum ApplicationEvent
    Disabled
    <DisplayInfo("After Project Loaded")> ProjectLoaded
    <DisplayInfo("After Project Encoded")> JobEncoded
    <DisplayInfo("Before Project Encoding")> BeforeEncoding
    <DisplayInfo("After Source Loaded")> AfterSourceLoaded
    <DisplayInfo("Application Exit")> ApplicationExit
    <DisplayInfo("After Project Or Source Loaded")> ProjectOrSourceLoaded
End Enum

Public Enum DynamicMenuItemID
    Audio1Profiles
    Audio2Profiles
    EncoderProfiles
    FilterSetupProfiles
    MuxerProfiles
    RecentProjects
    TemplateProjects
End Enum

Public Enum SourceInputMode
    Combine
    FileBatch
    DirectoryBatch
End Enum

Public Class MacrosEditTreeNodeSetting
    Inherits ListEditorTreeNodeSetting

    Public Sub New(ByVal path As String, ByVal text As String, ByVal items As IList, ByVal type As Type, ByVal description As String)
        MyBase.New(path, text, items, type, description)
    End Sub

    Public Overrides Sub OnClick(ByVal sender As Object, ByVal e As EventArgs)
        Using f As New MacroListEditor(TempItems)
            f.ShowDialog()
        End Using
    End Sub
End Class

Public Class BoolStringIntConverter
    Inherits TypeConverter

    Public Overloads Overrides Function GetStandardValuesSupported(ByVal context As ITypeDescriptorContext) As Boolean
        Return True
    End Function

    Public Overloads Overrides Function GetStandardValues(ByVal context As ITypeDescriptorContext) As StandardValuesCollection
        Return New StandardValuesCollection(New String() {"String", "Integer", "Boolean"})
    End Function

    Public Overloads Overrides Function CanConvertFrom(ByVal context As ITypeDescriptorContext, ByVal sourceType As Type) As Boolean
        If sourceType Is GetType(String) Then
            Return True
        End If

        Return MyBase.CanConvertFrom(context, sourceType)
    End Function

    Public Overloads Overrides Function CanConvertTo(ByVal context As ITypeDescriptorContext, ByVal destinationType As Type) As Boolean
        If destinationType Is GetType(String) Then
            Return True
        End If

        Return MyBase.CanConvertTo(context, destinationType)
    End Function

    Public Overloads Overrides Function ConvertFrom(ByVal context As ITypeDescriptorContext, ByVal culture As CultureInfo, ByVal value As Object) As Object
        If TypeOf value Is String Then
            Select Case value.ToString
                Case "String"
                    Return GetType(String)
                Case "Integer"
                    Return GetType(Integer)
                Case "Boolean"
                    Return GetType(Boolean)
            End Select
        End If

        Return MyBase.ConvertFrom(context, culture, value)
    End Function

    Public Overloads Overrides Function ConvertTo(ByVal context As ITypeDescriptorContext, ByVal culture As CultureInfo, ByVal value As Object, ByVal destinationType As Type) As Object
        If destinationType Is GetType(String) AndAlso TypeOf value Is Type Then
            If value Is GetType(Boolean) Then
                Return "Boolean"
            ElseIf value Is GetType(String) Then
                Return "String"
            ElseIf value Is GetType(Integer) Then
                Return "Integer"
            End If
        End If

        Return MyBase.ConvertTo(context, culture, value, destinationType)
    End Function
End Class

'legacy
Public NotInheritable Class LegacySerializationBinder
    Inherits SerializationBinder

    Public Overrides Function BindToType(ByVal assemblyName As String, ByVal typeName As String) As Type
        Dim ret As Type

        If typeName.Contains("CLIEncoder") Then
            typeName = typeName.Replace("CLIEncoder", "CmdlEncoder")
        End If

        If typeName.Contains("CLIEncoder") Then
            typeName = typeName.Replace("CLIEncoder", "CmdlEncoder")
        End If

        If typeName.Contains("MP4BoxMuxer") Then
            typeName = typeName.Replace("MP4BoxMuxer", "MP4Muxer")
        End If

        If typeName.Contains("mkvmergeMuxer") Then
            typeName = typeName.Replace("mkvmergeMuxer", "MkvMuxer")
        End If

        If typeName.Contains("CommandLineAudioProfile") Then
            typeName = typeName.Replace("CommandLineAudioProfile", "CmdlAudioProfile")
        End If

        If typeName.Contains("DMFMuxer") Then
            typeName = typeName.Replace("DMFMuxer", "DivXMuxer")
        End If

        ret = Type.GetType(typeName)

        Return ret
    End Function
End Class

Public Class x264ProfilesImport
    Private Dic As New Dictionary(Of String, String)

    Public Sub SetBool(ByVal bag As SettingBag(Of Boolean), ByVal key As String, Optional ByVal invers As Boolean = False)
        If ExistBool(key) Then
            If invers Then
                bag.Value = Not CBool(Dic(key))
            Else
                bag.Value = CBool(Dic(key))
            End If
        End If
    End Sub

    Public Sub SetInt(ByVal bag As SettingBag(Of Integer), ByVal key As String, Optional ByVal offset As Integer = 0)
        If ExistInt(key) Then
            bag.Value = CInt(Dic(key)) + offset
        End If
    End Sub

    Public Sub SetSng(ByVal bag As SettingBag(Of Single), ByVal key As String, Optional ByVal offset As Single = 0)
        If ExistSng(key) Then
            bag.Value = Convert.ToSingle(Dic(key), CultureInfo.InvariantCulture) + offset
        End If
    End Sub

    Private Function ExistInt(ByVal key As String) As Boolean
        Dim r = Dic.ContainsKey(key) AndAlso Integer.TryParse(Dic(key), Nothing)
        ShowWarning(r, key)
        Return r
    End Function

    Private Function ExistSng(ByVal key As String) As Boolean
        Dim r = Dic.ContainsKey(key) AndAlso Single.TryParse(Dic(key), Nothing)
        ShowWarning(r, key)
        Return r
    End Function

    Private Function ExistBool(ByVal key As String) As Boolean
        Dim r = Dic.ContainsKey(key) AndAlso Boolean.TryParse(Dic(key), Nothing)
        ShowWarning(r, key)
        Return r
    End Function

    Private Sub ShowWarning(ByVal show As Boolean, ByVal key As String)
        Static FailureList As New List(Of String)

        If Not show AndAlso Not FailureList.Contains(key) Then
            Msg.Show("Failed to import " + key + ".", MsgIcon.Warning)
            FailureList.Add(key)
        End If
    End Sub

    Public Shared Function ConvertName(ByVal value As String) As String
        For Each i In "Device DXVA Standalone Unrestricted".Split(" ")
            If value.StartsWith(i + " ") OrElse value.StartsWith(i + "-") Then
                value = i + "/" + value.Substring(i.Length + 1)
            End If
        Next

        value = "x264/" + value

        If value.Contains("_") Then
            value = value.Replace("_", " ")
        End If

        Return value
    End Function

    Public Shared Function GetCode(ByVal encoder As x264Encoder) As String
        Dim params = encoder.Params
        Dim defaultParams As New StaxRip.x264Params
        Dim varname = encoder.Name

        varname = varname.Replace("x264/", "")

        If varname.First.IsDigit() Then
            varname = "_" + varname
        End If

        For Each i In varname
            If Not Char.IsLetterOrDigit(i) Then
                varname = varname.Replace(i, "_"c)
            End If
        Next

        varname = varname.Replace("-", "")
        varname = varname.Replace("(", "_")
        varname = varname.Replace(")", "_")

        Dim l As New List(Of String)

        l.Add("Dim " + varname + " As New x264Encoder()")
        l.Add(varname + ".Name = """ + encoder.Name + """")

        For Each i In params.GetType.GetFields
            Dim value = i.FieldType.GetProperty("Value").GetValue(i.GetValue(params), Nothing)
            Dim defaultValue = i.FieldType.GetProperty("Value").GetValue(i.GetValue(defaultParams), Nothing)

            If value.Equals(defaultValue) Then
                Continue For
            End If

            If TypeOf value Is String Then
                value = """" + value.ToString + """"
            Else
                value = value.ToString.Replace(",", ".")
            End If

            l.Add(varname + ".Params." + i.Name + ".Value = " + value.ToString)
        Next

        l.Add("ret.Add(" + varname + ")")

        Return l.Join(CrLf)
    End Function

    Public Sub LoadProfiles(ByVal files As String())
        Dim code As String = Nothing

        For Each i In files
            Dic.Clear()
            Dim doc As New XmlDocument
            doc.Load(i)

            Dim encoder As New x264Encoder
            Dim params = encoder.Params
            encoder.Name = x264ProfilesImport.ConvertName(doc.GetElementsByTagName("Name")(0).InnerText)

            If encoder.Name = "x264/default" Then
                Continue For
            End If

            Dim e = doc.GetElementsByTagName("Settings")
            e = e(0).ChildNodes

            For Each iNode As XmlNode In e
                Dic(iNode.Name) = iNode.InnerText
            Next

            If ExistInt("EncodingMode") Then
                Select Case CInt(Dic(("EncodingMode")))
                    Case 0
                        params.Mode.Value = StaxRip.x264Mode.SingleBitrate
                    Case 1
                        params.Mode.Value = StaxRip.x264Mode.SingleQuant
                    Case 4
                        params.Mode.Value = StaxRip.x264Mode.TwoPass
                    Case 8
                        params.Mode.Value = StaxRip.x264Mode.ThreePass
                    Case 9
                        params.Mode.Value = StaxRip.x264Mode.SingleCRF
                End Select
            End If

            SetBool(params.AdaptiveDCT, "AdaptiveDCT")

            If Dic.ContainsKey("CustomEncoderOptions") Then
                params.AddAll.Value = Dic("CustomEncoderOptions")
            End If

            For Each i2 In New String() {"--psy-rd", "--nal-hrd", "--sar"}
                If params.AddAll.Value.Contains(i2) Then
                    Dim re As New Regex(" *" + i2 + ".*?(?= --)| *" + i2 + ".*")

                    If re.IsMatch(params.AddAll.Value) Then
                        params.AddAll.Value = re.Replace(params.AddAll.Value, "", RegexOptions.IgnoreCase)
                    End If
                End If
            Next

            SetBool(params.B8x8, "B8x8mv")
            SetBool(params.BFramesPyramid, "BFramePyramid")
            SetBool(params.BFramesWeightedPrediction, "WeightedBPrediction")
            SetBool(params.CABAC, "Cabac")
            SetBool(params.I4x4, "I4x4mv")
            SetBool(params.I8x8, "I8x8mv")
            SetBool(params.LoopFilter, "Deblock")
            SetBool(params.MixedRefs, "MixedRefs")
            SetBool(params.NoChromaMe, "ChromaME", True)
            SetBool(params.NoDCTDecimate, "NoDCTDecimate")
            SetBool(params.NoFastPSkip, "noFastPSkip")
            SetBool(params.NoPSNR, "PSNRCalculation", True)
            SetBool(params.NoSSIM, "SSIMCalculation", True)
            SetBool(params.P4x4, "P4x4mv")
            SetBool(params.P8x8, "P8x8mv")

            SetInt(params.BAdapt, "NewAdaptiveBFrames")
            SetInt(params.BFrames, "NbBframes")
            SetInt(params.BFramesBias, "BframeBias")
            SetInt(params.DeblockAlpha, "AlphaDeblock")
            SetInt(params.DeblockBeta, "BetaDeblock")
            SetInt(params.DirectMode, "BframePredictionMode")
            SetInt(params.GOPSizeMax, "KeyframeInterval")
            SetInt(params.GOPSizeMin, "MinGOPSize")
            SetInt(params.Level, "Level")
            SetInt(params.MEMethod, "METype")
            SetInt(params.MeRange, "MERange")
            SetInt(params.QPMin, "MinQuantizer")
            SetInt(params.AQMode, "AQmode")

            SetSng(params.IPRatio, "IPFactor")
            SetSng(params.PBRatio, "PBFactor")
            SetSng(params.PsyRD, "PsyRDO")
            SetSng(params.PsyTrellis, "PsyTrellis")
            SetSng(params.QComp, "QuantCompression")

            If ExistBool("Lossless") Then
                params.Quant.Value = If(CBool(Dic("Lossless")), 0, params.Quant.Value)
            End If

            If ExistInt("BitrateQuantizer") Then
                params.Quant.Value = If(CInt(Dic("BitrateQuantizer")) <= 51, CInt(Dic("BitrateQuantizer")), params.Quant.Value)
            End If

            SetInt(params.ReferenceFrames, "NbRefFrames")
            SetInt(params.SceneCut, "SCDSensitivity")
            SetInt(params.SubME, "SubPelRefinement", 1)
            SetInt(params.Threads, "NbThreads")
            SetInt(params.Trellis, "X264Trellis")

            If ExistInt("VBVBufferSize") Then
                params.VBVBufSize.Value = If(CInt(Dic("VBVBufferSize")) < 0, 0, CInt(Dic("VBVBufferSize")))
            End If

            SetSng(params.VBVInit, "VBVInitialBuffer")

            If ExistInt("VBVMaxBitrate") Then
                params.VBVMaxRate.Value = If(CInt(Dic("VBVMaxBitrate")) < 0, 0, CInt(Dic("VBVMaxBitrate")))
            End If

            For x = 0 To s.VideoEncoderProfiles.Count - 1
                If s.VideoEncoderProfiles(x).Name = encoder.Name Then
                    s.VideoEncoderProfiles(x) = encoder
                End If
            Next

            If Not s.VideoEncoderProfiles.Contains(encoder) Then
                s.VideoEncoderProfiles.Add(encoder)
            End If
#If DEBUG Then
            code += CrLf2 + x264ProfilesImport.GetCode(encoder)
#End If
        Next

        If OK(code) Then
            Clipboard.SetText(code)
        End If
    End Sub
End Class

Public Class CmdlBuilder
    Public SB As New StringBuilder

    Public Sub Add(ByVal value As Integer, ByVal defaultValue As Integer, ByVal prefix As String)
        If value <> defaultValue Then
            SB.Append(prefix + value.ToString)
        End If
    End Sub

    Public Sub Add(ByVal value As Boolean, ByVal switch As String)
        If value Then
            SB.Append(switch)
        End If
    End Sub
End Class