Imports System.Text
Imports System.Runtime.Serialization
Imports System.Reflection
Imports System.Text.RegularExpressions
Imports System.Globalization

Imports Stax
Imports Stax.UI
Imports Stax.AppFramework
Imports Stax.ComponentModel

<Serializable()> _
Public MustInherit Class Muxer
    Inherits Profile

    Public MustOverride Sub Mux()

    Public Sub New(ByVal name As String, ByVal typeName As String)
        MyBase.New(name, typeName)
        CanEditValue = True
    End Sub

    Private ChapterFileValue As String = ""

    Public Property ChapterFile() As String
        Get
            Return ChapterFileValue
        End Get
        Set(ByVal Value As String)
            ChapterFileValue = Value
        End Set
    End Property

    Private AdditionalSwitchesValue As String

    Public Property AdditionalSwitches() As String
        Get
            Return AdditionalSwitchesValue
        End Get
        Set(ByVal Value As String)
            If Not OK(Value) Then
                AdditionalSwitchesValue = Nothing
            Else
                AdditionalSwitchesValue = Value
            End If
        End Set
    End Property

    Public Overridable ReadOnly Property SupportsCutting() As Boolean
        Get
            Return False
        End Get
    End Property

    Protected InputTypesValue As StreamType()

    Public Overridable ReadOnly Property InputTypes() As StreamType()
        Get
            Return InputTypesValue
        End Get
    End Property

    Protected OutputTypeValue As StreamType

    Public Overridable ReadOnly Property OutputType() As StreamType
        Get
            Return OutputTypeValue
        End Get
    End Property

    Private SubtitlesValue2 As List(Of Subtitle)

    'legacy 1.1.0.1
    Public Property Subtitles2() As List(Of Subtitle)
        Get
            If SubtitlesValue2 Is Nothing Then
                SubtitlesValue2 = New List(Of Subtitle)
            End If

            Return SubtitlesValue2
        End Get
        Set(ByVal value As List(Of Subtitle))
            SubtitlesValue2 = value
        End Set
    End Property

    Public Overrides Sub Clean()
        Subtitles2 = Nothing
        ChapterFile = ""
    End Sub

    Public Overridable Function GetError() As String
        Return Nothing
    End Function

    Public Overridable Function GetCmdl() As String
        Return Nothing
    End Function

    Public Overridable Function GetExtension() As String
        Return StreamTypeUtils.GetExtension(OutputType)
    End Function

    Public Overridable Sub Init()
        If Not File.Exists(p.SourceFile) Then
            Exit Sub
        End If

        Dim files = Directory.GetFiles(Filepath.GetDir(p.SourceFile))
        Dim subs = "idx srt ssa smi ass ttxt".Split(" ").ToList

        For Each i In files
            If subs.Contains(Filepath.GetExtNoDot(i)) AndAlso g.IsSourceSameOrSimilar(i) Then
                For Each i2 In Subtitle.Create(i)
                    For Each i3 In p.AutoSubtitleLanguages
                        If i3.Equals(i2.Language) Then
                            Subtitles2.Add(i2)
                        End If
                    Next
                Next
            End If
        Next

        For Each i In files
            If i.EndsWith("Chapter Information - OGG.txt") AndAlso g.IsSourceSameOrSimilar(i) Then
                ChapterFile = i
                Exit For
            End If
        Next
    End Sub

    Public Overridable Function IsStreamSupported(ByVal type As StreamType) As Boolean
        Return Array.IndexOf(Of StreamType)(InputTypes, type) > -1
    End Function

    Public Shared Function GetDefaults() As List(Of Muxer)
        Dim ret As New List(Of Muxer)

        ret.Add(New MkvMuxer)
        ret.Add(New AviMuxer)
        ret.Add(New MP4Muxer)
        ret.Add(New DivXMuxer)
        ret.Add(New PMPMuxer)

        Dim mp4CLI As New CmdlMuxer(StreamType.MP4)
        mp4CLI.Name = "Misc/Command Line Example"
        mp4CLI.CommandLines.Value = """%application:MP4Box%"" -nodrop -add ""%encoder_out_file%#video"" -add ""%audio_file1%"" -new ""%target_file%"""
        ret.Add(mp4CLI)

        ret.Add(New NullMuxer("Misc/Null Muxer"))

        Return ret
    End Function

    Public Overrides Function GetInfoValues() As Dictionary(Of String, String)
        Dim ret As New Dictionary(Of String, String)

        ret("Muxer Type") = TypeName
        ret("Output File Extension") = GetExtension()

        If Not InputTypes Is Nothing Then
            ret("Supported Input Formats") = StreamTypeUtils.GetNames(InputTypes)
        End If

        ret("Output Format") = DisplayInfoAttribute.GetValueForEnum(Of StreamType)(OutputType)

        Return ret
    End Function

    Public Overrides Function Edit() As DialogResult
        ShowInfoDialog()
    End Function
End Class

<Serializable()> _
Public MustInherit Class EncoderMuxerBase
    Inherits Muxer

    Public MustOverride ReadOnly Property Encoder() As Type

    Public Sub New(ByVal name As String, ByVal typeName As String)
        MyBase.New(name, typeName)
    End Sub

    Public Overrides Function GetExtension() As String
        Return p.VideoEncoder.GetExtension
    End Function

    Public Overrides Sub Mux()
    End Sub
End Class

'legacy 1.1.0.1
<Serializable()> _
Public Class EncoderMuxer
    Inherits EncoderMuxerBase

    Public Sub New()
        MyBase.New("", "")
    End Sub

    Public Overrides ReadOnly Property SupportsCutting() As Boolean
        Get
            Return True
        End Get
    End Property

    Private EncoderValue As Type

    Public Overrides ReadOnly Property Encoder() As Type
        Get
            Return GetType(VirtualDubEncoder)
        End Get
    End Property
End Class

<Serializable()> _
Public Class AviMuxer
    Inherits EncoderMuxerBase

    Public Sub New()
        MyBase.New("AVI", "Encoder Muxer")
    End Sub

    Public Overrides ReadOnly Property SupportsCutting() As Boolean
        Get
            Return True
        End Get
    End Property

    Public Overrides ReadOnly Property OutputType() As StreamType
        Get
            Return StaxRip.StreamType.AVI
        End Get
    End Property

    Public Overrides ReadOnly Property InputTypes() As StreamType()
        Get
            Return New StreamType() {StreamType.AC3, StreamType.MP3, StreamType.AVI, StreamType.MP2, StreamType.WAV}
        End Get
    End Property

    Public Overrides ReadOnly Property Encoder() As Type
        Get
            Return GetType(VirtualDubEncoder)
        End Get
    End Property
End Class

<Serializable()> _
Public Class MP4Muxer
    Inherits Muxer

    Public Sub New()
        MyBase.New("MP4", "MP4Box Muxer")
    End Sub

    Public Overrides ReadOnly Property OutputType() As StreamType
        Get
            Return StreamType.MP4
        End Get
    End Property

    Public Overrides Function Edit() As DialogResult
        Using f As New MuxerForm(Me)
            Return f.ShowDialog()
        End Using
    End Function

    Public Overrides Function GetCmdl() As String
        Return """" + Packs.MP4Box.Path + """ " + GetArgs()
    End Function

    Private Function GetArgs() As String
        Dim args As New StringBuilder

        args.Append(" -fps " + p.TargetAVSDoc.GetFramerate.ToString("f3", CultureInfo.InvariantCulture))

        Dim temp As String = Nothing
        Dim par = Calc.GetTargetPAR

        If TypeOf p.VideoEncoder Is NullEncoder Then
            temp = ":par=" & par.X & ":" & par.Y
        End If

        args.Append(" -add """ + p.VideoEncoder.OutputPath + "#video:name=Video" + temp + """")

        If File.Exists(p.Audio0.File) AndAlso IsStreamSupported(p.Audio0.StreamType) Then
            args.Append(" -add """ + p.Audio0.File + ":lang=" + p.Audio0.Language.ThreeLetterCode)

            If p.Audio0.Delay <> 0 AndAlso Not p.Audio0.HandlesDelay Then
                args.Append(":delay=" + p.Audio0.Delay.ToString)
            End If

            If p.Audio0.StreamName <> "" Then
                args.Append(":name=" + p.Audio0.SolveMacros(p.Audio0.StreamName, False))
            End If

            If g.IsAacHE(p.Audio0.File) Then
                args.Append(":sbr")
            End If

            args.Append("""")
        End If

        If File.Exists(p.Audio1.File) AndAlso IsStreamSupported(p.Audio1.StreamType) AndAlso p.Audio1.File <> "" Then
            args.Append(" -add """ + p.Audio1.File + ":lang=" + p.Audio1.Language.ThreeLetterCode)

            If p.Audio1.Delay <> 0 AndAlso Not p.Audio1.HandlesDelay Then
                args.Append(":delay=" + p.Audio1.Delay.ToString)
            End If

            If p.Audio1.StreamName <> "" Then
                args.Append(":name=" + p.Audio1.SolveMacros(p.Audio1.StreamName, False))
            End If

            If g.IsAacHE(p.Audio1.File) Then
                args.Append(":sbr")
            End If

            args.Append("""")
        End If

        For Each i In Subtitles2
            Dim path = g.GetFullPath(i.Path)

            If File.Exists(path) Then
                If Filepath.GetExt(path) = ".idx" Then
                    Dim name As String = Nothing

                    If OK(i.Name) Then
                        name = ":name=" + i.Name
                    End If

                    args.Append(" -add """ + path + "#" & i.IndexIDX + 1 & name & """")
                Else
                    Dim name As String = ""

                    If OK(i.Name) Then
                        name = ":name=" + i.Name
                    End If

                    args.Append(" -add """ + path + ":lang=" + i.Language.ThreeLetterCode + name + """")
                End If
            End If
        Next

        If File.Exists(ChapterFile) Then
            args.Append(" -chap """ + ChapterFile + """")
        End If

        If OK(AdditionalSwitches) Then
            args.Append(" " + Macro.Solve(AdditionalSwitches))
        End If

        args.Append(" -new """ + p.TargetFile + """")

        Return args.ToString.Trim
    End Function

    Public Overrides Sub Mux()
        Log.WriteHeader("Muxing")

        Dim start As DateTime = DateTime.Now

        Using proc As New Proc
            AddHandler proc.Process.OutputDataReceived, AddressOf ProcessForm.CmdlDataHandler
            proc.BeginOutputReadLine = True

            proc.File = Packs.MP4Box.Path
            proc.Arguments = GetArgs()

            Log.WriteLine(proc.CommandLine + CrLf2)

            proc.Wait = True
            proc.Priority = s.ProcessPriority.Value

            ProcessForm.Proc = proc

            proc.Start()

            RemoveHandler proc.Process.OutputDataReceived, AddressOf ProcessForm.CmdlDataHandler
            ProcessForm.LogCmdlOutput("|")

            If proc.ExitCode <> 0 Then
                Dim msg = "MP4 muxing failed with exit code " & proc.ExitCode
                Log.WriteHeader("Error")
                Log.WriteLine(msg)
                Throw New ErrorAbortException(msg)
            ElseIf Not g.FileWasJustWritten(p.TargetFile) Then
                Log.WriteHeader("Error")
                Dim msg = "MP4 output file is missing"
                Log.WriteLine(msg)
                Throw New ErrorAbortException(msg)
            End If
        End Using

        Log.WriteHeader("Muxing Statistics")
        Log.WriteStats(start, p.TargetAVSDoc.GetFrames)
        Log.WriteFilesize(p.TargetFile)
    End Sub

    Public Overrides Sub Clean()
        Subtitles2 = Nothing
    End Sub

    Public Overrides ReadOnly Property InputTypes() As StreamType()
        Get
            Return New StreamType() {StreamType.AVI, StreamType.RAW264, StreamType.MP4, StreamType.AAC, StreamType.MP3, StreamType.MP2, StreamType.MPA}
        End Get
    End Property
End Class

<Serializable()> _
Public Class NullMuxer
    Inherits Muxer

    Public Sub New(ByVal name As String)
        MyBase.New(name, "Null Muxer")
        CanEditValue = False
    End Sub

    Public Overrides Function IsStreamSupported(ByVal type As StreamType) As Boolean
        Return True
    End Function

    Public Overrides ReadOnly Property OutputType() As StreamType
        Get
            Return p.VideoEncoder.OutputStreamType
        End Get
    End Property

    Public Overrides Function GetExtension() As String
        Return p.VideoEncoder.GetExtension
    End Function

    Public Overrides Sub Mux()
    End Sub
End Class

<Serializable()> _
Public Class CmdlMuxer
    Inherits Muxer

    Public Sub New(ByVal outputType As StreamType)
        MyBase.New("Command Line", "Command Line Muxer")
        OutputTypeValue = outputType
    End Sub

    Private CommandLinesValue As New SettingBag(Of String)("")

    Public Property CommandLines() As SettingBag(Of String)
        Get
            Return CommandLinesValue
        End Get
        Set(ByVal Value As SettingBag(Of String))
            CommandLinesValue = Value
        End Set
    End Property

    Public Overrides Function IsStreamSupported(ByVal type As StreamType) As Boolean
        Return True
    End Function

    Public Overrides Sub Mux()
        Log.WriteHeader("Muxing")

        For Each i As String In Macro.Solve(CommandLines.Value).SplitLines
            Dim start As DateTime = DateTime.Now

            Dim pw As New Proc

            pw.CommandLine = i
            pw.Wait = True
            pw.Minimized = s.RunAppsMinimized.Value
            pw.Priority = s.ProcessPriority.Value

            If Not ProcessForm.IsVisible Then
                pw.HideAfterStart = True
            End If

            Log.WriteLine(i)
            ProcessForm.Proc = pw

            pw.Start()

            Log.WriteHeader("Muxing Statistics")
            Log.WriteStats(start, p.TargetAVSDoc.GetFrames)
        Next
    End Sub

    Public Overrides Function Edit() As DialogResult
        Using f As New SettingTreeViewForm(Name, "The Command Line Muxer dialog allows to configure StaxRip to use any command line compatible muxer.")
            Dim outputTypeSetting As New SettingBag(Of StreamType)(OutputType)

            f.SettingTreeView.AddSetting(New ComboboxTreeNodeSetting(Of StreamType)("General", "Output Type", outputTypeSetting, "Stream type of the output file."))
            f.SettingTreeView.AddSetting(New CmdlEditorTreeNodeSetting("General", "Command Lines", CommandLines, "Command lines to be executed performing the muxing. It's possible to use multible command lines separated by a new line. The command lines may contain macros."))

            f.SettingTreeView.Nodes(0).ExpandAll()
            f.Document.WriteTable("Macros", Strings.MacrosHelp, Macro.GetTips())

            Dim ret As DialogResult = f.ShowDialog()

            If ret = DialogResult.OK Then
                OutputTypeValue = outputTypeSetting.Value
            End If

            Return ret
        End Using
    End Function
End Class

<Serializable()> _
Public Class MkvMuxer
    Inherits Muxer

    Public VideoTrackName As String = "" 'Reflection
    Public VideoTrackLanguage As New Language(CultureInfo.InvariantCulture) 'Reflection
    Public Title As String = "" 'Reflection

    Public Sub New()
        MyBase.New("MKV", "mkvmerge Muxer")
    End Sub

    Public Overrides ReadOnly Property OutputType() As StreamType
        Get
            Return StreamType.MKV
        End Get
    End Property

    Public Overrides Function Edit() As DialogResult
        Using f As New MuxerForm(Me)
            Return f.ShowDialog()
        End Using
    End Function

    Public Overrides Sub Mux()
        Log.WriteHeader("Muxing")

        Dim start = DateTime.Now

        Using proc As New Proc
            AddHandler proc.Process.OutputDataReceived, AddressOf ProcessForm.CmdlDataHandler
            proc.BeginOutputReadLine = True
            proc.File = Packs.Mkvmerge.Path
            proc.Arguments = GetArgs()

            Log.WriteLine(proc.CommandLine + CrLf2)

            ProcessForm.Proc = proc

            proc.Wait = True
            proc.Priority = s.ProcessPriority.Value
            proc.Start()

            RemoveHandler proc.Process.OutputDataReceived, AddressOf ProcessForm.CmdlDataHandler
            ProcessForm.LogCmdlOutput("progress: ")

            If Not IsOneOf(proc.ExitCode, 0, 1) Then
                Log.WriteHeader("Error")
                Dim msg = "MKV muxing failed with exit code " & proc.ExitCode
                Log.WriteLine(msg)
                Throw New ErrorAbortException(msg)
            ElseIf Not g.FileWasJustWritten(p.TargetFile) Then
                Log.WriteHeader("Error")
                Dim msg = "MKV output file is missing."
                Log.WriteLine(msg)
                Throw New ErrorAbortException(msg)
            End If
        End Using

        Log.WriteHeader("Muxing Statistics")
        Log.WriteStats(start, p.TargetAVSDoc.GetFrames)
        Log.WriteFilesize(p.TargetFile)
    End Sub

    Public Overrides Function GetCmdl() As String
        Return """" + Packs.Mkvmerge.Path + """ " + GetArgs()
    End Function

    Private Function GetArgs() As String
        Dim args As New StringBuilder("-o """ + p.TargetFile + """")

        args.Append(" --noaudio --nosubs")

        If VideoTrackLanguage.ThreeLetterCode <> "und" Then
            args.Append(" --language 0:" + VideoTrackLanguage.ThreeLetterCode)
        End If

        If OK(VideoTrackName) Then
            args.Append(" --track-name ""0:" + Macro.Solve(VideoTrackName) + """")
        End If

        If p.VideoEncoder.OutputStreamType = StreamType.RAW264 Then
            args.Append(" --default-duration 0:" + p.TargetAVSDoc.GetFramerate.ToString("f3").Replace(",", ".") + "fps")
        End If

        args.Append(" """ + p.VideoEncoder.OutputPath + """")

        If File.Exists(p.Audio0.File) AndAlso IsStreamSupported(p.Audio0.StreamType) Then
            Dim id = "0"

            If Filepath.GetExt(p.Audio0.File) = ".mp4" Then
                id = MediaInfo.GetInfo(p.Audio0.File, MediaInfoStreamKind.Audio, "ID")
            End If

            If g.IsAacHE(p.Audio0.File) Then
                args.Append(" --aac-is-sbr " + id + ":1")
            End If

            args.Append(" --language " + id + ":" + p.Audio0.Language.ThreeLetterCode)

            If p.Audio0.StreamName <> "" Then
                args.Append(" --track-name """ + id + ":" + p.Audio0.SolveMacros(p.Audio0.StreamName, False) + """")
            End If

            If p.Audio0.Delay <> 0 AndAlso Not p.Audio0.HandlesDelay Then
                args.Append(" --sync " + id + ":" + p.Audio0.Delay.ToString)
            End If

            args.Append(" """ + p.Audio0.File + """")
        End If

        If File.Exists(p.Audio1.File) AndAlso IsStreamSupported(p.Audio1.StreamType) Then
            Dim id = "0"

            If Filepath.GetExt(p.Audio1.File) = ".mp4" Then
                id = MediaInfo.GetInfo(p.Audio1.File, MediaInfoStreamKind.Audio, "ID")
            End If

            If g.IsAacHE(p.Audio1.File) Then
                args.Append(" --aac-is-sbr " + id + ":1")
            End If

            args.Append(" --language " + id + ":" + p.Audio1.Language.ThreeLetterCode)

            If p.Audio1.StreamName <> "" Then
                args.Append(" --track-name """ + id + ":" + p.Audio1.SolveMacros(p.Audio1.StreamName, False) + """")
            End If

            If p.Audio1.Delay <> 0 AndAlso Not p.Audio1.HandlesDelay Then
                args.Append(" --sync " + id + ":" + p.Audio1.Delay.ToString)
            End If

            args.Append(" """ + p.Audio1.File + """")
        End If

        For Each i In Subtitles2
            Dim path = g.GetFullPath(i.Path)

            If File.Exists(path) Then
                If Filepath.GetExt(path) = ".idx" Then
                    args.Append(" --language " & i.IndexData & ":" + i.Language.ThreeLetterCode)

                    If OK(i.Name) Then
                        args.Append(" --track-name """ & i.IndexData & ":" + i.Name + """")
                    End If

                    args.Append(" -s " & i.IndexData)
                    args.Append(" """ + path + """")
                Else
                    args.Append(" --language 0:" + i.Language.ThreeLetterCode)

                    If OK(i.Name) Then
                        args.Append(" --track-name ""0:" + i.Name + """")
                    End If

                    args.Append(" """ + path + """")
                End If
            End If
        Next

        If File.Exists(ChapterFile) Then
            args.Append(" --chapters """ + ChapterFile + """")
        End If

        If OK(Title) Then
            args.Append(" --title """ + Macro.Solve(Title) + """")
        End If

        args.Append(" --engage keep_bitstream_ar_info")

        If OK(AdditionalSwitches) Then
            args.Append(" " + Macro.Solve(AdditionalSwitches))
        End If

        Return args.ToString
    End Function

    Public Overrides ReadOnly Property InputTypes() As StreamType()
        Get
            Return New StreamType() {StreamType.AVI, StreamType.RAW264, StreamType.MKV, StreamType.MP4, StreamType.AAC, StreamType.AAC_LC_RAW, StreamType.AC3, StreamType.MP3, StreamType.Vorbis, StreamType.DTS, StreamType.MP2}
        End Get
    End Property
End Class

<Serializable()> _
Public Class DivXMuxer
    Inherits Muxer

    Public Sub New()
        MyBase.New("DIVX", "DivXMux")
        InputTypesValue = New StreamType() {StreamType.AVI, StreamType.DMF, StreamType.AC3, StreamType.MP3}
    End Sub

    Public Overrides ReadOnly Property OutputType() As StreamType
        Get
            Return StreamType.DMF
        End Get
    End Property

    Public Overrides Function Edit() As DialogResult
        Using f As New MuxerForm(Me)
            Return f.ShowDialog()
        End Using
    End Function

    Public Overrides Function GetCmdl() As String
        Return """" + Packs.DivXMux.Path + """ " + GetArgs()
    End Function

    Private Function GetArgs() As String
        Dim args As New StringBuilder("-v """ + p.VideoEncoder.OutputPath + """")

        If File.Exists(p.Audio0.File) AndAlso IsStreamSupported(p.Audio0.StreamType) Then
            args.Append(" -a (Audio -" + p.Audio0.Language.TwoLetterCode + ";01)")

            If p.Audio0.Delay <> 0 AndAlso Not p.Audio0.HandlesDelay Then
                args.Append(" o" + p.Audio0.Delay.ToString)
            End If

            p.Audio0.File = Filepath.GetChangeExt(p.Audio0.File, Filepath.GetExt(p.Audio0.File))
            args.Append(" """ + p.Audio0.File + """")
        End If

        If File.Exists(p.Audio1.File) AndAlso IsStreamSupported(p.Audio1.StreamType) Then
            args.Append(" -a (Audio -" + p.Audio1.Language.TwoLetterCode + ";01)")

            If p.Audio1.Delay <> 0 AndAlso Not p.Audio1.HandlesDelay Then
                args.Append(" o" + p.Audio1.Delay.ToString)
            End If

            p.Audio1.File = Filepath.GetChangeExt(p.Audio1.File, Filepath.GetExt(p.Audio1.File))
            args.Append(" """ + p.Audio1.File + """")
        End If

        For Each i In Subtitles2
            Dim path = g.GetFullPath(i.Path)

            If File.Exists(path) Then
                If Filepath.GetExt(path) = ".idx" Then
                    Dim content = StringHelp.ReadFile(path)
                    content = Regex.Replace(content, "langidx: \d+", "langidx: " + i.IndexIDX.ToString)
                    Dim newPath = Filepath.GetDir(p.SourceFile) + i.Language.TwoLetterCode + i.IndexIDX.ToString + ".idx"
                    content.WriteFile(newPath)

                    FileHelp.Copy(Filepath.GetDirAndBase(path) + ".sub", Filepath.GetDir(p.SourceFile) + _
                                  i.Language.TwoLetterCode + i.IndexIDX.ToString + ".sub")

                    args.Append(" -s (Subtitle -" + i.Language.TwoLetterCode + ";01) """ + newPath + """")
                Else
                    args.Append(" -s (Subtitle -" + i.Language.TwoLetterCode + ";01) """ + path + """")
                End If
            End If
        Next

        args.Append(" -o """ + p.TargetFile + """")

        Return args.ToString
    End Function

    Public Overrides Sub Mux()
        Log.WriteHeader("Muxing")

        Dim start As DateTime = DateTime.Now

        Using proc As New Proc
            proc.File = Packs.DivXMux.Path
            proc.Arguments = GetArgs()

            Log.WriteLine(proc.CommandLine)

            If Not ProcessForm.IsVisible Then
                proc.HideAfterStart = True
            End If

            proc.Minimized = s.RunAppsMinimized.Value
            proc.Wait = True
            proc.Priority = s.ProcessPriority.Value

            ProcessForm.Proc = proc

            proc.Start()

            If proc.ExitCode <> 0 Then
                Log.WriteHeader("Error")
                Dim msg = "DivX muxing failed with exit code " & proc.ExitCode
                Log.WriteLine(msg)
                Throw New ErrorAbortException(msg)
            ElseIf Not g.FileWasJustWritten(p.TargetFile) Then
                Log.WriteHeader("Error")
                Dim msg = "DivX output file is missing"
                Log.WriteLine(msg)
                Throw New ErrorAbortException(msg)
            End If
        End Using

        Log.WriteHeader("Muxing Statistics")
        Log.WriteStats(start, p.TargetAVSDoc.GetFrames)
        Log.WriteFilesize(p.TargetFile)
    End Sub
End Class

<Serializable()> _
Public Class PMPMuxer
    Inherits Muxer

    Public Sub New()
        MyBase.New("Misc/PMP (PlayStation Portable)", "PMP Muxer")
        InputTypesValue = New StreamType() {StreamType.AVI, StreamType.MP3}
    End Sub

    Public Overrides ReadOnly Property OutputType() As StreamType
        Get
            Return StreamType.PMP
        End Get
    End Property

    Public Overrides Sub Mux()
        Log.WriteHeader("Muxing")

        Dim start As DateTime = DateTime.Now

        Dim pw As New Proc

        pw.File = Packs.PMPMuxer.Path
        pw.Arguments = GetArgs()
        pw.Wait = True
        pw.Minimized = s.RunAppsMinimized.Value
        pw.Priority = s.ProcessPriority.Value

        If Not ProcessForm.IsVisible Then
            pw.HideAfterStart = True
        End If

        Log.WriteLine(pw.CommandLine)
        ProcessForm.Proc = pw

        pw.Start()

        Log.WriteHeader("Muxing Statistics")
        Log.WriteStats(start, p.TargetAVSDoc.GetFrames)
    End Sub

    Private Function GetArgs() As String
        Dim sb As New StringBuilder

        sb.Append("-v """ + p.VideoEncoder.OutputPath + """")

        If File.Exists(p.Audio0.File) AndAlso IsStreamSupported(p.Audio0.StreamType) Then
            sb.Append(" -a """ + p.Audio0.File + """")
        End If

        If File.Exists(p.Audio1.File) AndAlso IsStreamSupported(p.Audio1.StreamType) Then
            sb.Append(" -a """ + p.Audio1.File + """")
        End If

        sb.Append(" -o """ + p.TargetFile + """")

        Return sb.ToString
    End Function
End Class