Imports Stax
Imports Stax.UI
Imports Stax.AppFramework

Imports System.Text.RegularExpressions

<Serializable()> _
Public MustInherit Class Preparer
    Public MustOverride Sub Run()

    Private NameValue As String = ""

    Public Property Name() As String
        Get
            Return NameValue
        End Get
        Set(ByVal Value As String)
            NameValue = Value
        End Set
    End Property

    Private InputExtensionsValue As String()

    Public Overridable Property InputExtensions() As String()
        Get
            If InputExtensionsValue Is Nothing Then
                Return New String() {}
            End If

            Return InputExtensionsValue
        End Get
        Set(ByVal value As String())
            InputExtensionsValue = value
        End Set
    End Property

    Private InputFormatsValue As String()

    Public Overridable Property InputFormats() As String()
        Get
            If InputFormatsValue Is Nothing Then
                Return New String() {}
            End If

            Return InputFormatsValue
        End Get
        Set(ByVal value As String())
            InputFormatsValue = value
        End Set
    End Property

    Private VideoOutputExtensionsValue As String()

    Public Overridable Property VideoOutputExtensions() As String()
        Get
            If VideoOutputExtensionsValue Is Nothing Then
                Return New String() {}
            End If

            Return VideoOutputExtensionsValue
        End Get
        Set(ByVal value As String())
            VideoOutputExtensionsValue = value
        End Set
    End Property

    Private InputFormatsBlacklistValue As String()

    Public Overridable Property InputFormatsBlacklist() As String()
        Get
            If InputFormatsBlacklistValue Is Nothing Then
                Return New String() {}
            End If

            Return InputFormatsBlacklistValue
        End Get
        Set(ByVal value As String())
            InputFormatsBlacklistValue = value
        End Set
    End Property

    Private ActiveValue As Boolean = True

    Public Property Active() As Boolean
        Get
            Return ActiveValue
        End Get
        Set(ByVal Value As Boolean)
            ActiveValue = Value
        End Set
    End Property

    Public Overridable ReadOnly Property HasConfigDialog() As Boolean
        Get
            Return False
        End Get
    End Property

    Public Overridable Function ShowConfigDialog() As DialogResult
        Return DialogResult.OK
    End Function

    Public Overrides Function ToString() As String
        Return Name
    End Function

    Public Shared Function GetDefaults(ByVal all As Boolean) As List(Of Preparer)
        Dim ret As New List(Of Preparer)

        Dim projectX As New CmdlPreparer
        projectX.Name = "Demux MPEG-2 using ProjectX"
        projectX.InputExtensions = "mpg mpeg ts pva vdr".Split(" ")
        projectX.VideoOutputExtensions = New String() {"m2v"}
        projectX.InputFormatsBlacklist = New String() {"AVC"}
        projectX.CommandLines.Value = """%application:Java%"" -jar ""%application:ProjectX%"" %source_files_blank% -out ""%working_dir%"""
        ret.Add(projectX)

        Dim dgi As New CmdlPreparer
        dgi.Name = "Demux audio and index MPEG-2 using DGIndex"
        dgi.InputExtensions = "mpg mpeg vob mpv m2v ts pva dat".Split(" ")
        dgi.VideoOutputExtensions = New String() {"d2v"}
        dgi.InputFormatsBlacklist = New String() {"AVC"}
        dgi.CommandLines.Value = """%application:DGIndex%"" -SD=| -IF=|%source_files_comma_no_quotes%| -IA=2 -FO=0 -YR=1 -TN=1 -OM=2 -DRC=2 -DSD=0 -DSA=0 -OF=|%working_dir%\%source_name%| -minimize -exit"
        ret.Add(dgi)

        ret.Add(New DemuxPreparer)

        Dim dgaByFormat As New CmdlPreparer
        dgaByFormat.Name = "Demux and index AVC using DGAVCIndex by format"
        dgaByFormat.InputExtensions = "m2ts ts".Split(" ")
        dgaByFormat.VideoOutputExtensions = "dga".Split(" ")
        dgaByFormat.InputFormats = "AVC".Split(" ")
        dgaByFormat.CommandLines.Value = """%application:DGAVCIndex%"" -i ""%source_file%"" -o ""%working_dir%\%source_name%.dga"" -a -h -e"
        ret.Add(dgaByFormat)

        Dim dgaByExtension As New CmdlPreparer
        dgaByExtension.Name = "Demux and index AVC using DGAVCIndex by extension"
        dgaByExtension.InputExtensions = "avc 264 h264".Split(" ")
        dgaByExtension.VideoOutputExtensions = "dga".Split(" ")
        dgaByExtension.CommandLines.Value = """%application:DGAVCIndex%"" -i ""%source_file%"" -o ""%working_dir%\%source_name%.dga"" -a -h -e"
        ret.Add(dgaByExtension)

        ret.Add(New DirectShowDecoderPreparer())

        If all Then
            Dim cliPreparer As New CmdlPreparer
            cliPreparer.Name = "Command Line Preparer"
            ret.Add(cliPreparer)
        End If

        Return ret
    End Function
End Class

<Serializable()> _
Public Class CmdlPreparer
    Inherits Preparer

    Public CommandLines As New SettingBag(Of String)("")

    Public Overrides ReadOnly Property HasConfigDialog() As Boolean
        Get
            Return True
        End Get
    End Property

    Public Overrides Function ShowConfigDialog() As DialogResult
        Using t As New SettingTreeViewForm("Command Line Preparer", "A command line preparer can be used to perform different tasks with source files like extracting audio to a separate file.")
            Dim nameBag As New SettingBag(Of String)(Name)
            t.SettingTreeView.AddSetting(New TextBoxTreeNodeSetting("General", "Name", nameBag, "Name of this preparer."))

            Dim inputBag As New SettingBag(Of String)(InputExtensions.ToArray.Join(", "))
            t.SettingTreeView.AddSetting(New TextBoxTreeNodeSetting("General", "Input Extensions", inputBag, "The file types the preparer should handle."))

            Dim inputFormatsBag As New SettingBag(Of String)(InputFormats.ToArray.Join(", "))
            t.SettingTreeView.AddSetting(New TextBoxTreeNodeSetting("General", "Input Formats", inputFormatsBag, "Video formats to be handled, empty handles everything except blacklisted formats. The format as shown by [http://mediainfo.sourceforge.net/de MediaInfo's] 'Format' property."))

            Dim inputFormatsBlackListBag As New SettingBag(Of String)(InputFormatsBlacklist.ToArray.Join(", "))
            t.SettingTreeView.AddSetting(New TextBoxTreeNodeSetting("General", "Input Formats Blacklist", inputFormatsBlackListBag, "Video formats that should not be handled. The format as shown by [http://mediainfo.sourceforge.net/de MediaInfo's] 'Format' property."))

            Dim outputVideoBag As New SettingBag(Of String)(VideoOutputExtensions.ToArray.Join(", "))
            t.SettingTreeView.AddSetting(New TextBoxTreeNodeSetting("General", "Video Output Extensions", outputVideoBag, "The video output file types the preparer might create."))

            t.SettingTreeView.AddSetting(New CmdlEditorTreeNodeSetting("General", "Command Lines", CommandLines, Strings.CommandLine))

            t.Document.WriteTable("Macros", Strings.MacrosHelp, Macro.GetTips())

            t.SettingTreeView.Nodes(0).ExpandAll()

            Dim ret = t.ShowDialog

            If ret = DialogResult.OK Then
                Name = nameBag.Value
                InputExtensions = inputBag.Value.ToLower.SplitNonLetterOrDigit
                VideoOutputExtensions = outputVideoBag.Value.ToLower.SplitNonLetterOrDigit
                InputFormatsBlacklist = inputFormatsBlackListBag.Value.Split(True, ",", ";")
                InputFormats = inputFormatsBag.Value.Split(True, ",", ";")
            End If

            Return ret
        End Using
    End Function

    Public Overrides Sub Run()
        For Each i In Macro.Solve(CommandLines.Value).SplitLines
            If i.Contains("%application:") OrElse i.Contains("%application_dir:") Then
                Throw New AbortException
            End If

            Using proc As New Proc
                ProcessForm.Proc = proc
                ProcessForm.ShowForm(False)

                Log.WriteLine(i + CrLf2)
                Log.WriteLine("Preparer: " + Name)

                If Not ProcessForm.IsVisible Then
                    proc.HideAfterStart = True
                End If

                Dim start = DateTime.Now

                proc.CommandLine = i
                proc.Minimized = s.RunAppsMinimized.Value
                proc.Priority = s.ProcessPriority.Value
                proc.Directory = Filepath.GetDir(proc.File)
                proc.Process.StartInfo.UseShellExecute = False
                proc.Wait = True
                proc.ReTrowException = True
                proc.Start()

                Log.WriteStats(start)
            End Using
        Next
    End Sub
End Class

<Serializable()> _
Public Class DirectShowDecoderPreparer
    Inherits Preparer

    Public Sub New()
        Name = "Decode audio using DirectShow"
        InputExtensions = New String() {"wmv"}
    End Sub

    Public Overrides ReadOnly Property HasConfigDialog() As Boolean
        Get
            Return True
        End Get
    End Property

    Public Overrides Sub Run()
        If OK(p.Audio0.File) OrElse OK(p.Audio1.File) Then
            Log.WriteLine("There is a file already.")
            Exit Sub
        End If

        If Not Packs.VirtualDubMod.VerifyOK(True) Then
            Throw New AbortException
        End If

        If IsOneOf(Filepath.GetExt(p.SourceFile), ".mp4", ".mkv") AndAlso Not Packs.HaaliSplitter.VerifyOK(True) Then
            Throw New AbortException
        End If

        Dim start = DateTime.Now

        Dim avs As New AviSynthDocument
        avs.Filters.Add(New AviSynthFilter("Source", "DirectShow", "DirectShowSource(""%source_file%"", convertfps=true).ConvertAudioTo16bit()", True))
        avs.Path = g.TempDir + Filepath.GetBase(p.SourceFile) + "_DirectShowDecode.avs"
        avs.Synchronize()

        Dim script As New VirtualDubScript
        script.Open(avs.Path)
        script.SaveWAV(Filepath.GetChangeExt(p.SourceFile, ".wav"))
        script.SaveScript(p.SourceFile.Path.Dir + Filepath.GetBase(p.SourceFile) + "_DirectShowDecode.vcf")

        Using pw As New Proc
            ProcessForm.Proc = pw
            ProcessForm.ShowForm(False)

            pw.File = Packs.VirtualDubMod.Path
            pw.Arguments = "/s""" + p.SourceFile.Path.Dir + Filepath.GetBase(p.SourceFile) + "_DirectShowDecode.vcf"" /x"
            pw.Minimized = s.RunAppsMinimized.Value
            pw.Wait = True
            pw.Priority = s.ProcessPriority.Value

            If Not ProcessForm.IsVisible Then
                pw.HideAfterStart = True
            End If

            Log.WriteLine(Macro.Solve(avs.GetScript()))

            pw.Start()

            Log.WriteHeader("Audio Decoding Statistics")
            Log.WriteStats(start, avs.GetFrames)
        End Using
    End Sub

    Public Overrides Function ShowConfigDialog() As DialogResult
        Using t As New SettingTreeViewForm("DirectShow Audio Decoding Preparer", "The DirectShow Audio Decoding Preparer extracts the audio as wav file using DirectShow.")
            Dim inputBag As New SettingBag(Of String)(String.Join(", ", InputExtensions.ToArray))
            t.SettingTreeView.AddSetting(New TextBoxTreeNodeSetting("General", "Input Extensions", inputBag, "The file types the preparer should handle."))
            t.SettingTreeView.Nodes(0).ExpandAll()

            Dim ret = t.ShowDialog

            If ret = DialogResult.OK Then
                InputExtensions = inputBag.Value.Replace(" ", "").Split(",")
            End If

            Return ret
        End Using
    End Function
End Class

<Serializable()> _
Public Class DemuxPreparer
    Inherits Preparer

    Public Sub New()
        Name = "Demux MKV, MP4, AVI, MOV and DIVX files"
        InputExtensions = New String() {"avi", "mkv", "mp4", "divx", "mov"}
        VideoOutputExtensions = New String() {"avi", "264", "h264"}
    End Sub

    Public Overrides ReadOnly Property HasConfigDialog() As Boolean
        Get
            Return True
        End Get
    End Property

    Public Overrides Function ShowConfigDialog() As DialogResult
        Using t As New SettingTreeViewForm("Demux Preparer", "The Demux preparer extracts video, audio and subtitle streams from MKV, MP4 and AVI files.")
            Dim inputBag As New SettingBag(Of String)(String.Join(", ", InputExtensions.ToArray))
            t.SettingTreeView.AddSetting(New TextBoxTreeNodeSetting("General", "Input Extensions", inputBag, "The file types the preparer should handle."))

            Dim outputVideoBag As New SettingBag(Of String)(VideoOutputExtensions.ToArray.Join(", "))
            t.SettingTreeView.AddSetting(New TextBoxTreeNodeSetting("General", "Video Output Extensions", outputVideoBag, "The video output file types the preparer might create."))

            t.SettingTreeView.Nodes(0).ExpandAll()

            Dim ret = t.ShowDialog

            If ret = DialogResult.OK Then
                InputExtensions = inputBag.Value.ToLower.SplitNonLetterOrDigit
                VideoOutputExtensions = outputVideoBag.Value.ToLower.SplitNonLetterOrDigit
            End If

            Return ret
        End Using
    End Function

    Public Overrides Sub Run()
        Select Case Filepath.GetExt(p.SourceFile)
            Case ".mkv"
                DemuxMKV()
            Case ".mp4", ".mov"
                DemuxMP4()
            Case ".avi", ".divx"
                DemuxAVI()
        End Select
    End Sub

    Public Shared Sub DemuxMP4()
        If Not Packs.MP4Box.VerifyOK Then
            Throw New AbortException
        End If

        Dim output As String

        Using o As New Proc
            o.File = Packs.MP4Box.Path
            o.Arguments = " -info """ + p.SourceFile + """"
            o.Redirect = True
            o.Wait = True
            o.Start()
            output = o.RedirectOutput
        End Using

        Dim a = output.Split("Track # ").ToList
        Dim files As New List(Of String)(Directory.GetFiles(Filepath.GetDir(p.SourceFile)))

        For x = 1 To a.Count - 1
            Dim startTime = DateTime.Now

            Using o As New Proc
                AddHandler o.Process.OutputDataReceived, AddressOf ProcessForm.CmdlDataHandler
                o.BeginOutputReadLine = True
                o.File = Packs.MP4Box.Path

                Dim switch As String

                If a(x).Contains("Type ""vide:mp4v""") Then
                    switch = " -avi "
                ElseIf a(x).Contains("Type ""soun:mp4a""") AndAlso Filepath.GetExt(p.SourceFile) <> ".mov" Then
                    switch = " -single "
                Else
                    switch = " -raw "
                End If

                o.Arguments = switch & x & " """ + p.SourceFile + """"

                ProcessForm.Proc = o
                ProcessForm.ShowForm(False)

                Log.WriteLine(o.CommandLine + CrLf2)

                o.Wait = True
                o.Priority = s.ProcessPriority.Value
                o.Start()

                RemoveHandler o.Process.OutputDataReceived, AddressOf ProcessForm.CmdlDataHandler
                ProcessForm.LogCmdlOutput("Media Export: |", "File Export: |", "ISO File Writing: |")
            End Using

            Log.WriteHeader("MP4 Demuxing Statistics Track" & x)
            Log.WriteStats(startTime)

            For Each i In Directory.GetFiles(Filepath.GetDir(p.SourceFile))
                If Not files.Contains(i) Then
                    Dim newPath As String

                    If Not DirPath.IsSameDir(Filepath.GetDir(p.SourceFile), g.TempDir) Then
                        newPath = g.TempDir + Filepath.GetName(i)
                        File.Move(i, newPath)
                        i = newPath
                    End If

                    If i.Contains("_track" & x) Then
                        newPath = i.Replace("_track" & x, "")

                        If Not a(x).Contains("Type ""vide:") Then
                            Dim lang = Regex.Match(a(x), "Language ""(.+?)""")

                            If lang.Success Then
                                Dim l = lang.Groups(1).Value

                                For Each i2 In Language.Languages
                                    If i2.Name = l Then
                                        newPath = Filepath.GetDirAndBase(newPath) + " - " + i2.TwoLetterCode + _
                                            "-" + i2.Name + Filepath.GetExt(newPath)
                                        Exit For
                                    End If
                                Next
                            End If
                        End If

                        If FileHelp.Delete(newPath) Then
                            FileHelp.Move(i, newPath)
                        End If
                    End If
                End If
            Next
        Next
    End Sub

    Private Sub DemuxMKV()
        If Not Packs.Mkvmerge.VerifyOK Then
            Throw New AbortException
        End If

        Dim output As String

        Log.WriteDebug("mkvinfo exist", File.Exists(Packs.Mkvmerge.Path.Path.Dir + "mkvinfo.exe").ToString) 'debug
        Log.WriteDebug("source exist", File.Exists(p.SourceFile).ToString) 'debug

        Using o As New Proc
            o.File = Packs.Mkvmerge.Path.Path.Dir + "mkvinfo.exe"
            o.Arguments = """" + p.SourceFile + """"
            o.Redirect = True
            o.Wait = True
            o.Start()
            output = o.RedirectOutput
        End Using

        Log.WriteDebug("mkvinfo output", output) 'debug

        Dim args = "tracks """ + p.SourceFile + """"
        Dim dic As New Dictionary(Of String, String)

        dic("A_AAC") = "aac"
        dic("A_AC3") = "ac3"
        dic("A_DTS") = "dts"
        dic("A_FLAC") = "flac"
        dic("A_MPEG/L1") = "mpa"
        dic("A_MPEG/L2") = "mp2"
        dic("A_MPEG/L3") = "mp3"
        dic("A_PCM/INT/LIT") = "wav"
        dic("A_QUICKTIME") = "qdm"
        dic("A_REAL") = "ra"
        dic("A_TTA1") = "tta"
        dic("A_VORBIS") = "ogg"
        dic("A_WAVPACK4") = "wv"
        dic("Attachment") = "att"
        dic("Chapters") = "Chapters"
        dic("S_TEXT/ASS") = "ass"
        dic("S_TEXT/SSA") = "ssa"
        dic("S_TEXT/USF") = "usf"
        dic("S_TEXT/UTF8") = "srt"
        dic("S_VOBSUB") = "idx"
        dic("Tags") = "Tags"
        dic("V_MPEG1") = "mpg"
        dic("V_MPEG4/ISO/ASP") = "m4v"
        dic("V_MPEG4/ISO/AVC") = "264"
        dic("V_MS/VFW/FOURCC") = "avi"
        dic("V_REAL") = "rmvb"
        dic("V_THEORA") = "ogg"

        Dim a = output.Split("+ Track number: ")
        Dim audioIndex As Integer

        For x = 1 To a.Length - 1
            Dim codecMatch = Regex.Match(a(x), " Codec ID: (.+)")

            If codecMatch.Success Then
                Dim codec = codecMatch.Groups(1).Value.Trim

                If dic.ContainsKey(codec) Then
                    Dim base = g.TempDir + Filepath.GetBase(p.SourceFile)

                    If Not a(x).Contains("Track type: video") Then
                        Dim lang = Regex.Match(a(x), " Language: (.+)")

                        If lang.Success Then
                            Dim l = lang.Groups(1).Value.Trim

                            For Each i In Language.Languages
                                If i.ThreeLetterCode = l Then
                                    base += " - " + i.TwoLetterCode + "-" + i.Name
                                    Exit For
                                End If
                            Next
                        End If

                        If a(x).Contains("Track type: audio") Then
                            Using mi As New MediaInfo(p.SourceFile)
                                If mi.AudioTracks()(audioIndex).AudioType = MediaInfoAudioType.AACHE Then
                                    base += " - sbr"
                                End If
                            End Using

                            audioIndex += 1
                        End If
                    End If

                    args += " " & x & ":""" + base + "." + dic(codec) + """"
                End If
            End If
        Next

        Dim startTime = DateTime.Now

        Using o As New Proc
            AddHandler o.Process.OutputDataReceived, AddressOf ProcessForm.CmdlDataHandler
            o.BeginOutputReadLine = True
            Log.WriteDebug("mkvextract exist", File.Exists(Packs.Mkvmerge.Path.Path.Dir + "mkvextract.exe").ToString) 'debug
            o.File = Packs.Mkvmerge.Path.Path.Dir + "mkvextract.exe"
            o.Arguments = args

            ProcessForm.Proc = o
            ProcessForm.ShowForm(False)

            Log.WriteLine(o.CommandLine + CrLf2)

            o.Wait = True
            o.Priority = s.ProcessPriority.Value
            o.Start()

            RemoveHandler o.Process.OutputDataReceived, AddressOf ProcessForm.CmdlDataHandler
            ProcessForm.LogCmdlOutput("progress: ")
        End Using

        Log.WriteHeader("MKV Demuxing Statistics")
        Log.WriteStats(startTime)
    End Sub

    Public Sub DemuxAVI()
        If Not Packs.VirtualDubMod.VerifyOK(True) Then
            Throw New AbortException
        End If

        Dim script As New VirtualDubScript
        script.Open(p.SourceFile)

        If p.SourceFiles.Count > 1 Then
            For i = 1 To p.SourceFiles.Count - 1
                script.Append(p.SourceFiles(i))
            Next
        End If

        Dim found As Boolean
        Dim tracks = MediaInfo.GetAudioTracks(p.SourceFile)

        If tracks.Count > 0 Then
            For x = 0 To tracks.Count - 1
                Dim index = ""

                If tracks.Count > 1 Then
                    index = " " + x.ToString
                End If

                If tracks(x).AudioType = MediaInfoAudioType.WAV Then
                    script.SaveWAV(g.TempDir + Filepath.GetBase(p.SourceFile) + index + tracks(x).Extension)
                    found = True
                ElseIf tracks(x).AudioType <> MediaInfoAudioType.Unknown Then
                    script.StreamDemux(x, g.TempDir + Filepath.GetBase(p.SourceFile) + index + tracks(x).Extension)
                    found = True
                End If
            Next
        End If

        If Not found Then
            Exit Sub
        End If

        script.SaveScript(g.TempDir + Filepath.GetBase(p.SourceFile) + "_Demux.vcf")

        Using pw As New Proc
            ProcessForm.Proc = pw
            ProcessForm.ShowForm(False)

            If Not ProcessForm.IsVisible Then
                pw.HideAfterStart = True
            End If

            pw.File = Packs.VirtualDubMod.Path
            pw.Arguments = "/s""" + g.TempDir + Filepath.GetBase(p.SourceFile) + "_Demux.vcf"" /x"

            Log.WriteLine("Preparer: " + Name)
            Log.WriteLine(pw.CommandLine)

            pw.Minimized = s.RunAppsMinimized.Value
            pw.Wait = True
            pw.Priority = s.ProcessPriority.Value
            pw.Start()
        End Using
    End Sub
End Class