Imports System.Collections

Imports System.Runtime.Serialization
Imports System.Runtime.Serialization.Formatters.Binary
Imports System.Reflection
Imports System.Globalization
Imports System.Text

Imports Stax
Imports Stax.UI
Imports Stax.Win32
Imports Stax.DynamicObject

<Serializable()> _
Public Class Project
    Implements ISafeSerialization

    Private Storage As ObjectStorage

    Public Audio0 As AudioProfile
    Public Audio1 As AudioProfile
    Public AutoARSignaling As New SettingBag(Of Boolean)(True)
    Public AutoCompCheck As Boolean
    Public AutoCorrectCropValues As New SettingBag(Of Boolean)(True)
    Public AutoCrop As Boolean = True
    Public AutoForceFilmThreshold As New SettingBag(Of Integer)(95)
    Public AutoResizeImage As Integer = 250000
    Public AutoSmartCrop As New SettingBag(Of Boolean)(False)
    Public AutoSmartOvercrop As New SettingBag(Of Single)(0)
    Public AutoSubtitleLanguages As List(Of Language)
    Public AutoVSRip As Boolean = True
    Public BatchMode As New SettingBag(Of Boolean)(False)
    Public CompCheckAction As CompCheckAction = StaxRip.CompCheckAction.AdjustImageSize
    Public CompCheckRange As New SettingBag(Of Integer)(5)
    Public Compressibility As Double
    Public CropBottom As Integer
    Public CropLeft As Integer
    Public CropRight As Integer
    Public CropTop As Integer
    Public DefaultTargetFolder As String = ""
    Public DefaultTargetName As String = ""
    Public ForcedOutputMod As New SettingBag(Of Integer)(16)
    Public ITU As New SettingBag(Of Boolean)(True)
    Public MaxAspectRatioError As New SettingBag(Of Integer)(2)
    Public Name As String
    Public Preparers As List(Of Preparer)
    Public PreviewScript As New SettingBag(Of String)("Info()")
    Public Ranges As List(Of Range)
    Public RemindToCrop As New SettingBag(Of Boolean)(True)
    Public RemindToCut As New SettingBag(Of Boolean)(True)
    Public RemindToDoCompCheck As New SettingBag(Of Boolean)(True)
    Public RemindToSetFilters As New SettingBag(Of Boolean)(True)
    Public RemindOversize As New SettingBag(Of Boolean)(True)
    Public Size As Integer = 700
    Public SkippedAssistantTips As List(Of String)
    Public SourceAnamorphic As Boolean
    Public SourceAviSynthDocument As SourceAviSynthDocument
    Public SourceFile As String
    Public SourceFiles As List(Of String)
    Public SourceFramerate As Double
    Public SourceFrames As Integer
    Public SourceHeight As Integer = 480
    Public SourceSeconds As Integer
    Public SourceWidth As Integer = 640
    Public TargetAVSDoc As TargetAviSynthDocument
    Public TargetFile As String
    Public TargetHeight As Integer = 480
    Public TargetSeconds As Integer = 5400
    Public TargetWidth As Integer = 640
    Public TemplateName As String = ""
    Public UseAviSynthToCut As New SettingBag(Of Boolean)(False)
    Public Versions As Dictionary(Of String, Integer)
    Public VideoBitrate As Integer = 1000
    Public VideoEncoder As VideoEncoder
    Public Log As StringBuilder
    Public ResizeSliderMaxWidth As New SettingBag(Of Integer)(0)
    Public TempDir As String
    Public CustomPAR As String = ""
    Public DeleteTempFilesDir As Boolean

    Private WasUpdatedValue As Boolean

    Public Property WasUpdated() As Boolean Implements Stax.ISafeSerialization.WasUpdated
        Get
            Return WasUpdatedValue
        End Get
        Set(ByVal value As Boolean)
            WasUpdatedValue = value
        End Set
    End Property

    Public ReadOnly Property VersionsProperty() As Dictionary(Of String, Integer) Implements ISafeSerialization.Versions
        Get
            Return Versions
        End Get
    End Property

    Private Function Check(ByVal obj As Object, ByVal key As String, ByVal minimalVersion As Integer, ByVal currentVersion As Integer) As Boolean
        Return SafeSerialization.Check(Me, obj, key, minimalVersion, currentVersion)
    End Function

    Public Sub Init() Implements ISafeSerialization.Init
        If Versions Is Nothing Then
            Versions = New Dictionary(Of String, Integer)
        End If

        If TempDir Is Nothing Then
            TempDir = ""
        End If

        If Log Is Nothing Then
            Log = New StringBuilder
        End If

        If Storage Is Nothing Then
            Storage = New ObjectStorage
        End If

        If AutoSubtitleLanguages Is Nothing Then
            AutoSubtitleLanguages = New List(Of Language)
            AutoSubtitleLanguages.Add(New Language())

            If Not g.IsCulture("en") Then
                AutoSubtitleLanguages.Add(New Language(New CultureInfo("en")))
            End If
        End If

        If Ranges Is Nothing Then
            Ranges = New List(Of Range)
        End If

        If TargetFile Is Nothing Then
            TargetFile = ""
        End If

        If SourceAviSynthDocument Is Nothing Then
            SourceAviSynthDocument = New SourceAviSynthDocument
        End If

        If Name Is Nothing Then
            Name = ""
        End If

        If SourceFile Is Nothing Then
            SourceFile = ""
        End If

        If SkippedAssistantTips Is Nothing Then
            SkippedAssistantTips = New List(Of String)
        End If

        If SourceFiles Is Nothing Then
            SourceFiles = New List(Of String)
        End If

        If Check(Preparers, "Preparers", 38, 38) Then
            Preparers = Preparer.GetDefaults(False)
        End If

        If Check(VideoEncoder, "Video Encoder", 20, 31) Then
            Dim enc As New x264Encoder()
            enc.Name = "x264 Constant Quality Balanced"

            enc.Params.AdaptiveDCT.Value = True
            enc.Params.BFrames.Value = 16
            enc.Params.BFramesPyramid.Value = True
            enc.Params.BFramesWeightedPrediction.Value = True
            enc.Params.DeblockAlpha.Value = -1
            enc.Params.DeblockBeta.Value = -1
            enc.Params.I8x8.Value = True
            enc.Params.MixedRefs.Value = True
            enc.Params.Mode.Value = 2
            enc.Params.P4x4.Value = False
            enc.Params.Quant.Value = 22
            enc.Params.ReferenceFrames.Value = 3
            enc.Params.Trellis.Value = 1

            VideoEncoder = enc
        End If

        If Check(Audio0, "Audio Track 1", 14, 15) Then
            If TypeOf VideoEncoder Is x264Encoder Then
                Audio0 = AudioProfile.GetDefaults(False)(0)
            Else
                Audio0 = AudioProfile.GetDefaults(False)(3)
            End If
        End If

        If Check(Audio1, "Audio Track 2", 14, 15) Then
            Audio1 = New NullAudioProfile
            Audio1.Language = New Language(New CultureInfo("en"))
        End If

        If Check(TargetAVSDoc, "Filter Setup", 1, 4) Then
            TargetAVSDoc = AviSynthDocument.GetDefaults()(0)
        End If

        'legacy 1.1.0.1
        If IsOneOf(0, SourceWidth, SourceHeight, TargetWidth, TargetHeight) Then
            SourceWidth = 640
            SourceHeight = 480
            TargetWidth = 640
            TargetHeight = 480

            WasUpdated = True
        End If

        'legacy 1.1.0.1
        For Each i In New AudioProfile() {Audio0, Audio1}
            If i.StreamType = StreamType.MP3CBR OrElse i.StreamType = StreamType.MP3VBR Then
                i.StreamType = StreamType.MP3
                WasUpdated = True
            End If

            If i.StreamType = StreamType.AAC_HE_MP4 OrElse i.StreamType = StreamType.AAC_LC_MP4 Then
                i.StreamType = StreamType.MP4
                WasUpdated = True
            End If

            If i.StreamType = StreamType.AAC_HE_RAW OrElse i.StreamType = StreamType.AAC_LC_RAW Then
                i.StreamType = StreamType.AAC
                WasUpdated = True
            End If
        Next

        'legacy 1.1.0.1
        If TypeOf VideoEncoder.Muxer Is EncoderMuxer Then
            VideoEncoder.Muxer = New AviMuxer
        ElseIf TypeOf VideoEncoder.Muxer Is MkvMuxer Then
            Dim m = DirectCast(VideoEncoder.Muxer, MkvMuxer)

            If m.Title Is Nothing Then
                m.Title = ""
                m.VideoTrackLanguage = New Language(CultureInfo.InvariantCulture)
                m.VideoTrackName = ""
                WasUpdated = True
            End If
        End If

        'legacy 1.1.1.0
        If TypeOf VideoEncoder Is x264Encoder Then
            Dim b = DirectCast(VideoEncoder, x264Encoder).Params.AQMode

            If b.Value > 1 Then
                b.Value = 1
                WasUpdated = True
            End If
        End If
    End Sub
End Class