Imports Microsoft.Win32

Imports System.Globalization
Imports System.Runtime.Serialization
Imports System.Reflection
Imports System.Text.RegularExpressions
Imports System.Text

Imports Stax
Imports Stax.UI
Imports Stax.AppFramework
Imports Stax.ComponentModel

<Serializable()> _
Public MustInherit Class VideoEncoder
    Inherits Profile
    Implements IComparable(Of VideoEncoder)

    Public MustOverride Sub Encode()
    Public MustOverride Function GetMenu() As List(Of OptionListBoxItem)
    Protected MustOverride Sub ShowConfigDialog()

    Public Sub New(ByVal profileName As String, ByVal profileTypeName As String)
        MyBase.New(profileName, profileTypeName)
        CanEditValue = True
    End Sub

    Private OutputPathValue As String

    Public Overridable ReadOnly Property OutputPath() As String
        Get
            If TypeOf Muxer Is EncoderMuxerBase OrElse TypeOf Muxer Is NullMuxer Then
                Return p.TargetFile
            Else
                Return g.TempDir + p.Name + "_EncoderOutput" + GetExtension()
            End If
        End Get
    End Property

    Private PassesValue As Integer

    Public Overridable Property Passes() As Integer
        Get
            Return PassesValue
        End Get
        Set(ByVal Value As Integer)
            PassesValue = Value
        End Set
    End Property

    Private QualityModeValue As Boolean

    Public Overridable Property QualityMode() As Boolean
        Get
            Return QualityModeValue
        End Get
        Set(ByVal Value As Boolean)
            QualityModeValue = Value
        End Set
    End Property

    Private MuxerValue As Muxer

    Public Property Muxer() As Muxer
        Get
            Return MuxerValue
        End Get
        Set(ByVal Value As Muxer)
            MuxerValue = Value
        End Set
    End Property

    Public Overrides Function CreateEditControl() As System.Windows.Forms.Control
        Dim lb As New OptionListBox
        lb.Dock = DockStyle.Fill
        lb.IntegralHeight = False
        lb.HelpID = Me.GetType.FullName
        lb.Items.AddRange(GetMenu.ToArray)
        Return lb
    End Function

    Public Overridable ReadOnly Property IsCompCheckEnabled() As Boolean
        Get
            Return Not QualityMode
        End Get
    End Property

    Private AutoCompCheckValueValue As Integer = 70

    Public Property AutoCompCheckValue() As Integer
        Get
            Return AutoCompCheckValueValue
        End Get
        Set(ByVal Value As Integer)
            AutoCompCheckValueValue = Value
        End Set
    End Property

    Private OutputStreamTypeValue As StreamType

    Public Overridable Property OutputStreamType() As StreamType
        Get
            Return OutputStreamTypeValue
        End Get
        Set(ByVal value As StreamType)
            OutputStreamTypeValue = value
        End Set
    End Property

    Protected Sub OnAfterCompCheck()
        If p.CompCheckAction = CompCheckAction.AdjustFileSize Then
            g.MainForm.tbSize.Text = g.GetAutoSize(AutoCompCheckValue).ToString
        ElseIf p.CompCheckAction = CompCheckAction.AdjustImageSize Then
            AutoSetImageSize()
        End If
    End Sub

    Public Sub AutoSetImageSize()
        If p.VideoEncoder.AutoCompCheckValue > 0 AndAlso _
            Calc.GetPercent <> 0 AndAlso p.TargetAVSDoc.IsFilterActive("Resize") Then
            p.TargetWidth = Calc.FixMod16(CInt((p.SourceHeight - p.CropTop - p.CropBottom) * Calc.GetTargetDAR()))

            Dim cropw = p.SourceWidth - p.CropLeft - p.CropRight

            If p.TargetWidth > cropw Then
                p.TargetWidth = cropw
            End If

            p.TargetHeight = Calc.FixMod16(CInt(p.TargetWidth / Calc.GetTargetDAR()))

            While Calc.GetPercent < p.VideoEncoder.AutoCompCheckValue
                If p.TargetWidth - 16 >= 320 Then
                    p.TargetWidth -= 16
                    p.TargetHeight = Calc.FixMod16(CInt(p.TargetWidth / Calc.GetTargetDAR()))
                Else
                    Exit While
                End If
            End While

            g.MainForm.tbTargetWidth.Text = p.TargetWidth.ToString
            g.MainForm.tbTargetHeight.Text = p.TargetHeight.ToString

            If p.AutoSmartCrop.Value Then
                g.MainForm.PerformSmartCrop()
            End If
        End If
    End Sub

    Public Overrides Sub Clean()
        Muxer.Clean()
    End Sub

    Public Overridable Function GetExtension() As String
        Return StreamTypeUtils.GetExtension(OutputStreamType)
    End Function

    Public Overridable Function GetError() As String
        Return Nothing
    End Function

    Protected Sub OnStateChange()
        g.MainForm.UpdateEncoderStateRelatedControls()
        g.MainForm.SetEncoderControl(CreateEditControl)
        g.MainForm.Assistant()
    End Sub

    Public Enum Modes
        First = 1
        Second = 2
        CompCheck = 4
    End Enum

    Protected Sub OpenMuxerConfigDialog()
        Dim m = ObjectHelp.GetCopy(Of Muxer)(Muxer)

        If m.Edit = DialogResult.OK Then
            Muxer = m
            g.MainForm.Refresh()
            g.MainForm.tbSize_TextChanged()
            g.MainForm.Assistant()
        End If
    End Sub

    Public Function OpenMuxerProfilesDialog() As DialogResult
        Using f As New ProfilesForm("Muxer Profiles", s.MuxerProfiles, AddressOf LoadMuxer, AddressOf GetMuxerProfile)
            Return f.ShowDialog()
        End Using
    End Function

    Public Sub LoadMuxer(ByVal profile As Profile)
        Muxer = DirectCast(ObjectHelp.GetCopy(profile), Muxer)
        Muxer.Init()

        g.MainForm.llMuxer.Text = g.ConvertPath(DisplayInfoAttribute.GetValueForEnum(Of StreamType)(Muxer.OutputType)).Shorten(4)
        g.MainForm.tbTargetPath.Text = Filepath.GetChangeExt(p.TargetFile, Muxer.GetExtension)
        g.MainForm.RecalcBitrate()
        g.MainForm.Assistant()
    End Sub

    Private Function GetMuxerProfile() As Profile
        Dim sb As New SelectionBox(Of Muxer)

        sb.Title = "New Profile"
        sb.Text = Strings.AddProfile

        sb.AddItem("Current Project", Muxer)

        For Each i As Muxer In StaxRip.Muxer.GetDefaults()
            sb.AddItem(i)
        Next

        If sb.Show = DialogResult.OK Then
            Return sb.SelectedItem
        End If

        Return Nothing
    End Function

    Public Shared Function GetEncoder(ByVal name As String) As VideoEncoder
        For Each i In GetDefaults()
            If i.Name = name Then
                Return i
            End If
        Next

        Return GetDefaults()(0)
    End Function

    Public Shared Function GetDefaults() As List(Of VideoEncoder)
        Dim ret As New List(Of VideoEncoder)

        Dim Device_Android_G1 As New x264Encoder()
        Device_Android_G1.Name = "x264/Device/Android G1"
        Device_Android_G1.Params.BFrames.Value = 0
        Device_Android_G1.Params.CABAC.Value = False
        Device_Android_G1.Params.DeblockAlpha.Value = 1
        Device_Android_G1.Params.DirectMode.Value = 0
        Device_Android_G1.Params.Level.Value = 7
        Device_Android_G1.Params.MEMethod.Value = 2
        Device_Android_G1.Params.Mode.Value = 3
        Device_Android_G1.Params.P4x4.Value = False
        Device_Android_G1.Params.QPMin.Value = 12
        Device_Android_G1.Params.ReferenceFrames.Value = 1
        Device_Android_G1.Params.SubME.Value = 8
        Device_Android_G1.Params.VBVBufSize.Value = 2500
        Device_Android_G1.Params.VBVMaxRate.Value = 2500
        ret.Add(Device_Android_G1)

        Dim Device_Archos605 As New x264Encoder()
        Device_Archos605.Name = "x264/Device/Archos605"
        Device_Archos605.Params.BFrames.Value = 0
        Device_Archos605.Params.CABAC.Value = False
        Device_Archos605.Params.DirectMode.Value = 0
        Device_Archos605.Params.Level.Value = 10
        Device_Archos605.Params.MEMethod.Value = 2
        Device_Archos605.Params.MixedRefs.Value = True
        Device_Archos605.Params.Mode.Value = 3
        Device_Archos605.Params.P4x4.Value = False
        Device_Archos605.Params.ReferenceFrames.Value = 2
        Device_Archos605.Params.VBVBufSize.Value = 5000
        Device_Archos605.Params.VBVMaxRate.Value = 5000
        ret.Add(Device_Archos605)

        Dim Device_Cellphones_PDAs_Level1_0 As New x264Encoder()
        Device_Cellphones_PDAs_Level1_0.Name = "x264/Device/Cellphones PDAs-Level1.0"
        Device_Cellphones_PDAs_Level1_0.Params.BFrames.Value = 0
        Device_Cellphones_PDAs_Level1_0.Params.CABAC.Value = False
        Device_Cellphones_PDAs_Level1_0.Params.DirectMode.Value = 0
        Device_Cellphones_PDAs_Level1_0.Params.Level.Value = 0
        Device_Cellphones_PDAs_Level1_0.Params.MEMethod.Value = 2
        Device_Cellphones_PDAs_Level1_0.Params.MeRange.Value = 12
        Device_Cellphones_PDAs_Level1_0.Params.Mode.Value = 3
        Device_Cellphones_PDAs_Level1_0.Params.QPMin.Value = 16
        Device_Cellphones_PDAs_Level1_0.Params.ReferenceFrames.Value = 1
        Device_Cellphones_PDAs_Level1_0.Params.VBVBufSize.Value = 175
        Device_Cellphones_PDAs_Level1_0.Params.VBVMaxRate.Value = 64
        ret.Add(Device_Cellphones_PDAs_Level1_0)

        Dim Device_Cellphones_PDAs_Level1_1 As New x264Encoder()
        Device_Cellphones_PDAs_Level1_1.Name = "x264/Device/Cellphones PDAs-Level1.1"
        Device_Cellphones_PDAs_Level1_1.Params.BFrames.Value = 0
        Device_Cellphones_PDAs_Level1_1.Params.CABAC.Value = False
        Device_Cellphones_PDAs_Level1_1.Params.DirectMode.Value = 0
        Device_Cellphones_PDAs_Level1_1.Params.Level.Value = 1
        Device_Cellphones_PDAs_Level1_1.Params.MEMethod.Value = 2
        Device_Cellphones_PDAs_Level1_1.Params.MeRange.Value = 12
        Device_Cellphones_PDAs_Level1_1.Params.Mode.Value = 3
        Device_Cellphones_PDAs_Level1_1.Params.QPMin.Value = 16
        Device_Cellphones_PDAs_Level1_1.Params.ReferenceFrames.Value = 1
        Device_Cellphones_PDAs_Level1_1.Params.VBVBufSize.Value = 500
        Device_Cellphones_PDAs_Level1_1.Params.VBVMaxRate.Value = 192
        ret.Add(Device_Cellphones_PDAs_Level1_1)

        Dim Device_Cellphones_PDAs_Level1_2 As New x264Encoder()
        Device_Cellphones_PDAs_Level1_2.Name = "x264/Device/Cellphones PDAs-Level1.2"
        Device_Cellphones_PDAs_Level1_2.Params.BFrames.Value = 0
        Device_Cellphones_PDAs_Level1_2.Params.CABAC.Value = False
        Device_Cellphones_PDAs_Level1_2.Params.DirectMode.Value = 0
        Device_Cellphones_PDAs_Level1_2.Params.Level.Value = 2
        Device_Cellphones_PDAs_Level1_2.Params.MEMethod.Value = 2
        Device_Cellphones_PDAs_Level1_2.Params.MeRange.Value = 12
        Device_Cellphones_PDAs_Level1_2.Params.Mode.Value = 3
        Device_Cellphones_PDAs_Level1_2.Params.QPMin.Value = 16
        Device_Cellphones_PDAs_Level1_2.Params.ReferenceFrames.Value = 1
        Device_Cellphones_PDAs_Level1_2.Params.VBVBufSize.Value = 1000
        Device_Cellphones_PDAs_Level1_2.Params.VBVMaxRate.Value = 384
        ret.Add(Device_Cellphones_PDAs_Level1_2)

        Dim Device_Cellphones_PDAs_Level1_3 As New x264Encoder()
        Device_Cellphones_PDAs_Level1_3.Name = "x264/Device/Cellphones PDAs-Level1.3"
        Device_Cellphones_PDAs_Level1_3.Params.BFrames.Value = 0
        Device_Cellphones_PDAs_Level1_3.Params.CABAC.Value = False
        Device_Cellphones_PDAs_Level1_3.Params.DirectMode.Value = 0
        Device_Cellphones_PDAs_Level1_3.Params.Level.Value = 3
        Device_Cellphones_PDAs_Level1_3.Params.MEMethod.Value = 2
        Device_Cellphones_PDAs_Level1_3.Params.MeRange.Value = 12
        Device_Cellphones_PDAs_Level1_3.Params.Mode.Value = 3
        Device_Cellphones_PDAs_Level1_3.Params.QPMin.Value = 16
        Device_Cellphones_PDAs_Level1_3.Params.ReferenceFrames.Value = 1
        Device_Cellphones_PDAs_Level1_3.Params.VBVBufSize.Value = 2000
        Device_Cellphones_PDAs_Level1_3.Params.VBVMaxRate.Value = 768
        ret.Add(Device_Cellphones_PDAs_Level1_3)

        Dim Device_iPhone As New x264Encoder()
        Device_iPhone.Name = "x264/Device/iPhone"
        Device_iPhone.Params.BFrames.Value = 0
        Device_iPhone.Params.CABAC.Value = False
        Device_iPhone.Params.DirectMode.Value = 0
        Device_iPhone.Params.Level.Value = 7
        Device_iPhone.Params.Mode.Value = 3
        Device_iPhone.Params.P4x4.Value = False
        Device_iPhone.Params.ReferenceFrames.Value = 1
        Device_iPhone.Params.VBVBufSize.Value = 10000
        Device_iPhone.Params.VBVMaxRate.Value = 10000
        ret.Add(Device_iPhone)

        Dim Device_iPod_5_5G As New x264Encoder()
        Device_iPod_5_5G.Name = "x264/Device/iPod 5.5G"
        Device_iPod_5_5G.Params.BFrames.Value = 0
        Device_iPod_5_5G.Params.CABAC.Value = False
        Device_iPod_5_5G.Params.DirectMode.Value = 0
        Device_iPod_5_5G.Params.Level.Value = 7
        Device_iPod_5_5G.Params.MEMethod.Value = 2
        Device_iPod_5_5G.Params.MeRange.Value = 12
        Device_iPod_5_5G.Params.Mode.Value = 3
        Device_iPod_5_5G.Params.P4x4.Value = False
        Device_iPod_5_5G.Params.QPMin.Value = 16
        Device_iPod_5_5G.Params.ReferenceFrames.Value = 1
        Device_iPod_5_5G.Params.VBVBufSize.Value = 1500
        Device_iPod_5_5G.Params.VBVMaxRate.Value = 1500
        ret.Add(Device_iPod_5_5G)

        Dim Device_iPod As New x264Encoder()
        Device_iPod.Name = "x264/Device/iPod"
        Device_iPod.Params.BFrames.Value = 0
        Device_iPod.Params.CABAC.Value = False
        Device_iPod.Params.DirectMode.Value = 0
        Device_iPod.Params.Level.Value = 3
        Device_iPod.Params.MEMethod.Value = 2
        Device_iPod.Params.MeRange.Value = 12
        Device_iPod.Params.Mode.Value = 3
        Device_iPod.Params.QPMin.Value = 16
        Device_iPod.Params.ReferenceFrames.Value = 1
        Device_iPod.Params.VBVBufSize.Value = 768
        Device_iPod.Params.VBVMaxRate.Value = 768
        ret.Add(Device_iPod)

        Dim Device_PSP As New x264Encoder()
        Device_PSP.Name = "x264/Device/PSP"
        Device_PSP.Params.BAdapt.Value = 2
        Device_PSP.Params.BFrames.Value = 3
        Device_PSP.Params.BFramesWeightedPrediction.Value = True
        Device_PSP.Params.DirectMode.Value = 3
        Device_PSP.Params.Level.Value = 7
        Device_PSP.Params.MEMethod.Value = 2
        Device_PSP.Params.MixedRefs.Value = True
        Device_PSP.Params.Mode.Value = 3
        Device_PSP.Params.P4x4.Value = False
        Device_PSP.Params.ReferenceFrames.Value = 3
        Device_PSP.Params.SubME.Value = 7
        Device_PSP.Params.Trellis.Value = 1
        Device_PSP.Params.VBVBufSize.Value = 10000
        Device_PSP.Params.VBVMaxRate.Value = 10000
        ret.Add(Device_PSP)

        Dim Device_Zune As New x264Encoder()
        Device_Zune.Name = "x264/Device/Zune"
        Device_Zune.Params.BFrames.Value = 0
        Device_Zune.Params.CABAC.Value = False
        Device_Zune.Params.DirectMode.Value = 0
        Device_Zune.Params.Level.Value = 7
        Device_Zune.Params.LoopFilter.Value = False
        Device_Zune.Params.MEMethod.Value = 2
        Device_Zune.Params.MeRange.Value = 12
        Device_Zune.Params.Mode.Value = 3
        Device_Zune.Params.P4x4.Value = False
        Device_Zune.Params.QPMin.Value = 16
        Device_Zune.Params.ReferenceFrames.Value = 1
        Device_Zune.Params.VBVBufSize.Value = 2500
        Device_Zune.Params.VBVMaxRate.Value = 2500
        ret.Add(Device_Zune)

        Dim DXVA_HD_Anime_Toons_Fast As New x264Encoder()
        DXVA_HD_Anime_Toons_Fast.Name = "x264/DXVA/HD-Anime Toons Fast"
        DXVA_HD_Anime_Toons_Fast.Params.AdaptiveDCT.Value = True
        DXVA_HD_Anime_Toons_Fast.Params.AQMode.Value = 0
        DXVA_HD_Anime_Toons_Fast.Params.BFrames.Value = 3
        DXVA_HD_Anime_Toons_Fast.Params.DeblockAlpha.Value = 1
        DXVA_HD_Anime_Toons_Fast.Params.DeblockBeta.Value = 1
        DXVA_HD_Anime_Toons_Fast.Params.DirectMode.Value = 3
        DXVA_HD_Anime_Toons_Fast.Params.I8x8.Value = True
        DXVA_HD_Anime_Toons_Fast.Params.Level.Value = 11
        DXVA_HD_Anime_Toons_Fast.Params.Mode.Value = 3
        DXVA_HD_Anime_Toons_Fast.Params.P4x4.Value = False
        DXVA_HD_Anime_Toons_Fast.Params.SubME.Value = 5
        DXVA_HD_Anime_Toons_Fast.Params.VBVBufSize.Value = 50000
        DXVA_HD_Anime_Toons_Fast.Params.VBVMaxRate.Value = 50000
        ret.Add(DXVA_HD_Anime_Toons_Fast)

        Dim DXVA_HD_Anime_Toons_HQ As New x264Encoder()
        DXVA_HD_Anime_Toons_HQ.Name = "x264/DXVA/HD-Anime Toons HQ"
        DXVA_HD_Anime_Toons_HQ.Params.AdaptiveDCT.Value = True
        DXVA_HD_Anime_Toons_HQ.Params.AQMode.Value = 0
        DXVA_HD_Anime_Toons_HQ.Params.BAdapt.Value = 2
        DXVA_HD_Anime_Toons_HQ.Params.BFrames.Value = 3
        DXVA_HD_Anime_Toons_HQ.Params.DeblockAlpha.Value = 1
        DXVA_HD_Anime_Toons_HQ.Params.DeblockBeta.Value = 1
        DXVA_HD_Anime_Toons_HQ.Params.DirectMode.Value = 3
        DXVA_HD_Anime_Toons_HQ.Params.I8x8.Value = True
        DXVA_HD_Anime_Toons_HQ.Params.Level.Value = 11
        DXVA_HD_Anime_Toons_HQ.Params.MEMethod.Value = 2
        DXVA_HD_Anime_Toons_HQ.Params.MixedRefs.Value = True
        DXVA_HD_Anime_Toons_HQ.Params.Mode.Value = 3
        DXVA_HD_Anime_Toons_HQ.Params.P4x4.Value = False
        DXVA_HD_Anime_Toons_HQ.Params.PsyRD.Value = 0.6
        DXVA_HD_Anime_Toons_HQ.Params.SubME.Value = 7
        DXVA_HD_Anime_Toons_HQ.Params.Trellis.Value = 1
        DXVA_HD_Anime_Toons_HQ.Params.VBVBufSize.Value = 50000
        DXVA_HD_Anime_Toons_HQ.Params.VBVMaxRate.Value = 50000
        ret.Add(DXVA_HD_Anime_Toons_HQ)

        Dim DXVA_HD_Balanced As New x264Encoder()
        DXVA_HD_Balanced.Name = "x264/DXVA/HD-Balanced"
        DXVA_HD_Balanced.Params.AdaptiveDCT.Value = True
        DXVA_HD_Balanced.Params.BFrames.Value = 3
        DXVA_HD_Balanced.Params.BFramesWeightedPrediction.Value = True
        DXVA_HD_Balanced.Params.DeblockAlpha.Value = -1
        DXVA_HD_Balanced.Params.DeblockBeta.Value = -1
        DXVA_HD_Balanced.Params.DirectMode.Value = 3
        DXVA_HD_Balanced.Params.I8x8.Value = True
        DXVA_HD_Balanced.Params.Level.Value = 11
        DXVA_HD_Balanced.Params.MixedRefs.Value = True
        DXVA_HD_Balanced.Params.Mode.Value = 3
        DXVA_HD_Balanced.Params.P4x4.Value = False
        DXVA_HD_Balanced.Params.ReferenceFrames.Value = 3
        DXVA_HD_Balanced.Params.Trellis.Value = 1
        DXVA_HD_Balanced.Params.VBVBufSize.Value = 50000
        DXVA_HD_Balanced.Params.VBVMaxRate.Value = 50000
        ret.Add(DXVA_HD_Balanced)

        Dim DXVA_HD_Extra_Quality As New x264Encoder()
        DXVA_HD_Extra_Quality.Name = "x264/DXVA/HD-Extra Quality"
        DXVA_HD_Extra_Quality.Params.AdaptiveDCT.Value = True
        DXVA_HD_Extra_Quality.Params.BAdapt.Value = 2
        DXVA_HD_Extra_Quality.Params.BFrames.Value = 3
        DXVA_HD_Extra_Quality.Params.BFramesWeightedPrediction.Value = True
        DXVA_HD_Extra_Quality.Params.DeblockAlpha.Value = -1
        DXVA_HD_Extra_Quality.Params.DeblockBeta.Value = -1
        DXVA_HD_Extra_Quality.Params.DirectMode.Value = 3
        DXVA_HD_Extra_Quality.Params.I8x8.Value = True
        DXVA_HD_Extra_Quality.Params.Level.Value = 11
        DXVA_HD_Extra_Quality.Params.MEMethod.Value = 2
        DXVA_HD_Extra_Quality.Params.MixedRefs.Value = True
        DXVA_HD_Extra_Quality.Params.Mode.Value = 3
        DXVA_HD_Extra_Quality.Params.P4x4.Value = False
        DXVA_HD_Extra_Quality.Params.SubME.Value = 8
        DXVA_HD_Extra_Quality.Params.Trellis.Value = 2
        DXVA_HD_Extra_Quality.Params.VBVBufSize.Value = 50000
        DXVA_HD_Extra_Quality.Params.VBVMaxRate.Value = 50000
        ret.Add(DXVA_HD_Extra_Quality)

        Dim DXVA_HD_Fast As New x264Encoder()
        DXVA_HD_Fast.Name = "x264/DXVA/HD-Fast"
        DXVA_HD_Fast.Params.AdaptiveDCT.Value = True
        DXVA_HD_Fast.Params.BFrames.Value = 3
        DXVA_HD_Fast.Params.BFramesWeightedPrediction.Value = True
        DXVA_HD_Fast.Params.DeblockAlpha.Value = -1
        DXVA_HD_Fast.Params.DeblockBeta.Value = -1
        DXVA_HD_Fast.Params.DirectMode.Value = 3
        DXVA_HD_Fast.Params.I8x8.Value = True
        DXVA_HD_Fast.Params.Level.Value = 11
        DXVA_HD_Fast.Params.Mode.Value = 3
        DXVA_HD_Fast.Params.P4x4.Value = False
        DXVA_HD_Fast.Params.ReferenceFrames.Value = 2
        DXVA_HD_Fast.Params.SubME.Value = 5
        DXVA_HD_Fast.Params.VBVBufSize.Value = 50000
        DXVA_HD_Fast.Params.VBVMaxRate.Value = 50000
        ret.Add(DXVA_HD_Fast)

        Dim DXVA_HD_HQ As New x264Encoder()
        DXVA_HD_HQ.Name = "x264/DXVA/HD-HQ"
        DXVA_HD_HQ.Params.AdaptiveDCT.Value = True
        DXVA_HD_HQ.Params.BAdapt.Value = 2
        DXVA_HD_HQ.Params.BFrames.Value = 3
        DXVA_HD_HQ.Params.BFramesWeightedPrediction.Value = True
        DXVA_HD_HQ.Params.DeblockAlpha.Value = -1
        DXVA_HD_HQ.Params.DeblockBeta.Value = -1
        DXVA_HD_HQ.Params.DirectMode.Value = 3
        DXVA_HD_HQ.Params.I8x8.Value = True
        DXVA_HD_HQ.Params.Level.Value = 11
        DXVA_HD_HQ.Params.MEMethod.Value = 2
        DXVA_HD_HQ.Params.MixedRefs.Value = True
        DXVA_HD_HQ.Params.Mode.Value = 3
        DXVA_HD_HQ.Params.P4x4.Value = False
        DXVA_HD_HQ.Params.SubME.Value = 7
        DXVA_HD_HQ.Params.Trellis.Value = 2
        DXVA_HD_HQ.Params.VBVBufSize.Value = 50000
        DXVA_HD_HQ.Params.VBVMaxRate.Value = 50000
        ret.Add(DXVA_HD_HQ)

        Dim DXVA_HD_Insane As New x264Encoder()
        DXVA_HD_Insane.Name = "x264/DXVA/HD-Insane"
        DXVA_HD_Insane.Params.AdaptiveDCT.Value = True
        DXVA_HD_Insane.Params.BAdapt.Value = 2
        DXVA_HD_Insane.Params.BFrames.Value = 3
        DXVA_HD_Insane.Params.BFramesWeightedPrediction.Value = True
        DXVA_HD_Insane.Params.DeblockAlpha.Value = -1
        DXVA_HD_Insane.Params.DeblockBeta.Value = -1
        DXVA_HD_Insane.Params.DirectMode.Value = 3
        DXVA_HD_Insane.Params.I8x8.Value = True
        DXVA_HD_Insane.Params.Level.Value = 11
        DXVA_HD_Insane.Params.MEMethod.Value = 4
        DXVA_HD_Insane.Params.MeRange.Value = 32
        DXVA_HD_Insane.Params.MixedRefs.Value = True
        DXVA_HD_Insane.Params.Mode.Value = 3
        DXVA_HD_Insane.Params.NoDCTDecimate.Value = True
        DXVA_HD_Insane.Params.NoFastPSkip.Value = True
        DXVA_HD_Insane.Params.P4x4.Value = False
        DXVA_HD_Insane.Params.SubME.Value = 9
        DXVA_HD_Insane.Params.Trellis.Value = 2
        DXVA_HD_Insane.Params.VBVBufSize.Value = 50000
        DXVA_HD_Insane.Params.VBVMaxRate.Value = 50000
        ret.Add(DXVA_HD_Insane)

        Dim DXVA_SD_Anime_Toons_Fast As New x264Encoder()
        DXVA_SD_Anime_Toons_Fast.Name = "x264/DXVA/SD-Anime Toons Fast"
        DXVA_SD_Anime_Toons_Fast.Params.AdaptiveDCT.Value = True
        DXVA_SD_Anime_Toons_Fast.Params.AQMode.Value = 0
        DXVA_SD_Anime_Toons_Fast.Params.BFrames.Value = 3
        DXVA_SD_Anime_Toons_Fast.Params.DeblockAlpha.Value = 1
        DXVA_SD_Anime_Toons_Fast.Params.DeblockBeta.Value = 1
        DXVA_SD_Anime_Toons_Fast.Params.DirectMode.Value = 3
        DXVA_SD_Anime_Toons_Fast.Params.I8x8.Value = True
        DXVA_SD_Anime_Toons_Fast.Params.Level.Value = 8
        DXVA_SD_Anime_Toons_Fast.Params.Mode.Value = 3
        DXVA_SD_Anime_Toons_Fast.Params.P4x4.Value = False
        DXVA_SD_Anime_Toons_Fast.Params.SubME.Value = 5
        DXVA_SD_Anime_Toons_Fast.Params.VBVBufSize.Value = 14000
        DXVA_SD_Anime_Toons_Fast.Params.VBVMaxRate.Value = 17500
        ret.Add(DXVA_SD_Anime_Toons_Fast)

        Dim DXVA_SD_Anime_Toons_HQ As New x264Encoder()
        DXVA_SD_Anime_Toons_HQ.Name = "x264/DXVA/SD-Anime Toons HQ"
        DXVA_SD_Anime_Toons_HQ.Params.AdaptiveDCT.Value = True
        DXVA_SD_Anime_Toons_HQ.Params.AQMode.Value = 0
        DXVA_SD_Anime_Toons_HQ.Params.BAdapt.Value = 2
        DXVA_SD_Anime_Toons_HQ.Params.BFrames.Value = 3
        DXVA_SD_Anime_Toons_HQ.Params.DeblockAlpha.Value = 1
        DXVA_SD_Anime_Toons_HQ.Params.DeblockBeta.Value = 1
        DXVA_SD_Anime_Toons_HQ.Params.DirectMode.Value = 3
        DXVA_SD_Anime_Toons_HQ.Params.I8x8.Value = True
        DXVA_SD_Anime_Toons_HQ.Params.Level.Value = 8
        DXVA_SD_Anime_Toons_HQ.Params.MEMethod.Value = 2
        DXVA_SD_Anime_Toons_HQ.Params.MixedRefs.Value = True
        DXVA_SD_Anime_Toons_HQ.Params.Mode.Value = 3
        DXVA_SD_Anime_Toons_HQ.Params.P4x4.Value = False
        DXVA_SD_Anime_Toons_HQ.Params.PsyRD.Value = 0.6
        DXVA_SD_Anime_Toons_HQ.Params.ReferenceFrames.Value = 8
        DXVA_SD_Anime_Toons_HQ.Params.SubME.Value = 7
        DXVA_SD_Anime_Toons_HQ.Params.Trellis.Value = 2
        DXVA_SD_Anime_Toons_HQ.Params.VBVBufSize.Value = 14000
        DXVA_SD_Anime_Toons_HQ.Params.VBVMaxRate.Value = 17500
        ret.Add(DXVA_SD_Anime_Toons_HQ)

        Dim DXVA_SD_Balanced As New x264Encoder()
        DXVA_SD_Balanced.Name = "x264/DXVA/SD-Balanced"
        DXVA_SD_Balanced.Params.AdaptiveDCT.Value = True
        DXVA_SD_Balanced.Params.BFrames.Value = 3
        DXVA_SD_Balanced.Params.BFramesWeightedPrediction.Value = True
        DXVA_SD_Balanced.Params.DeblockAlpha.Value = -1
        DXVA_SD_Balanced.Params.DeblockBeta.Value = -1
        DXVA_SD_Balanced.Params.DirectMode.Value = 3
        DXVA_SD_Balanced.Params.I8x8.Value = True
        DXVA_SD_Balanced.Params.Level.Value = 8
        DXVA_SD_Balanced.Params.MixedRefs.Value = True
        DXVA_SD_Balanced.Params.Mode.Value = 3
        DXVA_SD_Balanced.Params.P4x4.Value = False
        DXVA_SD_Balanced.Params.ReferenceFrames.Value = 3
        DXVA_SD_Balanced.Params.Trellis.Value = 1
        DXVA_SD_Balanced.Params.VBVBufSize.Value = 14000
        DXVA_SD_Balanced.Params.VBVMaxRate.Value = 17500
        ret.Add(DXVA_SD_Balanced)

        Dim DXVA_SD_Extra_Quality As New x264Encoder()
        DXVA_SD_Extra_Quality.Name = "x264/DXVA/SD-Extra Quality"
        DXVA_SD_Extra_Quality.Params.AdaptiveDCT.Value = True
        DXVA_SD_Extra_Quality.Params.BAdapt.Value = 2
        DXVA_SD_Extra_Quality.Params.BFrames.Value = 3
        DXVA_SD_Extra_Quality.Params.BFramesWeightedPrediction.Value = True
        DXVA_SD_Extra_Quality.Params.DeblockAlpha.Value = -1
        DXVA_SD_Extra_Quality.Params.DeblockBeta.Value = -1
        DXVA_SD_Extra_Quality.Params.DirectMode.Value = 3
        DXVA_SD_Extra_Quality.Params.I8x8.Value = True
        DXVA_SD_Extra_Quality.Params.Level.Value = 8
        DXVA_SD_Extra_Quality.Params.MEMethod.Value = 2
        DXVA_SD_Extra_Quality.Params.MixedRefs.Value = True
        DXVA_SD_Extra_Quality.Params.Mode.Value = 3
        DXVA_SD_Extra_Quality.Params.P4x4.Value = False
        DXVA_SD_Extra_Quality.Params.ReferenceFrames.Value = 8
        DXVA_SD_Extra_Quality.Params.SubME.Value = 8
        DXVA_SD_Extra_Quality.Params.Trellis.Value = 2
        DXVA_SD_Extra_Quality.Params.VBVBufSize.Value = 14000
        DXVA_SD_Extra_Quality.Params.VBVMaxRate.Value = 17500
        ret.Add(DXVA_SD_Extra_Quality)

        Dim DXVA_SD_Fast As New x264Encoder()
        DXVA_SD_Fast.Name = "x264/DXVA/SD-Fast"
        DXVA_SD_Fast.Params.AdaptiveDCT.Value = True
        DXVA_SD_Fast.Params.BFrames.Value = 3
        DXVA_SD_Fast.Params.BFramesWeightedPrediction.Value = True
        DXVA_SD_Fast.Params.DeblockAlpha.Value = -1
        DXVA_SD_Fast.Params.DeblockBeta.Value = -1
        DXVA_SD_Fast.Params.DirectMode.Value = 3
        DXVA_SD_Fast.Params.I8x8.Value = True
        DXVA_SD_Fast.Params.Level.Value = 8
        DXVA_SD_Fast.Params.Mode.Value = 3
        DXVA_SD_Fast.Params.P4x4.Value = False
        DXVA_SD_Fast.Params.ReferenceFrames.Value = 2
        DXVA_SD_Fast.Params.SubME.Value = 5
        DXVA_SD_Fast.Params.VBVBufSize.Value = 14000
        DXVA_SD_Fast.Params.VBVMaxRate.Value = 17500
        ret.Add(DXVA_SD_Fast)

        Dim DXVA_SD_HQ As New x264Encoder()
        DXVA_SD_HQ.Name = "x264/DXVA/SD-HQ"
        DXVA_SD_HQ.Params.AdaptiveDCT.Value = True
        DXVA_SD_HQ.Params.BAdapt.Value = 2
        DXVA_SD_HQ.Params.BFrames.Value = 3
        DXVA_SD_HQ.Params.BFramesWeightedPrediction.Value = True
        DXVA_SD_HQ.Params.DeblockAlpha.Value = -1
        DXVA_SD_HQ.Params.DeblockBeta.Value = -1
        DXVA_SD_HQ.Params.DirectMode.Value = 3
        DXVA_SD_HQ.Params.I8x8.Value = True
        DXVA_SD_HQ.Params.Level.Value = 8
        DXVA_SD_HQ.Params.MEMethod.Value = 2
        DXVA_SD_HQ.Params.MixedRefs.Value = True
        DXVA_SD_HQ.Params.Mode.Value = 3
        DXVA_SD_HQ.Params.P4x4.Value = False
        DXVA_SD_HQ.Params.ReferenceFrames.Value = 8
        DXVA_SD_HQ.Params.SubME.Value = 7
        DXVA_SD_HQ.Params.Trellis.Value = 2
        DXVA_SD_HQ.Params.VBVBufSize.Value = 14000
        DXVA_SD_HQ.Params.VBVMaxRate.Value = 17500
        ret.Add(DXVA_SD_HQ)

        Dim DXVA_SD_Insane As New x264Encoder()
        DXVA_SD_Insane.Name = "x264/DXVA/SD-Insane"
        DXVA_SD_Insane.Params.AdaptiveDCT.Value = True
        DXVA_SD_Insane.Params.BAdapt.Value = 2
        DXVA_SD_Insane.Params.BFrames.Value = 3
        DXVA_SD_Insane.Params.BFramesWeightedPrediction.Value = True
        DXVA_SD_Insane.Params.DeblockAlpha.Value = -1
        DXVA_SD_Insane.Params.DeblockBeta.Value = -1
        DXVA_SD_Insane.Params.DirectMode.Value = 3
        DXVA_SD_Insane.Params.I8x8.Value = True
        DXVA_SD_Insane.Params.Level.Value = 8
        DXVA_SD_Insane.Params.MEMethod.Value = 4
        DXVA_SD_Insane.Params.MeRange.Value = 32
        DXVA_SD_Insane.Params.MixedRefs.Value = True
        DXVA_SD_Insane.Params.Mode.Value = 3
        DXVA_SD_Insane.Params.NoDCTDecimate.Value = True
        DXVA_SD_Insane.Params.NoFastPSkip.Value = True
        DXVA_SD_Insane.Params.P4x4.Value = False
        DXVA_SD_Insane.Params.ReferenceFrames.Value = 8
        DXVA_SD_Insane.Params.SubME.Value = 9
        DXVA_SD_Insane.Params.Trellis.Value = 2
        DXVA_SD_Insane.Params.VBVBufSize.Value = 14000
        DXVA_SD_Insane.Params.VBVMaxRate.Value = 17500
        ret.Add(DXVA_SD_Insane)

        Dim Standalone_AppleTV As New x264Encoder()
        Standalone_AppleTV.Name = "x264/Standalone/AppleTV"
        Standalone_AppleTV.Params.BAdapt.Value = 2
        Standalone_AppleTV.Params.BFrames.Value = 3
        Standalone_AppleTV.Params.BFramesWeightedPrediction.Value = True
        Standalone_AppleTV.Params.CABAC.Value = False
        Standalone_AppleTV.Params.DirectMode.Value = 0
        Standalone_AppleTV.Params.Level.Value = 8
        Standalone_AppleTV.Params.MEMethod.Value = 2
        Standalone_AppleTV.Params.MeRange.Value = 12
        Standalone_AppleTV.Params.Mode.Value = 3
        Standalone_AppleTV.Params.P4x4.Value = False
        Standalone_AppleTV.Params.QPMin.Value = 16
        Standalone_AppleTV.Params.ReferenceFrames.Value = 1
        Standalone_AppleTV.Params.SubME.Value = 7
        Standalone_AppleTV.Params.VBVBufSize.Value = 5000
        Standalone_AppleTV.Params.VBVMaxRate.Value = 12000
        ret.Add(Standalone_AppleTV)

        Dim Standalone_AVC_HD As New x264Encoder()
        Standalone_AVC_HD.Name = "x264/Standalone/AVC-HD"
        Standalone_AVC_HD.Params.AdaptiveDCT.Value = True
        Standalone_AVC_HD.Params.AddAll.Value = "--mvrange 511"
        Standalone_AVC_HD.Params.BAdapt.Value = 2
        Standalone_AVC_HD.Params.BFrames.Value = 3
        Standalone_AVC_HD.Params.BFramesWeightedPrediction.Value = True
        Standalone_AVC_HD.Params.DirectMode.Value = 3
        Standalone_AVC_HD.Params.GOPSizeMax.Value = 24
        Standalone_AVC_HD.Params.GOPSizeMin.Value = 2
        Standalone_AVC_HD.Params.I8x8.Value = True
        Standalone_AVC_HD.Params.IPRatio.Value = 1.1
        Standalone_AVC_HD.Params.Level.Value = 11
        Standalone_AVC_HD.Params.MEMethod.Value = 2
        Standalone_AVC_HD.Params.MixedRefs.Value = True
        Standalone_AVC_HD.Params.Mode.Value = 3
        Standalone_AVC_HD.Params.P4x4.Value = False
        Standalone_AVC_HD.Params.PBRatio.Value = 1.1
        Standalone_AVC_HD.Params.QComp.Value = 0.5
        Standalone_AVC_HD.Params.ReferenceFrames.Value = 3
        Standalone_AVC_HD.Params.SubME.Value = 7
        Standalone_AVC_HD.Params.Trellis.Value = 1
        Standalone_AVC_HD.Params.VBVBufSize.Value = 16500
        Standalone_AVC_HD.Params.VBVMaxRate.Value = 16500
        ret.Add(Standalone_AVC_HD)

        Dim Standalone_Blu_ray As New x264Encoder()
        Standalone_Blu_ray.Name = "x264/Standalone/Blu-ray"
        Standalone_Blu_ray.Params.AdaptiveDCT.Value = True
        Standalone_Blu_ray.Params.AddAll.Value = "--mvrange 511"
        Standalone_Blu_ray.Params.BAdapt.Value = 2
        Standalone_Blu_ray.Params.BFrames.Value = 3
        Standalone_Blu_ray.Params.BFramesWeightedPrediction.Value = True
        Standalone_Blu_ray.Params.DeblockAlpha.Value = -1
        Standalone_Blu_ray.Params.DeblockBeta.Value = -1
        Standalone_Blu_ray.Params.DirectMode.Value = 3
        Standalone_Blu_ray.Params.GOPSizeMax.Value = 24
        Standalone_Blu_ray.Params.GOPSizeMin.Value = 2
        Standalone_Blu_ray.Params.I8x8.Value = True
        Standalone_Blu_ray.Params.IPRatio.Value = 1.1
        Standalone_Blu_ray.Params.Level.Value = 11
        Standalone_Blu_ray.Params.MEMethod.Value = 2
        Standalone_Blu_ray.Params.MixedRefs.Value = True
        Standalone_Blu_ray.Params.Mode.Value = 3
        Standalone_Blu_ray.Params.P4x4.Value = False
        Standalone_Blu_ray.Params.PBRatio.Value = 1.1
        Standalone_Blu_ray.Params.QComp.Value = 0.5
        Standalone_Blu_ray.Params.ReferenceFrames.Value = 3
        Standalone_Blu_ray.Params.SubME.Value = 7
        Standalone_Blu_ray.Params.Trellis.Value = 2
        Standalone_Blu_ray.Params.VBVBufSize.Value = 40000
        Standalone_Blu_ray.Params.VBVMaxRate.Value = 40000
        ret.Add(Standalone_Blu_ray)

        Dim Standalone_Blu_ray_Fast As New x264Encoder()
        Standalone_Blu_ray_Fast.Name = "x264/Standalone/Blu-ray Fast"
        Standalone_Blu_ray_Fast.Params.AdaptiveDCT.Value = True
        Standalone_Blu_ray_Fast.Params.AddAll.Value = "--mvrange 511"
        Standalone_Blu_ray_Fast.Params.BFrames.Value = 3
        Standalone_Blu_ray_Fast.Params.BFramesWeightedPrediction.Value = True
        Standalone_Blu_ray_Fast.Params.DeblockAlpha.Value = -1
        Standalone_Blu_ray_Fast.Params.DeblockBeta.Value = -1
        Standalone_Blu_ray_Fast.Params.DirectMode.Value = 3
        Standalone_Blu_ray_Fast.Params.GOPSizeMax.Value = 24
        Standalone_Blu_ray_Fast.Params.GOPSizeMin.Value = 2
        Standalone_Blu_ray_Fast.Params.I8x8.Value = True
        Standalone_Blu_ray_Fast.Params.IPRatio.Value = 1.1
        Standalone_Blu_ray_Fast.Params.Level.Value = 11
        Standalone_Blu_ray_Fast.Params.MeRange.Value = 12
        Standalone_Blu_ray_Fast.Params.MixedRefs.Value = True
        Standalone_Blu_ray_Fast.Params.Mode.Value = 3
        Standalone_Blu_ray_Fast.Params.P4x4.Value = False
        Standalone_Blu_ray_Fast.Params.PBRatio.Value = 1.1
        Standalone_Blu_ray_Fast.Params.QComp.Value = 0.5
        Standalone_Blu_ray_Fast.Params.ReferenceFrames.Value = 3
        Standalone_Blu_ray_Fast.Params.SubME.Value = 5
        Standalone_Blu_ray_Fast.Params.VBVBufSize.Value = 40000
        Standalone_Blu_ray_Fast.Params.VBVMaxRate.Value = 40000
        ret.Add(Standalone_Blu_ray_Fast)

        Dim Standalone_HD_DVD As New x264Encoder()
        Standalone_HD_DVD.Name = "x264/Standalone/HD-DVD"
        Standalone_HD_DVD.Params.AdaptiveDCT.Value = True
        Standalone_HD_DVD.Params.AddAll.Value = "--mvrange 511"
        Standalone_HD_DVD.Params.BAdapt.Value = 2
        Standalone_HD_DVD.Params.BFrames.Value = 2
        Standalone_HD_DVD.Params.BFramesWeightedPrediction.Value = True
        Standalone_HD_DVD.Params.DeblockAlpha.Value = -1
        Standalone_HD_DVD.Params.DeblockBeta.Value = -1
        Standalone_HD_DVD.Params.DirectMode.Value = 3
        Standalone_HD_DVD.Params.GOPSizeMax.Value = 14
        Standalone_HD_DVD.Params.GOPSizeMin.Value = 2
        Standalone_HD_DVD.Params.I8x8.Value = True
        Standalone_HD_DVD.Params.IPRatio.Value = 1.1
        Standalone_HD_DVD.Params.Level.Value = 11
        Standalone_HD_DVD.Params.MEMethod.Value = 2
        Standalone_HD_DVD.Params.MixedRefs.Value = True
        Standalone_HD_DVD.Params.Mode.Value = 3
        Standalone_HD_DVD.Params.P4x4.Value = False
        Standalone_HD_DVD.Params.PBRatio.Value = 1.1
        Standalone_HD_DVD.Params.QComp.Value = 0.5
        Standalone_HD_DVD.Params.ReferenceFrames.Value = 3
        Standalone_HD_DVD.Params.SubME.Value = 7
        Standalone_HD_DVD.Params.Trellis.Value = 2
        Standalone_HD_DVD.Params.VBVBufSize.Value = 14475
        Standalone_HD_DVD.Params.VBVMaxRate.Value = 24000
        ret.Add(Standalone_HD_DVD)

        Dim Standalone_HD_DVD_Fast As New x264Encoder()
        Standalone_HD_DVD_Fast.Name = "x264/Standalone/HD-DVD Fast"
        Standalone_HD_DVD_Fast.Params.AdaptiveDCT.Value = True
        Standalone_HD_DVD_Fast.Params.AddAll.Value = "--mvrange 511"
        Standalone_HD_DVD_Fast.Params.BFrames.Value = 2
        Standalone_HD_DVD_Fast.Params.BFramesWeightedPrediction.Value = True
        Standalone_HD_DVD_Fast.Params.DeblockAlpha.Value = -1
        Standalone_HD_DVD_Fast.Params.DeblockBeta.Value = -1
        Standalone_HD_DVD_Fast.Params.DirectMode.Value = 3
        Standalone_HD_DVD_Fast.Params.GOPSizeMax.Value = 14
        Standalone_HD_DVD_Fast.Params.GOPSizeMin.Value = 2
        Standalone_HD_DVD_Fast.Params.I8x8.Value = True
        Standalone_HD_DVD_Fast.Params.IPRatio.Value = 1.1
        Standalone_HD_DVD_Fast.Params.Level.Value = 11
        Standalone_HD_DVD_Fast.Params.MeRange.Value = 12
        Standalone_HD_DVD_Fast.Params.MixedRefs.Value = True
        Standalone_HD_DVD_Fast.Params.Mode.Value = 3
        Standalone_HD_DVD_Fast.Params.P4x4.Value = False
        Standalone_HD_DVD_Fast.Params.PBRatio.Value = 1.1
        Standalone_HD_DVD_Fast.Params.QComp.Value = 0.5
        Standalone_HD_DVD_Fast.Params.ReferenceFrames.Value = 2
        Standalone_HD_DVD_Fast.Params.SubME.Value = 5
        Standalone_HD_DVD_Fast.Params.VBVBufSize.Value = 14475
        Standalone_HD_DVD_Fast.Params.VBVMaxRate.Value = 24000
        ret.Add(Standalone_HD_DVD_Fast)

        Dim Standalone_PS3_Xbox360 As New x264Encoder()
        Standalone_PS3_Xbox360.Name = "x264/Standalone/PS3-Xbox360"
        Standalone_PS3_Xbox360.Params.AdaptiveDCT.Value = True
        Standalone_PS3_Xbox360.Params.BAdapt.Value = 2
        Standalone_PS3_Xbox360.Params.BFrames.Value = 3
        Standalone_PS3_Xbox360.Params.BFramesWeightedPrediction.Value = True
        Standalone_PS3_Xbox360.Params.DirectMode.Value = 3
        Standalone_PS3_Xbox360.Params.I8x8.Value = True
        Standalone_PS3_Xbox360.Params.IPRatio.Value = 1.1
        Standalone_PS3_Xbox360.Params.Level.Value = 11
        Standalone_PS3_Xbox360.Params.MEMethod.Value = 2
        Standalone_PS3_Xbox360.Params.MeRange.Value = 12
        Standalone_PS3_Xbox360.Params.MixedRefs.Value = True
        Standalone_PS3_Xbox360.Params.Mode.Value = 3
        Standalone_PS3_Xbox360.Params.P4x4.Value = False
        Standalone_PS3_Xbox360.Params.PBRatio.Value = 1.1
        Standalone_PS3_Xbox360.Params.QComp.Value = 0.5
        Standalone_PS3_Xbox360.Params.ReferenceFrames.Value = 3
        Standalone_PS3_Xbox360.Params.SubME.Value = 7
        Standalone_PS3_Xbox360.Params.Trellis.Value = 2
        Standalone_PS3_Xbox360.Params.VBVBufSize.Value = 24000
        Standalone_PS3_Xbox360.Params.VBVMaxRate.Value = 24000
        ret.Add(Standalone_PS3_Xbox360)

        Dim Standalone_PS3_Xbox360_Fast As New x264Encoder()
        Standalone_PS3_Xbox360_Fast.Name = "x264/Standalone/PS3-Xbox360 Fast"
        Standalone_PS3_Xbox360_Fast.Params.AdaptiveDCT.Value = True
        Standalone_PS3_Xbox360_Fast.Params.BFrames.Value = 3
        Standalone_PS3_Xbox360_Fast.Params.BFramesWeightedPrediction.Value = True
        Standalone_PS3_Xbox360_Fast.Params.DirectMode.Value = 3
        Standalone_PS3_Xbox360_Fast.Params.I8x8.Value = True
        Standalone_PS3_Xbox360_Fast.Params.IPRatio.Value = 1.1
        Standalone_PS3_Xbox360_Fast.Params.Level.Value = 11
        Standalone_PS3_Xbox360_Fast.Params.Mode.Value = 3
        Standalone_PS3_Xbox360_Fast.Params.P4x4.Value = False
        Standalone_PS3_Xbox360_Fast.Params.PBRatio.Value = 1.1
        Standalone_PS3_Xbox360_Fast.Params.QComp.Value = 0.5
        Standalone_PS3_Xbox360_Fast.Params.ReferenceFrames.Value = 1
        Standalone_PS3_Xbox360_Fast.Params.SubME.Value = 5
        Standalone_PS3_Xbox360_Fast.Params.VBVBufSize.Value = 24000
        Standalone_PS3_Xbox360_Fast.Params.VBVMaxRate.Value = 24000
        ret.Add(Standalone_PS3_Xbox360_Fast)

        Dim Standalone_Xbox_HiRes As New x264Encoder()
        Standalone_Xbox_HiRes.Name = "x264/Standalone/Xbox HiRes"
        Standalone_Xbox_HiRes.Params.AdaptiveDCT.Value = True
        Standalone_Xbox_HiRes.Params.BAdapt.Value = 2
        Standalone_Xbox_HiRes.Params.BFrames.Value = 2
        Standalone_Xbox_HiRes.Params.CABAC.Value = False
        Standalone_Xbox_HiRes.Params.DirectMode.Value = 3
        Standalone_Xbox_HiRes.Params.I8x8.Value = True
        Standalone_Xbox_HiRes.Params.LoopFilter.Value = False
        Standalone_Xbox_HiRes.Params.MEMethod.Value = 2
        Standalone_Xbox_HiRes.Params.Mode.Value = 3
        Standalone_Xbox_HiRes.Params.NoFastPSkip.Value = True
        Standalone_Xbox_HiRes.Params.QComp.Value = 0.3
        Standalone_Xbox_HiRes.Params.QPMin.Value = 18
        Standalone_Xbox_HiRes.Params.ReferenceFrames.Value = 1
        Standalone_Xbox_HiRes.Params.SubME.Value = 7
        Standalone_Xbox_HiRes.Params.VBVBufSize.Value = 1500
        Standalone_Xbox_HiRes.Params.VBVMaxRate.Value = 1500
        ret.Add(Standalone_Xbox_HiRes)

        Dim Standalone_Xbox_LoRes As New x264Encoder()
        Standalone_Xbox_LoRes.Name = "x264/Standalone/Xbox LoRes"
        Standalone_Xbox_LoRes.Params.AdaptiveDCT.Value = True
        Standalone_Xbox_LoRes.Params.BAdapt.Value = 2
        Standalone_Xbox_LoRes.Params.BFrames.Value = 3
        Standalone_Xbox_LoRes.Params.BFramesPyramid.Value = True
        Standalone_Xbox_LoRes.Params.BFramesWeightedPrediction.Value = True
        Standalone_Xbox_LoRes.Params.DirectMode.Value = 3
        Standalone_Xbox_LoRes.Params.I8x8.Value = True
        Standalone_Xbox_LoRes.Params.MEMethod.Value = 2
        Standalone_Xbox_LoRes.Params.Mode.Value = 3
        Standalone_Xbox_LoRes.Params.NoFastPSkip.Value = True
        Standalone_Xbox_LoRes.Params.QComp.Value = 0.3
        Standalone_Xbox_LoRes.Params.QPMin.Value = 18
        Standalone_Xbox_LoRes.Params.ReferenceFrames.Value = 1
        Standalone_Xbox_LoRes.Params.SubME.Value = 7
        Standalone_Xbox_LoRes.Params.Trellis.Value = 2
        Standalone_Xbox_LoRes.Params.VBVBufSize.Value = 1500
        Standalone_Xbox_LoRes.Params.VBVMaxRate.Value = 1500
        ret.Add(Standalone_Xbox_LoRes)

        Dim Unrestricted_1pass_ABR_Fast As New x264Encoder()
        Unrestricted_1pass_ABR_Fast.Name = "x264/Unrestricted/1pass ABR Fast"
        Unrestricted_1pass_ABR_Fast.Params.B8x8.Value = False
        Unrestricted_1pass_ABR_Fast.Params.BFrames.Value = 16
        Unrestricted_1pass_ABR_Fast.Params.I4x4.Value = False
        Unrestricted_1pass_ABR_Fast.Params.Mode.Value = 0
        Unrestricted_1pass_ABR_Fast.Params.P4x4.Value = False
        Unrestricted_1pass_ABR_Fast.Params.P8x8.Value = False
        Unrestricted_1pass_ABR_Fast.Params.ReferenceFrames.Value = 1
        Unrestricted_1pass_ABR_Fast.Params.SubME.Value = 5
        ret.Add(Unrestricted_1pass_ABR_Fast)

        Dim Unrestricted_1pass_ABR As New x264Encoder()
        Unrestricted_1pass_ABR.Name = "x264/Unrestricted/1pass ABR"
        Unrestricted_1pass_ABR.Params.AdaptiveDCT.Value = True
        Unrestricted_1pass_ABR.Params.BFrames.Value = 16
        Unrestricted_1pass_ABR.Params.BFramesPyramid.Value = True
        Unrestricted_1pass_ABR.Params.BFramesWeightedPrediction.Value = True
        Unrestricted_1pass_ABR.Params.I8x8.Value = True
        Unrestricted_1pass_ABR.Params.MEMethod.Value = 2
        Unrestricted_1pass_ABR.Params.Mode.Value = 0
        Unrestricted_1pass_ABR.Params.ReferenceFrames.Value = 3
        Unrestricted_1pass_ABR.Params.SubME.Value = 7
        Unrestricted_1pass_ABR.Params.Trellis.Value = 1
        ret.Add(Unrestricted_1pass_ABR)

        Dim Unrestricted_1pass_Const__Quality_Anime_Toons As New x264Encoder()
        Unrestricted_1pass_Const__Quality_Anime_Toons.Name = "x264/Unrestricted/1pass Const. Quality Anime Toons"
        Unrestricted_1pass_Const__Quality_Anime_Toons.Params.AdaptiveDCT.Value = True
        Unrestricted_1pass_Const__Quality_Anime_Toons.Params.AQMode.Value = 0
        Unrestricted_1pass_Const__Quality_Anime_Toons.Params.BFrames.Value = 16
        Unrestricted_1pass_Const__Quality_Anime_Toons.Params.BFramesPyramid.Value = True
        Unrestricted_1pass_Const__Quality_Anime_Toons.Params.DeblockAlpha.Value = 1
        Unrestricted_1pass_Const__Quality_Anime_Toons.Params.DeblockBeta.Value = 1
        Unrestricted_1pass_Const__Quality_Anime_Toons.Params.I8x8.Value = True
        Unrestricted_1pass_Const__Quality_Anime_Toons.Params.MEMethod.Value = 2
        Unrestricted_1pass_Const__Quality_Anime_Toons.Params.MixedRefs.Value = True
        Unrestricted_1pass_Const__Quality_Anime_Toons.Params.Mode.Value = 2
        Unrestricted_1pass_Const__Quality_Anime_Toons.Params.PsyRD.Value = 0.6
        Unrestricted_1pass_Const__Quality_Anime_Toons.Params.Quant.Value = 18
        Unrestricted_1pass_Const__Quality_Anime_Toons.Params.ReferenceFrames.Value = 8
        Unrestricted_1pass_Const__Quality_Anime_Toons.Params.SubME.Value = 7
        Unrestricted_1pass_Const__Quality_Anime_Toons.Params.Trellis.Value = 2
        ret.Add(Unrestricted_1pass_Const__Quality_Anime_Toons)

        Dim Unrestricted_1pass_Const__Quality_Balanced As New x264Encoder()
        Unrestricted_1pass_Const__Quality_Balanced.Name = "x264/Unrestricted/1pass Const. Quality Balanced"
        Unrestricted_1pass_Const__Quality_Balanced.Params.AdaptiveDCT.Value = True
        Unrestricted_1pass_Const__Quality_Balanced.Params.BFrames.Value = 16
        Unrestricted_1pass_Const__Quality_Balanced.Params.BFramesPyramid.Value = True
        Unrestricted_1pass_Const__Quality_Balanced.Params.BFramesWeightedPrediction.Value = True
        Unrestricted_1pass_Const__Quality_Balanced.Params.DeblockAlpha.Value = -1
        Unrestricted_1pass_Const__Quality_Balanced.Params.DeblockBeta.Value = -1
        Unrestricted_1pass_Const__Quality_Balanced.Params.I8x8.Value = True
        Unrestricted_1pass_Const__Quality_Balanced.Params.MixedRefs.Value = True
        Unrestricted_1pass_Const__Quality_Balanced.Params.Mode.Value = 2
        Unrestricted_1pass_Const__Quality_Balanced.Params.Quant.Value = 18
        Unrestricted_1pass_Const__Quality_Balanced.Params.ReferenceFrames.Value = 3
        Unrestricted_1pass_Const__Quality_Balanced.Params.Trellis.Value = 1
        ret.Add(Unrestricted_1pass_Const__Quality_Balanced)

        Dim Unrestricted_1pass_Const__Quality_Extra_Q_ As New x264Encoder()
        Unrestricted_1pass_Const__Quality_Extra_Q_.Name = "x264/Unrestricted/1pass Const. Quality Extra Q."
        Unrestricted_1pass_Const__Quality_Extra_Q_.Params.AdaptiveDCT.Value = True
        Unrestricted_1pass_Const__Quality_Extra_Q_.Params.BAdapt.Value = 2
        Unrestricted_1pass_Const__Quality_Extra_Q_.Params.BFrames.Value = 3
        Unrestricted_1pass_Const__Quality_Extra_Q_.Params.BFramesPyramid.Value = True
        Unrestricted_1pass_Const__Quality_Extra_Q_.Params.BFramesWeightedPrediction.Value = True
        Unrestricted_1pass_Const__Quality_Extra_Q_.Params.DeblockAlpha.Value = -1
        Unrestricted_1pass_Const__Quality_Extra_Q_.Params.DeblockBeta.Value = -1
        Unrestricted_1pass_Const__Quality_Extra_Q_.Params.I8x8.Value = True
        Unrestricted_1pass_Const__Quality_Extra_Q_.Params.MEMethod.Value = 2
        Unrestricted_1pass_Const__Quality_Extra_Q_.Params.MixedRefs.Value = True
        Unrestricted_1pass_Const__Quality_Extra_Q_.Params.Mode.Value = 2
        Unrestricted_1pass_Const__Quality_Extra_Q_.Params.Quant.Value = 18
        Unrestricted_1pass_Const__Quality_Extra_Q_.Params.ReferenceFrames.Value = 5
        Unrestricted_1pass_Const__Quality_Extra_Q_.Params.SubME.Value = 8
        Unrestricted_1pass_Const__Quality_Extra_Q_.Params.Trellis.Value = 2
        ret.Add(Unrestricted_1pass_Const__Quality_Extra_Q_)

        Dim Unrestricted_1pass_Const__Quality_Fast As New x264Encoder()
        Unrestricted_1pass_Const__Quality_Fast.Name = "x264/Unrestricted/1pass Const. Quality Fast"
        Unrestricted_1pass_Const__Quality_Fast.Params.AdaptiveDCT.Value = True
        Unrestricted_1pass_Const__Quality_Fast.Params.BFrames.Value = 16
        Unrestricted_1pass_Const__Quality_Fast.Params.DeblockAlpha.Value = -1
        Unrestricted_1pass_Const__Quality_Fast.Params.DeblockBeta.Value = -1
        Unrestricted_1pass_Const__Quality_Fast.Params.I8x8.Value = True
        Unrestricted_1pass_Const__Quality_Fast.Params.Mode.Value = 2
        Unrestricted_1pass_Const__Quality_Fast.Params.Quant.Value = 18
        Unrestricted_1pass_Const__Quality_Fast.Params.ReferenceFrames.Value = 3
        Unrestricted_1pass_Const__Quality_Fast.Params.SubME.Value = 5
        ret.Add(Unrestricted_1pass_Const__Quality_Fast)

        Dim Unrestricted_1pass_Const__Quality_HQ As New x264Encoder()
        Unrestricted_1pass_Const__Quality_HQ.Name = "x264/Unrestricted/1pass Const. Quality HQ"
        Unrestricted_1pass_Const__Quality_HQ.Params.AdaptiveDCT.Value = True
        Unrestricted_1pass_Const__Quality_HQ.Params.BAdapt.Value = 2
        Unrestricted_1pass_Const__Quality_HQ.Params.BFrames.Value = 3
        Unrestricted_1pass_Const__Quality_HQ.Params.BFramesPyramid.Value = True
        Unrestricted_1pass_Const__Quality_HQ.Params.BFramesWeightedPrediction.Value = True
        Unrestricted_1pass_Const__Quality_HQ.Params.DeblockAlpha.Value = -1
        Unrestricted_1pass_Const__Quality_HQ.Params.DeblockBeta.Value = -1
        Unrestricted_1pass_Const__Quality_HQ.Params.I8x8.Value = True
        Unrestricted_1pass_Const__Quality_HQ.Params.MEMethod.Value = 2
        Unrestricted_1pass_Const__Quality_HQ.Params.MixedRefs.Value = True
        Unrestricted_1pass_Const__Quality_HQ.Params.Mode.Value = 2
        Unrestricted_1pass_Const__Quality_HQ.Params.Quant.Value = 18
        Unrestricted_1pass_Const__Quality_HQ.Params.ReferenceFrames.Value = 5
        Unrestricted_1pass_Const__Quality_HQ.Params.Trellis.Value = 2
        ret.Add(Unrestricted_1pass_Const__Quality_HQ)

        Dim Unrestricted_1pass_Const__Quality_Insane As New x264Encoder()
        Unrestricted_1pass_Const__Quality_Insane.Name = "x264/Unrestricted/1pass Const. Quality Insane"
        Unrestricted_1pass_Const__Quality_Insane.Params.AdaptiveDCT.Value = True
        Unrestricted_1pass_Const__Quality_Insane.Params.BAdapt.Value = 2
        Unrestricted_1pass_Const__Quality_Insane.Params.BFramesPyramid.Value = True
        Unrestricted_1pass_Const__Quality_Insane.Params.BFramesWeightedPrediction.Value = True
        Unrestricted_1pass_Const__Quality_Insane.Params.DeblockAlpha.Value = -1
        Unrestricted_1pass_Const__Quality_Insane.Params.DeblockBeta.Value = -1
        Unrestricted_1pass_Const__Quality_Insane.Params.I8x8.Value = True
        Unrestricted_1pass_Const__Quality_Insane.Params.MEMethod.Value = 4
        Unrestricted_1pass_Const__Quality_Insane.Params.MeRange.Value = 32
        Unrestricted_1pass_Const__Quality_Insane.Params.MixedRefs.Value = True
        Unrestricted_1pass_Const__Quality_Insane.Params.Mode.Value = 2
        Unrestricted_1pass_Const__Quality_Insane.Params.NoDCTDecimate.Value = True
        Unrestricted_1pass_Const__Quality_Insane.Params.NoFastPSkip.Value = True
        Unrestricted_1pass_Const__Quality_Insane.Params.Quant.Value = 18
        Unrestricted_1pass_Const__Quality_Insane.Params.ReferenceFrames.Value = 16
        Unrestricted_1pass_Const__Quality_Insane.Params.SubME.Value = 9
        Unrestricted_1pass_Const__Quality_Insane.Params.Trellis.Value = 2
        ret.Add(Unrestricted_1pass_Const__Quality_Insane)

        Dim Unrestricted_1pass_Lossless As New x264Encoder()
        Unrestricted_1pass_Lossless.Name = "x264/Unrestricted/1pass Lossless"
        Unrestricted_1pass_Lossless.Params.BFrames.Value = 0
        Unrestricted_1pass_Lossless.Params.DirectMode.Value = 2
        Unrestricted_1pass_Lossless.Params.LoopFilter.Value = False
        Unrestricted_1pass_Lossless.Params.Quant.Value = 1
        Unrestricted_1pass_Lossless.Params.ReferenceFrames.Value = 1
        Unrestricted_1pass_Lossless.Params.SubME.Value = 1
        Unrestricted_1pass_Lossless.Params.VBVMaxRate.Value = 25000
        ret.Add(Unrestricted_1pass_Lossless)

        Dim Unrestricted_1pass_MaxSpeed As New x264Encoder()
        Unrestricted_1pass_MaxSpeed.Name = "x264/Unrestricted/1pass MaxSpeed"
        Unrestricted_1pass_MaxSpeed.Params.AQMode.Value = 0
        Unrestricted_1pass_MaxSpeed.Params.B8x8.Value = False
        Unrestricted_1pass_MaxSpeed.Params.BFrames.Value = 0
        Unrestricted_1pass_MaxSpeed.Params.CABAC.Value = False
        Unrestricted_1pass_MaxSpeed.Params.I4x4.Value = False
        Unrestricted_1pass_MaxSpeed.Params.LoopFilter.Value = False
        Unrestricted_1pass_MaxSpeed.Params.MEMethod.Value = 0
        Unrestricted_1pass_MaxSpeed.Params.Mode.Value = 2
        Unrestricted_1pass_MaxSpeed.Params.P4x4.Value = False
        Unrestricted_1pass_MaxSpeed.Params.P8x8.Value = False
        Unrestricted_1pass_MaxSpeed.Params.ReferenceFrames.Value = 1
        Unrestricted_1pass_MaxSpeed.Params.SubME.Value = 1
        ret.Add(Unrestricted_1pass_MaxSpeed)

        Dim Unrestricted_2pass_Anime_Toons_Balanced As New x264Encoder()
        Unrestricted_2pass_Anime_Toons_Balanced.Name = "x264/Unrestricted/2pass Anime Toons Balanced"
        Unrestricted_2pass_Anime_Toons_Balanced.Params.AdaptiveDCT.Value = True
        Unrestricted_2pass_Anime_Toons_Balanced.Params.AQMode.Value = 0
        Unrestricted_2pass_Anime_Toons_Balanced.Params.BFrames.Value = 16
        Unrestricted_2pass_Anime_Toons_Balanced.Params.BFramesPyramid.Value = True
        Unrestricted_2pass_Anime_Toons_Balanced.Params.DeblockAlpha.Value = 1
        Unrestricted_2pass_Anime_Toons_Balanced.Params.DeblockBeta.Value = 1
        Unrestricted_2pass_Anime_Toons_Balanced.Params.DirectMode.Value = 3
        Unrestricted_2pass_Anime_Toons_Balanced.Params.I8x8.Value = True
        Unrestricted_2pass_Anime_Toons_Balanced.Params.MixedRefs.Value = True
        Unrestricted_2pass_Anime_Toons_Balanced.Params.Mode.Value = 3
        Unrestricted_2pass_Anime_Toons_Balanced.Params.PsyRD.Value = 0.6
        Unrestricted_2pass_Anime_Toons_Balanced.Params.ReferenceFrames.Value = 5
        Unrestricted_2pass_Anime_Toons_Balanced.Params.Trellis.Value = 1
        ret.Add(Unrestricted_2pass_Anime_Toons_Balanced)

        Dim Unrestricted_2pass_Anime_Toons_Extra_Q_ As New x264Encoder()
        Unrestricted_2pass_Anime_Toons_Extra_Q_.Name = "x264/Unrestricted/2pass Anime Toons Extra Q."
        Unrestricted_2pass_Anime_Toons_Extra_Q_.Params.AdaptiveDCT.Value = True
        Unrestricted_2pass_Anime_Toons_Extra_Q_.Params.AQMode.Value = 0
        Unrestricted_2pass_Anime_Toons_Extra_Q_.Params.BFrames.Value = 16
        Unrestricted_2pass_Anime_Toons_Extra_Q_.Params.BFramesPyramid.Value = True
        Unrestricted_2pass_Anime_Toons_Extra_Q_.Params.DeblockAlpha.Value = 1
        Unrestricted_2pass_Anime_Toons_Extra_Q_.Params.DeblockBeta.Value = 1
        Unrestricted_2pass_Anime_Toons_Extra_Q_.Params.DirectMode.Value = 3
        Unrestricted_2pass_Anime_Toons_Extra_Q_.Params.I8x8.Value = True
        Unrestricted_2pass_Anime_Toons_Extra_Q_.Params.MEMethod.Value = 2
        Unrestricted_2pass_Anime_Toons_Extra_Q_.Params.MixedRefs.Value = True
        Unrestricted_2pass_Anime_Toons_Extra_Q_.Params.Mode.Value = 3
        Unrestricted_2pass_Anime_Toons_Extra_Q_.Params.PsyRD.Value = 0.6
        Unrestricted_2pass_Anime_Toons_Extra_Q_.Params.ReferenceFrames.Value = 12
        Unrestricted_2pass_Anime_Toons_Extra_Q_.Params.SubME.Value = 8
        Unrestricted_2pass_Anime_Toons_Extra_Q_.Params.Trellis.Value = 2
        ret.Add(Unrestricted_2pass_Anime_Toons_Extra_Q_)

        Dim Unrestricted_2pass_Anime_Toons_Fast As New x264Encoder()
        Unrestricted_2pass_Anime_Toons_Fast.Name = "x264/Unrestricted/2pass Anime Toons Fast"
        Unrestricted_2pass_Anime_Toons_Fast.Params.AdaptiveDCT.Value = True
        Unrestricted_2pass_Anime_Toons_Fast.Params.AQMode.Value = 0
        Unrestricted_2pass_Anime_Toons_Fast.Params.BFrames.Value = 16
        Unrestricted_2pass_Anime_Toons_Fast.Params.DeblockAlpha.Value = 1
        Unrestricted_2pass_Anime_Toons_Fast.Params.DeblockBeta.Value = 1
        Unrestricted_2pass_Anime_Toons_Fast.Params.DirectMode.Value = 3
        Unrestricted_2pass_Anime_Toons_Fast.Params.I8x8.Value = True
        Unrestricted_2pass_Anime_Toons_Fast.Params.Mode.Value = 3
        Unrestricted_2pass_Anime_Toons_Fast.Params.SubME.Value = 5
        ret.Add(Unrestricted_2pass_Anime_Toons_Fast)

        Dim Unrestricted_2pass_Anime_Toons_HQ As New x264Encoder()
        Unrestricted_2pass_Anime_Toons_HQ.Name = "x264/Unrestricted/2pass Anime Toons HQ"
        Unrestricted_2pass_Anime_Toons_HQ.Params.AdaptiveDCT.Value = True
        Unrestricted_2pass_Anime_Toons_HQ.Params.AQMode.Value = 0
        Unrestricted_2pass_Anime_Toons_HQ.Params.BFrames.Value = 16
        Unrestricted_2pass_Anime_Toons_HQ.Params.BFramesPyramid.Value = True
        Unrestricted_2pass_Anime_Toons_HQ.Params.DeblockAlpha.Value = 1
        Unrestricted_2pass_Anime_Toons_HQ.Params.DeblockBeta.Value = 1
        Unrestricted_2pass_Anime_Toons_HQ.Params.DirectMode.Value = 3
        Unrestricted_2pass_Anime_Toons_HQ.Params.I8x8.Value = True
        Unrestricted_2pass_Anime_Toons_HQ.Params.MEMethod.Value = 2
        Unrestricted_2pass_Anime_Toons_HQ.Params.MixedRefs.Value = True
        Unrestricted_2pass_Anime_Toons_HQ.Params.Mode.Value = 3
        Unrestricted_2pass_Anime_Toons_HQ.Params.PsyRD.Value = 0.6
        Unrestricted_2pass_Anime_Toons_HQ.Params.ReferenceFrames.Value = 8
        Unrestricted_2pass_Anime_Toons_HQ.Params.SubME.Value = 7
        Unrestricted_2pass_Anime_Toons_HQ.Params.Trellis.Value = 2
        ret.Add(Unrestricted_2pass_Anime_Toons_HQ)

        Dim Unrestricted_2pass_Anime_Toons_Insane As New x264Encoder()
        Unrestricted_2pass_Anime_Toons_Insane.Name = "x264/Unrestricted/2pass Anime Toons Insane"
        Unrestricted_2pass_Anime_Toons_Insane.Params.AdaptiveDCT.Value = True
        Unrestricted_2pass_Anime_Toons_Insane.Params.AQMode.Value = 0
        Unrestricted_2pass_Anime_Toons_Insane.Params.BFrames.Value = 16
        Unrestricted_2pass_Anime_Toons_Insane.Params.BFramesPyramid.Value = True
        Unrestricted_2pass_Anime_Toons_Insane.Params.DeblockAlpha.Value = 1
        Unrestricted_2pass_Anime_Toons_Insane.Params.DeblockBeta.Value = 1
        Unrestricted_2pass_Anime_Toons_Insane.Params.DirectMode.Value = 3
        Unrestricted_2pass_Anime_Toons_Insane.Params.I8x8.Value = True
        Unrestricted_2pass_Anime_Toons_Insane.Params.MEMethod.Value = 4
        Unrestricted_2pass_Anime_Toons_Insane.Params.MeRange.Value = 32
        Unrestricted_2pass_Anime_Toons_Insane.Params.MixedRefs.Value = True
        Unrestricted_2pass_Anime_Toons_Insane.Params.Mode.Value = 3
        Unrestricted_2pass_Anime_Toons_Insane.Params.NoDCTDecimate.Value = True
        Unrestricted_2pass_Anime_Toons_Insane.Params.NoFastPSkip.Value = True
        Unrestricted_2pass_Anime_Toons_Insane.Params.PsyRD.Value = 0.6
        Unrestricted_2pass_Anime_Toons_Insane.Params.ReferenceFrames.Value = 16
        Unrestricted_2pass_Anime_Toons_Insane.Params.SubME.Value = 9
        Unrestricted_2pass_Anime_Toons_Insane.Params.Trellis.Value = 2
        ret.Add(Unrestricted_2pass_Anime_Toons_Insane)

        Dim Unrestricted_2pass_Balanced As New x264Encoder()
        Unrestricted_2pass_Balanced.Name = "x264/Unrestricted/2pass Balanced"
        Unrestricted_2pass_Balanced.Params.AdaptiveDCT.Value = True
        Unrestricted_2pass_Balanced.Params.BFrames.Value = 16
        Unrestricted_2pass_Balanced.Params.BFramesPyramid.Value = True
        Unrestricted_2pass_Balanced.Params.BFramesWeightedPrediction.Value = True
        Unrestricted_2pass_Balanced.Params.DeblockAlpha.Value = -1
        Unrestricted_2pass_Balanced.Params.DeblockBeta.Value = -1
        Unrestricted_2pass_Balanced.Params.DirectMode.Value = 3
        Unrestricted_2pass_Balanced.Params.I8x8.Value = True
        Unrestricted_2pass_Balanced.Params.MixedRefs.Value = True
        Unrestricted_2pass_Balanced.Params.Mode.Value = 3
        Unrestricted_2pass_Balanced.Params.ReferenceFrames.Value = 3
        Unrestricted_2pass_Balanced.Params.Trellis.Value = 1
        ret.Add(Unrestricted_2pass_Balanced)

        Dim Unrestricted_2pass_Extra_Quality As New x264Encoder()
        Unrestricted_2pass_Extra_Quality.Name = "x264/Unrestricted/2pass Extra Quality"
        Unrestricted_2pass_Extra_Quality.Params.AdaptiveDCT.Value = True
        Unrestricted_2pass_Extra_Quality.Params.BAdapt.Value = 2
        Unrestricted_2pass_Extra_Quality.Params.BFrames.Value = 3
        Unrestricted_2pass_Extra_Quality.Params.BFramesPyramid.Value = True
        Unrestricted_2pass_Extra_Quality.Params.BFramesWeightedPrediction.Value = True
        Unrestricted_2pass_Extra_Quality.Params.DeblockAlpha.Value = -1
        Unrestricted_2pass_Extra_Quality.Params.DeblockBeta.Value = -1
        Unrestricted_2pass_Extra_Quality.Params.DirectMode.Value = 3
        Unrestricted_2pass_Extra_Quality.Params.I8x8.Value = True
        Unrestricted_2pass_Extra_Quality.Params.MEMethod.Value = 2
        Unrestricted_2pass_Extra_Quality.Params.MixedRefs.Value = True
        Unrestricted_2pass_Extra_Quality.Params.Mode.Value = 3
        Unrestricted_2pass_Extra_Quality.Params.ReferenceFrames.Value = 8
        Unrestricted_2pass_Extra_Quality.Params.SubME.Value = 8
        Unrestricted_2pass_Extra_Quality.Params.Trellis.Value = 2
        ret.Add(Unrestricted_2pass_Extra_Quality)

        Dim Unrestricted_2pass_Fast As New x264Encoder()
        Unrestricted_2pass_Fast.Name = "x264/Unrestricted/2pass Fast"
        Unrestricted_2pass_Fast.Params.AdaptiveDCT.Value = True
        Unrestricted_2pass_Fast.Params.BFrames.Value = 16
        Unrestricted_2pass_Fast.Params.BFramesWeightedPrediction.Value = True
        Unrestricted_2pass_Fast.Params.DeblockAlpha.Value = -1
        Unrestricted_2pass_Fast.Params.DeblockBeta.Value = -1
        Unrestricted_2pass_Fast.Params.DirectMode.Value = 3
        Unrestricted_2pass_Fast.Params.I8x8.Value = True
        Unrestricted_2pass_Fast.Params.Mode.Value = 3
        Unrestricted_2pass_Fast.Params.ReferenceFrames.Value = 3
        Unrestricted_2pass_Fast.Params.SubME.Value = 5
        ret.Add(Unrestricted_2pass_Fast)

        Dim Unrestricted_2pass_HQ As New x264Encoder()
        Unrestricted_2pass_HQ.Name = "x264/Unrestricted/2pass HQ"
        Unrestricted_2pass_HQ.Params.AdaptiveDCT.Value = True
        Unrestricted_2pass_HQ.Params.BAdapt.Value = 2
        Unrestricted_2pass_HQ.Params.BFrames.Value = 3
        Unrestricted_2pass_HQ.Params.BFramesPyramid.Value = True
        Unrestricted_2pass_HQ.Params.BFramesWeightedPrediction.Value = True
        Unrestricted_2pass_HQ.Params.DeblockAlpha.Value = -1
        Unrestricted_2pass_HQ.Params.DeblockBeta.Value = -1
        Unrestricted_2pass_HQ.Params.DirectMode.Value = 3
        Unrestricted_2pass_HQ.Params.I8x8.Value = True
        Unrestricted_2pass_HQ.Params.MEMethod.Value = 2
        Unrestricted_2pass_HQ.Params.MixedRefs.Value = True
        Unrestricted_2pass_HQ.Params.Mode.Value = 3
        Unrestricted_2pass_HQ.Params.ReferenceFrames.Value = 5
        Unrestricted_2pass_HQ.Params.SubME.Value = 7
        Unrestricted_2pass_HQ.Params.Trellis.Value = 2
        ret.Add(Unrestricted_2pass_HQ)

        Dim Unrestricted_2pass_Insane As New x264Encoder()
        Unrestricted_2pass_Insane.Name = "x264/Unrestricted/2pass Insane"
        Unrestricted_2pass_Insane.Params.AdaptiveDCT.Value = True
        Unrestricted_2pass_Insane.Params.BAdapt.Value = 2
        Unrestricted_2pass_Insane.Params.BFramesPyramid.Value = True
        Unrestricted_2pass_Insane.Params.BFramesWeightedPrediction.Value = True
        Unrestricted_2pass_Insane.Params.DeblockAlpha.Value = -1
        Unrestricted_2pass_Insane.Params.DeblockBeta.Value = -1
        Unrestricted_2pass_Insane.Params.DirectMode.Value = 3
        Unrestricted_2pass_Insane.Params.I8x8.Value = True
        Unrestricted_2pass_Insane.Params.MEMethod.Value = 4
        Unrestricted_2pass_Insane.Params.MeRange.Value = 32
        Unrestricted_2pass_Insane.Params.MixedRefs.Value = True
        Unrestricted_2pass_Insane.Params.Mode.Value = 3
        Unrestricted_2pass_Insane.Params.NoDCTDecimate.Value = True
        Unrestricted_2pass_Insane.Params.NoFastPSkip.Value = True
        Unrestricted_2pass_Insane.Params.ReferenceFrames.Value = 16
        Unrestricted_2pass_Insane.Params.SubME.Value = 9
        Unrestricted_2pass_Insane.Params.Trellis.Value = 2
        ret.Add(Unrestricted_2pass_Insane)

        Dim Unrestricted_2pass_QuickTime_for_Windows As New x264Encoder()
        Unrestricted_2pass_QuickTime_for_Windows.Name = "x264/Unrestricted/2pass QuickTime for Windows"
        Unrestricted_2pass_QuickTime_for_Windows.Params.BAdapt.Value = 2
        Unrestricted_2pass_QuickTime_for_Windows.Params.BFrames.Value = 1
        Unrestricted_2pass_QuickTime_for_Windows.Params.DirectMode.Value = 3
        Unrestricted_2pass_QuickTime_for_Windows.Params.MEMethod.Value = 2
        Unrestricted_2pass_QuickTime_for_Windows.Params.Mode.Value = 3
        Unrestricted_2pass_QuickTime_for_Windows.Params.ReferenceFrames.Value = 5
        Unrestricted_2pass_QuickTime_for_Windows.Params.SubME.Value = 7
        Unrestricted_2pass_QuickTime_for_Windows.Params.Trellis.Value = 2
        ret.Add(Unrestricted_2pass_QuickTime_for_Windows)

        Dim xvidQuality As New XviDVirtualDubEncoder
        xvidQuality.Name = "XviD/Constant Quality"
        xvidQuality.Passes = 1
        xvidQuality.QualityMode = True
        xvidQuality.SetQualityModeRegDefaults()
        ret.Add(xvidQuality)

        Dim xvidTwoPass As New XviDVirtualDubEncoder
        xvidTwoPass.Name = "XviD/Exact File Size"
        xvidTwoPass.Passes = 2
        xvidTwoPass.QualityMode = False
        xvidTwoPass.SetTwoPassRegDefaults()
        xvidTwoPass.SetCompCheckRegDefaults()
        ret.Add(xvidTwoPass)

        Dim divxQuality As New DivXVirtualDubEncoder
        divxQuality.Name = "DivX/Constant Quality"
        divxQuality.QualityMode = True
        divxQuality.Passes = 1
        divxQuality.Verify()
        ret.Add(divxQuality)

        Dim divxTwoPass As New DivXVirtualDubEncoder
        divxTwoPass.Name = "DivX/Exact File Size"
        ret.Add(divxTwoPass)

        Dim r = Package.GetSearchResult("DivX264.exe", Nothing, "DivX")

        If r.Count > 0 Then
            Dim divxH264 As New DivXH264Encoder
            divxH264.Name = "DivX/H.264"
            ret.Add(divxH264)
        End If

        Dim x264cli As New CmdlEncoder(StaxRip.StreamType.RAW264)
        x264cli.Name = "Misc/Command Line Example"
        x264cli.Muxer = New MP4Muxer
        x264cli.CommandLines = """%application:x264%"" --pass 1 --bitrate %video_bitrate% --stats ""%working_dir%%target_name%.stats"" --bframes 3 --b-pyramid --deblock -2:-1 --progress --no-psnr --subme 1 --partitions none --me dia --ref 1 --output NUL ""%working_dir%%target_name%.avs""" + CrLf + """%application:x264%"" --pass 2 --bitrate %video_bitrate% --stats ""%working_dir%%target_name%.stats"" --ref 5 --bframes 3 --b-pyramid --weightb --deblock -2:-1 --partitions all --8x8dct --subme 6 --trellis 1 --progress --no-psnr --output ""%encoder_out_file%"" ""%working_dir%%target_name%.avs"""
        x264cli.CompCheckCommandLines = """%application:x264%"" --qp 18 --ref 5 --bframes 3 --b-pyramid --weightb --deblock -2:-1 --partitions all --8x8dct --subme 6 --trellis 1 --progress --no-psnr --output ""%working_dir%%target_name%_CompCheck%encoder_ext%"" ""%working_dir%%target_name%_CompCheck.avs"""
        ret.Add(x264cli)

        ret.Add(New NullEncoder())

        Return ret
    End Function

    Public Function CompareToVideoEncoder(ByVal other As VideoEncoder) As Integer Implements System.IComparable(Of VideoEncoder).CompareTo
        Return Name.CompareTo(other.Name)
    End Function

    Public Overridable Sub RunCompCheck()
    End Sub

    Public Overrides Function GetInfoValues() As Dictionary(Of String, String)
        Dim ret As New Dictionary(Of String, String)

        ret("Encoder Type") = TypeName
        ret("Output File Extension") = GetExtension()
        ret("Muxer Type") = Muxer.TypeName
        ret("Output Format") = DisplayInfoAttribute.GetValueForEnum(Of StreamType)(OutputStreamType)

        Return ret
    End Function

    Public Overrides Function Edit() As DialogResult
        Using f As New ControlHostForm(Name)
            f.AddControl(CreateEditControl)
            f.ShowDialog()
        End Using

        Return DialogResult.OK
    End Function
End Class

<Serializable()> _
Public MustInherit Class VirtualDubEncoder
    Inherits VideoEncoder

    Protected VDMSetCompression As String
    Protected RegistryKey As String
    Protected RegistryRoot As RegistryRoot

    Public RegFirstPass As New SortedList
    Public RegSecondPass As New SortedList
    Public RegCompCheck As New SortedList

    Protected MustOverride Function VerifyRequirements() As Boolean
    Public MustOverride Sub ModifyValues(ByVal sl As SortedList)

    Public Sub New(ByVal name As String, ByVal typeName As String)
        MyBase.New(name, typeName)
        Muxer = New AviMuxer
        OutputStreamType = StaxRip.StreamType.AVI
    End Sub

    Private SupportedPassesValue As Integer

    Public Property SupportedPasses() As Integer
        Get
            Return SupportedPassesValue
        End Get
        Set(ByVal Value As Integer)
            SupportedPassesValue = Value
        End Set
    End Property

    Private SplitSizeValue As Integer

    Public Property SplitSize() As Integer
        Get
            Return SplitSizeValue
        End Get
        Set(ByVal Value As Integer)
            SplitSizeValue = Value
        End Set
    End Property

    Private FourCCValue As String

    Public Property FourCC() As String
        Get
            Return FourCCValue
        End Get
        Set(ByVal Value As String)
            FourCCValue = Value
        End Set
    End Property

    Private UseVirtualDubValue As Boolean

    Public Property UseVirtualDub() As Boolean
        Get
            Return UseVirtualDubValue
        End Get
        Set(ByVal Value As Boolean)
            UseVirtualDubValue = Value
        End Set
    End Property

    Protected Overrides Sub ShowConfigDialog()
        Using f As New VirtualDubEncoderOptionsForm(Me)
            If f.ShowDialog() = DialogResult.OK Then
                OnStateChange()
            End If
        End Using
    End Sub

    Public Overrides Sub RunCompCheck()
        If Not VerifyRequirements() Then Exit Sub
        If Not g.IsValidSource Then Exit Sub

        ProcessForm.ShowForm(False)
        Log.WriteHeader("Compressibility Check")

        Dim vd As New VirtualDubScript
        Dim script As String = Macro.Solve(p.TargetAVSDoc.GetScript) + CrLf

        script += "SelectRangeEvery(" + ((100 \ p.CompCheckRange.Value) * 14).ToString + ",14)" + CrLf

        script.WriteFile(g.TempDir + p.Name + "_CompCheck.avs")

        SetReg(RegCompCheck, False)

        vd.Open(g.TempDir + p.Name + "_CompCheck.avs")
        vd.VideoSetModeFastRecompress()
        vd.VideoSetCompression(VDMSetCompression, 0)
        vd.Save(g.TempDir + p.Name + "_CompCheck.avi", 0)
        vd.SaveScript(g.TempDir + p.Name + "_CompCheck.vcf")

        Dim pw As New Proc

        pw.File = GetVirtualDubModPath()
        pw.Arguments = "/s""" + g.TempDir + p.Name + "_CompCheck.vcf"" /x"
        pw.Minimized = s.RunAppsMinimized.Value
        pw.Wait = True
        pw.Priority = s.ProcessPriority.Value

        ProcessForm.Proc = pw

        Try
            pw.Start()
        Catch ex As AbortException
            ProcessForm.CloseProcessForm()
            Exit Sub
        End Try

        Dim bits As Long = (New FileInfo(g.TempDir + p.Name + "_CompCheck.avi").Length) * 8
        Dim af As New AVIFile

        af.Open(g.TempDir + p.Name + "_CompCheck.avs")
        p.Compressibility = (bits / af.FrameCount) / (p.TargetWidth * p.TargetHeight)
        af.Close()

        OnAfterCompCheck()

        g.MainForm.Assistant()

        Log.WriteLine(CInt(Calc.GetPercent).ToString() + " %")

        ProcessForm.CloseProcessForm()
    End Sub

    Public Overrides Sub Encode()
        Dim startTime As DateTime
        Dim script As New VirtualDubScript

        If Passes > 1 Then
            Log.WriteHeader("First Pass Codec Parameters")
            SetReg(RegFirstPass, True)
            script.Open(p.TargetAVSDoc.Path)
            script.VideoSetModeFastRecompress()
            script.VideoSetCompression(VDMSetCompression, p.VideoBitrate / 8)
            script.Save(g.TempDir + p.Name + "_FirstPass.avi", 0)

            Log.WriteHeader("First Pass Encoding")
            Log.WriteLine(script.Text)

            script.SaveScript(g.TempDir + p.Name + "_FirstPass.vcf")
            startTime = DateTime.Now

            Dim pw As New Proc
            ProcessForm.Proc = pw
            p.TargetAVSDoc.Synchronize()

            pw.File = GetVirtualDubModPath()
            pw.Arguments = "/s""" + g.TempDir + p.Name + "_FirstPass.vcf"" /x"
            pw.Minimized = s.RunAppsMinimized.Value
            pw.Wait = True
            pw.Priority = s.ProcessPriority.Value
            pw.Start()

            Log.WriteHeader("First Pass Encoding Statistics")
            Log.WriteStats(startTime, p.TargetAVSDoc.GetFrames)
        End If

        Dim index As Integer = Passes

        If Passes > 1 Then
            index -= 1
        End If

        For i = 1 To index
            Log.WriteHeader("Single, Second or NTh Pass Codec Parameters")
            SetReg(RegSecondPass, True)
            script.Open(p.TargetAVSDoc.Path)

            If Not UseVirtualDub Then
                script.RemoveInputStreams()
            End If

            script.VideoSetModeFastRecompress()
            script.VideoSetCompression(VDMSetCompression, p.VideoBitrate / 8)

            If TypeOf Muxer Is AviMuxer AndAlso Not UseVirtualDub Then
                Dim indexTrack2 As Integer

                If p.Audio0.File <> "" AndAlso Muxer.IsStreamSupported(p.Audio0.StreamType) Then
                    indexTrack2 = 1
                    script.StreamSetSource(0, p.Audio0.File)
                    script.StreamSetMode(0, AudioMode.DirectStreamCopy)

                    If Not p.Audio0.HandlesDelay AndAlso p.Audio0.Delay <> 0 Then
                        script.StreamSetInterleave(0, p.Audio0.Delay)
                    End If
                End If

                If p.Audio1.File <> "" AndAlso Muxer.IsStreamSupported(p.Audio1.StreamType) Then
                    script.StreamSetSource(indexTrack2, p.Audio1.File)
                    script.StreamSetMode(indexTrack2, AudioMode.DirectStreamCopy)

                    If Not p.Audio1.HandlesDelay AndAlso p.Audio1.Delay <> 0 Then
                        script.StreamSetInterleave(1, p.Audio1.Delay)
                    End If
                End If
            End If

            If i = index Then
                script.Save(OutputPath, SplitSize)
            Else
                script.Save(OutputPath, 0)
            End If

            Log.WriteHeader("Single, Second or NTh Pass Encoding")
            Log.WriteLine(script.Text)

            script.SaveScript(g.TempDir + p.Name + "_SecondPass.vcf")
            startTime = DateTime.Now

            Dim pw As New Proc
            ProcessForm.Proc = pw
            p.TargetAVSDoc.Synchronize()

            If Not ProcessForm.IsVisible Then
                pw.HideAfterStart = True
            End If

            pw.File = GetVirtualDubModPath()
            pw.Arguments = "/s""" + g.TempDir + p.Name + "_SecondPass.vcf"" /x"
            pw.Minimized = s.RunAppsMinimized.Value
            pw.Wait = True
            pw.Priority = s.ProcessPriority.Value
            pw.Start()

            Log.WriteHeader("Single, Second or NTh Pass Encoding Statistics")
            Log.WriteStats(startTime, p.TargetAVSDoc.GetFrames)
        Next

        If SplitSize = 0 AndAlso Not g.FileWasJustWritten(OutputPath) Then
            Log.WriteHeader("Error")
            Log.WriteLine("encoding failed")
            Throw New ErrorAbortException("encoding failed")
        End If
    End Sub

    Public Function GetVirtualDubModPath() As String
        If UseVirtualDub Then
            Return Packs.VirtualDub.Path
        Else
            Return Packs.VirtualDubMod.Path
        End If
    End Function

    Public Sub SetRegList(ByVal key As String, ByVal value As Object)
        SetRegList(key, value, Modes.CompCheck Or Modes.First Or Modes.Second)
    End Sub

    Public Sub SetRegList(ByVal key As String, ByVal value As Object, ByVal flags As Modes)
        If (flags And Modes.First) = Modes.First Then
            RegFirstPass(key) = value
        End If

        If (flags And Modes.Second) = Modes.Second Then
            RegSecondPass(key) = value
        End If

        If (flags And Modes.CompCheck) = Modes.CompCheck Then
            RegCompCheck(key) = value
        End If
    End Sub

    Public Overridable Sub SetReg(ByVal name As String, ByVal value As Object, ByVal writeLog As Boolean)
        If RegistryRoot = RegistryRoot.CurrentUser Then
            Registry.CurrentUser.CreateSubKey(RegistryKey).SetValue(name, value)
        ElseIf RegistryRoot = RegistryRoot.LocalMachine Then
            Registry.LocalMachine.CreateSubKey(RegistryKey).SetValue(name, value)
        End If

        If writeLog AndAlso TypeOf value Is IConvertible Then
            Log.WriteLine(name + ": " + CStr(value))
        End If
    End Sub

    Public Sub SetReg(ByVal sl As SortedList, ByVal log As Boolean)
        If Not sl Is Nothing Then
            ModifyValues(sl)

            For Each i As String In sl.Keys
                SetReg(i, sl(i), log)
            Next
        End If
    End Sub

    Public Function GetReg(ByVal key As String, ByVal value As String) As Object
        If RegistryRoot = RegistryRoot.CurrentUser Then
            Return Registry.CurrentUser.OpenSubKey(key).GetValue(value)
        ElseIf RegistryRoot = RegistryRoot.LocalMachine Then
            Return Registry.LocalMachine.OpenSubKey(key).GetValue(value)
        End If

        Return Nothing
    End Function

    Protected Overridable Function GetRegAll() As SortedList
        Dim sl As New SortedList
        Dim st As String

        If RegistryRoot = RegistryRoot.CurrentUser Then
            For Each st In Registry.CurrentUser.CreateSubKey(RegistryKey).GetValueNames
                sl.Add(st, GetReg(RegistryKey, st))
            Next
        ElseIf RegistryRoot = RegistryRoot.LocalMachine Then
            For Each st In Registry.LocalMachine.CreateSubKey(RegistryKey).GetValueNames
                sl.Add(st, GetReg(RegistryKey, st))
            Next
        End If

        Return sl
    End Function

    Public Sub ShowFirstPassOptionsDialog()
        If Not VerifyRequirements() Then
            Exit Sub
        End If

        SetReg(RegFirstPass, False)
        VFW.ShowEncoderForm(FourCC, g.MainForm.Handle)
        RegFirstPass = GetRegAll()
    End Sub

    Public Sub ShowSecondPassOptionsDialog()
        If Not VerifyRequirements() Then
            Exit Sub
        End If

        SetReg(RegSecondPass, False)
        VFW.ShowEncoderForm(FourCC, g.MainForm.Handle)
        RegSecondPass = GetRegAll()
    End Sub

    Public Sub ShowCompCheckOptionsDialog()
        If Not VerifyRequirements() Then
            Exit Sub
        End If

        SetReg(RegCompCheck, False)
        VFW.ShowEncoderForm(FourCC, g.MainForm.Handle)
        RegCompCheck = GetRegAll()
    End Sub

    Public Overrides Function GetMenu() As List(Of OptionListBoxItem)
        Dim ret As New List(Of OptionListBoxItem)

        If Passes = 1 Then
            Dim secondPassDialog As New OptionListBoxItem
            secondPassDialog.Text = "Codec Configuration"
            secondPassDialog.ToolTip = "Opens the codec configuration dialog"
            secondPassDialog.ClickDelegate = AddressOf ShowSecondPassOptionsDialog
            ret.Add(secondPassDialog)
        Else
            Dim firstPassDialog As New OptionListBoxItem
            firstPassDialog.Text = "First Pass"
            firstPassDialog.ToolTip = "Opens the configuration dialog for the first pass which performs a analysis"
            firstPassDialog.ClickDelegate = AddressOf ShowFirstPassOptionsDialog
            ret.Add(firstPassDialog)

            Dim secondPassDialog As New OptionListBoxItem
            secondPassDialog.Text = "Second Pass"
            secondPassDialog.ToolTip = "Opens the configuration dialog for the second pass which performs the encoding"
            secondPassDialog.ClickDelegate = AddressOf ShowSecondPassOptionsDialog
            ret.Add(secondPassDialog)
        End If

        If Not QualityMode Then
            Dim compCheckDialog As New OptionListBoxItem
            compCheckDialog.Text = "Compressibility Check"
            compCheckDialog.ToolTip = "Opens the configuration dialog for the compressibility check which helps to find an optimal bitrate"
            compCheckDialog.ClickDelegate = AddressOf ShowCompCheckOptionsDialog
            ret.Add(compCheckDialog)

            Dim startCompCheckDialog As New OptionListBoxItem
            startCompCheckDialog.Text = "Run Compressibility Check"
            startCompCheckDialog.ToolTip = "Runs the compressibility check which helps to find an optimal bitrate"
            startCompCheckDialog.ClickDelegate = AddressOf RunCompCheck
            ret.Add(startCompCheckDialog)
        End If

        Dim muxConfig As New OptionListBoxItem
        muxConfig.Text = "Container Configuration"
        muxConfig.ToolTip = "Opens the container configuration dialog"
        muxConfig.ClickDelegate = AddressOf OpenMuxerConfigDialog
        ret.Add(muxConfig)

        Dim optionsDialog As New OptionListBoxItem
        optionsDialog.Text = "Options"
        optionsDialog.ToolTip = "Opens the encoder options dialog"
        optionsDialog.ClickDelegate = AddressOf ShowConfigDialog
        ret.Add(optionsDialog)

        Return ret
    End Function
End Class

<Serializable()> _
Public Class XviDVirtualDubEncoder
    Inherits VirtualDubEncoder

    Public Sub New()
        MyBase.New("XviD", "XviD VirtualDub(Mod) Encoder")
        AutoCompCheckValue = 60
        RegistryKey = "Software\GNU\XviD"
        FourCC = "xvid"
        VDMSetCompression = "0x64697678,0,10000,0"
        SupportedPasses = 2
        RegistryRoot = RegistryRoot.CurrentUser
    End Sub

    Public Sub SetTwoPassRegDefaults()
        RegFirstPass.Clear()

        RegFirstPass("ar_mode") = 0
        RegFirstPass("ar_x") = 4
        RegFirstPass("ar_y") = 3
        RegFirstPass("aspect_ratio") = 0
        RegFirstPass("audio_mode") = 0
        RegFirstPass("audio_rate") = 128
        RegFirstPass("audio_size") = 0
        RegFirstPass("audio_type") = 0
        RegFirstPass("bitrate") = 700
        RegFirstPass("bquant_offset") = 100
        RegFirstPass("bquant_ratio") = 150
        RegFirstPass("Brightness") = 0
        RegFirstPass("chromame") = 1
        RegFirstPass("container_type") = 1
        RegFirstPass("curve_compression_high") = 0
        RegFirstPass("curve_compression_low") = 0
        RegFirstPass("Deblock_UV") = 0
        RegFirstPass("Deblock_Y") = 0
        RegFirstPass("debug") = 0
        RegFirstPass("Dering_UV") = 0
        RegFirstPass("Dering_Y") = 0
        RegFirstPass("desired_quant") = 200
        RegFirstPass("desired_size") = 570000
        RegFirstPass("discard1pass") = 1
        RegFirstPass("display_status") = 0
        RegFirstPass("FilmEffect") = 0
        RegFirstPass("fourcc_used") = 0
        RegFirstPass("fps") = 2
        RegFirstPass("frame_drop_ratio") = 0
        RegFirstPass("full1pass") = 0
        RegFirstPass("gmc") = 0
        RegFirstPass("hours") = 1
        RegFirstPass("interlacing") = 0
        RegFirstPass("keyframe_boost") = 10
        RegFirstPass("kfreduction") = 20
        RegFirstPass("kfthreshold") = 1
        RegFirstPass("lum_masking") = 0
        RegFirstPass("max_bframes") = 2
        RegFirstPass("max_bquant") = 31
        RegFirstPass("max_iquant") = 31
        RegFirstPass("max_key_interval") = 300
        RegFirstPass("max_pquant") = 31
        RegFirstPass("min_bquant") = 1
        RegFirstPass("min_iquant") = 1
        RegFirstPass("min_pquant") = 1
        RegFirstPass("minutes") = 30
        RegFirstPass("mode") = 1
        RegFirstPass("motion_search") = 6
        RegFirstPass("num_zones") = 1
        RegFirstPass("overflow_control_strength") = 5
        RegFirstPass("packed") = 1
        RegFirstPass("par_x") = 1
        RegFirstPass("par_y") = 1
        RegFirstPass("profile") = "(unrestricted)"
        RegFirstPass("qmatrix_inter") = New Byte() {16, 17, 18, 19, 20, 21, 22, 23, 17, 18, 19, 20, 21, 22, 23, 24, 18, 19, 20, 21, 22, 23, 24, 25, 19, 20, 21, 22, 23, 24, 26, 27, 20, 21, 22, 23, 25, 26, 27, 28, 21, 22, 23, 24, 26, 27, 28, 30, 22, 23, 24, 26, 27, 28, 30, 31, 23, 24, 25, 27, 28, 30, 31, 33}
        RegFirstPass("qmatrix_intra") = New Byte() {8, 17, 18, 19, 21, 23, 25, 27, 17, 18, 19, 21, 23, 25, 27, 28, 20, 21, 22, 23, 24, 26, 28, 30, 21, 22, 23, 24, 26, 28, 30, 32, 22, 23, 24, 26, 28, 30, 32, 35, 23, 24, 26, 28, 30, 32, 35, 38, 25, 26, 28, 30, 32, 35, 38, 41, 27, 28, 30, 32, 35, 38, 41, 45}
        RegFirstPass("qpel") = 0
        RegFirstPass("quality") = "General purpose"
        RegFirstPass("quant_type") = 0
        RegFirstPass("rc_averaging_period") = 100
        RegFirstPass("rc_buffer") = 100
        RegFirstPass("rc_reaction_delay_factor") = 16
        RegFirstPass("seconds") = 0
        RegFirstPass("stats") = "\video.pass"
        RegFirstPass("subtitle_size") = 0
        RegFirstPass("target_size") = 665600
        RegFirstPass("tff") = 0
        RegFirstPass("trellis_quant") = 1
        RegFirstPass("turbo") = 0
        RegFirstPass("twopass_max_overflow_degradation") = 5
        RegFirstPass("twopass_max_overflow_improvement") = 5
        RegFirstPass("use_2pass_bitrate") = 0
        RegFirstPass("use_bvop") = 1
        RegFirstPass("vhq_bframe") = 0
        RegFirstPass("vhq_mode") = 1
        RegFirstPass("vop_debug") = 0
        RegFirstPass("zone0_bvop_threshold") = 0
        RegFirstPass("zone0_cartoon_mode") = 0
        RegFirstPass("zone0_chroma_opt") = 0
        RegFirstPass("zone0_frame") = 0
        RegFirstPass("zone0_greyscale") = 0
        RegFirstPass("zone0_mode") = 0
        RegFirstPass("zone0_quant") = 500
        RegFirstPass("zone0_type") = 0
        RegFirstPass("zone0_weight") = 100

        RegSecondPass.Clear()

        RegSecondPass("ar_mode") = 0
        RegSecondPass("ar_x") = 4
        RegSecondPass("ar_y") = 3
        RegSecondPass("aspect_ratio") = 0
        RegSecondPass("audio_mode") = 0
        RegSecondPass("audio_rate") = 128
        RegSecondPass("audio_size") = 0
        RegSecondPass("audio_type") = 0
        RegSecondPass("bitrate") = 1000
        RegSecondPass("bquant_offset") = 100
        RegSecondPass("bquant_ratio") = 150
        RegSecondPass("Brightness") = 0
        RegSecondPass("chromame") = 1
        RegSecondPass("container_type") = 1
        RegSecondPass("curve_compression_high") = 0
        RegSecondPass("curve_compression_low") = 0
        RegSecondPass("Deblock_UV") = 0
        RegSecondPass("Deblock_Y") = 0
        RegSecondPass("debug") = 0
        RegSecondPass("Dering_UV") = 0
        RegSecondPass("Dering_Y") = 0
        RegSecondPass("desired_quant") = 200
        RegSecondPass("desired_size") = 570000
        RegSecondPass("discard1pass") = 1
        RegSecondPass("display_status") = 0
        RegSecondPass("FilmEffect") = 0
        RegSecondPass("fourcc_used") = 0
        RegSecondPass("fps") = 2
        RegSecondPass("frame_drop_ratio") = 0
        RegSecondPass("full1pass") = 0
        RegSecondPass("gmc") = 0
        RegSecondPass("hours") = 1
        RegSecondPass("interlacing") = 0
        RegSecondPass("keyframe_boost") = 10
        RegSecondPass("kfreduction") = 20
        RegSecondPass("kfthreshold") = 1
        RegSecondPass("lum_masking") = 0
        RegSecondPass("max_bframes") = 2
        RegSecondPass("max_bquant") = 31
        RegSecondPass("max_iquant") = 31
        RegSecondPass("max_key_interval") = 300
        RegSecondPass("max_pquant") = 31
        RegSecondPass("min_bquant") = 1
        RegSecondPass("min_iquant") = 1
        RegSecondPass("min_pquant") = 1
        RegSecondPass("minutes") = 30
        RegSecondPass("mode") = 2
        RegSecondPass("motion_search") = 6
        RegSecondPass("num_zones") = 1
        RegSecondPass("overflow_control_strength") = 5
        RegSecondPass("packed") = 1
        RegSecondPass("par_x") = 1
        RegSecondPass("par_y") = 1
        RegSecondPass("profile") = "(unrestricted)"
        RegSecondPass("qmatrix_inter") = New Byte() {16, 17, 18, 19, 20, 21, 22, 23, 17, 18, 19, 20, 21, 22, 23, 24, 18, 19, 20, 21, 22, 23, 24, 25, 19, 20, 21, 22, 23, 24, 26, 27, 20, 21, 22, 23, 25, 26, 27, 28, 21, 22, 23, 24, 26, 27, 28, 30, 22, 23, 24, 26, 27, 28, 30, 31, 23, 24, 25, 27, 28, 30, 31, 33}
        RegSecondPass("qmatrix_intra") = New Byte() {8, 17, 18, 19, 21, 23, 25, 27, 17, 18, 19, 21, 23, 25, 27, 28, 20, 21, 22, 23, 24, 26, 28, 30, 21, 22, 23, 24, 26, 28, 30, 32, 22, 23, 24, 26, 28, 30, 32, 35, 23, 24, 26, 28, 30, 32, 35, 38, 25, 26, 28, 30, 32, 35, 38, 41, 27, 28, 30, 32, 35, 38, 41, 45}
        RegSecondPass("qpel") = 0
        RegSecondPass("quality") = "General purpose"
        RegSecondPass("quant_type") = 0
        RegSecondPass("rc_averaging_period") = 100
        RegSecondPass("rc_buffer") = 100
        RegSecondPass("rc_reaction_delay_factor") = 16
        RegSecondPass("seconds") = 0
        RegSecondPass("stats") = "\video.pass"
        RegSecondPass("subtitle_size") = 0
        RegSecondPass("target_size") = 665600
        RegSecondPass("tff") = 0
        RegSecondPass("trellis_quant") = 1
        RegSecondPass("turbo") = 0
        RegSecondPass("twopass_max_overflow_degradation") = 5
        RegSecondPass("twopass_max_overflow_improvement") = 5
        RegSecondPass("use_2pass_bitrate") = 1
        RegSecondPass("use_bvop") = 1
        RegSecondPass("vhq_bframe") = 0
        RegSecondPass("vhq_mode") = 1
        RegSecondPass("vop_debug") = 0
        RegSecondPass("zone0_bvop_threshold") = 0
        RegSecondPass("zone0_cartoon_mode") = 0
        RegSecondPass("zone0_chroma_opt") = 0
        RegSecondPass("zone0_frame") = 0
        RegSecondPass("zone0_greyscale") = 0
        RegSecondPass("zone0_mode") = 0
        RegSecondPass("zone0_quant") = 500
        RegSecondPass("zone0_type") = 0
        RegSecondPass("zone0_weight") = 100
    End Sub

    Public Sub SetQualityModeRegDefaults()
        RegSecondPass.Clear()

        RegSecondPass("ar_mode") = 0
        RegSecondPass("ar_x") = 4
        RegSecondPass("ar_y") = 3
        RegSecondPass("aspect_ratio") = 0
        RegSecondPass("audio_mode") = 0
        RegSecondPass("audio_rate") = 128
        RegSecondPass("audio_size") = 0
        RegSecondPass("audio_type") = 0
        RegSecondPass("bitrate") = 700
        RegSecondPass("bquant_offset") = 100
        RegSecondPass("bquant_ratio") = 150
        RegSecondPass("Brightness") = 0
        RegSecondPass("chromame") = 1
        RegSecondPass("container_type") = 1
        RegSecondPass("curve_compression_high") = 0
        RegSecondPass("curve_compression_low") = 0
        RegSecondPass("Deblock_UV") = 0
        RegSecondPass("Deblock_Y") = 0
        RegSecondPass("debug") = 0
        RegSecondPass("Dering_UV") = 0
        RegSecondPass("Dering_Y") = 0
        RegSecondPass("desired_quant") = 300
        RegSecondPass("desired_size") = 570000
        RegSecondPass("discard1pass") = 1
        RegSecondPass("display_status") = 0
        RegSecondPass("FilmEffect") = 0
        RegSecondPass("fourcc_used") = 0
        RegSecondPass("fps") = 2
        RegSecondPass("frame_drop_ratio") = 0
        RegSecondPass("full1pass") = 0
        RegSecondPass("gmc") = 0
        RegSecondPass("hours") = 1
        RegSecondPass("interlacing") = 0
        RegSecondPass("keyframe_boost") = 10
        RegSecondPass("kfreduction") = 20
        RegSecondPass("kfthreshold") = 1
        RegSecondPass("lum_masking") = 0
        RegSecondPass("max_bframes") = 2
        RegSecondPass("max_bquant") = 31
        RegSecondPass("max_iquant") = 31
        RegSecondPass("max_key_interval") = 300
        RegSecondPass("max_pquant") = 31
        RegSecondPass("min_bquant") = 1
        RegSecondPass("min_iquant") = 1
        RegSecondPass("min_pquant") = 1
        RegSecondPass("minutes") = 30
        RegSecondPass("mode") = 0
        RegSecondPass("motion_search") = 6
        RegSecondPass("num_zones") = 1
        RegSecondPass("overflow_control_strength") = 5
        RegSecondPass("packed") = 1
        RegSecondPass("par_x") = 1
        RegSecondPass("par_y") = 1
        RegSecondPass("profile") = "(unrestricted)"
        RegSecondPass("qmatrix_inter") = New Byte() {16, 17, 18, 19, 20, 21, 22, 23, 17, 18, 19, 20, 21, 22, 23, 24, 18, 19, 20, 21, 22, 23, 24, 25, 19, 20, 21, 22, 23, 24, 26, 27, 20, 21, 22, 23, 25, 26, 27, 28, 21, 22, 23, 24, 26, 27, 28, 30, 22, 23, 24, 26, 27, 28, 30, 31, 23, 24, 25, 27, 28, 30, 31, 33}
        RegSecondPass("qmatrix_intra") = New Byte() {8, 17, 18, 19, 21, 23, 25, 27, 17, 18, 19, 21, 23, 25, 27, 28, 20, 21, 22, 23, 24, 26, 28, 30, 21, 22, 23, 24, 26, 28, 30, 32, 22, 23, 24, 26, 28, 30, 32, 35, 23, 24, 26, 28, 30, 32, 35, 38, 25, 26, 28, 30, 32, 35, 38, 41, 27, 28, 30, 32, 35, 38, 41, 45}
        RegSecondPass("qpel") = 0
        RegSecondPass("quality") = "General purpose"
        RegSecondPass("quant_type") = 0
        RegSecondPass("rc_averaging_period") = 100
        RegSecondPass("rc_buffer") = 100
        RegSecondPass("rc_reaction_delay_factor") = 16
        RegSecondPass("seconds") = 0
        RegSecondPass("stats") = "\video.pass"
        RegSecondPass("subtitle_size") = 0
        RegSecondPass("target_size") = 665600
        RegSecondPass("tff") = 0
        RegSecondPass("trellis_quant") = 1
        RegSecondPass("turbo") = 0
        RegSecondPass("twopass_max_overflow_degradation") = 5
        RegSecondPass("twopass_max_overflow_improvement") = 5
        RegSecondPass("use_2pass_bitrate") = 0
        RegSecondPass("use_bvop") = 1
        RegSecondPass("vhq_bframe") = 0
        RegSecondPass("vhq_mode") = 1
        RegSecondPass("vop_debug") = 0
        RegSecondPass("zone0_bvop_threshold") = 0
        RegSecondPass("zone0_cartoon_mode") = 0
        RegSecondPass("zone0_chroma_opt") = 0
        RegSecondPass("zone0_frame") = 0
        RegSecondPass("zone0_greyscale") = 0
        RegSecondPass("zone0_mode") = 0
        RegSecondPass("zone0_quant") = 500
        RegSecondPass("zone0_type") = 0
        RegSecondPass("zone0_weight") = 100
    End Sub

    Public Sub SetCompCheckRegDefaults()
        RegCompCheck.Clear()

        RegCompCheck("ar_mode") = 0
        RegCompCheck("ar_x") = 4
        RegCompCheck("ar_y") = 3
        RegCompCheck("aspect_ratio") = 0
        RegCompCheck("audio_mode") = 0
        RegCompCheck("audio_rate") = 128
        RegCompCheck("audio_size") = 0
        RegCompCheck("audio_type") = 0
        RegCompCheck("bitrate") = 700
        RegCompCheck("bquant_offset") = 100
        RegCompCheck("bquant_ratio") = 150
        RegCompCheck("Brightness") = 0
        RegCompCheck("chromame") = 1
        RegCompCheck("container_type") = 1
        RegCompCheck("curve_compression_high") = 0
        RegCompCheck("curve_compression_low") = 0
        RegCompCheck("Deblock_UV") = 0
        RegCompCheck("Deblock_Y") = 0
        RegCompCheck("debug") = 0
        RegCompCheck("Dering_UV") = 0
        RegCompCheck("Dering_Y") = 0
        RegCompCheck("desired_quant") = 200
        RegCompCheck("desired_size") = 570000
        RegCompCheck("discard1pass") = 1
        RegCompCheck("display_status") = 0
        RegCompCheck("FilmEffect") = 0
        RegCompCheck("fourcc_used") = 0
        RegCompCheck("fps") = 2
        RegCompCheck("frame_drop_ratio") = 0
        RegCompCheck("full1pass") = 0
        RegCompCheck("gmc") = 0
        RegCompCheck("hours") = 1
        RegCompCheck("interlacing") = 0
        RegCompCheck("keyframe_boost") = 10
        RegCompCheck("kfreduction") = 20
        RegCompCheck("kfthreshold") = 1
        RegCompCheck("lum_masking") = 0
        RegCompCheck("max_bframes") = 2
        RegCompCheck("max_bquant") = 31
        RegCompCheck("max_iquant") = 31
        RegCompCheck("max_key_interval") = 300
        RegCompCheck("max_pquant") = 31
        RegCompCheck("min_bquant") = 1
        RegCompCheck("min_iquant") = 1
        RegCompCheck("min_pquant") = 1
        RegCompCheck("minutes") = 30
        RegCompCheck("mode") = 0
        RegCompCheck("motion_search") = 6
        RegCompCheck("num_zones") = 1
        RegCompCheck("overflow_control_strength") = 5
        RegCompCheck("packed") = 1
        RegCompCheck("par_x") = 1
        RegCompCheck("par_y") = 1
        RegCompCheck("profile") = "(unrestricted)"
        RegCompCheck("qmatrix_inter") = New Byte() {16, 17, 18, 19, 20, 21, 22, 23, 17, 18, 19, 20, 21, 22, 23, 24, 18, 19, 20, 21, 22, 23, 24, 25, 19, 20, 21, 22, 23, 24, 26, 27, 20, 21, 22, 23, 25, 26, 27, 28, 21, 22, 23, 24, 26, 27, 28, 30, 22, 23, 24, 26, 27, 28, 30, 31, 23, 24, 25, 27, 28, 30, 31, 33}
        RegCompCheck("qmatrix_intra") = New Byte() {8, 17, 18, 19, 21, 23, 25, 27, 17, 18, 19, 21, 23, 25, 27, 28, 20, 21, 22, 23, 24, 26, 28, 30, 21, 22, 23, 24, 26, 28, 30, 32, 22, 23, 24, 26, 28, 30, 32, 35, 23, 24, 26, 28, 30, 32, 35, 38, 25, 26, 28, 30, 32, 35, 38, 41, 27, 28, 30, 32, 35, 38, 41, 45}
        RegCompCheck("qpel") = 0
        RegCompCheck("quality") = "General purpose"
        RegCompCheck("quant_type") = 0
        RegCompCheck("rc_averaging_period") = 100
        RegCompCheck("rc_buffer") = 100
        RegCompCheck("rc_reaction_delay_factor") = 16
        RegCompCheck("seconds") = 0
        RegCompCheck("stats") = "\video.pass"
        RegCompCheck("subtitle_size") = 0
        RegCompCheck("target_size") = 665600
        RegCompCheck("tff") = 0
        RegCompCheck("trellis_quant") = 1
        RegCompCheck("turbo") = 0
        RegCompCheck("twopass_max_overflow_degradation") = 5
        RegCompCheck("twopass_max_overflow_improvement") = 5
        RegCompCheck("use_2pass_bitrate") = 0
        RegCompCheck("use_bvop") = 1
        RegCompCheck("vhq_bframe") = 0
        RegCompCheck("vhq_mode") = 1
        RegCompCheck("vop_debug") = 0
        RegCompCheck("zone0_bvop_threshold") = 0
        RegCompCheck("zone0_cartoon_mode") = 0
        RegCompCheck("zone0_chroma_opt") = 0
        RegCompCheck("zone0_frame") = 0
        RegCompCheck("zone0_greyscale") = 0
        RegCompCheck("zone0_mode") = 0
        RegCompCheck("zone0_quant") = 500
        RegCompCheck("zone0_type") = 0
        RegCompCheck("zone0_weight") = 100
    End Sub

    Private Sub Verify()
        If Passes = 1 Then
            If CInt(RegSecondPass("mode")) <> XviDPasses.Single Then
                SetQualityModeRegDefaults()
            End If
        Else
            If CInt(RegSecondPass("mode")) <> XviDPasses.Second Then
                SetTwoPassRegDefaults()
            End If
        End If

        If CInt(RegCompCheck("mode")) <> XviDPasses.Single Then
            SetCompCheckRegDefaults()
        End If
    End Sub

    Public Overrides Sub ModifyValues(ByVal sl As SortedList)
        Verify()
        Dim frames As Integer = p.TargetAVSDoc.GetFrames
        Dim rate As Double = frames / p.TargetSeconds
        Dim desired_size As Integer = CInt((((p.VideoBitrate * p.TargetSeconds) / 8) / 1.024) + (p.TargetSeconds * rate * 0.024))
        sl("desired_size") = desired_size
        sl("stats") = g.TempDir + p.Name + ".pass"
        sl("bitrate") = CInt(p.VideoBitrate + ((frames * 0.024 * 8 * 1.024) / p.TargetSeconds))

        If p.AutoARSignaling.Value Then
            If Calc.IsARSignalingRequired Then
                Dim par = Calc.GetTargetPAR
                Dim mp4 As PointF() = {New PointF(12, 11), New PointF(10, 11), New PointF(16, 11), New PointF(40, 33)}

                sl("aspect_ratio") = 5

                For x = 0 To mp4.Length - 1
                    If par = mp4(x) Then
                        sl("aspect_ratio") = x + 1
                    End If
                Next

                sl("par_x") = par.X
                sl("par_y") = par.Y
            Else
                sl("aspect_ratio") = 0
                sl("par_x") = 1
                sl("par_y") = 1
            End If
        End If
    End Sub

    Public Enum XviDPasses
        [Single]
        First
        Second
    End Enum

    Protected Overrides Function VerifyRequirements() As Boolean
        If Not Paths.VerifyRequirements Then
            Return False
        End If

        If Not Packs.XviD.VerifyOK(True) Then
            Return False
        End If

        If UseVirtualDub Then
            If Not Packs.VirtualDub.VerifyOK(True) Then
                Return False
            End If
        Else
            If Not Packs.VirtualDubMod.VerifyOK(True) Then
                Return False
            End If
        End If

        Return True
    End Function
End Class

<Serializable()> _
Public Class DivXVirtualDubEncoder
    Inherits VirtualDubEncoder

    Private VersionDate As DateTime
    Private BuildName As String

    Public Sub New()
        MyBase.New("DivX", "DivX VirtualDub(Mod) Encoder")
        AutoCompCheckValue = 60
        RegistryKey = "Software\DivXNetworks\DivX4Windows\State"
        FourCC = "divx"
        VDMSetCompression = "0x78766964,0,10000,0"
        SupportedPasses = 3
        Passes = 2
        RegistryRoot = RegistryRoot.CurrentUser
        Verify()
    End Sub

    Public Sub Verify()
        Dim k = Registry.CurrentUser.OpenSubKey(RegistryKey, True)

        If Not k Is Nothing Then
            k.SetValue(Nothing, "")
        End If

        Dim h = VFW.GetEncoderHandle(FourCC, VFW.ICMODE.QUERY)

        If h = 0 Then
            Exit Sub
        End If

        VFW.ICClose(h)

        BuildName = RegistryHelp.CurrentUser.GetString(RegistryKey, Nothing)

        If BuildName Is Nothing Then
            Exit Sub
        End If

        Dim settingsDir As String = CommonDirs.UserAppDataRoaming + "DivX\DivX Codec"

        If Not Directory.Exists(settingsDir) Then
            settingsDir = DirPath.TrimTrailingSeparator(CommonDirs.Temp)
        End If

        Dim clPass1 As String = "-bvn1 780000 -vbv 4854000,3145728,2359296 -dir """ + settingsDir + """ -w -b 1 -profile=3 -preset=4"
        Dim clPassN As String = "-bvnn 780000 -vbv 4854000,3145728,2359296 -dir """ + settingsDir + """ -w -b 1 -profile=3 -preset=4"
        Dim clCompCheck As String = "-bv1q 2 -vbv 4854000,3145728,2359296 -dir """ + settingsDir + """ -b 1 -profile=3 -preset=4"
        Dim clQualityMode As String = "-bv1q 4 -vbv 4854000,3145728,2359296 -dir """ + settingsDir + """ -b 1 -profile=3 -preset=4"

        If Not RegFirstPass.ContainsKey("args") Then
            SetRegList("args", clPass1, VirtualDubEncoder.Modes.First)
            SetRegList("args", clPassN, VirtualDubEncoder.Modes.Second)
            SetRegList("args", clCompCheck, VirtualDubEncoder.Modes.CompCheck)
        End If

        If QualityMode Then
            If Not RegSecondPass("args").ToString.Contains("-bv1q") AndAlso Not _
                RegSecondPass("args").ToString.Contains("-b1q") Then

                RegSecondPass("args") = clQualityMode
            End If
        ElseIf Passes > 1 Then
            If Not RegSecondPass("args").ToString.Contains("-bvnn") Then
                RegSecondPass("args") = clPassN
            End If
        End If

        If Not RegCompCheck("args").ToString.Contains("-bv1q 2") Then
            RegCompCheck("args") = clCompCheck
        End If
    End Sub

    Public Overrides Sub ModifyValues(ByVal sl As SortedList)
        Verify()
        Dim value As String = sl("args").ToString
        Dim bitrate As New Regex("(-bvn\w) \d+")

        If bitrate.IsMatch(value) Then
            sl("args") = bitrate.Replace(value, "$1 " + (p.VideoBitrate * 1000).ToString)
        End If

        If p.AutoARSignaling.Value Then
            Dim re As New Regex(" -parout \d+:\d+")

            If Calc.IsARSignalingRequired Then
                Dim par = Calc.GetTargetPAR
                Dim arg = " -parout " + par.X.ToString + ":" + par.Y.ToString

                If re.IsMatch(value) Then
                    If Not value.Contains(arg) Then
                        sl("args") = re.Replace(value, arg)
                    End If
                Else
                    sl("args") = value + arg
                End If
            Else
                If re.IsMatch(value) Then
                    sl("args") = re.Replace(value, "")
                End If
            End If
        End If
    End Sub

    Protected Overrides Function VerifyRequirements() As Boolean
        If Not Paths.VerifyRequirements Then
            Return False
        End If

        If Not Packs.DivX.VerifyOK(True) Then
            Return False
        End If

        If UseVirtualDub Then
            If Not Packs.VirtualDub.VerifyOK(True) Then
                Return False
            End If
        Else
            If Not Packs.VirtualDubMod.VerifyOK(True) Then
                Return False
            End If
        End If

        Return True
    End Function

    Public Overrides Sub SetReg(ByVal name As String, ByVal value As Object, ByVal writeLog As Boolean)
        If name = "args" Then
            name = BuildName
        End If

        If RegistryRoot = RegistryRoot.CurrentUser Then
            Registry.CurrentUser.CreateSubKey(RegistryKey).SetValue(name, value)
        ElseIf RegistryRoot = RegistryRoot.LocalMachine Then
            Registry.LocalMachine.CreateSubKey(RegistryKey).SetValue(name, value)
        End If

        If writeLog AndAlso TypeOf value Is IConvertible Then
            Log.WriteLine(name + ": " + CStr(value))
        End If
    End Sub

    Protected Overrides Function GetRegAll() As SortedList
        Dim sl As New SortedList
        Dim k As RegistryKey = Nothing

        If RegistryRoot = RegistryRoot.CurrentUser Then
            k = Registry.CurrentUser
        ElseIf RegistryRoot = RegistryRoot.LocalMachine Then
            k = Registry.LocalMachine
        End If

        For Each i In k.CreateSubKey(RegistryKey).GetValueNames
            If i = BuildName Then
                sl.Add("args", GetReg(RegistryKey, i))
            End If
        Next

        Return sl
    End Function
End Class

<Serializable()> _
Public Class CmdlEncoder
    Inherits VideoEncoder

    Public Sub New(ByVal streamType As StreamType)
        MyBase.New("Command Line", "Command Line Encoder")
        Muxer = New NullMuxer("Misc/Null Muxer")
        OutputStreamType = streamType
    End Sub

    Private CommandLinesValue As String = ""

    Public Property CommandLines() As String
        Get
            Return CommandLinesValue
        End Get
        Set(ByVal Value As String)
            CommandLinesValue = Value
        End Set
    End Property

    Private CompCheckCommandLinesValue As String

    Public Property CompCheckCommandLines() As String
        Get
            Return CompCheckCommandLinesValue
        End Get
        Set(ByVal Value As String)
            CompCheckCommandLinesValue = Value
        End Set
    End Property

    Public Overrides Sub Encode()
        For Each i As String In Macro.Solve(CommandLines).SplitLines
            Log.WriteHeader("Video Encoding")
            Log.WriteLine("Encoder Type: " + TypeName)

            Dim startTime As DateTime = DateTime.Now

            Dim pw As New Proc

            ProcessForm.Proc = pw
            p.TargetAVSDoc.Synchronize()

            pw.CommandLine = i

            Log.WriteLine(i)

            pw.Minimized = s.RunAppsMinimized.Value
            pw.Wait = True
            pw.Priority = s.ProcessPriority.Value
            pw.Start()

            Log.WriteHeader("Video Encoding Statistics")
            Log.WriteStats(startTime, p.TargetAVSDoc.GetFrames)
        Next
    End Sub

    Protected Overrides Sub ShowConfigDialog()
        Using f As New CmdlEncoderForm(Me)
            f.ShowDialog()
        End Using

        OnStateChange()
    End Sub

    Public Overrides Function GetMenu() As List(Of OptionListBoxItem)
        Dim ret As New List(Of OptionListBoxItem)

        Dim configuration As New OptionListBoxItem
        configuration.Text = "Codec Configuration"
        configuration.ToolTip = "Opens the codec configuration dialog"
        configuration.ClickDelegate = AddressOf ShowConfigDialog
        ret.Add(configuration)

        If IsCompCheckEnabled Then
            Dim startCompCheckDialog As New OptionListBoxItem
            startCompCheckDialog.Text = "Run Compressibility Check"
            startCompCheckDialog.ToolTip = "Runs the compressibility check which helps to find a optimal bitrate"
            startCompCheckDialog.ClickDelegate = AddressOf RunCompCheck
            ret.Add(startCompCheckDialog)
        End If

        Dim muxConfig As New OptionListBoxItem
        muxConfig.Text = "Container Configuration"
        muxConfig.ToolTip = "Opens the container configuration dialog"
        muxConfig.ClickDelegate = AddressOf OpenMuxerConfigDialog
        ret.Add(muxConfig)

        Return ret
    End Function

    Public Overrides Sub RunCompCheck()
        If Not Paths.VerifyRequirements Then Exit Sub
        If Not g.IsValidSource Then Exit Sub

        ProcessForm.ShowForm(False)
        Log.WriteHeader("Compressibility Check")

        Dim doc As New AviSynthDocument

        For Each i As AviSynthFilter In p.TargetAVSDoc.Filters
            doc.Filters.Add(i.GetCopy)
        Next

        doc.Filters.Add(New AviSynthFilter("SelectRangeEvery", "SelectRangeEvery", "SelectRangeEvery(" + ((100 \ p.CompCheckRange.Value) * 14).ToString + ",14)", True))
        doc.Path = g.TempDir + p.Name + "_CompCheck.avs"
        doc.Synchronize()

        For Each i As String In Macro.Solve(CompCheckCommandLines).SplitLines
            Dim pw As New Proc

            ProcessForm.Proc = pw

            pw.CommandLine = i

            Log.WriteLine("Command Line: " + pw.CommandLine)

            pw.Minimized = s.RunAppsMinimized.Value
            pw.Wait = True
            pw.Priority = s.ProcessPriority.Value

            Try
                pw.Start()
            Catch ex As AbortException
                ProcessForm.CloseProcessForm()
                Exit Sub
            End Try
        Next

        Dim bits As Long = (New FileInfo(g.TempDir + p.Name + "_CompCheck" + GetExtension()).Length) * 8
        p.Compressibility = (bits / doc.GetFrames) / (p.TargetWidth * p.TargetHeight)

        OnAfterCompCheck()

        g.MainForm.Assistant()

        Log.WriteLine(CInt(Calc.GetPercent).ToString() + " %")

        ProcessForm.CloseProcessForm()
    End Sub
End Class

'legacy 1.1.0.1
<Serializable()> _
Public Class SourceNullEncoder
    Inherits NullEncoder

    Protected Overrides Sub ShowConfigDialog()
    End Sub
End Class

<Serializable()> _
Public Class NullEncoder
    Inherits VideoEncoder

    Public Sub New()
        MyBase.New("Misc/Pass through to remux", "Null Encoder")
        Muxer = New NullMuxer("Misc/Null Muxer")
        QualityMode = True
    End Sub

    Public Overrides ReadOnly Property OutputPath() As String
        Get
            Dim base = g.TempDir + p.Name + "_EncoderOutput"

            If File.Exists(base + ".264") Then
                Return base + ".264"
            ElseIf File.Exists(base + ".h264") Then
                Return base + ".h264"
            End If

            If Filepath.GetExt(p.SourceFile) = ".dga" Then
                For Each i In ".avc .264 .h264".Split(" ")
                    base = Filepath.GetDirAndBase(p.SourceFile) + i

                    If File.Exists(base) Then
                        Return base
                    End If
                Next
            End If

            Return p.SourceFile
        End Get
    End Property

    Public Overrides Property OutputStreamType() As StreamType
        Get
            Return StreamTypeUtils.GetStreamType(Filepath.GetExt(OutputPath))
        End Get
        Set(ByVal value As StreamType)
        End Set
    End Property

    Public Overrides Sub Encode()
    End Sub

    Public Overrides Function GetError() As String
        If Not File.Exists(OutputPath) Then
            Return "Encoder output file not found."
        End If

        Return Nothing
    End Function

    Public Overrides Function GetMenu() As List(Of OptionListBoxItem)
        Dim ret As New List(Of OptionListBoxItem)

        Dim muxConfig As New OptionListBoxItem
        muxConfig.Text = "Container Configuration"
        muxConfig.ToolTip = "Opens the container configuration dialog"
        muxConfig.ClickDelegate = AddressOf OpenMuxerConfigDialog
        ret.Add(muxConfig)

        Return ret
    End Function

    Protected Overrides Sub ShowConfigDialog()
    End Sub
End Class

<Serializable()> _
Public Class x264Encoder
    Inherits H264CmdlEncoder

    Public Sub New()
        MyBase.New("x264", "x264 Encoder")
    End Sub

    Private ParamsValue As New x264Params

    Public Property Params() As x264Params
        Get
            Return ParamsValue
        End Get
        Set(ByVal Value As x264Params)
            ParamsValue = Value
        End Set
    End Property

    Public Overrides Sub Encode()
        p.TargetAVSDoc.Synchronize()
        Dim exe = Packs.x264.Path
        Dim skip = " kb/s, eta "

        Encode("x264", exe, GetArgs(1), True, skip)

        If Params.Mode.Value = x264Mode.TwoPass OrElse Params.Mode.Value = x264Mode.ThreePass Then
            Encode("x264 Second Pass", exe, GetArgs(2), True, skip)
        End If

        If Params.Mode.Value = x264Mode.ThreePass Then
            Encode("x264 Third Pass", exe, GetArgs(3), True, skip)
        End If

        If Not g.FileWasJustWritten(OutputPath) Then
            Log.WriteHeader("Error")
            Log.WriteLine("x264 encoding failed")
            Throw New ErrorAbortException("x264 encoding failed")
        Else
            Log.WriteFilesize(OutputPath)
        End If
    End Sub

    Protected Overrides Sub ShowConfigDialog()
        Using f As New x264Form(Me)
            If f.ShowDialog() = DialogResult.OK Then
                OnStateChange()
            End If
        End Using
    End Sub

    Private Function IsTurboPass(ByVal pass As Integer) As Boolean
        If Params.Mode.Value = x264Mode.TwoPass AndAlso pass <> 2 Then
            Return True
        End If

        If Params.Mode.Value = x264Mode.ThreePass AndAlso pass <> 3 Then
            Return True
        End If

        Return False
    End Function

    Public Function GetArgs(ByVal pass As Integer) As String
        Return GetArgs(pass, p.TargetAVSDoc.Path, Filepath.GetDirAndBase(OutputPath) + GetExtension())
    End Function

    Public Function GetArgs(ByVal pass As Integer, ByVal sourcePath As String, ByVal targetPath As String) As String
        Dim sb As New StringBuilder

        If Params.Mode.Value = x264Mode.TwoPass Then
            sb.Append(" --pass " & pass)
        ElseIf Params.Mode.Value = x264Mode.ThreePass Then
            Select Case pass
                Case 1
                    sb.Append(" --pass 1")
                Case 2
                    sb.Append(" --pass 3")
                Case 3
                    sb.Append(" --pass 2")
            End Select
        End If

        If Params.Mode.Value = x264Mode.SingleQuant Then
            sb.Append(" --qp " + CInt(Params.Quant.Value).ToString)
        ElseIf Params.Mode.Value = x264Mode.SingleCRF Then
            sb.Append(" --crf " + Params.Quant.Value.ToString(CultureInfo.InvariantCulture))
        Else
            sb.Append(" --bitrate " + p.VideoBitrate.ToString)
        End If

        If Params.Mode.Value = x264Mode.TwoPass OrElse Params.Mode.Value = x264Mode.ThreePass Then
            sb.Append(" --stats """ + g.TempDir + p.Name + ".stats""")
        End If

        If Params.Level.Value <> 15 Then
            sb.Append(" --level " + DisplayInfoAttribute.GetValueForEnum(Of x264LevelMode)(CType(Params.Level.Value, x264LevelMode)))
        End If

        If Params.GOPSizeMax.Value <> 250 Then
            sb.Append(" --keyint " + Params.GOPSizeMax.Value.ToString)
        End If

        If Params.GOPSizeMin.Value <> 25 Then
            sb.Append(" --min-keyint " + Params.GOPSizeMin.Value.ToString)
        End If

        If Params.ReferenceFrames.Value <> 1 Then
            sb.Append(" --ref " + Params.ReferenceFrames.Value.ToString)
        End If

        If Params.MixedRefs.Value Then
            sb.Append(" --mixed-refs")
        End If

        If Params.NoFastPSkip.Value Then
            sb.Append(" --no-fast-pskip")
        End If

        If Params.BFrames.Value > 0 Then
            sb.Append(" --bframes " + Params.BFrames.Value.ToString)

            If Params.BAdapt.Value <> x264BAdaptMode.Fast Then
                sb.Append(" --b-adapt " + Params.BAdapt.Value.ToString)
            End If

            If Params.BFramesPyramid.Value Then
                sb.Append(" --b-pyramid")
            End If

            If Params.BFramesBias.Value <> 0 Then
                sb.Append(" --b-bias " + Params.BFramesBias.Value.ToString)
            End If

            If Params.BFramesWeightedPrediction.Value Then
                sb.Append(" --weightb")
            End If

            If Params.DirectMode.Value <> x264DirectMode.spatial Then
                sb.Append(" --direct " + CType(Params.DirectMode.Value, x264DirectMode).ToString)
            End If
        End If

        If Not Params.LoopFilter.Value Then
            sb.Append(" --nf")
        Else
            If Params.DeblockAlpha.Value <> 0 OrElse Params.DeblockBeta.Value <> 0 Then
                sb.Append(" --deblock " + Params.DeblockAlpha.Value.ToString + ":" + Params.DeblockBeta.Value.ToString)
            End If
        End If

        If Not Params.CABAC.Value Then
            sb.Append(" --no-cabac")
        End If

        If Params.SubME.Value <> 6 Then
            sb.Append(" --subme " + Params.SubME.Value.ToString)
        End If

        If Params.Trellis.Value <> 0 Then
            sb.Append(" --trellis " + Params.Trellis.Value.ToString)
        End If

        If Params.SubME.Value > 5 AndAlso (Params.PsyRD.Value <> 1.0F OrElse Params.PsyTrellis.Value <> 0.0F) Then
            sb.Append(" --psy-rd " + Params.PsyRD.Value.ToString(CultureInfo.InvariantCulture) + ":" + Params.PsyTrellis.Value.ToString(CultureInfo.InvariantCulture))
        End If

        Dim l As New List(Of String)

        If Params.P8x8.Value Then
            l.Add("p8x8")
        End If

        If Params.B8x8.Value Then
            l.Add("b8x8")
        End If

        If Params.I4x4.Value Then
            l.Add("i4x4")
        End If

        If Params.P4x4.Value Then
            l.Add("p4x4")
        End If

        If Params.I8x8.Value Then
            l.Add("i8x8")
        End If

        If l.Count = 0 Then
            sb.Append(" --partitions none")
        ElseIf l.Count = 5 Then
            sb.Append(" --partitions all")
        Else
            sb.Append(" --partitions " + String.Join(",", l.ToArray))
        End If

        If Params.AdaptiveDCT.Value Then
            sb.Append(" --8x8dct")
        End If

        If Params.IPRatio.Value <> 1.4F Then
            sb.Append(" --ipratio " + Params.IPRatio.Value.ToString(CultureInfo.InvariantCulture))
        End If

        If Params.PBRatio.Value <> 1.3F Then
            sb.Append(" --pbratio " + Params.PBRatio.Value.ToString(CultureInfo.InvariantCulture))
        End If

        If Params.QPMin.Value <> 10 Then
            sb.Append(" --qpmin " + Params.QPMin.Value.ToString)
        End If

        If Params.VBVBufSize.Value > 0 Then
            sb.Append(" --vbv-bufsize " + Params.VBVBufSize.Value.ToString)
        End If

        If Params.VBVMaxRate.Value > 0 Then
            sb.Append(" --vbv-maxrate " + Params.VBVMaxRate.Value.ToString)
        End If

        If Params.VBVInit.Value <> 0.9F Then
            sb.Append(" --vbv-init " + Params.VBVInit.Value.ToString(CultureInfo.InvariantCulture))
        End If

        If Params.QComp.Value <> 0.6F Then
            sb.Append(" --qcomp " + Params.QComp.Value.ToString(CultureInfo.InvariantCulture))
        End If

        If Params.NoChromaMe.Value Then
            sb.Append(" --no-chroma-me")
        End If

        If Params.MEMethod.Value <> MEMethodMode.hex Then
            sb.Append(" --me " + CType(Params.MEMethod.Value, MEMethodMode).ToString)
        End If

        If Params.MeRange.Value <> 16 Then
            sb.Append(" --merange " + Params.MeRange.Value.ToString)
        End If

        If Params.SceneCut.Value <> 40 Then
            sb.Append(" --scenecut " + Params.SceneCut.Value.ToString)
        End If

        If Params.Threads.Value = 0 Then
            sb.Append(" --threads auto")
        ElseIf Params.Threads.Value > 1 Then
            sb.Append(" --threads " + Params.Threads.Value.ToString)
        End If

        If Params.AQMode.Value <> 1 Then
            sb.Append(" --aq-mode " + Params.AQMode.Value.ToString)
        End If

        If Params.AQMode.Value <> 0 AndAlso Params.AQStrength.Value <> 10 Then
            sb.Append(" --aq-strength " + (Params.AQStrength.Value / 10).ToString("f1", CultureInfo.InvariantCulture))
        End If

        If Params.Progress.Value Then
            sb.Append(" --progress")
        End If

        If Params.NoDCTDecimate.Value Then
            sb.Append(" --no-dct-decimate")
        End If

        If Params.ThreadInput.Value AndAlso Params.Threads.Value <> 0 Then
            sb.Append(" --thread-input")
        End If

        If Calc.IsARSignalingRequired Then
            Dim par = Calc.GetTargetPAR
            sb.Append(" --sar " & par.X & ":" & par.Y)
        End If

        If Params.NoPSNR.Value Then
            sb.Append(" --no-psnr")
        End If

        If Params.NoSSIM.Value Then
            sb.Append(" --no-ssim")
        End If

        If Params.AddAll.Value <> "" Then
            sb.Append(" " + Params.AddAll.Value)
        End If

        Dim ret = sb.ToString

        If IsTurboPass(pass) Then
            Dim switches = Params.TurboRemove.Value.Split(" ")

            For Each i In switches
                Dim re As New Regex(" *" + i + ".*?(?= --)| *" + i + ".*")

                If re.IsMatch(ret) Then
                    ret = re.Replace(ret, "", RegexOptions.IgnoreCase)
                End If
            Next

            If Params.TurboAdd.Value <> "" Then
                ret += " " + Params.TurboAdd.Value
            End If
        End If

        If (Params.Mode.Value = x264Mode.ThreePass AndAlso (pass = 1 OrElse pass = 2)) OrElse Params.Mode.Value = x264Mode.TwoPass AndAlso pass = 1 Then
            ret += " --output NUL """ + sourcePath + """"
        Else
            ret += " --output """ + targetPath + """ """ + sourcePath + """"
        End If

        Return ret.Trim
    End Function

    Public Overrides Sub RunCompCheck()
        Dim avsPath = g.TempDir + p.Name + "_CompCheck.avs"
        Dim enc As New x264Encoder
        enc.Params = ObjectHelp.GetCopy(Of x264Params)(Params)
        enc.Params.Mode.Value = x264Mode.SingleCRF
        enc.Params.Quant.Value = enc.Params.QuantCompCheck.Value
        Dim args = enc.GetArgs(0, avsPath, g.TempDir + p.Name + "_CompCheck" + GetExtension())
        RunCompCheck(Packs.x264.Path, args, True, "encoded frames: ")
    End Sub

    Public Overrides Property QualityMode() As Boolean
        Get
            Return Params.Mode.Value = x264Mode.SingleQuant OrElse Params.Mode.Value = x264Mode.SingleCRF
        End Get
        Set(ByVal Value As Boolean)
        End Set
    End Property
End Class

<Serializable()> _
Public MustInherit Class H264CmdlEncoder
    Inherits VideoEncoder

    Public Sub New(ByVal profileName As String, ByVal profileTypeName As String)
        MyBase.New(profileName, profileTypeName)
        AutoCompCheckValue = 50
        Muxer = New MkvMuxer
    End Sub

    Public Overrides Property OutputStreamType() As StreamType
        Get
            Return StaxRip.StreamType.RAW264
        End Get
        Set(ByVal value As StreamType)
        End Set
    End Property

    Public Overrides Function GetMenu() As List(Of OptionListBoxItem)
        Dim r As New List(Of OptionListBoxItem)

        Dim conf As New OptionListBoxItem
        conf.Text = "Codec Configuration"
        conf.ToolTip = "Opens the codec configuration dialog"
        conf.ClickDelegate = AddressOf ShowConfigDialog
        r.Add(conf)

        If IsCompCheckEnabled Then
            Dim compCheck As New OptionListBoxItem
            compCheck.Text = "Run Compressibility Check"
            compCheck.ToolTip = "Runs the compressibility check which helps to find a optimal bitrate"
            compCheck.ClickDelegate = AddressOf RunCompCheck
            r.Add(compCheck)
        End If

        Dim mux As New OptionListBoxItem
        mux.Text = "Container Configuration"
        mux.ToolTip = "Opens the container configuration dialog"
        mux.ClickDelegate = AddressOf OpenMuxerConfigDialog
        r.Add(mux)

        Return r
    End Function

    Public Overloads Sub Encode(ByVal passName As String, ByVal exe As String, ByVal args As String, ByVal isErrorStream As Boolean, ByVal ParamArray logValuesToSkip As String())
        Dim startTime = DateTime.Now

        Using proc As New Proc
            If isErrorStream Then
                AddHandler proc.Process.ErrorDataReceived, AddressOf ProcessForm.CmdlDataHandler
                proc.BeginErrorReadLine = True
            Else
                AddHandler proc.Process.OutputDataReceived, AddressOf ProcessForm.CmdlDataHandler
                proc.BeginOutputReadLine = True
            End If

            proc.File = exe
            proc.Arguments = args

            Log.WriteHeader(passName)
            Log.WriteLine(proc.CommandLine + CrLf2)

            ProcessForm.Proc = proc

            proc.Wait = True
            proc.Priority = s.ProcessPriority.Value
            proc.Start()

            If isErrorStream Then
                RemoveHandler proc.Process.ErrorDataReceived, AddressOf ProcessForm.CmdlDataHandler
            Else
                RemoveHandler proc.Process.OutputDataReceived, AddressOf ProcessForm.CmdlDataHandler
            End If

            If proc.ExitCode <> 0 Then
                Log.WriteHeader("Error")
                Dim err = TypeName + " failed with exit code " & proc.ExitCode & CrLf2 + ProcessForm.CmdlLog.ToString()
                Log.WriteLine(err)
                Throw New ErrorAbortException(err)
            End If

            ProcessForm.LogCmdlOutput(logValuesToSkip)
        End Using

        Log.WriteHeader(passName + " Statistics")
        Log.WriteStats(startTime)
    End Sub

    Protected Overloads Sub RunCompCheck(ByVal executable As String, ByVal arguments As String, ByVal isErrorStream As Boolean, ByVal ParamArray logValuesToSkip As String())
        If Not Paths.VerifyRequirements Then Exit Sub
        If Not g.IsValidSource Then Exit Sub

        ProcessForm.ShowForm(False)
        Log.WriteHeader("Compressibility Check")

        Dim avsPath = g.TempDir + p.Name + "_CompCheck.avs"
        Dim script = Macro.Solve(p.TargetAVSDoc.GetScript) + CrLf + _
            "SelectRangeEvery(" + ((100 \ p.CompCheckRange.Value) * 14).ToString + ",14)" + CrLf + _
            "ConvertToYV12()"

        script.WriteFile(avsPath)

        Using proc As New Proc
            If isErrorStream Then
                AddHandler proc.Process.ErrorDataReceived, AddressOf ProcessForm.CmdlDataHandler
            Else
                AddHandler proc.Process.OutputDataReceived, AddressOf ProcessForm.CmdlDataHandler
            End If

            proc.BeginErrorReadLine = True
            proc.File = executable
            proc.Arguments = arguments

            Log.WriteLine(proc.CommandLine)

            proc.Wait = True
            proc.Priority = s.ProcessPriority.Value

            ProcessForm.Proc = proc

            Try
                proc.Start()

                If proc.ExitCode <> 0 Then
                    ProcessForm.CloseProcessForm()
                    Msg.Show(TypeName + " failed with exit code " & proc.ExitCode & CrLf2 + ProcessForm.CmdlLog.ToString(), MsgIcon.Error)
                    Exit Sub
                End If
            Catch ex As AbortException
                ProcessForm.CloseProcessForm()
                Exit Sub
            Finally
                If isErrorStream Then
                    RemoveHandler proc.Process.ErrorDataReceived, AddressOf ProcessForm.CmdlDataHandler
                Else
                    RemoveHandler proc.Process.OutputDataReceived, AddressOf ProcessForm.CmdlDataHandler
                End If
            End Try
        End Using

        ProcessForm.LogCmdlOutput(logValuesToSkip)

        Dim bits = (New FileInfo(g.TempDir + p.Name + "_CompCheck" + GetExtension()).Length) * 8
        Dim af As New AVIFile

        af.Open(avsPath)
        p.Compressibility = (bits / af.FrameCount) / (p.TargetWidth * p.TargetHeight)
        af.Close()

        OnAfterCompCheck()
        g.MainForm.Assistant()
        Log.WriteLine(CInt(Calc.GetPercent).ToString() + " %")
        ProcessForm.CloseProcessForm()
    End Sub
End Class

<Serializable()> _
Public Class DivXH264Encoder
    Inherits H264CmdlEncoder

    Public BRef As Boolean = True
    Public FieldType As Integer
    Public GopLength As Integer = 4
    Public InterlaceMode As Integer
    Public MaxConsBframes As Integer = 2
    Public Mode As DivXH264Mode = DivXH264Mode.TwoPass
    Public NoProgress As Boolean
    Public Pyramid As Boolean = True
    Public QualityOptimization As Integer = 1
    Public RefFrames As Integer = 4
    Public Threads As Integer
    Public VerboseLevel As Integer = -1

    Public Sub New()
        MyBase.New("DivX", "DivX H.264 Encoder")
    End Sub

    Public Overrides Property QualityMode() As Boolean
        Get
            Return False
        End Get
        Set(ByVal Value As Boolean)
        End Set
    End Property

    Protected Overrides Sub ShowConfigDialog()
        Using f As New SettingTreeViewForm("DivX H.264 Encoder Options")
            Dim tv = f.SettingTreeView
            tv.FieldOwner = Me

            tv.AddBool("Input/Output", "No Progress", "NoProgress", "Do not display progress information.")
            tv.AddCombo("Input/Output", "Verbose Level", "VerboseLevel", Nothing, -1, "Disabled", "Simple", "Verbose")
            tv.AddInt("Input/Output", "Threads", "Threads", 0, 32, 1)
            tv.AddCombo(Of DivXH264Mode)("Rate Control", "Mode", "Mode")
            tv.AddCombo("Rate Control", "Quality Optimization", "QualityOptimization", Nothing, "Fast", "Balanced", "High Quality")
            tv.AddInt("Frame Options", "Gop Length", "GopLength", Nothing, 1, 4, 1)
            tv.AddCombo("Frame Options", "Field Type", "FieldType", Nothing, "Progressive", "Top Field First", "Bottom Field First")
            tv.AddCombo("Frame Options", "Interlace Mode", "InterlaceMode", Nothing, "Disabled", "MBAFF", "Field")
            tv.AddInt("Frame Options", "Reference Frames", "RefFrames", 1, 4, 1)
            tv.AddBool("Frame Options", "B-Frames as references", "BRef")
            tv.AddBool("Frame Options", "B-Frames Pyramid", "Pyramid")
            tv.AddInt("Frame Options", "Maximum consecutive B-Frames", "MaxConsBframes", 1, 3, 1)

            f.ShowDialog()
        End Using
    End Sub

    Public Overrides Sub Encode()
        p.TargetAVSDoc.Synchronize()
        Dim exe = Package.GetSearchResult("DivX264.exe", Nothing, "DivX")(0)
        Encode("DivX H.264", exe, GetArgs(1), False, "[time: ")

        If Mode = DivXH264Mode.TwoPass Then
            Encode("DivX H.264 Second Pass", exe, GetArgs(2), False, "[time: ")
        End If

        If Not g.FileWasJustWritten(OutputPath) Then
            Log.WriteHeader("Error")
            Log.WriteLine("DivX H.264 encoding failed")
            Throw New ErrorAbortException("DivX H.264 encoding failed")
        Else
            Log.WriteFilesize(OutputPath)
        End If
    End Sub

    Public Function GetArgs(ByVal pass As Integer) As String
        Return GetArgs(pass, p.TargetAVSDoc.Path, Filepath.GetDirAndBase(OutputPath) + GetExtension())
    End Function

    Public Function GetArgs(ByVal pass As Integer, ByVal sourcePath As String, ByVal targetPath As String) As String
        Dim cb As New CmdlBuilder

        cb.SB.Append("-i """ + sourcePath + """")
        cb.SB.Append(" -o """ + targetPath + """")

        Select Case Mode
            Case DivXH264Mode.TwoPass
                cb.SB.Append(" -npass " & pass)
                cb.SB.Append(" -sf """ + g.TempDir + p.Name + ".stats""")
                cb.SB.Append(" -br " & p.VideoBitrate)
            Case DivXH264Mode.Bitrate
                cb.SB.Append(" -br " & p.VideoBitrate)
        End Select

        cb.Add(BRef, " -bref")
        cb.Add(Pyramid AndAlso BRef, " -pyramid")
        cb.Add(FieldType = 1, " -tff")
        cb.Add(FieldType = 2, " -bff")
        cb.Add(GopLength, 4, " -I ")
        cb.Add(InterlaceMode, 0, " -fmode ")
        cb.Add(MaxConsBframes, 2, " -bf ")
        cb.Add(NoProgress, " -noprogress")
        cb.Add(QualityOptimization, 1, " -aqo ")
        cb.Add(RefFrames, 4, " -ref ")
        cb.Add(Threads, 0, " -threads ")
        cb.Add(VerboseLevel, -1, " -v ")

        Return cb.SB.ToString.Trim
    End Function

    Public Overrides Sub RunCompCheck()
        Dim exe = Package.GetSearchResult("DivX264.exe", Nothing, "DivX")(0)
        Dim enc = ObjectHelp.GetCopy(Of DivXH264Encoder)(Me)
        enc.Mode = DivXH264Mode.Bitrate
        Dim args = enc.GetArgs(0, g.TempDir + p.Name + "_CompCheck.avs", g.TempDir + p.Name + "_CompCheck" + GetExtension())
        RunCompCheck(exe, args, False)
    End Sub

    Public Overrides ReadOnly Property IsCompCheckEnabled() As Boolean
        Get
            Return False
        End Get
    End Property
End Class

Public Enum DivXH264Mode
    Bitrate
    <DisplayInfo("Two Pass")> TwoPass
End Enum